var Q1=Object.defineProperty;var K1=(l,i,o)=>i in l?Q1(l,i,{enumerable:!0,configurable:!0,writable:!0,value:o}):l[i]=o;var Ut=(l,i,o)=>K1(l,typeof i!="symbol"?i+"":i,o);function Z1(l,i){for(var o=0;o<i.length;o++){const r=i[o];if(typeof r!="string"&&!Array.isArray(r)){for(const c in r)if(c!=="default"&&!(c in l)){const f=Object.getOwnPropertyDescriptor(r,c);f&&Object.defineProperty(l,c,f.get?f:{enumerable:!0,get:()=>r[c]})}}}return Object.freeze(Object.defineProperty(l,Symbol.toStringTag,{value:"Module"}))}(function(){const i=document.createElement("link").relList;if(i&&i.supports&&i.supports("modulepreload"))return;for(const c of document.querySelectorAll('link[rel="modulepreload"]'))r(c);new MutationObserver(c=>{for(const f of c)if(f.type==="childList")for(const d of f.addedNodes)d.tagName==="LINK"&&d.rel==="modulepreload"&&r(d)}).observe(document,{childList:!0,subtree:!0});function o(c){const f={};return c.integrity&&(f.integrity=c.integrity),c.referrerPolicy&&(f.referrerPolicy=c.referrerPolicy),c.crossOrigin==="use-credentials"?f.credentials="include":c.crossOrigin==="anonymous"?f.credentials="omit":f.credentials="same-origin",f}function r(c){if(c.ep)return;c.ep=!0;const f=o(c);fetch(c.href,f)}})();function Bv(l){return l&&l.__esModule&&Object.prototype.hasOwnProperty.call(l,"default")?l.default:l}var gf={exports:{}},Oo={};/**
 * @license React
 * react-jsx-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Sp;function I1(){if(Sp)return Oo;Sp=1;var l=Symbol.for("react.transitional.element"),i=Symbol.for("react.fragment");function o(r,c,f){var d=null;if(f!==void 0&&(d=""+f),c.key!==void 0&&(d=""+c.key),"key"in c){f={};for(var p in c)p!=="key"&&(f[p]=c[p])}else f=c;return c=f.ref,{$$typeof:l,type:r,key:d,ref:c!==void 0?c:null,props:f}}return Oo.Fragment=i,Oo.jsx=o,Oo.jsxs=o,Oo}var Ep;function F1(){return Ep||(Ep=1,gf.exports=I1()),gf.exports}var L=F1(),pf={exports:{}},Ke={};/**
 * @license React
 * react.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var wp;function J1(){if(wp)return Ke;wp=1;var l=Symbol.for("react.transitional.element"),i=Symbol.for("react.portal"),o=Symbol.for("react.fragment"),r=Symbol.for("react.strict_mode"),c=Symbol.for("react.profiler"),f=Symbol.for("react.consumer"),d=Symbol.for("react.context"),p=Symbol.for("react.forward_ref"),v=Symbol.for("react.suspense"),m=Symbol.for("react.memo"),y=Symbol.for("react.lazy"),h=Symbol.for("react.activity"),E=Symbol.iterator;function w(A){return A===null||typeof A!="object"?null:(A=E&&A[E]||A["@@iterator"],typeof A=="function"?A:null)}var S={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},R=Object.assign,M={};function T(A,O,W){this.props=A,this.context=O,this.refs=M,this.updater=W||S}T.prototype.isReactComponent={},T.prototype.setState=function(A,O){if(typeof A!="object"&&typeof A!="function"&&A!=null)throw Error("takes an object of state variables to update or a function which returns an object of state variables.");this.updater.enqueueSetState(this,A,O,"setState")},T.prototype.forceUpdate=function(A){this.updater.enqueueForceUpdate(this,A,"forceUpdate")};function B(){}B.prototype=T.prototype;function _(A,O,W){this.props=A,this.context=O,this.refs=M,this.updater=W||S}var N=_.prototype=new B;N.constructor=_,R(N,T.prototype),N.isPureReactComponent=!0;var U=Array.isArray;function G(){}var Y={H:null,A:null,T:null,S:null},D=Object.prototype.hasOwnProperty;function le(A,O,W){var Q=W.ref;return{$$typeof:l,type:A,key:O,ref:Q!==void 0?Q:null,props:W}}function ge(A,O){return le(A.type,O,A.props)}function me(A){return typeof A=="object"&&A!==null&&A.$$typeof===l}function ue(A){var O={"=":"=0",":":"=2"};return"$"+A.replace(/[=:]/g,function(W){return O[W]})}var F=/\/+/g;function ae(A,O){return typeof A=="object"&&A!==null&&A.key!=null?ue(""+A.key):O.toString(36)}function he(A){switch(A.status){case"fulfilled":return A.value;case"rejected":throw A.reason;default:switch(typeof A.status=="string"?A.then(G,G):(A.status="pending",A.then(function(O){A.status==="pending"&&(A.status="fulfilled",A.value=O)},function(O){A.status==="pending"&&(A.status="rejected",A.reason=O)})),A.status){case"fulfilled":return A.value;case"rejected":throw A.reason}}throw A}function C(A,O,W,Q,te){var P=typeof A;(P==="undefined"||P==="boolean")&&(A=null);var de=!1;if(A===null)de=!0;else switch(P){case"bigint":case"string":case"number":de=!0;break;case"object":switch(A.$$typeof){case l:case i:de=!0;break;case y:return de=A._init,C(de(A._payload),O,W,Q,te)}}if(de)return te=te(A),de=Q===""?"."+ae(A,0):Q,U(te)?(W="",de!=null&&(W=de.replace(F,"$&/")+"/"),C(te,O,W,"",function(De){return De})):te!=null&&(me(te)&&(te=ge(te,W+(te.key==null||A&&A.key===te.key?"":(""+te.key).replace(F,"$&/")+"/")+de)),O.push(te)),1;de=0;var ce=Q===""?".":Q+":";if(U(A))for(var be=0;be<A.length;be++)Q=A[be],P=ce+ae(Q,be),de+=C(Q,O,W,P,te);else if(be=w(A),typeof be=="function")for(A=be.call(A),be=0;!(Q=A.next()).done;)Q=Q.value,P=ce+ae(Q,be++),de+=C(Q,O,W,P,te);else if(P==="object"){if(typeof A.then=="function")return C(he(A),O,W,Q,te);throw O=String(A),Error("Objects are not valid as a React child (found: "+(O==="[object Object]"?"object with keys {"+Object.keys(A).join(", ")+"}":O)+"). If you meant to render a collection of children, use an array instead.")}return de}function k(A,O,W){if(A==null)return A;var Q=[],te=0;return C(A,Q,"","",function(P){return O.call(W,P,te++)}),Q}function X(A){if(A._status===-1){var O=A._result;O=O(),O.then(function(W){(A._status===0||A._status===-1)&&(A._status=1,A._result=W)},function(W){(A._status===0||A._status===-1)&&(A._status=2,A._result=W)}),A._status===-1&&(A._status=0,A._result=O)}if(A._status===1)return A._result.default;throw A._result}var I=typeof reportError=="function"?reportError:function(A){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var O=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof A=="object"&&A!==null&&typeof A.message=="string"?String(A.message):String(A),error:A});if(!window.dispatchEvent(O))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",A);return}console.error(A)},q={map:k,forEach:function(A,O,W){k(A,function(){O.apply(this,arguments)},W)},count:function(A){var O=0;return k(A,function(){O++}),O},toArray:function(A){return k(A,function(O){return O})||[]},only:function(A){if(!me(A))throw Error("React.Children.only expected to receive a single React element child.");return A}};return Ke.Activity=h,Ke.Children=q,Ke.Component=T,Ke.Fragment=o,Ke.Profiler=c,Ke.PureComponent=_,Ke.StrictMode=r,Ke.Suspense=v,Ke.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=Y,Ke.__COMPILER_RUNTIME={__proto__:null,c:function(A){return Y.H.useMemoCache(A)}},Ke.cache=function(A){return function(){return A.apply(null,arguments)}},Ke.cacheSignal=function(){return null},Ke.cloneElement=function(A,O,W){if(A==null)throw Error("The argument must be a React element, but you passed "+A+".");var Q=R({},A.props),te=A.key;if(O!=null)for(P in O.key!==void 0&&(te=""+O.key),O)!D.call(O,P)||P==="key"||P==="__self"||P==="__source"||P==="ref"&&O.ref===void 0||(Q[P]=O[P]);var P=arguments.length-2;if(P===1)Q.children=W;else if(1<P){for(var de=Array(P),ce=0;ce<P;ce++)de[ce]=arguments[ce+2];Q.children=de}return le(A.type,te,Q)},Ke.createContext=function(A){return A={$$typeof:d,_currentValue:A,_currentValue2:A,_threadCount:0,Provider:null,Consumer:null},A.Provider=A,A.Consumer={$$typeof:f,_context:A},A},Ke.createElement=function(A,O,W){var Q,te={},P=null;if(O!=null)for(Q in O.key!==void 0&&(P=""+O.key),O)D.call(O,Q)&&Q!=="key"&&Q!=="__self"&&Q!=="__source"&&(te[Q]=O[Q]);var de=arguments.length-2;if(de===1)te.children=W;else if(1<de){for(var ce=Array(de),be=0;be<de;be++)ce[be]=arguments[be+2];te.children=ce}if(A&&A.defaultProps)for(Q in de=A.defaultProps,de)te[Q]===void 0&&(te[Q]=de[Q]);return le(A,P,te)},Ke.createRef=function(){return{current:null}},Ke.forwardRef=function(A){return{$$typeof:p,render:A}},Ke.isValidElement=me,Ke.lazy=function(A){return{$$typeof:y,_payload:{_status:-1,_result:A},_init:X}},Ke.memo=function(A,O){return{$$typeof:m,type:A,compare:O===void 0?null:O}},Ke.startTransition=function(A){var O=Y.T,W={};Y.T=W;try{var Q=A(),te=Y.S;te!==null&&te(W,Q),typeof Q=="object"&&Q!==null&&typeof Q.then=="function"&&Q.then(G,I)}catch(P){I(P)}finally{O!==null&&W.types!==null&&(O.types=W.types),Y.T=O}},Ke.unstable_useCacheRefresh=function(){return Y.H.useCacheRefresh()},Ke.use=function(A){return Y.H.use(A)},Ke.useActionState=function(A,O,W){return Y.H.useActionState(A,O,W)},Ke.useCallback=function(A,O){return Y.H.useCallback(A,O)},Ke.useContext=function(A){return Y.H.useContext(A)},Ke.useDebugValue=function(){},Ke.useDeferredValue=function(A,O){return Y.H.useDeferredValue(A,O)},Ke.useEffect=function(A,O){return Y.H.useEffect(A,O)},Ke.useEffectEvent=function(A){return Y.H.useEffectEvent(A)},Ke.useId=function(){return Y.H.useId()},Ke.useImperativeHandle=function(A,O,W){return Y.H.useImperativeHandle(A,O,W)},Ke.useInsertionEffect=function(A,O){return Y.H.useInsertionEffect(A,O)},Ke.useLayoutEffect=function(A,O){return Y.H.useLayoutEffect(A,O)},Ke.useMemo=function(A,O){return Y.H.useMemo(A,O)},Ke.useOptimistic=function(A,O){return Y.H.useOptimistic(A,O)},Ke.useReducer=function(A,O,W){return Y.H.useReducer(A,O,W)},Ke.useRef=function(A){return Y.H.useRef(A)},Ke.useState=function(A){return Y.H.useState(A)},Ke.useSyncExternalStore=function(A,O,W){return Y.H.useSyncExternalStore(A,O,W)},Ke.useTransition=function(){return Y.H.useTransition()},Ke.version="19.2.4",Ke}var Tp;function Io(){return Tp||(Tp=1,pf.exports=J1()),pf.exports}var x=Io();const ye=Bv(x),Hv=Z1({__proto__:null,default:ye},[x]);var vf={exports:{}},zo={},yf={exports:{}},bf={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Rp;function W1(){return Rp||(Rp=1,(function(l){function i(C,k){var X=C.length;C.push(k);e:for(;0<X;){var I=X-1>>>1,q=C[I];if(0<c(q,k))C[I]=k,C[X]=q,X=I;else break e}}function o(C){return C.length===0?null:C[0]}function r(C){if(C.length===0)return null;var k=C[0],X=C.pop();if(X!==k){C[0]=X;e:for(var I=0,q=C.length,A=q>>>1;I<A;){var O=2*(I+1)-1,W=C[O],Q=O+1,te=C[Q];if(0>c(W,X))Q<q&&0>c(te,W)?(C[I]=te,C[Q]=X,I=Q):(C[I]=W,C[O]=X,I=O);else if(Q<q&&0>c(te,X))C[I]=te,C[Q]=X,I=Q;else break e}}return k}function c(C,k){var X=C.sortIndex-k.sortIndex;return X!==0?X:C.id-k.id}if(l.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var f=performance;l.unstable_now=function(){return f.now()}}else{var d=Date,p=d.now();l.unstable_now=function(){return d.now()-p}}var v=[],m=[],y=1,h=null,E=3,w=!1,S=!1,R=!1,M=!1,T=typeof setTimeout=="function"?setTimeout:null,B=typeof clearTimeout=="function"?clearTimeout:null,_=typeof setImmediate<"u"?setImmediate:null;function N(C){for(var k=o(m);k!==null;){if(k.callback===null)r(m);else if(k.startTime<=C)r(m),k.sortIndex=k.expirationTime,i(v,k);else break;k=o(m)}}function U(C){if(R=!1,N(C),!S)if(o(v)!==null)S=!0,G||(G=!0,ue());else{var k=o(m);k!==null&&he(U,k.startTime-C)}}var G=!1,Y=-1,D=5,le=-1;function ge(){return M?!0:!(l.unstable_now()-le<D)}function me(){if(M=!1,G){var C=l.unstable_now();le=C;var k=!0;try{e:{S=!1,R&&(R=!1,B(Y),Y=-1),w=!0;var X=E;try{t:{for(N(C),h=o(v);h!==null&&!(h.expirationTime>C&&ge());){var I=h.callback;if(typeof I=="function"){h.callback=null,E=h.priorityLevel;var q=I(h.expirationTime<=C);if(C=l.unstable_now(),typeof q=="function"){h.callback=q,N(C),k=!0;break t}h===o(v)&&r(v),N(C)}else r(v);h=o(v)}if(h!==null)k=!0;else{var A=o(m);A!==null&&he(U,A.startTime-C),k=!1}}break e}finally{h=null,E=X,w=!1}k=void 0}}finally{k?ue():G=!1}}}var ue;if(typeof _=="function")ue=function(){_(me)};else if(typeof MessageChannel<"u"){var F=new MessageChannel,ae=F.port2;F.port1.onmessage=me,ue=function(){ae.postMessage(null)}}else ue=function(){T(me,0)};function he(C,k){Y=T(function(){C(l.unstable_now())},k)}l.unstable_IdlePriority=5,l.unstable_ImmediatePriority=1,l.unstable_LowPriority=4,l.unstable_NormalPriority=3,l.unstable_Profiling=null,l.unstable_UserBlockingPriority=2,l.unstable_cancelCallback=function(C){C.callback=null},l.unstable_forceFrameRate=function(C){0>C||125<C?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):D=0<C?Math.floor(1e3/C):5},l.unstable_getCurrentPriorityLevel=function(){return E},l.unstable_next=function(C){switch(E){case 1:case 2:case 3:var k=3;break;default:k=E}var X=E;E=k;try{return C()}finally{E=X}},l.unstable_requestPaint=function(){M=!0},l.unstable_runWithPriority=function(C,k){switch(C){case 1:case 2:case 3:case 4:case 5:break;default:C=3}var X=E;E=C;try{return k()}finally{E=X}},l.unstable_scheduleCallback=function(C,k,X){var I=l.unstable_now();switch(typeof X=="object"&&X!==null?(X=X.delay,X=typeof X=="number"&&0<X?I+X:I):X=I,C){case 1:var q=-1;break;case 2:q=250;break;case 5:q=1073741823;break;case 4:q=1e4;break;default:q=5e3}return q=X+q,C={id:y++,callback:k,priorityLevel:C,startTime:X,expirationTime:q,sortIndex:-1},X>I?(C.sortIndex=X,i(m,C),o(v)===null&&C===o(m)&&(R?(B(Y),Y=-1):R=!0,he(U,X-I))):(C.sortIndex=q,i(v,C),S||w||(S=!0,G||(G=!0,ue()))),C},l.unstable_shouldYield=ge,l.unstable_wrapCallback=function(C){var k=E;return function(){var X=E;E=k;try{return C.apply(this,arguments)}finally{E=X}}}})(bf)),bf}var Ap;function $1(){return Ap||(Ap=1,yf.exports=W1()),yf.exports}var xf={exports:{}},fn={};/**
 * @license React
 * react-dom.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Cp;function P1(){if(Cp)return fn;Cp=1;var l=Io();function i(v){var m="https://react.dev/errors/"+v;if(1<arguments.length){m+="?args[]="+encodeURIComponent(arguments[1]);for(var y=2;y<arguments.length;y++)m+="&args[]="+encodeURIComponent(arguments[y])}return"Minified React error #"+v+"; visit "+m+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function o(){}var r={d:{f:o,r:function(){throw Error(i(522))},D:o,C:o,L:o,m:o,X:o,S:o,M:o},p:0,findDOMNode:null},c=Symbol.for("react.portal");function f(v,m,y){var h=3<arguments.length&&arguments[3]!==void 0?arguments[3]:null;return{$$typeof:c,key:h==null?null:""+h,children:v,containerInfo:m,implementation:y}}var d=l.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE;function p(v,m){if(v==="font")return"";if(typeof m=="string")return m==="use-credentials"?m:""}return fn.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=r,fn.createPortal=function(v,m){var y=2<arguments.length&&arguments[2]!==void 0?arguments[2]:null;if(!m||m.nodeType!==1&&m.nodeType!==9&&m.nodeType!==11)throw Error(i(299));return f(v,m,null,y)},fn.flushSync=function(v){var m=d.T,y=r.p;try{if(d.T=null,r.p=2,v)return v()}finally{d.T=m,r.p=y,r.d.f()}},fn.preconnect=function(v,m){typeof v=="string"&&(m?(m=m.crossOrigin,m=typeof m=="string"?m==="use-credentials"?m:"":void 0):m=null,r.d.C(v,m))},fn.prefetchDNS=function(v){typeof v=="string"&&r.d.D(v)},fn.preinit=function(v,m){if(typeof v=="string"&&m&&typeof m.as=="string"){var y=m.as,h=p(y,m.crossOrigin),E=typeof m.integrity=="string"?m.integrity:void 0,w=typeof m.fetchPriority=="string"?m.fetchPriority:void 0;y==="style"?r.d.S(v,typeof m.precedence=="string"?m.precedence:void 0,{crossOrigin:h,integrity:E,fetchPriority:w}):y==="script"&&r.d.X(v,{crossOrigin:h,integrity:E,fetchPriority:w,nonce:typeof m.nonce=="string"?m.nonce:void 0})}},fn.preinitModule=function(v,m){if(typeof v=="string")if(typeof m=="object"&&m!==null){if(m.as==null||m.as==="script"){var y=p(m.as,m.crossOrigin);r.d.M(v,{crossOrigin:y,integrity:typeof m.integrity=="string"?m.integrity:void 0,nonce:typeof m.nonce=="string"?m.nonce:void 0})}}else m==null&&r.d.M(v)},fn.preload=function(v,m){if(typeof v=="string"&&typeof m=="object"&&m!==null&&typeof m.as=="string"){var y=m.as,h=p(y,m.crossOrigin);r.d.L(v,y,{crossOrigin:h,integrity:typeof m.integrity=="string"?m.integrity:void 0,nonce:typeof m.nonce=="string"?m.nonce:void 0,type:typeof m.type=="string"?m.type:void 0,fetchPriority:typeof m.fetchPriority=="string"?m.fetchPriority:void 0,referrerPolicy:typeof m.referrerPolicy=="string"?m.referrerPolicy:void 0,imageSrcSet:typeof m.imageSrcSet=="string"?m.imageSrcSet:void 0,imageSizes:typeof m.imageSizes=="string"?m.imageSizes:void 0,media:typeof m.media=="string"?m.media:void 0})}},fn.preloadModule=function(v,m){if(typeof v=="string")if(m){var y=p(m.as,m.crossOrigin);r.d.m(v,{as:typeof m.as=="string"&&m.as!=="script"?m.as:void 0,crossOrigin:y,integrity:typeof m.integrity=="string"?m.integrity:void 0})}else r.d.m(v)},fn.requestFormReset=function(v){r.d.r(v)},fn.unstable_batchedUpdates=function(v,m){return v(m)},fn.useFormState=function(v,m,y){return d.H.useFormState(v,m,y)},fn.useFormStatus=function(){return d.H.useHostTransitionStatus()},fn.version="19.2.4",fn}var Mp;function Vv(){if(Mp)return xf.exports;Mp=1;function l(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(l)}catch(i){console.error(i)}}return l(),xf.exports=P1(),xf.exports}/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var Op;function ex(){if(Op)return zo;Op=1;var l=$1(),i=Io(),o=Vv();function r(e){var t="https://react.dev/errors/"+e;if(1<arguments.length){t+="?args[]="+encodeURIComponent(arguments[1]);for(var n=2;n<arguments.length;n++)t+="&args[]="+encodeURIComponent(arguments[n])}return"Minified React error #"+e+"; visit "+t+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function c(e){return!(!e||e.nodeType!==1&&e.nodeType!==9&&e.nodeType!==11)}function f(e){var t=e,n=e;if(e.alternate)for(;t.return;)t=t.return;else{e=t;do t=e,(t.flags&4098)!==0&&(n=t.return),e=t.return;while(e)}return t.tag===3?n:null}function d(e){if(e.tag===13){var t=e.memoizedState;if(t===null&&(e=e.alternate,e!==null&&(t=e.memoizedState)),t!==null)return t.dehydrated}return null}function p(e){if(e.tag===31){var t=e.memoizedState;if(t===null&&(e=e.alternate,e!==null&&(t=e.memoizedState)),t!==null)return t.dehydrated}return null}function v(e){if(f(e)!==e)throw Error(r(188))}function m(e){var t=e.alternate;if(!t){if(t=f(e),t===null)throw Error(r(188));return t!==e?null:e}for(var n=e,a=t;;){var s=n.return;if(s===null)break;var u=s.alternate;if(u===null){if(a=s.return,a!==null){n=a;continue}break}if(s.child===u.child){for(u=s.child;u;){if(u===n)return v(s),e;if(u===a)return v(s),t;u=u.sibling}throw Error(r(188))}if(n.return!==a.return)n=s,a=u;else{for(var g=!1,b=s.child;b;){if(b===n){g=!0,n=s,a=u;break}if(b===a){g=!0,a=s,n=u;break}b=b.sibling}if(!g){for(b=u.child;b;){if(b===n){g=!0,n=u,a=s;break}if(b===a){g=!0,a=u,n=s;break}b=b.sibling}if(!g)throw Error(r(189))}}if(n.alternate!==a)throw Error(r(190))}if(n.tag!==3)throw Error(r(188));return n.stateNode.current===n?e:t}function y(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e;for(e=e.child;e!==null;){if(t=y(e),t!==null)return t;e=e.sibling}return null}var h=Object.assign,E=Symbol.for("react.element"),w=Symbol.for("react.transitional.element"),S=Symbol.for("react.portal"),R=Symbol.for("react.fragment"),M=Symbol.for("react.strict_mode"),T=Symbol.for("react.profiler"),B=Symbol.for("react.consumer"),_=Symbol.for("react.context"),N=Symbol.for("react.forward_ref"),U=Symbol.for("react.suspense"),G=Symbol.for("react.suspense_list"),Y=Symbol.for("react.memo"),D=Symbol.for("react.lazy"),le=Symbol.for("react.activity"),ge=Symbol.for("react.memo_cache_sentinel"),me=Symbol.iterator;function ue(e){return e===null||typeof e!="object"?null:(e=me&&e[me]||e["@@iterator"],typeof e=="function"?e:null)}var F=Symbol.for("react.client.reference");function ae(e){if(e==null)return null;if(typeof e=="function")return e.$$typeof===F?null:e.displayName||e.name||null;if(typeof e=="string")return e;switch(e){case R:return"Fragment";case T:return"Profiler";case M:return"StrictMode";case U:return"Suspense";case G:return"SuspenseList";case le:return"Activity"}if(typeof e=="object")switch(e.$$typeof){case S:return"Portal";case _:return e.displayName||"Context";case B:return(e._context.displayName||"Context")+".Consumer";case N:var t=e.render;return e=e.displayName,e||(e=t.displayName||t.name||"",e=e!==""?"ForwardRef("+e+")":"ForwardRef"),e;case Y:return t=e.displayName||null,t!==null?t:ae(e.type)||"Memo";case D:t=e._payload,e=e._init;try{return ae(e(t))}catch{}}return null}var he=Array.isArray,C=i.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,k=o.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,X={pending:!1,data:null,method:null,action:null},I=[],q=-1;function A(e){return{current:e}}function O(e){0>q||(e.current=I[q],I[q]=null,q--)}function W(e,t){q++,I[q]=e.current,e.current=t}var Q=A(null),te=A(null),P=A(null),de=A(null);function ce(e,t){switch(W(P,t),W(te,e),W(Q,null),t.nodeType){case 9:case 11:e=(e=t.documentElement)&&(e=e.namespaceURI)?Xg(e):0;break;default:if(e=t.tagName,t=t.namespaceURI)t=Xg(t),e=Qg(t,e);else switch(e){case"svg":e=1;break;case"math":e=2;break;default:e=0}}O(Q),W(Q,e)}function be(){O(Q),O(te),O(P)}function De(e){e.memoizedState!==null&&W(de,e);var t=Q.current,n=Qg(t,e.type);t!==n&&(W(te,e),W(Q,n))}function we(e){te.current===e&&(O(Q),O(te)),de.current===e&&(O(de),Ro._currentValue=X)}var Ye,xe;function pe(e){if(Ye===void 0)try{throw Error()}catch(n){var t=n.stack.trim().match(/\n( *(at )?)/);Ye=t&&t[1]||"",xe=-1<n.stack.indexOf(`
    at`)?" (<anonymous>)":-1<n.stack.indexOf("@")?"@unknown:0:0":""}return`
`+Ye+e+xe}var Re=!1;function Ge(e,t){if(!e||Re)return"";Re=!0;var n=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var a={DetermineComponentFrameRoot:function(){try{if(t){var se=function(){throw Error()};if(Object.defineProperty(se.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(se,[])}catch(ee){var J=ee}Reflect.construct(e,[],se)}else{try{se.call()}catch(ee){J=ee}e.call(se.prototype)}}else{try{throw Error()}catch(ee){J=ee}(se=e())&&typeof se.catch=="function"&&se.catch(function(){})}}catch(ee){if(ee&&J&&typeof ee.stack=="string")return[ee.stack,J.stack]}return[null,null]}};a.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var s=Object.getOwnPropertyDescriptor(a.DetermineComponentFrameRoot,"name");s&&s.configurable&&Object.defineProperty(a.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var u=a.DetermineComponentFrameRoot(),g=u[0],b=u[1];if(g&&b){var z=g.split(`
`),Z=b.split(`
`);for(s=a=0;a<z.length&&!z[a].includes("DetermineComponentFrameRoot");)a++;for(;s<Z.length&&!Z[s].includes("DetermineComponentFrameRoot");)s++;if(a===z.length||s===Z.length)for(a=z.length-1,s=Z.length-1;1<=a&&0<=s&&z[a]!==Z[s];)s--;for(;1<=a&&0<=s;a--,s--)if(z[a]!==Z[s]){if(a!==1||s!==1)do if(a--,s--,0>s||z[a]!==Z[s]){var ne=`
`+z[a].replace(" at new "," at ");return e.displayName&&ne.includes("<anonymous>")&&(ne=ne.replace("<anonymous>",e.displayName)),ne}while(1<=a&&0<=s);break}}}finally{Re=!1,Error.prepareStackTrace=n}return(n=e?e.displayName||e.name:"")?pe(n):""}function ve(e,t){switch(e.tag){case 26:case 27:case 5:return pe(e.type);case 16:return pe("Lazy");case 13:return e.child!==t&&t!==null?pe("Suspense Fallback"):pe("Suspense");case 19:return pe("SuspenseList");case 0:case 15:return Ge(e.type,!1);case 11:return Ge(e.type.render,!1);case 1:return Ge(e.type,!0);case 31:return pe("Activity");default:return""}}function Ee(e){try{var t="",n=null;do t+=ve(e,n),n=e,e=e.return;while(e);return t}catch(a){return`
Error generating stack: `+a.message+`
`+a.stack}}var fe=Object.prototype.hasOwnProperty,oe=l.unstable_scheduleCallback,H=l.unstable_cancelCallback,$=l.unstable_shouldYield,re=l.unstable_requestPaint,Se=l.unstable_now,Le=l.unstable_getCurrentPriorityLevel,Xe=l.unstable_ImmediatePriority,ut=l.unstable_UserBlockingPriority,Je=l.unstable_NormalPriority,je=l.unstable_LowPriority,Qe=l.unstable_IdlePriority,He=l.log,rt=l.unstable_setDisableYieldValue,gt=null,pt=null;function Lt(e){if(typeof He=="function"&&rt(e),pt&&typeof pt.setStrictMode=="function")try{pt.setStrictMode(gt,e)}catch{}}var it=Math.clz32?Math.clz32:qe,ot=Math.log,Tt=Math.LN2;function qe(e){return e>>>=0,e===0?32:31-(ot(e)/Tt|0)|0}var ct=256,Ot=262144,st=4194304;function _t(e){var t=e&42;if(t!==0)return t;switch(e&-e){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:return 128;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:return e&261888;case 262144:case 524288:case 1048576:case 2097152:return e&3932160;case 4194304:case 8388608:case 16777216:case 33554432:return e&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return e}}function Te(e,t,n){var a=e.pendingLanes;if(a===0)return 0;var s=0,u=e.suspendedLanes,g=e.pingedLanes;e=e.warmLanes;var b=a&134217727;return b!==0?(a=b&~u,a!==0?s=_t(a):(g&=b,g!==0?s=_t(g):n||(n=b&~e,n!==0&&(s=_t(n))))):(b=a&~u,b!==0?s=_t(b):g!==0?s=_t(g):n||(n=a&~e,n!==0&&(s=_t(n)))),s===0?0:t!==0&&t!==s&&(t&u)===0&&(u=s&-s,n=t&-t,u>=n||u===32&&(n&4194048)!==0)?t:s}function $e(e,t){return(e.pendingLanes&~(e.suspendedLanes&~e.pingedLanes)&t)===0}function bt(e,t){switch(e){case 1:case 2:case 4:case 8:case 64:return t+250;case 16:case 32:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return t+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function xt(){var e=st;return st<<=1,(st&62914560)===0&&(st=4194304),e}function tn(e){for(var t=[],n=0;31>n;n++)t.push(e);return t}function ft(e,t){e.pendingLanes|=t,t!==268435456&&(e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0)}function ke(e,t,n,a,s,u){var g=e.pendingLanes;e.pendingLanes=n,e.suspendedLanes=0,e.pingedLanes=0,e.warmLanes=0,e.expiredLanes&=n,e.entangledLanes&=n,e.errorRecoveryDisabledLanes&=n,e.shellSuspendCounter=0;var b=e.entanglements,z=e.expirationTimes,Z=e.hiddenUpdates;for(n=g&~n;0<n;){var ne=31-it(n),se=1<<ne;b[ne]=0,z[ne]=-1;var J=Z[ne];if(J!==null)for(Z[ne]=null,ne=0;ne<J.length;ne++){var ee=J[ne];ee!==null&&(ee.lane&=-536870913)}n&=~se}a!==0&&St(e,a,0),u!==0&&s===0&&e.tag!==0&&(e.suspendedLanes|=u&~(g&~t))}function St(e,t,n){e.pendingLanes|=t,e.suspendedLanes&=~t;var a=31-it(t);e.entangledLanes|=t,e.entanglements[a]=e.entanglements[a]|1073741824|n&261930}function Pe(e,t){var n=e.entangledLanes|=t;for(e=e.entanglements;n;){var a=31-it(n),s=1<<a;s&t|e[a]&t&&(e[a]|=t),n&=~s}}function vt(e,t){var n=t&-t;return n=(n&42)!==0?1:Ht(n),(n&(e.suspendedLanes|t))!==0?0:n}function Ht(e){switch(e){case 2:e=1;break;case 8:e=4;break;case 32:e=16;break;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:e=128;break;case 268435456:e=134217728;break;default:e=0}return e}function Vt(e){return e&=-e,2<e?8<e?(e&134217727)!==0?32:268435456:8:2}function pn(){var e=k.p;return e!==0?e:(e=window.event,e===void 0?32:hp(e.type))}function lr(e,t){var n=k.p;try{return k.p=e,t()}finally{k.p=n}}var nl=Math.random().toString(36).slice(2),Wt="__reactFiber$"+nl,on="__reactProps$"+nl,dl="__reactContainer$"+nl,Xa="__reactEvents$"+nl,k0="__reactListeners$"+nl,j0="__reactHandles$"+nl,_d="__reactResources$"+nl,Hi="__reactMarker$"+nl;function ou(e){delete e[Wt],delete e[on],delete e[Xa],delete e[k0],delete e[j0]}function Qa(e){var t=e[Wt];if(t)return t;for(var n=e.parentNode;n;){if(t=n[dl]||n[Wt]){if(n=t.alternate,t.child!==null||n!==null&&n.child!==null)for(e=$g(e);e!==null;){if(n=e[Wt])return n;e=$g(e)}return t}e=n,n=e.parentNode}return null}function Ka(e){if(e=e[Wt]||e[dl]){var t=e.tag;if(t===5||t===6||t===13||t===31||t===26||t===27||t===3)return e}return null}function Vi(e){var t=e.tag;if(t===5||t===26||t===27||t===6)return e.stateNode;throw Error(r(33))}function Za(e){var t=e[_d];return t||(t=e[_d]={hoistableStyles:new Map,hoistableScripts:new Map}),t}function nn(e){e[Hi]=!0}var Nd=new Set,Dd={};function va(e,t){Ia(e,t),Ia(e+"Capture",t)}function Ia(e,t){for(Dd[e]=t,e=0;e<t.length;e++)Nd.add(t[e])}var L0=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),kd={},jd={};function U0(e){return fe.call(jd,e)?!0:fe.call(kd,e)?!1:L0.test(e)?jd[e]=!0:(kd[e]=!0,!1)}function ar(e,t,n){if(U0(t))if(n===null)e.removeAttribute(t);else{switch(typeof n){case"undefined":case"function":case"symbol":e.removeAttribute(t);return;case"boolean":var a=t.toLowerCase().slice(0,5);if(a!=="data-"&&a!=="aria-"){e.removeAttribute(t);return}}e.setAttribute(t,""+n)}}function ir(e,t,n){if(n===null)e.removeAttribute(t);else{switch(typeof n){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(t);return}e.setAttribute(t,""+n)}}function ml(e,t,n,a){if(a===null)e.removeAttribute(n);else{switch(typeof a){case"undefined":case"function":case"symbol":case"boolean":e.removeAttribute(n);return}e.setAttributeNS(t,n,""+a)}}function Bn(e){switch(typeof e){case"bigint":case"boolean":case"number":case"string":case"undefined":return e;case"object":return e;default:return""}}function Ld(e){var t=e.type;return(e=e.nodeName)&&e.toLowerCase()==="input"&&(t==="checkbox"||t==="radio")}function B0(e,t,n){var a=Object.getOwnPropertyDescriptor(e.constructor.prototype,t);if(!e.hasOwnProperty(t)&&typeof a<"u"&&typeof a.get=="function"&&typeof a.set=="function"){var s=a.get,u=a.set;return Object.defineProperty(e,t,{configurable:!0,get:function(){return s.call(this)},set:function(g){n=""+g,u.call(this,g)}}),Object.defineProperty(e,t,{enumerable:a.enumerable}),{getValue:function(){return n},setValue:function(g){n=""+g},stopTracking:function(){e._valueTracker=null,delete e[t]}}}}function ru(e){if(!e._valueTracker){var t=Ld(e)?"checked":"value";e._valueTracker=B0(e,t,""+e[t])}}function Ud(e){if(!e)return!1;var t=e._valueTracker;if(!t)return!0;var n=t.getValue(),a="";return e&&(a=Ld(e)?e.checked?"true":"false":e.value),e=a,e!==n?(t.setValue(e),!0):!1}function or(e){if(e=e||(typeof document<"u"?document:void 0),typeof e>"u")return null;try{return e.activeElement||e.body}catch{return e.body}}var H0=/[\n"\\]/g;function Hn(e){return e.replace(H0,function(t){return"\\"+t.charCodeAt(0).toString(16)+" "})}function su(e,t,n,a,s,u,g,b){e.name="",g!=null&&typeof g!="function"&&typeof g!="symbol"&&typeof g!="boolean"?e.type=g:e.removeAttribute("type"),t!=null?g==="number"?(t===0&&e.value===""||e.value!=t)&&(e.value=""+Bn(t)):e.value!==""+Bn(t)&&(e.value=""+Bn(t)):g!=="submit"&&g!=="reset"||e.removeAttribute("value"),t!=null?uu(e,g,Bn(t)):n!=null?uu(e,g,Bn(n)):a!=null&&e.removeAttribute("value"),s==null&&u!=null&&(e.defaultChecked=!!u),s!=null&&(e.checked=s&&typeof s!="function"&&typeof s!="symbol"),b!=null&&typeof b!="function"&&typeof b!="symbol"&&typeof b!="boolean"?e.name=""+Bn(b):e.removeAttribute("name")}function Bd(e,t,n,a,s,u,g,b){if(u!=null&&typeof u!="function"&&typeof u!="symbol"&&typeof u!="boolean"&&(e.type=u),t!=null||n!=null){if(!(u!=="submit"&&u!=="reset"||t!=null)){ru(e);return}n=n!=null?""+Bn(n):"",t=t!=null?""+Bn(t):n,b||t===e.value||(e.value=t),e.defaultValue=t}a=a??s,a=typeof a!="function"&&typeof a!="symbol"&&!!a,e.checked=b?e.checked:!!a,e.defaultChecked=!!a,g!=null&&typeof g!="function"&&typeof g!="symbol"&&typeof g!="boolean"&&(e.name=g),ru(e)}function uu(e,t,n){t==="number"&&or(e.ownerDocument)===e||e.defaultValue===""+n||(e.defaultValue=""+n)}function Fa(e,t,n,a){if(e=e.options,t){t={};for(var s=0;s<n.length;s++)t["$"+n[s]]=!0;for(n=0;n<e.length;n++)s=t.hasOwnProperty("$"+e[n].value),e[n].selected!==s&&(e[n].selected=s),s&&a&&(e[n].defaultSelected=!0)}else{for(n=""+Bn(n),t=null,s=0;s<e.length;s++){if(e[s].value===n){e[s].selected=!0,a&&(e[s].defaultSelected=!0);return}t!==null||e[s].disabled||(t=e[s])}t!==null&&(t.selected=!0)}}function Hd(e,t,n){if(t!=null&&(t=""+Bn(t),t!==e.value&&(e.value=t),n==null)){e.defaultValue!==t&&(e.defaultValue=t);return}e.defaultValue=n!=null?""+Bn(n):""}function Vd(e,t,n,a){if(t==null){if(a!=null){if(n!=null)throw Error(r(92));if(he(a)){if(1<a.length)throw Error(r(93));a=a[0]}n=a}n==null&&(n=""),t=n}n=Bn(t),e.defaultValue=n,a=e.textContent,a===n&&a!==""&&a!==null&&(e.value=a),ru(e)}function Ja(e,t){if(t){var n=e.firstChild;if(n&&n===e.lastChild&&n.nodeType===3){n.nodeValue=t;return}}e.textContent=t}var V0=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function Yd(e,t,n){var a=t.indexOf("--")===0;n==null||typeof n=="boolean"||n===""?a?e.setProperty(t,""):t==="float"?e.cssFloat="":e[t]="":a?e.setProperty(t,n):typeof n!="number"||n===0||V0.has(t)?t==="float"?e.cssFloat=n:e[t]=(""+n).trim():e[t]=n+"px"}function qd(e,t,n){if(t!=null&&typeof t!="object")throw Error(r(62));if(e=e.style,n!=null){for(var a in n)!n.hasOwnProperty(a)||t!=null&&t.hasOwnProperty(a)||(a.indexOf("--")===0?e.setProperty(a,""):a==="float"?e.cssFloat="":e[a]="");for(var s in t)a=t[s],t.hasOwnProperty(s)&&n[s]!==a&&Yd(e,s,a)}else for(var u in t)t.hasOwnProperty(u)&&Yd(e,u,t[u])}function cu(e){if(e.indexOf("-")===-1)return!1;switch(e){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var Y0=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),q0=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function rr(e){return q0.test(""+e)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":e}function hl(){}var fu=null;function du(e){return e=e.target||e.srcElement||window,e.correspondingUseElement&&(e=e.correspondingUseElement),e.nodeType===3?e.parentNode:e}var Wa=null,$a=null;function Gd(e){var t=Ka(e);if(t&&(e=t.stateNode)){var n=e[on]||null;e:switch(e=t.stateNode,t.type){case"input":if(su(e,n.value,n.defaultValue,n.defaultValue,n.checked,n.defaultChecked,n.type,n.name),t=n.name,n.type==="radio"&&t!=null){for(n=e;n.parentNode;)n=n.parentNode;for(n=n.querySelectorAll('input[name="'+Hn(""+t)+'"][type="radio"]'),t=0;t<n.length;t++){var a=n[t];if(a!==e&&a.form===e.form){var s=a[on]||null;if(!s)throw Error(r(90));su(a,s.value,s.defaultValue,s.defaultValue,s.checked,s.defaultChecked,s.type,s.name)}}for(t=0;t<n.length;t++)a=n[t],a.form===e.form&&Ud(a)}break e;case"textarea":Hd(e,n.value,n.defaultValue);break e;case"select":t=n.value,t!=null&&Fa(e,!!n.multiple,t,!1)}}}var mu=!1;function Xd(e,t,n){if(mu)return e(t,n);mu=!0;try{var a=e(t);return a}finally{if(mu=!1,(Wa!==null||$a!==null)&&(Ir(),Wa&&(t=Wa,e=$a,$a=Wa=null,Gd(t),e)))for(t=0;t<e.length;t++)Gd(e[t])}}function Yi(e,t){var n=e.stateNode;if(n===null)return null;var a=n[on]||null;if(a===null)return null;n=a[t];e:switch(t){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(a=!a.disabled)||(e=e.type,a=!(e==="button"||e==="input"||e==="select"||e==="textarea")),e=!a;break e;default:e=!1}if(e)return null;if(n&&typeof n!="function")throw Error(r(231,t,typeof n));return n}var gl=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),hu=!1;if(gl)try{var qi={};Object.defineProperty(qi,"passive",{get:function(){hu=!0}}),window.addEventListener("test",qi,qi),window.removeEventListener("test",qi,qi)}catch{hu=!1}var Hl=null,gu=null,sr=null;function Qd(){if(sr)return sr;var e,t=gu,n=t.length,a,s="value"in Hl?Hl.value:Hl.textContent,u=s.length;for(e=0;e<n&&t[e]===s[e];e++);var g=n-e;for(a=1;a<=g&&t[n-a]===s[u-a];a++);return sr=s.slice(e,1<a?1-a:void 0)}function ur(e){var t=e.keyCode;return"charCode"in e?(e=e.charCode,e===0&&t===13&&(e=13)):e=t,e===10&&(e=13),32<=e||e===13?e:0}function cr(){return!0}function Kd(){return!1}function vn(e){function t(n,a,s,u,g){this._reactName=n,this._targetInst=s,this.type=a,this.nativeEvent=u,this.target=g,this.currentTarget=null;for(var b in e)e.hasOwnProperty(b)&&(n=e[b],this[b]=n?n(u):u[b]);return this.isDefaultPrevented=(u.defaultPrevented!=null?u.defaultPrevented:u.returnValue===!1)?cr:Kd,this.isPropagationStopped=Kd,this}return h(t.prototype,{preventDefault:function(){this.defaultPrevented=!0;var n=this.nativeEvent;n&&(n.preventDefault?n.preventDefault():typeof n.returnValue!="unknown"&&(n.returnValue=!1),this.isDefaultPrevented=cr)},stopPropagation:function(){var n=this.nativeEvent;n&&(n.stopPropagation?n.stopPropagation():typeof n.cancelBubble!="unknown"&&(n.cancelBubble=!0),this.isPropagationStopped=cr)},persist:function(){},isPersistent:cr}),t}var ya={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(e){return e.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},fr=vn(ya),Gi=h({},ya,{view:0,detail:0}),G0=vn(Gi),pu,vu,Xi,dr=h({},Gi,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:bu,button:0,buttons:0,relatedTarget:function(e){return e.relatedTarget===void 0?e.fromElement===e.srcElement?e.toElement:e.fromElement:e.relatedTarget},movementX:function(e){return"movementX"in e?e.movementX:(e!==Xi&&(Xi&&e.type==="mousemove"?(pu=e.screenX-Xi.screenX,vu=e.screenY-Xi.screenY):vu=pu=0,Xi=e),pu)},movementY:function(e){return"movementY"in e?e.movementY:vu}}),Zd=vn(dr),X0=h({},dr,{dataTransfer:0}),Q0=vn(X0),K0=h({},Gi,{relatedTarget:0}),yu=vn(K0),Z0=h({},ya,{animationName:0,elapsedTime:0,pseudoElement:0}),I0=vn(Z0),F0=h({},ya,{clipboardData:function(e){return"clipboardData"in e?e.clipboardData:window.clipboardData}}),J0=vn(F0),W0=h({},ya,{data:0}),Id=vn(W0),$0={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},P0={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},eb={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function tb(e){var t=this.nativeEvent;return t.getModifierState?t.getModifierState(e):(e=eb[e])?!!t[e]:!1}function bu(){return tb}var nb=h({},Gi,{key:function(e){if(e.key){var t=$0[e.key]||e.key;if(t!=="Unidentified")return t}return e.type==="keypress"?(e=ur(e),e===13?"Enter":String.fromCharCode(e)):e.type==="keydown"||e.type==="keyup"?P0[e.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:bu,charCode:function(e){return e.type==="keypress"?ur(e):0},keyCode:function(e){return e.type==="keydown"||e.type==="keyup"?e.keyCode:0},which:function(e){return e.type==="keypress"?ur(e):e.type==="keydown"||e.type==="keyup"?e.keyCode:0}}),lb=vn(nb),ab=h({},dr,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),Fd=vn(ab),ib=h({},Gi,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:bu}),ob=vn(ib),rb=h({},ya,{propertyName:0,elapsedTime:0,pseudoElement:0}),sb=vn(rb),ub=h({},dr,{deltaX:function(e){return"deltaX"in e?e.deltaX:"wheelDeltaX"in e?-e.wheelDeltaX:0},deltaY:function(e){return"deltaY"in e?e.deltaY:"wheelDeltaY"in e?-e.wheelDeltaY:"wheelDelta"in e?-e.wheelDelta:0},deltaZ:0,deltaMode:0}),cb=vn(ub),fb=h({},ya,{newState:0,oldState:0}),db=vn(fb),mb=[9,13,27,32],xu=gl&&"CompositionEvent"in window,Qi=null;gl&&"documentMode"in document&&(Qi=document.documentMode);var hb=gl&&"TextEvent"in window&&!Qi,Jd=gl&&(!xu||Qi&&8<Qi&&11>=Qi),Wd=" ",$d=!1;function Pd(e,t){switch(e){case"keyup":return mb.indexOf(t.keyCode)!==-1;case"keydown":return t.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function em(e){return e=e.detail,typeof e=="object"&&"data"in e?e.data:null}var Pa=!1;function gb(e,t){switch(e){case"compositionend":return em(t);case"keypress":return t.which!==32?null:($d=!0,Wd);case"textInput":return e=t.data,e===Wd&&$d?null:e;default:return null}}function pb(e,t){if(Pa)return e==="compositionend"||!xu&&Pd(e,t)?(e=Qd(),sr=gu=Hl=null,Pa=!1,e):null;switch(e){case"paste":return null;case"keypress":if(!(t.ctrlKey||t.altKey||t.metaKey)||t.ctrlKey&&t.altKey){if(t.char&&1<t.char.length)return t.char;if(t.which)return String.fromCharCode(t.which)}return null;case"compositionend":return Jd&&t.locale!=="ko"?null:t.data;default:return null}}var vb={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function tm(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t==="input"?!!vb[e.type]:t==="textarea"}function nm(e,t,n,a){Wa?$a?$a.push(a):$a=[a]:Wa=a,t=ts(t,"onChange"),0<t.length&&(n=new fr("onChange","change",null,n,a),e.push({event:n,listeners:t}))}var Ki=null,Zi=null;function yb(e){Bg(e,0)}function mr(e){var t=Vi(e);if(Ud(t))return e}function lm(e,t){if(e==="change")return t}var am=!1;if(gl){var Su;if(gl){var Eu="oninput"in document;if(!Eu){var im=document.createElement("div");im.setAttribute("oninput","return;"),Eu=typeof im.oninput=="function"}Su=Eu}else Su=!1;am=Su&&(!document.documentMode||9<document.documentMode)}function om(){Ki&&(Ki.detachEvent("onpropertychange",rm),Zi=Ki=null)}function rm(e){if(e.propertyName==="value"&&mr(Zi)){var t=[];nm(t,Zi,e,du(e)),Xd(yb,t)}}function bb(e,t,n){e==="focusin"?(om(),Ki=t,Zi=n,Ki.attachEvent("onpropertychange",rm)):e==="focusout"&&om()}function xb(e){if(e==="selectionchange"||e==="keyup"||e==="keydown")return mr(Zi)}function Sb(e,t){if(e==="click")return mr(t)}function Eb(e,t){if(e==="input"||e==="change")return mr(t)}function wb(e,t){return e===t&&(e!==0||1/e===1/t)||e!==e&&t!==t}var An=typeof Object.is=="function"?Object.is:wb;function Ii(e,t){if(An(e,t))return!0;if(typeof e!="object"||e===null||typeof t!="object"||t===null)return!1;var n=Object.keys(e),a=Object.keys(t);if(n.length!==a.length)return!1;for(a=0;a<n.length;a++){var s=n[a];if(!fe.call(t,s)||!An(e[s],t[s]))return!1}return!0}function sm(e){for(;e&&e.firstChild;)e=e.firstChild;return e}function um(e,t){var n=sm(e);e=0;for(var a;n;){if(n.nodeType===3){if(a=e+n.textContent.length,e<=t&&a>=t)return{node:n,offset:t-e};e=a}e:{for(;n;){if(n.nextSibling){n=n.nextSibling;break e}n=n.parentNode}n=void 0}n=sm(n)}}function cm(e,t){return e&&t?e===t?!0:e&&e.nodeType===3?!1:t&&t.nodeType===3?cm(e,t.parentNode):"contains"in e?e.contains(t):e.compareDocumentPosition?!!(e.compareDocumentPosition(t)&16):!1:!1}function fm(e){e=e!=null&&e.ownerDocument!=null&&e.ownerDocument.defaultView!=null?e.ownerDocument.defaultView:window;for(var t=or(e.document);t instanceof e.HTMLIFrameElement;){try{var n=typeof t.contentWindow.location.href=="string"}catch{n=!1}if(n)e=t.contentWindow;else break;t=or(e.document)}return t}function wu(e){var t=e&&e.nodeName&&e.nodeName.toLowerCase();return t&&(t==="input"&&(e.type==="text"||e.type==="search"||e.type==="tel"||e.type==="url"||e.type==="password")||t==="textarea"||e.contentEditable==="true")}var Tb=gl&&"documentMode"in document&&11>=document.documentMode,ei=null,Tu=null,Fi=null,Ru=!1;function dm(e,t,n){var a=n.window===n?n.document:n.nodeType===9?n:n.ownerDocument;Ru||ei==null||ei!==or(a)||(a=ei,"selectionStart"in a&&wu(a)?a={start:a.selectionStart,end:a.selectionEnd}:(a=(a.ownerDocument&&a.ownerDocument.defaultView||window).getSelection(),a={anchorNode:a.anchorNode,anchorOffset:a.anchorOffset,focusNode:a.focusNode,focusOffset:a.focusOffset}),Fi&&Ii(Fi,a)||(Fi=a,a=ts(Tu,"onSelect"),0<a.length&&(t=new fr("onSelect","select",null,t,n),e.push({event:t,listeners:a}),t.target=ei)))}function ba(e,t){var n={};return n[e.toLowerCase()]=t.toLowerCase(),n["Webkit"+e]="webkit"+t,n["Moz"+e]="moz"+t,n}var ti={animationend:ba("Animation","AnimationEnd"),animationiteration:ba("Animation","AnimationIteration"),animationstart:ba("Animation","AnimationStart"),transitionrun:ba("Transition","TransitionRun"),transitionstart:ba("Transition","TransitionStart"),transitioncancel:ba("Transition","TransitionCancel"),transitionend:ba("Transition","TransitionEnd")},Au={},mm={};gl&&(mm=document.createElement("div").style,"AnimationEvent"in window||(delete ti.animationend.animation,delete ti.animationiteration.animation,delete ti.animationstart.animation),"TransitionEvent"in window||delete ti.transitionend.transition);function xa(e){if(Au[e])return Au[e];if(!ti[e])return e;var t=ti[e],n;for(n in t)if(t.hasOwnProperty(n)&&n in mm)return Au[e]=t[n];return e}var hm=xa("animationend"),gm=xa("animationiteration"),pm=xa("animationstart"),Rb=xa("transitionrun"),Ab=xa("transitionstart"),Cb=xa("transitioncancel"),vm=xa("transitionend"),ym=new Map,Cu="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");Cu.push("scrollEnd");function Jn(e,t){ym.set(e,t),va(t,[e])}var hr=typeof reportError=="function"?reportError:function(e){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var t=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof e=="object"&&e!==null&&typeof e.message=="string"?String(e.message):String(e),error:e});if(!window.dispatchEvent(t))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",e);return}console.error(e)},Vn=[],ni=0,Mu=0;function gr(){for(var e=ni,t=Mu=ni=0;t<e;){var n=Vn[t];Vn[t++]=null;var a=Vn[t];Vn[t++]=null;var s=Vn[t];Vn[t++]=null;var u=Vn[t];if(Vn[t++]=null,a!==null&&s!==null){var g=a.pending;g===null?s.next=s:(s.next=g.next,g.next=s),a.pending=s}u!==0&&bm(n,s,u)}}function pr(e,t,n,a){Vn[ni++]=e,Vn[ni++]=t,Vn[ni++]=n,Vn[ni++]=a,Mu|=a,e.lanes|=a,e=e.alternate,e!==null&&(e.lanes|=a)}function Ou(e,t,n,a){return pr(e,t,n,a),vr(e)}function Sa(e,t){return pr(e,null,null,t),vr(e)}function bm(e,t,n){e.lanes|=n;var a=e.alternate;a!==null&&(a.lanes|=n);for(var s=!1,u=e.return;u!==null;)u.childLanes|=n,a=u.alternate,a!==null&&(a.childLanes|=n),u.tag===22&&(e=u.stateNode,e===null||e._visibility&1||(s=!0)),e=u,u=u.return;return e.tag===3?(u=e.stateNode,s&&t!==null&&(s=31-it(n),e=u.hiddenUpdates,a=e[s],a===null?e[s]=[t]:a.push(t),t.lane=n|536870912),u):null}function vr(e){if(50<yo)throw yo=0,Bc=null,Error(r(185));for(var t=e.return;t!==null;)e=t,t=e.return;return e.tag===3?e.stateNode:null}var li={};function Mb(e,t,n,a){this.tag=e,this.key=n,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=t,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=a,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function Cn(e,t,n,a){return new Mb(e,t,n,a)}function zu(e){return e=e.prototype,!(!e||!e.isReactComponent)}function pl(e,t){var n=e.alternate;return n===null?(n=Cn(e.tag,t,e.key,e.mode),n.elementType=e.elementType,n.type=e.type,n.stateNode=e.stateNode,n.alternate=e,e.alternate=n):(n.pendingProps=t,n.type=e.type,n.flags=0,n.subtreeFlags=0,n.deletions=null),n.flags=e.flags&65011712,n.childLanes=e.childLanes,n.lanes=e.lanes,n.child=e.child,n.memoizedProps=e.memoizedProps,n.memoizedState=e.memoizedState,n.updateQueue=e.updateQueue,t=e.dependencies,n.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext},n.sibling=e.sibling,n.index=e.index,n.ref=e.ref,n.refCleanup=e.refCleanup,n}function xm(e,t){e.flags&=65011714;var n=e.alternate;return n===null?(e.childLanes=0,e.lanes=t,e.child=null,e.subtreeFlags=0,e.memoizedProps=null,e.memoizedState=null,e.updateQueue=null,e.dependencies=null,e.stateNode=null):(e.childLanes=n.childLanes,e.lanes=n.lanes,e.child=n.child,e.subtreeFlags=0,e.deletions=null,e.memoizedProps=n.memoizedProps,e.memoizedState=n.memoizedState,e.updateQueue=n.updateQueue,e.type=n.type,t=n.dependencies,e.dependencies=t===null?null:{lanes:t.lanes,firstContext:t.firstContext}),e}function yr(e,t,n,a,s,u){var g=0;if(a=e,typeof e=="function")zu(e)&&(g=1);else if(typeof e=="string")g=D1(e,n,Q.current)?26:e==="html"||e==="head"||e==="body"?27:5;else e:switch(e){case le:return e=Cn(31,n,t,s),e.elementType=le,e.lanes=u,e;case R:return Ea(n.children,s,u,t);case M:g=8,s|=24;break;case T:return e=Cn(12,n,t,s|2),e.elementType=T,e.lanes=u,e;case U:return e=Cn(13,n,t,s),e.elementType=U,e.lanes=u,e;case G:return e=Cn(19,n,t,s),e.elementType=G,e.lanes=u,e;default:if(typeof e=="object"&&e!==null)switch(e.$$typeof){case _:g=10;break e;case B:g=9;break e;case N:g=11;break e;case Y:g=14;break e;case D:g=16,a=null;break e}g=29,n=Error(r(130,e===null?"null":typeof e,"")),a=null}return t=Cn(g,n,t,s),t.elementType=e,t.type=a,t.lanes=u,t}function Ea(e,t,n,a){return e=Cn(7,e,a,t),e.lanes=n,e}function _u(e,t,n){return e=Cn(6,e,null,t),e.lanes=n,e}function Sm(e){var t=Cn(18,null,null,0);return t.stateNode=e,t}function Nu(e,t,n){return t=Cn(4,e.children!==null?e.children:[],e.key,t),t.lanes=n,t.stateNode={containerInfo:e.containerInfo,pendingChildren:null,implementation:e.implementation},t}var Em=new WeakMap;function Yn(e,t){if(typeof e=="object"&&e!==null){var n=Em.get(e);return n!==void 0?n:(t={value:e,source:t,stack:Ee(t)},Em.set(e,t),t)}return{value:e,source:t,stack:Ee(t)}}var ai=[],ii=0,br=null,Ji=0,qn=[],Gn=0,Vl=null,ll=1,al="";function vl(e,t){ai[ii++]=Ji,ai[ii++]=br,br=e,Ji=t}function wm(e,t,n){qn[Gn++]=ll,qn[Gn++]=al,qn[Gn++]=Vl,Vl=e;var a=ll;e=al;var s=32-it(a)-1;a&=~(1<<s),n+=1;var u=32-it(t)+s;if(30<u){var g=s-s%5;u=(a&(1<<g)-1).toString(32),a>>=g,s-=g,ll=1<<32-it(t)+s|n<<s|a,al=u+e}else ll=1<<u|n<<s|a,al=e}function Du(e){e.return!==null&&(vl(e,1),wm(e,1,0))}function ku(e){for(;e===br;)br=ai[--ii],ai[ii]=null,Ji=ai[--ii],ai[ii]=null;for(;e===Vl;)Vl=qn[--Gn],qn[Gn]=null,al=qn[--Gn],qn[Gn]=null,ll=qn[--Gn],qn[Gn]=null}function Tm(e,t){qn[Gn++]=ll,qn[Gn++]=al,qn[Gn++]=Vl,ll=t.id,al=t.overflow,Vl=e}var rn=null,Nt=null,at=!1,Yl=null,Xn=!1,ju=Error(r(519));function ql(e){var t=Error(r(418,1<arguments.length&&arguments[1]!==void 0&&arguments[1]?"text":"HTML",""));throw Wi(Yn(t,e)),ju}function Rm(e){var t=e.stateNode,n=e.type,a=e.memoizedProps;switch(t[Wt]=e,t[on]=a,n){case"dialog":tt("cancel",t),tt("close",t);break;case"iframe":case"object":case"embed":tt("load",t);break;case"video":case"audio":for(n=0;n<xo.length;n++)tt(xo[n],t);break;case"source":tt("error",t);break;case"img":case"image":case"link":tt("error",t),tt("load",t);break;case"details":tt("toggle",t);break;case"input":tt("invalid",t),Bd(t,a.value,a.defaultValue,a.checked,a.defaultChecked,a.type,a.name,!0);break;case"select":tt("invalid",t);break;case"textarea":tt("invalid",t),Vd(t,a.value,a.defaultValue,a.children)}n=a.children,typeof n!="string"&&typeof n!="number"&&typeof n!="bigint"||t.textContent===""+n||a.suppressHydrationWarning===!0||qg(t.textContent,n)?(a.popover!=null&&(tt("beforetoggle",t),tt("toggle",t)),a.onScroll!=null&&tt("scroll",t),a.onScrollEnd!=null&&tt("scrollend",t),a.onClick!=null&&(t.onclick=hl),t=!0):t=!1,t||ql(e,!0)}function Am(e){for(rn=e.return;rn;)switch(rn.tag){case 5:case 31:case 13:Xn=!1;return;case 27:case 3:Xn=!0;return;default:rn=rn.return}}function oi(e){if(e!==rn)return!1;if(!at)return Am(e),at=!0,!1;var t=e.tag,n;if((n=t!==3&&t!==27)&&((n=t===5)&&(n=e.type,n=!(n!=="form"&&n!=="button")||Pc(e.type,e.memoizedProps)),n=!n),n&&Nt&&ql(e),Am(e),t===13){if(e=e.memoizedState,e=e!==null?e.dehydrated:null,!e)throw Error(r(317));Nt=Wg(e)}else if(t===31){if(e=e.memoizedState,e=e!==null?e.dehydrated:null,!e)throw Error(r(317));Nt=Wg(e)}else t===27?(t=Nt,na(e.type)?(e=af,af=null,Nt=e):Nt=t):Nt=rn?Kn(e.stateNode.nextSibling):null;return!0}function wa(){Nt=rn=null,at=!1}function Lu(){var e=Yl;return e!==null&&(Sn===null?Sn=e:Sn.push.apply(Sn,e),Yl=null),e}function Wi(e){Yl===null?Yl=[e]:Yl.push(e)}var Uu=A(null),Ta=null,yl=null;function Gl(e,t,n){W(Uu,t._currentValue),t._currentValue=n}function bl(e){e._currentValue=Uu.current,O(Uu)}function Bu(e,t,n){for(;e!==null;){var a=e.alternate;if((e.childLanes&t)!==t?(e.childLanes|=t,a!==null&&(a.childLanes|=t)):a!==null&&(a.childLanes&t)!==t&&(a.childLanes|=t),e===n)break;e=e.return}}function Hu(e,t,n,a){var s=e.child;for(s!==null&&(s.return=e);s!==null;){var u=s.dependencies;if(u!==null){var g=s.child;u=u.firstContext;e:for(;u!==null;){var b=u;u=s;for(var z=0;z<t.length;z++)if(b.context===t[z]){u.lanes|=n,b=u.alternate,b!==null&&(b.lanes|=n),Bu(u.return,n,e),a||(g=null);break e}u=b.next}}else if(s.tag===18){if(g=s.return,g===null)throw Error(r(341));g.lanes|=n,u=g.alternate,u!==null&&(u.lanes|=n),Bu(g,n,e),g=null}else g=s.child;if(g!==null)g.return=s;else for(g=s;g!==null;){if(g===e){g=null;break}if(s=g.sibling,s!==null){s.return=g.return,g=s;break}g=g.return}s=g}}function ri(e,t,n,a){e=null;for(var s=t,u=!1;s!==null;){if(!u){if((s.flags&524288)!==0)u=!0;else if((s.flags&262144)!==0)break}if(s.tag===10){var g=s.alternate;if(g===null)throw Error(r(387));if(g=g.memoizedProps,g!==null){var b=s.type;An(s.pendingProps.value,g.value)||(e!==null?e.push(b):e=[b])}}else if(s===de.current){if(g=s.alternate,g===null)throw Error(r(387));g.memoizedState.memoizedState!==s.memoizedState.memoizedState&&(e!==null?e.push(Ro):e=[Ro])}s=s.return}e!==null&&Hu(t,e,n,a),t.flags|=262144}function xr(e){for(e=e.firstContext;e!==null;){if(!An(e.context._currentValue,e.memoizedValue))return!0;e=e.next}return!1}function Ra(e){Ta=e,yl=null,e=e.dependencies,e!==null&&(e.firstContext=null)}function sn(e){return Cm(Ta,e)}function Sr(e,t){return Ta===null&&Ra(e),Cm(e,t)}function Cm(e,t){var n=t._currentValue;if(t={context:t,memoizedValue:n,next:null},yl===null){if(e===null)throw Error(r(308));yl=t,e.dependencies={lanes:0,firstContext:t},e.flags|=524288}else yl=yl.next=t;return n}var Ob=typeof AbortController<"u"?AbortController:function(){var e=[],t=this.signal={aborted:!1,addEventListener:function(n,a){e.push(a)}};this.abort=function(){t.aborted=!0,e.forEach(function(n){return n()})}},zb=l.unstable_scheduleCallback,_b=l.unstable_NormalPriority,Qt={$$typeof:_,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function Vu(){return{controller:new Ob,data:new Map,refCount:0}}function $i(e){e.refCount--,e.refCount===0&&zb(_b,function(){e.controller.abort()})}var Pi=null,Yu=0,si=0,ui=null;function Nb(e,t){if(Pi===null){var n=Pi=[];Yu=0,si=Xc(),ui={status:"pending",value:void 0,then:function(a){n.push(a)}}}return Yu++,t.then(Mm,Mm),t}function Mm(){if(--Yu===0&&Pi!==null){ui!==null&&(ui.status="fulfilled");var e=Pi;Pi=null,si=0,ui=null;for(var t=0;t<e.length;t++)(0,e[t])()}}function Db(e,t){var n=[],a={status:"pending",value:null,reason:null,then:function(s){n.push(s)}};return e.then(function(){a.status="fulfilled",a.value=t;for(var s=0;s<n.length;s++)(0,n[s])(t)},function(s){for(a.status="rejected",a.reason=s,s=0;s<n.length;s++)(0,n[s])(void 0)}),a}var Om=C.S;C.S=function(e,t){dg=Se(),typeof t=="object"&&t!==null&&typeof t.then=="function"&&Nb(e,t),Om!==null&&Om(e,t)};var Aa=A(null);function qu(){var e=Aa.current;return e!==null?e:Mt.pooledCache}function Er(e,t){t===null?W(Aa,Aa.current):W(Aa,t.pool)}function zm(){var e=qu();return e===null?null:{parent:Qt._currentValue,pool:e}}var ci=Error(r(460)),Gu=Error(r(474)),wr=Error(r(542)),Tr={then:function(){}};function _m(e){return e=e.status,e==="fulfilled"||e==="rejected"}function Nm(e,t,n){switch(n=e[n],n===void 0?e.push(t):n!==t&&(t.then(hl,hl),t=n),t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,km(e),e;default:if(typeof t.status=="string")t.then(hl,hl);else{if(e=Mt,e!==null&&100<e.shellSuspendCounter)throw Error(r(482));e=t,e.status="pending",e.then(function(a){if(t.status==="pending"){var s=t;s.status="fulfilled",s.value=a}},function(a){if(t.status==="pending"){var s=t;s.status="rejected",s.reason=a}})}switch(t.status){case"fulfilled":return t.value;case"rejected":throw e=t.reason,km(e),e}throw Ma=t,ci}}function Ca(e){try{var t=e._init;return t(e._payload)}catch(n){throw n!==null&&typeof n=="object"&&typeof n.then=="function"?(Ma=n,ci):n}}var Ma=null;function Dm(){if(Ma===null)throw Error(r(459));var e=Ma;return Ma=null,e}function km(e){if(e===ci||e===wr)throw Error(r(483))}var fi=null,eo=0;function Rr(e){var t=eo;return eo+=1,fi===null&&(fi=[]),Nm(fi,e,t)}function to(e,t){t=t.props.ref,e.ref=t!==void 0?t:null}function Ar(e,t){throw t.$$typeof===E?Error(r(525)):(e=Object.prototype.toString.call(t),Error(r(31,e==="[object Object]"?"object with keys {"+Object.keys(t).join(", ")+"}":e)))}function jm(e){function t(V,j){if(e){var K=V.deletions;K===null?(V.deletions=[j],V.flags|=16):K.push(j)}}function n(V,j){if(!e)return null;for(;j!==null;)t(V,j),j=j.sibling;return null}function a(V){for(var j=new Map;V!==null;)V.key!==null?j.set(V.key,V):j.set(V.index,V),V=V.sibling;return j}function s(V,j){return V=pl(V,j),V.index=0,V.sibling=null,V}function u(V,j,K){return V.index=K,e?(K=V.alternate,K!==null?(K=K.index,K<j?(V.flags|=67108866,j):K):(V.flags|=67108866,j)):(V.flags|=1048576,j)}function g(V){return e&&V.alternate===null&&(V.flags|=67108866),V}function b(V,j,K,ie){return j===null||j.tag!==6?(j=_u(K,V.mode,ie),j.return=V,j):(j=s(j,K),j.return=V,j)}function z(V,j,K,ie){var Ne=K.type;return Ne===R?ne(V,j,K.props.children,ie,K.key):j!==null&&(j.elementType===Ne||typeof Ne=="object"&&Ne!==null&&Ne.$$typeof===D&&Ca(Ne)===j.type)?(j=s(j,K.props),to(j,K),j.return=V,j):(j=yr(K.type,K.key,K.props,null,V.mode,ie),to(j,K),j.return=V,j)}function Z(V,j,K,ie){return j===null||j.tag!==4||j.stateNode.containerInfo!==K.containerInfo||j.stateNode.implementation!==K.implementation?(j=Nu(K,V.mode,ie),j.return=V,j):(j=s(j,K.children||[]),j.return=V,j)}function ne(V,j,K,ie,Ne){return j===null||j.tag!==7?(j=Ea(K,V.mode,ie,Ne),j.return=V,j):(j=s(j,K),j.return=V,j)}function se(V,j,K){if(typeof j=="string"&&j!==""||typeof j=="number"||typeof j=="bigint")return j=_u(""+j,V.mode,K),j.return=V,j;if(typeof j=="object"&&j!==null){switch(j.$$typeof){case w:return K=yr(j.type,j.key,j.props,null,V.mode,K),to(K,j),K.return=V,K;case S:return j=Nu(j,V.mode,K),j.return=V,j;case D:return j=Ca(j),se(V,j,K)}if(he(j)||ue(j))return j=Ea(j,V.mode,K,null),j.return=V,j;if(typeof j.then=="function")return se(V,Rr(j),K);if(j.$$typeof===_)return se(V,Sr(V,j),K);Ar(V,j)}return null}function J(V,j,K,ie){var Ne=j!==null?j.key:null;if(typeof K=="string"&&K!==""||typeof K=="number"||typeof K=="bigint")return Ne!==null?null:b(V,j,""+K,ie);if(typeof K=="object"&&K!==null){switch(K.$$typeof){case w:return K.key===Ne?z(V,j,K,ie):null;case S:return K.key===Ne?Z(V,j,K,ie):null;case D:return K=Ca(K),J(V,j,K,ie)}if(he(K)||ue(K))return Ne!==null?null:ne(V,j,K,ie,null);if(typeof K.then=="function")return J(V,j,Rr(K),ie);if(K.$$typeof===_)return J(V,j,Sr(V,K),ie);Ar(V,K)}return null}function ee(V,j,K,ie,Ne){if(typeof ie=="string"&&ie!==""||typeof ie=="number"||typeof ie=="bigint")return V=V.get(K)||null,b(j,V,""+ie,Ne);if(typeof ie=="object"&&ie!==null){switch(ie.$$typeof){case w:return V=V.get(ie.key===null?K:ie.key)||null,z(j,V,ie,Ne);case S:return V=V.get(ie.key===null?K:ie.key)||null,Z(j,V,ie,Ne);case D:return ie=Ca(ie),ee(V,j,K,ie,Ne)}if(he(ie)||ue(ie))return V=V.get(K)||null,ne(j,V,ie,Ne,null);if(typeof ie.then=="function")return ee(V,j,K,Rr(ie),Ne);if(ie.$$typeof===_)return ee(V,j,K,Sr(j,ie),Ne);Ar(j,ie)}return null}function Ae(V,j,K,ie){for(var Ne=null,dt=null,ze=j,Fe=j=0,lt=null;ze!==null&&Fe<K.length;Fe++){ze.index>Fe?(lt=ze,ze=null):lt=ze.sibling;var mt=J(V,ze,K[Fe],ie);if(mt===null){ze===null&&(ze=lt);break}e&&ze&&mt.alternate===null&&t(V,ze),j=u(mt,j,Fe),dt===null?Ne=mt:dt.sibling=mt,dt=mt,ze=lt}if(Fe===K.length)return n(V,ze),at&&vl(V,Fe),Ne;if(ze===null){for(;Fe<K.length;Fe++)ze=se(V,K[Fe],ie),ze!==null&&(j=u(ze,j,Fe),dt===null?Ne=ze:dt.sibling=ze,dt=ze);return at&&vl(V,Fe),Ne}for(ze=a(ze);Fe<K.length;Fe++)lt=ee(ze,V,Fe,K[Fe],ie),lt!==null&&(e&&lt.alternate!==null&&ze.delete(lt.key===null?Fe:lt.key),j=u(lt,j,Fe),dt===null?Ne=lt:dt.sibling=lt,dt=lt);return e&&ze.forEach(function(ra){return t(V,ra)}),at&&vl(V,Fe),Ne}function Ue(V,j,K,ie){if(K==null)throw Error(r(151));for(var Ne=null,dt=null,ze=j,Fe=j=0,lt=null,mt=K.next();ze!==null&&!mt.done;Fe++,mt=K.next()){ze.index>Fe?(lt=ze,ze=null):lt=ze.sibling;var ra=J(V,ze,mt.value,ie);if(ra===null){ze===null&&(ze=lt);break}e&&ze&&ra.alternate===null&&t(V,ze),j=u(ra,j,Fe),dt===null?Ne=ra:dt.sibling=ra,dt=ra,ze=lt}if(mt.done)return n(V,ze),at&&vl(V,Fe),Ne;if(ze===null){for(;!mt.done;Fe++,mt=K.next())mt=se(V,mt.value,ie),mt!==null&&(j=u(mt,j,Fe),dt===null?Ne=mt:dt.sibling=mt,dt=mt);return at&&vl(V,Fe),Ne}for(ze=a(ze);!mt.done;Fe++,mt=K.next())mt=ee(ze,V,Fe,mt.value,ie),mt!==null&&(e&&mt.alternate!==null&&ze.delete(mt.key===null?Fe:mt.key),j=u(mt,j,Fe),dt===null?Ne=mt:dt.sibling=mt,dt=mt);return e&&ze.forEach(function(X1){return t(V,X1)}),at&&vl(V,Fe),Ne}function Ct(V,j,K,ie){if(typeof K=="object"&&K!==null&&K.type===R&&K.key===null&&(K=K.props.children),typeof K=="object"&&K!==null){switch(K.$$typeof){case w:e:{for(var Ne=K.key;j!==null;){if(j.key===Ne){if(Ne=K.type,Ne===R){if(j.tag===7){n(V,j.sibling),ie=s(j,K.props.children),ie.return=V,V=ie;break e}}else if(j.elementType===Ne||typeof Ne=="object"&&Ne!==null&&Ne.$$typeof===D&&Ca(Ne)===j.type){n(V,j.sibling),ie=s(j,K.props),to(ie,K),ie.return=V,V=ie;break e}n(V,j);break}else t(V,j);j=j.sibling}K.type===R?(ie=Ea(K.props.children,V.mode,ie,K.key),ie.return=V,V=ie):(ie=yr(K.type,K.key,K.props,null,V.mode,ie),to(ie,K),ie.return=V,V=ie)}return g(V);case S:e:{for(Ne=K.key;j!==null;){if(j.key===Ne)if(j.tag===4&&j.stateNode.containerInfo===K.containerInfo&&j.stateNode.implementation===K.implementation){n(V,j.sibling),ie=s(j,K.children||[]),ie.return=V,V=ie;break e}else{n(V,j);break}else t(V,j);j=j.sibling}ie=Nu(K,V.mode,ie),ie.return=V,V=ie}return g(V);case D:return K=Ca(K),Ct(V,j,K,ie)}if(he(K))return Ae(V,j,K,ie);if(ue(K)){if(Ne=ue(K),typeof Ne!="function")throw Error(r(150));return K=Ne.call(K),Ue(V,j,K,ie)}if(typeof K.then=="function")return Ct(V,j,Rr(K),ie);if(K.$$typeof===_)return Ct(V,j,Sr(V,K),ie);Ar(V,K)}return typeof K=="string"&&K!==""||typeof K=="number"||typeof K=="bigint"?(K=""+K,j!==null&&j.tag===6?(n(V,j.sibling),ie=s(j,K),ie.return=V,V=ie):(n(V,j),ie=_u(K,V.mode,ie),ie.return=V,V=ie),g(V)):n(V,j)}return function(V,j,K,ie){try{eo=0;var Ne=Ct(V,j,K,ie);return fi=null,Ne}catch(ze){if(ze===ci||ze===wr)throw ze;var dt=Cn(29,ze,null,V.mode);return dt.lanes=ie,dt.return=V,dt}finally{}}}var Oa=jm(!0),Lm=jm(!1),Xl=!1;function Xu(e){e.updateQueue={baseState:e.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function Qu(e,t){e=e.updateQueue,t.updateQueue===e&&(t.updateQueue={baseState:e.baseState,firstBaseUpdate:e.firstBaseUpdate,lastBaseUpdate:e.lastBaseUpdate,shared:e.shared,callbacks:null})}function Ql(e){return{lane:e,tag:0,payload:null,callback:null,next:null}}function Kl(e,t,n){var a=e.updateQueue;if(a===null)return null;if(a=a.shared,(yt&2)!==0){var s=a.pending;return s===null?t.next=t:(t.next=s.next,s.next=t),a.pending=t,t=vr(e),bm(e,null,n),t}return pr(e,a,t,n),vr(e)}function no(e,t,n){if(t=t.updateQueue,t!==null&&(t=t.shared,(n&4194048)!==0)){var a=t.lanes;a&=e.pendingLanes,n|=a,t.lanes=n,Pe(e,n)}}function Ku(e,t){var n=e.updateQueue,a=e.alternate;if(a!==null&&(a=a.updateQueue,n===a)){var s=null,u=null;if(n=n.firstBaseUpdate,n!==null){do{var g={lane:n.lane,tag:n.tag,payload:n.payload,callback:null,next:null};u===null?s=u=g:u=u.next=g,n=n.next}while(n!==null);u===null?s=u=t:u=u.next=t}else s=u=t;n={baseState:a.baseState,firstBaseUpdate:s,lastBaseUpdate:u,shared:a.shared,callbacks:a.callbacks},e.updateQueue=n;return}e=n.lastBaseUpdate,e===null?n.firstBaseUpdate=t:e.next=t,n.lastBaseUpdate=t}var Zu=!1;function lo(){if(Zu){var e=ui;if(e!==null)throw e}}function ao(e,t,n,a){Zu=!1;var s=e.updateQueue;Xl=!1;var u=s.firstBaseUpdate,g=s.lastBaseUpdate,b=s.shared.pending;if(b!==null){s.shared.pending=null;var z=b,Z=z.next;z.next=null,g===null?u=Z:g.next=Z,g=z;var ne=e.alternate;ne!==null&&(ne=ne.updateQueue,b=ne.lastBaseUpdate,b!==g&&(b===null?ne.firstBaseUpdate=Z:b.next=Z,ne.lastBaseUpdate=z))}if(u!==null){var se=s.baseState;g=0,ne=Z=z=null,b=u;do{var J=b.lane&-536870913,ee=J!==b.lane;if(ee?(nt&J)===J:(a&J)===J){J!==0&&J===si&&(Zu=!0),ne!==null&&(ne=ne.next={lane:0,tag:b.tag,payload:b.payload,callback:null,next:null});e:{var Ae=e,Ue=b;J=t;var Ct=n;switch(Ue.tag){case 1:if(Ae=Ue.payload,typeof Ae=="function"){se=Ae.call(Ct,se,J);break e}se=Ae;break e;case 3:Ae.flags=Ae.flags&-65537|128;case 0:if(Ae=Ue.payload,J=typeof Ae=="function"?Ae.call(Ct,se,J):Ae,J==null)break e;se=h({},se,J);break e;case 2:Xl=!0}}J=b.callback,J!==null&&(e.flags|=64,ee&&(e.flags|=8192),ee=s.callbacks,ee===null?s.callbacks=[J]:ee.push(J))}else ee={lane:J,tag:b.tag,payload:b.payload,callback:b.callback,next:null},ne===null?(Z=ne=ee,z=se):ne=ne.next=ee,g|=J;if(b=b.next,b===null){if(b=s.shared.pending,b===null)break;ee=b,b=ee.next,ee.next=null,s.lastBaseUpdate=ee,s.shared.pending=null}}while(!0);ne===null&&(z=se),s.baseState=z,s.firstBaseUpdate=Z,s.lastBaseUpdate=ne,u===null&&(s.shared.lanes=0),Wl|=g,e.lanes=g,e.memoizedState=se}}function Um(e,t){if(typeof e!="function")throw Error(r(191,e));e.call(t)}function Bm(e,t){var n=e.callbacks;if(n!==null)for(e.callbacks=null,e=0;e<n.length;e++)Um(n[e],t)}var di=A(null),Cr=A(0);function Hm(e,t){e=Ml,W(Cr,e),W(di,t),Ml=e|t.baseLanes}function Iu(){W(Cr,Ml),W(di,di.current)}function Fu(){Ml=Cr.current,O(di),O(Cr)}var Mn=A(null),Qn=null;function Zl(e){var t=e.alternate;W(Gt,Gt.current&1),W(Mn,e),Qn===null&&(t===null||di.current!==null||t.memoizedState!==null)&&(Qn=e)}function Ju(e){W(Gt,Gt.current),W(Mn,e),Qn===null&&(Qn=e)}function Vm(e){e.tag===22?(W(Gt,Gt.current),W(Mn,e),Qn===null&&(Qn=e)):Il()}function Il(){W(Gt,Gt.current),W(Mn,Mn.current)}function On(e){O(Mn),Qn===e&&(Qn=null),O(Gt)}var Gt=A(0);function Mr(e){for(var t=e;t!==null;){if(t.tag===13){var n=t.memoizedState;if(n!==null&&(n=n.dehydrated,n===null||nf(n)||lf(n)))return t}else if(t.tag===19&&(t.memoizedProps.revealOrder==="forwards"||t.memoizedProps.revealOrder==="backwards"||t.memoizedProps.revealOrder==="unstable_legacy-backwards"||t.memoizedProps.revealOrder==="together")){if((t.flags&128)!==0)return t}else if(t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return null;t=t.return}t.sibling.return=t.return,t=t.sibling}return null}var xl=0,Ie=null,Rt=null,Kt=null,Or=!1,mi=!1,za=!1,zr=0,io=0,hi=null,kb=0;function Yt(){throw Error(r(321))}function Wu(e,t){if(t===null)return!1;for(var n=0;n<t.length&&n<e.length;n++)if(!An(e[n],t[n]))return!1;return!0}function $u(e,t,n,a,s,u){return xl=u,Ie=t,t.memoizedState=null,t.updateQueue=null,t.lanes=0,C.H=e===null||e.memoizedState===null?wh:mc,za=!1,u=n(a,s),za=!1,mi&&(u=qm(t,n,a,s)),Ym(e),u}function Ym(e){C.H=so;var t=Rt!==null&&Rt.next!==null;if(xl=0,Kt=Rt=Ie=null,Or=!1,io=0,hi=null,t)throw Error(r(300));e===null||Zt||(e=e.dependencies,e!==null&&xr(e)&&(Zt=!0))}function qm(e,t,n,a){Ie=e;var s=0;do{if(mi&&(hi=null),io=0,mi=!1,25<=s)throw Error(r(301));if(s+=1,Kt=Rt=null,e.updateQueue!=null){var u=e.updateQueue;u.lastEffect=null,u.events=null,u.stores=null,u.memoCache!=null&&(u.memoCache.index=0)}C.H=Th,u=t(n,a)}while(mi);return u}function jb(){var e=C.H,t=e.useState()[0];return t=typeof t.then=="function"?oo(t):t,e=e.useState()[0],(Rt!==null?Rt.memoizedState:null)!==e&&(Ie.flags|=1024),t}function Pu(){var e=zr!==0;return zr=0,e}function ec(e,t,n){t.updateQueue=e.updateQueue,t.flags&=-2053,e.lanes&=~n}function tc(e){if(Or){for(e=e.memoizedState;e!==null;){var t=e.queue;t!==null&&(t.pending=null),e=e.next}Or=!1}xl=0,Kt=Rt=Ie=null,mi=!1,io=zr=0,hi=null}function hn(){var e={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return Kt===null?Ie.memoizedState=Kt=e:Kt=Kt.next=e,Kt}function Xt(){if(Rt===null){var e=Ie.alternate;e=e!==null?e.memoizedState:null}else e=Rt.next;var t=Kt===null?Ie.memoizedState:Kt.next;if(t!==null)Kt=t,Rt=e;else{if(e===null)throw Ie.alternate===null?Error(r(467)):Error(r(310));Rt=e,e={memoizedState:Rt.memoizedState,baseState:Rt.baseState,baseQueue:Rt.baseQueue,queue:Rt.queue,next:null},Kt===null?Ie.memoizedState=Kt=e:Kt=Kt.next=e}return Kt}function _r(){return{lastEffect:null,events:null,stores:null,memoCache:null}}function oo(e){var t=io;return io+=1,hi===null&&(hi=[]),e=Nm(hi,e,t),t=Ie,(Kt===null?t.memoizedState:Kt.next)===null&&(t=t.alternate,C.H=t===null||t.memoizedState===null?wh:mc),e}function Nr(e){if(e!==null&&typeof e=="object"){if(typeof e.then=="function")return oo(e);if(e.$$typeof===_)return sn(e)}throw Error(r(438,String(e)))}function nc(e){var t=null,n=Ie.updateQueue;if(n!==null&&(t=n.memoCache),t==null){var a=Ie.alternate;a!==null&&(a=a.updateQueue,a!==null&&(a=a.memoCache,a!=null&&(t={data:a.data.map(function(s){return s.slice()}),index:0})))}if(t==null&&(t={data:[],index:0}),n===null&&(n=_r(),Ie.updateQueue=n),n.memoCache=t,n=t.data[t.index],n===void 0)for(n=t.data[t.index]=Array(e),a=0;a<e;a++)n[a]=ge;return t.index++,n}function Sl(e,t){return typeof t=="function"?t(e):t}function Dr(e){var t=Xt();return lc(t,Rt,e)}function lc(e,t,n){var a=e.queue;if(a===null)throw Error(r(311));a.lastRenderedReducer=n;var s=e.baseQueue,u=a.pending;if(u!==null){if(s!==null){var g=s.next;s.next=u.next,u.next=g}t.baseQueue=s=u,a.pending=null}if(u=e.baseState,s===null)e.memoizedState=u;else{t=s.next;var b=g=null,z=null,Z=t,ne=!1;do{var se=Z.lane&-536870913;if(se!==Z.lane?(nt&se)===se:(xl&se)===se){var J=Z.revertLane;if(J===0)z!==null&&(z=z.next={lane:0,revertLane:0,gesture:null,action:Z.action,hasEagerState:Z.hasEagerState,eagerState:Z.eagerState,next:null}),se===si&&(ne=!0);else if((xl&J)===J){Z=Z.next,J===si&&(ne=!0);continue}else se={lane:0,revertLane:Z.revertLane,gesture:null,action:Z.action,hasEagerState:Z.hasEagerState,eagerState:Z.eagerState,next:null},z===null?(b=z=se,g=u):z=z.next=se,Ie.lanes|=J,Wl|=J;se=Z.action,za&&n(u,se),u=Z.hasEagerState?Z.eagerState:n(u,se)}else J={lane:se,revertLane:Z.revertLane,gesture:Z.gesture,action:Z.action,hasEagerState:Z.hasEagerState,eagerState:Z.eagerState,next:null},z===null?(b=z=J,g=u):z=z.next=J,Ie.lanes|=se,Wl|=se;Z=Z.next}while(Z!==null&&Z!==t);if(z===null?g=u:z.next=b,!An(u,e.memoizedState)&&(Zt=!0,ne&&(n=ui,n!==null)))throw n;e.memoizedState=u,e.baseState=g,e.baseQueue=z,a.lastRenderedState=u}return s===null&&(a.lanes=0),[e.memoizedState,a.dispatch]}function ac(e){var t=Xt(),n=t.queue;if(n===null)throw Error(r(311));n.lastRenderedReducer=e;var a=n.dispatch,s=n.pending,u=t.memoizedState;if(s!==null){n.pending=null;var g=s=s.next;do u=e(u,g.action),g=g.next;while(g!==s);An(u,t.memoizedState)||(Zt=!0),t.memoizedState=u,t.baseQueue===null&&(t.baseState=u),n.lastRenderedState=u}return[u,a]}function Gm(e,t,n){var a=Ie,s=Xt(),u=at;if(u){if(n===void 0)throw Error(r(407));n=n()}else n=t();var g=!An((Rt||s).memoizedState,n);if(g&&(s.memoizedState=n,Zt=!0),s=s.queue,rc(Km.bind(null,a,s,e),[e]),s.getSnapshot!==t||g||Kt!==null&&Kt.memoizedState.tag&1){if(a.flags|=2048,gi(9,{destroy:void 0},Qm.bind(null,a,s,n,t),null),Mt===null)throw Error(r(349));u||(xl&127)!==0||Xm(a,t,n)}return n}function Xm(e,t,n){e.flags|=16384,e={getSnapshot:t,value:n},t=Ie.updateQueue,t===null?(t=_r(),Ie.updateQueue=t,t.stores=[e]):(n=t.stores,n===null?t.stores=[e]:n.push(e))}function Qm(e,t,n,a){t.value=n,t.getSnapshot=a,Zm(t)&&Im(e)}function Km(e,t,n){return n(function(){Zm(t)&&Im(e)})}function Zm(e){var t=e.getSnapshot;e=e.value;try{var n=t();return!An(e,n)}catch{return!0}}function Im(e){var t=Sa(e,2);t!==null&&En(t,e,2)}function ic(e){var t=hn();if(typeof e=="function"){var n=e;if(e=n(),za){Lt(!0);try{n()}finally{Lt(!1)}}}return t.memoizedState=t.baseState=e,t.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:Sl,lastRenderedState:e},t}function Fm(e,t,n,a){return e.baseState=n,lc(e,Rt,typeof a=="function"?a:Sl)}function Lb(e,t,n,a,s){if(Lr(e))throw Error(r(485));if(e=t.action,e!==null){var u={payload:s,action:e,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(g){u.listeners.push(g)}};C.T!==null?n(!0):u.isTransition=!1,a(u),n=t.pending,n===null?(u.next=t.pending=u,Jm(t,u)):(u.next=n.next,t.pending=n.next=u)}}function Jm(e,t){var n=t.action,a=t.payload,s=e.state;if(t.isTransition){var u=C.T,g={};C.T=g;try{var b=n(s,a),z=C.S;z!==null&&z(g,b),Wm(e,t,b)}catch(Z){oc(e,t,Z)}finally{u!==null&&g.types!==null&&(u.types=g.types),C.T=u}}else try{u=n(s,a),Wm(e,t,u)}catch(Z){oc(e,t,Z)}}function Wm(e,t,n){n!==null&&typeof n=="object"&&typeof n.then=="function"?n.then(function(a){$m(e,t,a)},function(a){return oc(e,t,a)}):$m(e,t,n)}function $m(e,t,n){t.status="fulfilled",t.value=n,Pm(t),e.state=n,t=e.pending,t!==null&&(n=t.next,n===t?e.pending=null:(n=n.next,t.next=n,Jm(e,n)))}function oc(e,t,n){var a=e.pending;if(e.pending=null,a!==null){a=a.next;do t.status="rejected",t.reason=n,Pm(t),t=t.next;while(t!==a)}e.action=null}function Pm(e){e=e.listeners;for(var t=0;t<e.length;t++)(0,e[t])()}function eh(e,t){return t}function th(e,t){if(at){var n=Mt.formState;if(n!==null){e:{var a=Ie;if(at){if(Nt){t:{for(var s=Nt,u=Xn;s.nodeType!==8;){if(!u){s=null;break t}if(s=Kn(s.nextSibling),s===null){s=null;break t}}u=s.data,s=u==="F!"||u==="F"?s:null}if(s){Nt=Kn(s.nextSibling),a=s.data==="F!";break e}}ql(a)}a=!1}a&&(t=n[0])}}return n=hn(),n.memoizedState=n.baseState=t,a={pending:null,lanes:0,dispatch:null,lastRenderedReducer:eh,lastRenderedState:t},n.queue=a,n=xh.bind(null,Ie,a),a.dispatch=n,a=ic(!1),u=dc.bind(null,Ie,!1,a.queue),a=hn(),s={state:t,dispatch:null,action:e,pending:null},a.queue=s,n=Lb.bind(null,Ie,s,u,n),s.dispatch=n,a.memoizedState=e,[t,n,!1]}function nh(e){var t=Xt();return lh(t,Rt,e)}function lh(e,t,n){if(t=lc(e,t,eh)[0],e=Dr(Sl)[0],typeof t=="object"&&t!==null&&typeof t.then=="function")try{var a=oo(t)}catch(g){throw g===ci?wr:g}else a=t;t=Xt();var s=t.queue,u=s.dispatch;return n!==t.memoizedState&&(Ie.flags|=2048,gi(9,{destroy:void 0},Ub.bind(null,s,n),null)),[a,u,e]}function Ub(e,t){e.action=t}function ah(e){var t=Xt(),n=Rt;if(n!==null)return lh(t,n,e);Xt(),t=t.memoizedState,n=Xt();var a=n.queue.dispatch;return n.memoizedState=e,[t,a,!1]}function gi(e,t,n,a){return e={tag:e,create:n,deps:a,inst:t,next:null},t=Ie.updateQueue,t===null&&(t=_r(),Ie.updateQueue=t),n=t.lastEffect,n===null?t.lastEffect=e.next=e:(a=n.next,n.next=e,e.next=a,t.lastEffect=e),e}function ih(){return Xt().memoizedState}function kr(e,t,n,a){var s=hn();Ie.flags|=e,s.memoizedState=gi(1|t,{destroy:void 0},n,a===void 0?null:a)}function jr(e,t,n,a){var s=Xt();a=a===void 0?null:a;var u=s.memoizedState.inst;Rt!==null&&a!==null&&Wu(a,Rt.memoizedState.deps)?s.memoizedState=gi(t,u,n,a):(Ie.flags|=e,s.memoizedState=gi(1|t,u,n,a))}function oh(e,t){kr(8390656,8,e,t)}function rc(e,t){jr(2048,8,e,t)}function Bb(e){Ie.flags|=4;var t=Ie.updateQueue;if(t===null)t=_r(),Ie.updateQueue=t,t.events=[e];else{var n=t.events;n===null?t.events=[e]:n.push(e)}}function rh(e){var t=Xt().memoizedState;return Bb({ref:t,nextImpl:e}),function(){if((yt&2)!==0)throw Error(r(440));return t.impl.apply(void 0,arguments)}}function sh(e,t){return jr(4,2,e,t)}function uh(e,t){return jr(4,4,e,t)}function ch(e,t){if(typeof t=="function"){e=e();var n=t(e);return function(){typeof n=="function"?n():t(null)}}if(t!=null)return e=e(),t.current=e,function(){t.current=null}}function fh(e,t,n){n=n!=null?n.concat([e]):null,jr(4,4,ch.bind(null,t,e),n)}function sc(){}function dh(e,t){var n=Xt();t=t===void 0?null:t;var a=n.memoizedState;return t!==null&&Wu(t,a[1])?a[0]:(n.memoizedState=[e,t],e)}function mh(e,t){var n=Xt();t=t===void 0?null:t;var a=n.memoizedState;if(t!==null&&Wu(t,a[1]))return a[0];if(a=e(),za){Lt(!0);try{e()}finally{Lt(!1)}}return n.memoizedState=[a,t],a}function uc(e,t,n){return n===void 0||(xl&1073741824)!==0&&(nt&261930)===0?e.memoizedState=t:(e.memoizedState=n,e=hg(),Ie.lanes|=e,Wl|=e,n)}function hh(e,t,n,a){return An(n,t)?n:di.current!==null?(e=uc(e,n,a),An(e,t)||(Zt=!0),e):(xl&42)===0||(xl&1073741824)!==0&&(nt&261930)===0?(Zt=!0,e.memoizedState=n):(e=hg(),Ie.lanes|=e,Wl|=e,t)}function gh(e,t,n,a,s){var u=k.p;k.p=u!==0&&8>u?u:8;var g=C.T,b={};C.T=b,dc(e,!1,t,n);try{var z=s(),Z=C.S;if(Z!==null&&Z(b,z),z!==null&&typeof z=="object"&&typeof z.then=="function"){var ne=Db(z,a);ro(e,t,ne,Nn(e))}else ro(e,t,a,Nn(e))}catch(se){ro(e,t,{then:function(){},status:"rejected",reason:se},Nn())}finally{k.p=u,g!==null&&b.types!==null&&(g.types=b.types),C.T=g}}function Hb(){}function cc(e,t,n,a){if(e.tag!==5)throw Error(r(476));var s=ph(e).queue;gh(e,s,t,X,n===null?Hb:function(){return vh(e),n(a)})}function ph(e){var t=e.memoizedState;if(t!==null)return t;t={memoizedState:X,baseState:X,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:Sl,lastRenderedState:X},next:null};var n={};return t.next={memoizedState:n,baseState:n,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:Sl,lastRenderedState:n},next:null},e.memoizedState=t,e=e.alternate,e!==null&&(e.memoizedState=t),t}function vh(e){var t=ph(e);t.next===null&&(t=e.alternate.memoizedState),ro(e,t.next.queue,{},Nn())}function fc(){return sn(Ro)}function yh(){return Xt().memoizedState}function bh(){return Xt().memoizedState}function Vb(e){for(var t=e.return;t!==null;){switch(t.tag){case 24:case 3:var n=Nn();e=Ql(n);var a=Kl(t,e,n);a!==null&&(En(a,t,n),no(a,t,n)),t={cache:Vu()},e.payload=t;return}t=t.return}}function Yb(e,t,n){var a=Nn();n={lane:a,revertLane:0,gesture:null,action:n,hasEagerState:!1,eagerState:null,next:null},Lr(e)?Sh(t,n):(n=Ou(e,t,n,a),n!==null&&(En(n,e,a),Eh(n,t,a)))}function xh(e,t,n){var a=Nn();ro(e,t,n,a)}function ro(e,t,n,a){var s={lane:a,revertLane:0,gesture:null,action:n,hasEagerState:!1,eagerState:null,next:null};if(Lr(e))Sh(t,s);else{var u=e.alternate;if(e.lanes===0&&(u===null||u.lanes===0)&&(u=t.lastRenderedReducer,u!==null))try{var g=t.lastRenderedState,b=u(g,n);if(s.hasEagerState=!0,s.eagerState=b,An(b,g))return pr(e,t,s,0),Mt===null&&gr(),!1}catch{}finally{}if(n=Ou(e,t,s,a),n!==null)return En(n,e,a),Eh(n,t,a),!0}return!1}function dc(e,t,n,a){if(a={lane:2,revertLane:Xc(),gesture:null,action:a,hasEagerState:!1,eagerState:null,next:null},Lr(e)){if(t)throw Error(r(479))}else t=Ou(e,n,a,2),t!==null&&En(t,e,2)}function Lr(e){var t=e.alternate;return e===Ie||t!==null&&t===Ie}function Sh(e,t){mi=Or=!0;var n=e.pending;n===null?t.next=t:(t.next=n.next,n.next=t),e.pending=t}function Eh(e,t,n){if((n&4194048)!==0){var a=t.lanes;a&=e.pendingLanes,n|=a,t.lanes=n,Pe(e,n)}}var so={readContext:sn,use:Nr,useCallback:Yt,useContext:Yt,useEffect:Yt,useImperativeHandle:Yt,useLayoutEffect:Yt,useInsertionEffect:Yt,useMemo:Yt,useReducer:Yt,useRef:Yt,useState:Yt,useDebugValue:Yt,useDeferredValue:Yt,useTransition:Yt,useSyncExternalStore:Yt,useId:Yt,useHostTransitionStatus:Yt,useFormState:Yt,useActionState:Yt,useOptimistic:Yt,useMemoCache:Yt,useCacheRefresh:Yt};so.useEffectEvent=Yt;var wh={readContext:sn,use:Nr,useCallback:function(e,t){return hn().memoizedState=[e,t===void 0?null:t],e},useContext:sn,useEffect:oh,useImperativeHandle:function(e,t,n){n=n!=null?n.concat([e]):null,kr(4194308,4,ch.bind(null,t,e),n)},useLayoutEffect:function(e,t){return kr(4194308,4,e,t)},useInsertionEffect:function(e,t){kr(4,2,e,t)},useMemo:function(e,t){var n=hn();t=t===void 0?null:t;var a=e();if(za){Lt(!0);try{e()}finally{Lt(!1)}}return n.memoizedState=[a,t],a},useReducer:function(e,t,n){var a=hn();if(n!==void 0){var s=n(t);if(za){Lt(!0);try{n(t)}finally{Lt(!1)}}}else s=t;return a.memoizedState=a.baseState=s,e={pending:null,lanes:0,dispatch:null,lastRenderedReducer:e,lastRenderedState:s},a.queue=e,e=e.dispatch=Yb.bind(null,Ie,e),[a.memoizedState,e]},useRef:function(e){var t=hn();return e={current:e},t.memoizedState=e},useState:function(e){e=ic(e);var t=e.queue,n=xh.bind(null,Ie,t);return t.dispatch=n,[e.memoizedState,n]},useDebugValue:sc,useDeferredValue:function(e,t){var n=hn();return uc(n,e,t)},useTransition:function(){var e=ic(!1);return e=gh.bind(null,Ie,e.queue,!0,!1),hn().memoizedState=e,[!1,e]},useSyncExternalStore:function(e,t,n){var a=Ie,s=hn();if(at){if(n===void 0)throw Error(r(407));n=n()}else{if(n=t(),Mt===null)throw Error(r(349));(nt&127)!==0||Xm(a,t,n)}s.memoizedState=n;var u={value:n,getSnapshot:t};return s.queue=u,oh(Km.bind(null,a,u,e),[e]),a.flags|=2048,gi(9,{destroy:void 0},Qm.bind(null,a,u,n,t),null),n},useId:function(){var e=hn(),t=Mt.identifierPrefix;if(at){var n=al,a=ll;n=(a&~(1<<32-it(a)-1)).toString(32)+n,t="_"+t+"R_"+n,n=zr++,0<n&&(t+="H"+n.toString(32)),t+="_"}else n=kb++,t="_"+t+"r_"+n.toString(32)+"_";return e.memoizedState=t},useHostTransitionStatus:fc,useFormState:th,useActionState:th,useOptimistic:function(e){var t=hn();t.memoizedState=t.baseState=e;var n={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return t.queue=n,t=dc.bind(null,Ie,!0,n),n.dispatch=t,[e,t]},useMemoCache:nc,useCacheRefresh:function(){return hn().memoizedState=Vb.bind(null,Ie)},useEffectEvent:function(e){var t=hn(),n={impl:e};return t.memoizedState=n,function(){if((yt&2)!==0)throw Error(r(440));return n.impl.apply(void 0,arguments)}}},mc={readContext:sn,use:Nr,useCallback:dh,useContext:sn,useEffect:rc,useImperativeHandle:fh,useInsertionEffect:sh,useLayoutEffect:uh,useMemo:mh,useReducer:Dr,useRef:ih,useState:function(){return Dr(Sl)},useDebugValue:sc,useDeferredValue:function(e,t){var n=Xt();return hh(n,Rt.memoizedState,e,t)},useTransition:function(){var e=Dr(Sl)[0],t=Xt().memoizedState;return[typeof e=="boolean"?e:oo(e),t]},useSyncExternalStore:Gm,useId:yh,useHostTransitionStatus:fc,useFormState:nh,useActionState:nh,useOptimistic:function(e,t){var n=Xt();return Fm(n,Rt,e,t)},useMemoCache:nc,useCacheRefresh:bh};mc.useEffectEvent=rh;var Th={readContext:sn,use:Nr,useCallback:dh,useContext:sn,useEffect:rc,useImperativeHandle:fh,useInsertionEffect:sh,useLayoutEffect:uh,useMemo:mh,useReducer:ac,useRef:ih,useState:function(){return ac(Sl)},useDebugValue:sc,useDeferredValue:function(e,t){var n=Xt();return Rt===null?uc(n,e,t):hh(n,Rt.memoizedState,e,t)},useTransition:function(){var e=ac(Sl)[0],t=Xt().memoizedState;return[typeof e=="boolean"?e:oo(e),t]},useSyncExternalStore:Gm,useId:yh,useHostTransitionStatus:fc,useFormState:ah,useActionState:ah,useOptimistic:function(e,t){var n=Xt();return Rt!==null?Fm(n,Rt,e,t):(n.baseState=e,[e,n.queue.dispatch])},useMemoCache:nc,useCacheRefresh:bh};Th.useEffectEvent=rh;function hc(e,t,n,a){t=e.memoizedState,n=n(a,t),n=n==null?t:h({},t,n),e.memoizedState=n,e.lanes===0&&(e.updateQueue.baseState=n)}var gc={enqueueSetState:function(e,t,n){e=e._reactInternals;var a=Nn(),s=Ql(a);s.payload=t,n!=null&&(s.callback=n),t=Kl(e,s,a),t!==null&&(En(t,e,a),no(t,e,a))},enqueueReplaceState:function(e,t,n){e=e._reactInternals;var a=Nn(),s=Ql(a);s.tag=1,s.payload=t,n!=null&&(s.callback=n),t=Kl(e,s,a),t!==null&&(En(t,e,a),no(t,e,a))},enqueueForceUpdate:function(e,t){e=e._reactInternals;var n=Nn(),a=Ql(n);a.tag=2,t!=null&&(a.callback=t),t=Kl(e,a,n),t!==null&&(En(t,e,n),no(t,e,n))}};function Rh(e,t,n,a,s,u,g){return e=e.stateNode,typeof e.shouldComponentUpdate=="function"?e.shouldComponentUpdate(a,u,g):t.prototype&&t.prototype.isPureReactComponent?!Ii(n,a)||!Ii(s,u):!0}function Ah(e,t,n,a){e=t.state,typeof t.componentWillReceiveProps=="function"&&t.componentWillReceiveProps(n,a),typeof t.UNSAFE_componentWillReceiveProps=="function"&&t.UNSAFE_componentWillReceiveProps(n,a),t.state!==e&&gc.enqueueReplaceState(t,t.state,null)}function _a(e,t){var n=t;if("ref"in t){n={};for(var a in t)a!=="ref"&&(n[a]=t[a])}if(e=e.defaultProps){n===t&&(n=h({},n));for(var s in e)n[s]===void 0&&(n[s]=e[s])}return n}function Ch(e){hr(e)}function Mh(e){console.error(e)}function Oh(e){hr(e)}function Ur(e,t){try{var n=e.onUncaughtError;n(t.value,{componentStack:t.stack})}catch(a){setTimeout(function(){throw a})}}function zh(e,t,n){try{var a=e.onCaughtError;a(n.value,{componentStack:n.stack,errorBoundary:t.tag===1?t.stateNode:null})}catch(s){setTimeout(function(){throw s})}}function pc(e,t,n){return n=Ql(n),n.tag=3,n.payload={element:null},n.callback=function(){Ur(e,t)},n}function _h(e){return e=Ql(e),e.tag=3,e}function Nh(e,t,n,a){var s=n.type.getDerivedStateFromError;if(typeof s=="function"){var u=a.value;e.payload=function(){return s(u)},e.callback=function(){zh(t,n,a)}}var g=n.stateNode;g!==null&&typeof g.componentDidCatch=="function"&&(e.callback=function(){zh(t,n,a),typeof s!="function"&&($l===null?$l=new Set([this]):$l.add(this));var b=a.stack;this.componentDidCatch(a.value,{componentStack:b!==null?b:""})})}function qb(e,t,n,a,s){if(n.flags|=32768,a!==null&&typeof a=="object"&&typeof a.then=="function"){if(t=n.alternate,t!==null&&ri(t,n,s,!0),n=Mn.current,n!==null){switch(n.tag){case 31:case 13:return Qn===null?Fr():n.alternate===null&&qt===0&&(qt=3),n.flags&=-257,n.flags|=65536,n.lanes=s,a===Tr?n.flags|=16384:(t=n.updateQueue,t===null?n.updateQueue=new Set([a]):t.add(a),Yc(e,a,s)),!1;case 22:return n.flags|=65536,a===Tr?n.flags|=16384:(t=n.updateQueue,t===null?(t={transitions:null,markerInstances:null,retryQueue:new Set([a])},n.updateQueue=t):(n=t.retryQueue,n===null?t.retryQueue=new Set([a]):n.add(a)),Yc(e,a,s)),!1}throw Error(r(435,n.tag))}return Yc(e,a,s),Fr(),!1}if(at)return t=Mn.current,t!==null?((t.flags&65536)===0&&(t.flags|=256),t.flags|=65536,t.lanes=s,a!==ju&&(e=Error(r(422),{cause:a}),Wi(Yn(e,n)))):(a!==ju&&(t=Error(r(423),{cause:a}),Wi(Yn(t,n))),e=e.current.alternate,e.flags|=65536,s&=-s,e.lanes|=s,a=Yn(a,n),s=pc(e.stateNode,a,s),Ku(e,s),qt!==4&&(qt=2)),!1;var u=Error(r(520),{cause:a});if(u=Yn(u,n),vo===null?vo=[u]:vo.push(u),qt!==4&&(qt=2),t===null)return!0;a=Yn(a,n),n=t;do{switch(n.tag){case 3:return n.flags|=65536,e=s&-s,n.lanes|=e,e=pc(n.stateNode,a,e),Ku(n,e),!1;case 1:if(t=n.type,u=n.stateNode,(n.flags&128)===0&&(typeof t.getDerivedStateFromError=="function"||u!==null&&typeof u.componentDidCatch=="function"&&($l===null||!$l.has(u))))return n.flags|=65536,s&=-s,n.lanes|=s,s=_h(s),Nh(s,e,n,a),Ku(n,s),!1}n=n.return}while(n!==null);return!1}var vc=Error(r(461)),Zt=!1;function un(e,t,n,a){t.child=e===null?Lm(t,null,n,a):Oa(t,e.child,n,a)}function Dh(e,t,n,a,s){n=n.render;var u=t.ref;if("ref"in a){var g={};for(var b in a)b!=="ref"&&(g[b]=a[b])}else g=a;return Ra(t),a=$u(e,t,n,g,u,s),b=Pu(),e!==null&&!Zt?(ec(e,t,s),El(e,t,s)):(at&&b&&Du(t),t.flags|=1,un(e,t,a,s),t.child)}function kh(e,t,n,a,s){if(e===null){var u=n.type;return typeof u=="function"&&!zu(u)&&u.defaultProps===void 0&&n.compare===null?(t.tag=15,t.type=u,jh(e,t,u,a,s)):(e=yr(n.type,null,a,t,t.mode,s),e.ref=t.ref,e.return=t,t.child=e)}if(u=e.child,!Rc(e,s)){var g=u.memoizedProps;if(n=n.compare,n=n!==null?n:Ii,n(g,a)&&e.ref===t.ref)return El(e,t,s)}return t.flags|=1,e=pl(u,a),e.ref=t.ref,e.return=t,t.child=e}function jh(e,t,n,a,s){if(e!==null){var u=e.memoizedProps;if(Ii(u,a)&&e.ref===t.ref)if(Zt=!1,t.pendingProps=a=u,Rc(e,s))(e.flags&131072)!==0&&(Zt=!0);else return t.lanes=e.lanes,El(e,t,s)}return yc(e,t,n,a,s)}function Lh(e,t,n,a){var s=a.children,u=e!==null?e.memoizedState:null;if(e===null&&t.stateNode===null&&(t.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null}),a.mode==="hidden"){if((t.flags&128)!==0){if(u=u!==null?u.baseLanes|n:n,e!==null){for(a=t.child=e.child,s=0;a!==null;)s=s|a.lanes|a.childLanes,a=a.sibling;a=s&~u}else a=0,t.child=null;return Uh(e,t,u,n,a)}if((n&536870912)!==0)t.memoizedState={baseLanes:0,cachePool:null},e!==null&&Er(t,u!==null?u.cachePool:null),u!==null?Hm(t,u):Iu(),Vm(t);else return a=t.lanes=536870912,Uh(e,t,u!==null?u.baseLanes|n:n,n,a)}else u!==null?(Er(t,u.cachePool),Hm(t,u),Il(),t.memoizedState=null):(e!==null&&Er(t,null),Iu(),Il());return un(e,t,s,n),t.child}function uo(e,t){return e!==null&&e.tag===22||t.stateNode!==null||(t.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null}),t.sibling}function Uh(e,t,n,a,s){var u=qu();return u=u===null?null:{parent:Qt._currentValue,pool:u},t.memoizedState={baseLanes:n,cachePool:u},e!==null&&Er(t,null),Iu(),Vm(t),e!==null&&ri(e,t,a,!0),t.childLanes=s,null}function Br(e,t){return t=Vr({mode:t.mode,children:t.children},e.mode),t.ref=e.ref,e.child=t,t.return=e,t}function Bh(e,t,n){return Oa(t,e.child,null,n),e=Br(t,t.pendingProps),e.flags|=2,On(t),t.memoizedState=null,e}function Gb(e,t,n){var a=t.pendingProps,s=(t.flags&128)!==0;if(t.flags&=-129,e===null){if(at){if(a.mode==="hidden")return e=Br(t,a),t.lanes=536870912,uo(null,e);if(Ju(t),(e=Nt)?(e=Jg(e,Xn),e=e!==null&&e.data==="&"?e:null,e!==null&&(t.memoizedState={dehydrated:e,treeContext:Vl!==null?{id:ll,overflow:al}:null,retryLane:536870912,hydrationErrors:null},n=Sm(e),n.return=t,t.child=n,rn=t,Nt=null)):e=null,e===null)throw ql(t);return t.lanes=536870912,null}return Br(t,a)}var u=e.memoizedState;if(u!==null){var g=u.dehydrated;if(Ju(t),s)if(t.flags&256)t.flags&=-257,t=Bh(e,t,n);else if(t.memoizedState!==null)t.child=e.child,t.flags|=128,t=null;else throw Error(r(558));else if(Zt||ri(e,t,n,!1),s=(n&e.childLanes)!==0,Zt||s){if(a=Mt,a!==null&&(g=vt(a,n),g!==0&&g!==u.retryLane))throw u.retryLane=g,Sa(e,g),En(a,e,g),vc;Fr(),t=Bh(e,t,n)}else e=u.treeContext,Nt=Kn(g.nextSibling),rn=t,at=!0,Yl=null,Xn=!1,e!==null&&Tm(t,e),t=Br(t,a),t.flags|=4096;return t}return e=pl(e.child,{mode:a.mode,children:a.children}),e.ref=t.ref,t.child=e,e.return=t,e}function Hr(e,t){var n=t.ref;if(n===null)e!==null&&e.ref!==null&&(t.flags|=4194816);else{if(typeof n!="function"&&typeof n!="object")throw Error(r(284));(e===null||e.ref!==n)&&(t.flags|=4194816)}}function yc(e,t,n,a,s){return Ra(t),n=$u(e,t,n,a,void 0,s),a=Pu(),e!==null&&!Zt?(ec(e,t,s),El(e,t,s)):(at&&a&&Du(t),t.flags|=1,un(e,t,n,s),t.child)}function Hh(e,t,n,a,s,u){return Ra(t),t.updateQueue=null,n=qm(t,a,n,s),Ym(e),a=Pu(),e!==null&&!Zt?(ec(e,t,u),El(e,t,u)):(at&&a&&Du(t),t.flags|=1,un(e,t,n,u),t.child)}function Vh(e,t,n,a,s){if(Ra(t),t.stateNode===null){var u=li,g=n.contextType;typeof g=="object"&&g!==null&&(u=sn(g)),u=new n(a,u),t.memoizedState=u.state!==null&&u.state!==void 0?u.state:null,u.updater=gc,t.stateNode=u,u._reactInternals=t,u=t.stateNode,u.props=a,u.state=t.memoizedState,u.refs={},Xu(t),g=n.contextType,u.context=typeof g=="object"&&g!==null?sn(g):li,u.state=t.memoizedState,g=n.getDerivedStateFromProps,typeof g=="function"&&(hc(t,n,g,a),u.state=t.memoizedState),typeof n.getDerivedStateFromProps=="function"||typeof u.getSnapshotBeforeUpdate=="function"||typeof u.UNSAFE_componentWillMount!="function"&&typeof u.componentWillMount!="function"||(g=u.state,typeof u.componentWillMount=="function"&&u.componentWillMount(),typeof u.UNSAFE_componentWillMount=="function"&&u.UNSAFE_componentWillMount(),g!==u.state&&gc.enqueueReplaceState(u,u.state,null),ao(t,a,u,s),lo(),u.state=t.memoizedState),typeof u.componentDidMount=="function"&&(t.flags|=4194308),a=!0}else if(e===null){u=t.stateNode;var b=t.memoizedProps,z=_a(n,b);u.props=z;var Z=u.context,ne=n.contextType;g=li,typeof ne=="object"&&ne!==null&&(g=sn(ne));var se=n.getDerivedStateFromProps;ne=typeof se=="function"||typeof u.getSnapshotBeforeUpdate=="function",b=t.pendingProps!==b,ne||typeof u.UNSAFE_componentWillReceiveProps!="function"&&typeof u.componentWillReceiveProps!="function"||(b||Z!==g)&&Ah(t,u,a,g),Xl=!1;var J=t.memoizedState;u.state=J,ao(t,a,u,s),lo(),Z=t.memoizedState,b||J!==Z||Xl?(typeof se=="function"&&(hc(t,n,se,a),Z=t.memoizedState),(z=Xl||Rh(t,n,z,a,J,Z,g))?(ne||typeof u.UNSAFE_componentWillMount!="function"&&typeof u.componentWillMount!="function"||(typeof u.componentWillMount=="function"&&u.componentWillMount(),typeof u.UNSAFE_componentWillMount=="function"&&u.UNSAFE_componentWillMount()),typeof u.componentDidMount=="function"&&(t.flags|=4194308)):(typeof u.componentDidMount=="function"&&(t.flags|=4194308),t.memoizedProps=a,t.memoizedState=Z),u.props=a,u.state=Z,u.context=g,a=z):(typeof u.componentDidMount=="function"&&(t.flags|=4194308),a=!1)}else{u=t.stateNode,Qu(e,t),g=t.memoizedProps,ne=_a(n,g),u.props=ne,se=t.pendingProps,J=u.context,Z=n.contextType,z=li,typeof Z=="object"&&Z!==null&&(z=sn(Z)),b=n.getDerivedStateFromProps,(Z=typeof b=="function"||typeof u.getSnapshotBeforeUpdate=="function")||typeof u.UNSAFE_componentWillReceiveProps!="function"&&typeof u.componentWillReceiveProps!="function"||(g!==se||J!==z)&&Ah(t,u,a,z),Xl=!1,J=t.memoizedState,u.state=J,ao(t,a,u,s),lo();var ee=t.memoizedState;g!==se||J!==ee||Xl||e!==null&&e.dependencies!==null&&xr(e.dependencies)?(typeof b=="function"&&(hc(t,n,b,a),ee=t.memoizedState),(ne=Xl||Rh(t,n,ne,a,J,ee,z)||e!==null&&e.dependencies!==null&&xr(e.dependencies))?(Z||typeof u.UNSAFE_componentWillUpdate!="function"&&typeof u.componentWillUpdate!="function"||(typeof u.componentWillUpdate=="function"&&u.componentWillUpdate(a,ee,z),typeof u.UNSAFE_componentWillUpdate=="function"&&u.UNSAFE_componentWillUpdate(a,ee,z)),typeof u.componentDidUpdate=="function"&&(t.flags|=4),typeof u.getSnapshotBeforeUpdate=="function"&&(t.flags|=1024)):(typeof u.componentDidUpdate!="function"||g===e.memoizedProps&&J===e.memoizedState||(t.flags|=4),typeof u.getSnapshotBeforeUpdate!="function"||g===e.memoizedProps&&J===e.memoizedState||(t.flags|=1024),t.memoizedProps=a,t.memoizedState=ee),u.props=a,u.state=ee,u.context=z,a=ne):(typeof u.componentDidUpdate!="function"||g===e.memoizedProps&&J===e.memoizedState||(t.flags|=4),typeof u.getSnapshotBeforeUpdate!="function"||g===e.memoizedProps&&J===e.memoizedState||(t.flags|=1024),a=!1)}return u=a,Hr(e,t),a=(t.flags&128)!==0,u||a?(u=t.stateNode,n=a&&typeof n.getDerivedStateFromError!="function"?null:u.render(),t.flags|=1,e!==null&&a?(t.child=Oa(t,e.child,null,s),t.child=Oa(t,null,n,s)):un(e,t,n,s),t.memoizedState=u.state,e=t.child):e=El(e,t,s),e}function Yh(e,t,n,a){return wa(),t.flags|=256,un(e,t,n,a),t.child}var bc={dehydrated:null,treeContext:null,retryLane:0,hydrationErrors:null};function xc(e){return{baseLanes:e,cachePool:zm()}}function Sc(e,t,n){return e=e!==null?e.childLanes&~n:0,t&&(e|=_n),e}function qh(e,t,n){var a=t.pendingProps,s=!1,u=(t.flags&128)!==0,g;if((g=u)||(g=e!==null&&e.memoizedState===null?!1:(Gt.current&2)!==0),g&&(s=!0,t.flags&=-129),g=(t.flags&32)!==0,t.flags&=-33,e===null){if(at){if(s?Zl(t):Il(),(e=Nt)?(e=Jg(e,Xn),e=e!==null&&e.data!=="&"?e:null,e!==null&&(t.memoizedState={dehydrated:e,treeContext:Vl!==null?{id:ll,overflow:al}:null,retryLane:536870912,hydrationErrors:null},n=Sm(e),n.return=t,t.child=n,rn=t,Nt=null)):e=null,e===null)throw ql(t);return lf(e)?t.lanes=32:t.lanes=536870912,null}var b=a.children;return a=a.fallback,s?(Il(),s=t.mode,b=Vr({mode:"hidden",children:b},s),a=Ea(a,s,n,null),b.return=t,a.return=t,b.sibling=a,t.child=b,a=t.child,a.memoizedState=xc(n),a.childLanes=Sc(e,g,n),t.memoizedState=bc,uo(null,a)):(Zl(t),Ec(t,b))}var z=e.memoizedState;if(z!==null&&(b=z.dehydrated,b!==null)){if(u)t.flags&256?(Zl(t),t.flags&=-257,t=wc(e,t,n)):t.memoizedState!==null?(Il(),t.child=e.child,t.flags|=128,t=null):(Il(),b=a.fallback,s=t.mode,a=Vr({mode:"visible",children:a.children},s),b=Ea(b,s,n,null),b.flags|=2,a.return=t,b.return=t,a.sibling=b,t.child=a,Oa(t,e.child,null,n),a=t.child,a.memoizedState=xc(n),a.childLanes=Sc(e,g,n),t.memoizedState=bc,t=uo(null,a));else if(Zl(t),lf(b)){if(g=b.nextSibling&&b.nextSibling.dataset,g)var Z=g.dgst;g=Z,a=Error(r(419)),a.stack="",a.digest=g,Wi({value:a,source:null,stack:null}),t=wc(e,t,n)}else if(Zt||ri(e,t,n,!1),g=(n&e.childLanes)!==0,Zt||g){if(g=Mt,g!==null&&(a=vt(g,n),a!==0&&a!==z.retryLane))throw z.retryLane=a,Sa(e,a),En(g,e,a),vc;nf(b)||Fr(),t=wc(e,t,n)}else nf(b)?(t.flags|=192,t.child=e.child,t=null):(e=z.treeContext,Nt=Kn(b.nextSibling),rn=t,at=!0,Yl=null,Xn=!1,e!==null&&Tm(t,e),t=Ec(t,a.children),t.flags|=4096);return t}return s?(Il(),b=a.fallback,s=t.mode,z=e.child,Z=z.sibling,a=pl(z,{mode:"hidden",children:a.children}),a.subtreeFlags=z.subtreeFlags&65011712,Z!==null?b=pl(Z,b):(b=Ea(b,s,n,null),b.flags|=2),b.return=t,a.return=t,a.sibling=b,t.child=a,uo(null,a),a=t.child,b=e.child.memoizedState,b===null?b=xc(n):(s=b.cachePool,s!==null?(z=Qt._currentValue,s=s.parent!==z?{parent:z,pool:z}:s):s=zm(),b={baseLanes:b.baseLanes|n,cachePool:s}),a.memoizedState=b,a.childLanes=Sc(e,g,n),t.memoizedState=bc,uo(e.child,a)):(Zl(t),n=e.child,e=n.sibling,n=pl(n,{mode:"visible",children:a.children}),n.return=t,n.sibling=null,e!==null&&(g=t.deletions,g===null?(t.deletions=[e],t.flags|=16):g.push(e)),t.child=n,t.memoizedState=null,n)}function Ec(e,t){return t=Vr({mode:"visible",children:t},e.mode),t.return=e,e.child=t}function Vr(e,t){return e=Cn(22,e,null,t),e.lanes=0,e}function wc(e,t,n){return Oa(t,e.child,null,n),e=Ec(t,t.pendingProps.children),e.flags|=2,t.memoizedState=null,e}function Gh(e,t,n){e.lanes|=t;var a=e.alternate;a!==null&&(a.lanes|=t),Bu(e.return,t,n)}function Tc(e,t,n,a,s,u){var g=e.memoizedState;g===null?e.memoizedState={isBackwards:t,rendering:null,renderingStartTime:0,last:a,tail:n,tailMode:s,treeForkCount:u}:(g.isBackwards=t,g.rendering=null,g.renderingStartTime=0,g.last=a,g.tail=n,g.tailMode=s,g.treeForkCount=u)}function Xh(e,t,n){var a=t.pendingProps,s=a.revealOrder,u=a.tail;a=a.children;var g=Gt.current,b=(g&2)!==0;if(b?(g=g&1|2,t.flags|=128):g&=1,W(Gt,g),un(e,t,a,n),a=at?Ji:0,!b&&e!==null&&(e.flags&128)!==0)e:for(e=t.child;e!==null;){if(e.tag===13)e.memoizedState!==null&&Gh(e,n,t);else if(e.tag===19)Gh(e,n,t);else if(e.child!==null){e.child.return=e,e=e.child;continue}if(e===t)break e;for(;e.sibling===null;){if(e.return===null||e.return===t)break e;e=e.return}e.sibling.return=e.return,e=e.sibling}switch(s){case"forwards":for(n=t.child,s=null;n!==null;)e=n.alternate,e!==null&&Mr(e)===null&&(s=n),n=n.sibling;n=s,n===null?(s=t.child,t.child=null):(s=n.sibling,n.sibling=null),Tc(t,!1,s,n,u,a);break;case"backwards":case"unstable_legacy-backwards":for(n=null,s=t.child,t.child=null;s!==null;){if(e=s.alternate,e!==null&&Mr(e)===null){t.child=s;break}e=s.sibling,s.sibling=n,n=s,s=e}Tc(t,!0,n,null,u,a);break;case"together":Tc(t,!1,null,null,void 0,a);break;default:t.memoizedState=null}return t.child}function El(e,t,n){if(e!==null&&(t.dependencies=e.dependencies),Wl|=t.lanes,(n&t.childLanes)===0)if(e!==null){if(ri(e,t,n,!1),(n&t.childLanes)===0)return null}else return null;if(e!==null&&t.child!==e.child)throw Error(r(153));if(t.child!==null){for(e=t.child,n=pl(e,e.pendingProps),t.child=n,n.return=t;e.sibling!==null;)e=e.sibling,n=n.sibling=pl(e,e.pendingProps),n.return=t;n.sibling=null}return t.child}function Rc(e,t){return(e.lanes&t)!==0?!0:(e=e.dependencies,!!(e!==null&&xr(e)))}function Xb(e,t,n){switch(t.tag){case 3:ce(t,t.stateNode.containerInfo),Gl(t,Qt,e.memoizedState.cache),wa();break;case 27:case 5:De(t);break;case 4:ce(t,t.stateNode.containerInfo);break;case 10:Gl(t,t.type,t.memoizedProps.value);break;case 31:if(t.memoizedState!==null)return t.flags|=128,Ju(t),null;break;case 13:var a=t.memoizedState;if(a!==null)return a.dehydrated!==null?(Zl(t),t.flags|=128,null):(n&t.child.childLanes)!==0?qh(e,t,n):(Zl(t),e=El(e,t,n),e!==null?e.sibling:null);Zl(t);break;case 19:var s=(e.flags&128)!==0;if(a=(n&t.childLanes)!==0,a||(ri(e,t,n,!1),a=(n&t.childLanes)!==0),s){if(a)return Xh(e,t,n);t.flags|=128}if(s=t.memoizedState,s!==null&&(s.rendering=null,s.tail=null,s.lastEffect=null),W(Gt,Gt.current),a)break;return null;case 22:return t.lanes=0,Lh(e,t,n,t.pendingProps);case 24:Gl(t,Qt,e.memoizedState.cache)}return El(e,t,n)}function Qh(e,t,n){if(e!==null)if(e.memoizedProps!==t.pendingProps)Zt=!0;else{if(!Rc(e,n)&&(t.flags&128)===0)return Zt=!1,Xb(e,t,n);Zt=(e.flags&131072)!==0}else Zt=!1,at&&(t.flags&1048576)!==0&&wm(t,Ji,t.index);switch(t.lanes=0,t.tag){case 16:e:{var a=t.pendingProps;if(e=Ca(t.elementType),t.type=e,typeof e=="function")zu(e)?(a=_a(e,a),t.tag=1,t=Vh(null,t,e,a,n)):(t.tag=0,t=yc(null,t,e,a,n));else{if(e!=null){var s=e.$$typeof;if(s===N){t.tag=11,t=Dh(null,t,e,a,n);break e}else if(s===Y){t.tag=14,t=kh(null,t,e,a,n);break e}}throw t=ae(e)||e,Error(r(306,t,""))}}return t;case 0:return yc(e,t,t.type,t.pendingProps,n);case 1:return a=t.type,s=_a(a,t.pendingProps),Vh(e,t,a,s,n);case 3:e:{if(ce(t,t.stateNode.containerInfo),e===null)throw Error(r(387));a=t.pendingProps;var u=t.memoizedState;s=u.element,Qu(e,t),ao(t,a,null,n);var g=t.memoizedState;if(a=g.cache,Gl(t,Qt,a),a!==u.cache&&Hu(t,[Qt],n,!0),lo(),a=g.element,u.isDehydrated)if(u={element:a,isDehydrated:!1,cache:g.cache},t.updateQueue.baseState=u,t.memoizedState=u,t.flags&256){t=Yh(e,t,a,n);break e}else if(a!==s){s=Yn(Error(r(424)),t),Wi(s),t=Yh(e,t,a,n);break e}else{switch(e=t.stateNode.containerInfo,e.nodeType){case 9:e=e.body;break;default:e=e.nodeName==="HTML"?e.ownerDocument.body:e}for(Nt=Kn(e.firstChild),rn=t,at=!0,Yl=null,Xn=!0,n=Lm(t,null,a,n),t.child=n;n;)n.flags=n.flags&-3|4096,n=n.sibling}else{if(wa(),a===s){t=El(e,t,n);break e}un(e,t,a,n)}t=t.child}return t;case 26:return Hr(e,t),e===null?(n=np(t.type,null,t.pendingProps,null))?t.memoizedState=n:at||(n=t.type,e=t.pendingProps,a=ns(P.current).createElement(n),a[Wt]=t,a[on]=e,cn(a,n,e),nn(a),t.stateNode=a):t.memoizedState=np(t.type,e.memoizedProps,t.pendingProps,e.memoizedState),null;case 27:return De(t),e===null&&at&&(a=t.stateNode=Pg(t.type,t.pendingProps,P.current),rn=t,Xn=!0,s=Nt,na(t.type)?(af=s,Nt=Kn(a.firstChild)):Nt=s),un(e,t,t.pendingProps.children,n),Hr(e,t),e===null&&(t.flags|=4194304),t.child;case 5:return e===null&&at&&((s=a=Nt)&&(a=x1(a,t.type,t.pendingProps,Xn),a!==null?(t.stateNode=a,rn=t,Nt=Kn(a.firstChild),Xn=!1,s=!0):s=!1),s||ql(t)),De(t),s=t.type,u=t.pendingProps,g=e!==null?e.memoizedProps:null,a=u.children,Pc(s,u)?a=null:g!==null&&Pc(s,g)&&(t.flags|=32),t.memoizedState!==null&&(s=$u(e,t,jb,null,null,n),Ro._currentValue=s),Hr(e,t),un(e,t,a,n),t.child;case 6:return e===null&&at&&((e=n=Nt)&&(n=S1(n,t.pendingProps,Xn),n!==null?(t.stateNode=n,rn=t,Nt=null,e=!0):e=!1),e||ql(t)),null;case 13:return qh(e,t,n);case 4:return ce(t,t.stateNode.containerInfo),a=t.pendingProps,e===null?t.child=Oa(t,null,a,n):un(e,t,a,n),t.child;case 11:return Dh(e,t,t.type,t.pendingProps,n);case 7:return un(e,t,t.pendingProps,n),t.child;case 8:return un(e,t,t.pendingProps.children,n),t.child;case 12:return un(e,t,t.pendingProps.children,n),t.child;case 10:return a=t.pendingProps,Gl(t,t.type,a.value),un(e,t,a.children,n),t.child;case 9:return s=t.type._context,a=t.pendingProps.children,Ra(t),s=sn(s),a=a(s),t.flags|=1,un(e,t,a,n),t.child;case 14:return kh(e,t,t.type,t.pendingProps,n);case 15:return jh(e,t,t.type,t.pendingProps,n);case 19:return Xh(e,t,n);case 31:return Gb(e,t,n);case 22:return Lh(e,t,n,t.pendingProps);case 24:return Ra(t),a=sn(Qt),e===null?(s=qu(),s===null&&(s=Mt,u=Vu(),s.pooledCache=u,u.refCount++,u!==null&&(s.pooledCacheLanes|=n),s=u),t.memoizedState={parent:a,cache:s},Xu(t),Gl(t,Qt,s)):((e.lanes&n)!==0&&(Qu(e,t),ao(t,null,null,n),lo()),s=e.memoizedState,u=t.memoizedState,s.parent!==a?(s={parent:a,cache:a},t.memoizedState=s,t.lanes===0&&(t.memoizedState=t.updateQueue.baseState=s),Gl(t,Qt,a)):(a=u.cache,Gl(t,Qt,a),a!==s.cache&&Hu(t,[Qt],n,!0))),un(e,t,t.pendingProps.children,n),t.child;case 29:throw t.pendingProps}throw Error(r(156,t.tag))}function wl(e){e.flags|=4}function Ac(e,t,n,a,s){if((t=(e.mode&32)!==0)&&(t=!1),t){if(e.flags|=16777216,(s&335544128)===s)if(e.stateNode.complete)e.flags|=8192;else if(yg())e.flags|=8192;else throw Ma=Tr,Gu}else e.flags&=-16777217}function Kh(e,t){if(t.type!=="stylesheet"||(t.state.loading&4)!==0)e.flags&=-16777217;else if(e.flags|=16777216,!rp(t))if(yg())e.flags|=8192;else throw Ma=Tr,Gu}function Yr(e,t){t!==null&&(e.flags|=4),e.flags&16384&&(t=e.tag!==22?xt():536870912,e.lanes|=t,bi|=t)}function co(e,t){if(!at)switch(e.tailMode){case"hidden":t=e.tail;for(var n=null;t!==null;)t.alternate!==null&&(n=t),t=t.sibling;n===null?e.tail=null:n.sibling=null;break;case"collapsed":n=e.tail;for(var a=null;n!==null;)n.alternate!==null&&(a=n),n=n.sibling;a===null?t||e.tail===null?e.tail=null:e.tail.sibling=null:a.sibling=null}}function Dt(e){var t=e.alternate!==null&&e.alternate.child===e.child,n=0,a=0;if(t)for(var s=e.child;s!==null;)n|=s.lanes|s.childLanes,a|=s.subtreeFlags&65011712,a|=s.flags&65011712,s.return=e,s=s.sibling;else for(s=e.child;s!==null;)n|=s.lanes|s.childLanes,a|=s.subtreeFlags,a|=s.flags,s.return=e,s=s.sibling;return e.subtreeFlags|=a,e.childLanes=n,t}function Qb(e,t,n){var a=t.pendingProps;switch(ku(t),t.tag){case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return Dt(t),null;case 1:return Dt(t),null;case 3:return n=t.stateNode,a=null,e!==null&&(a=e.memoizedState.cache),t.memoizedState.cache!==a&&(t.flags|=2048),bl(Qt),be(),n.pendingContext&&(n.context=n.pendingContext,n.pendingContext=null),(e===null||e.child===null)&&(oi(t)?wl(t):e===null||e.memoizedState.isDehydrated&&(t.flags&256)===0||(t.flags|=1024,Lu())),Dt(t),null;case 26:var s=t.type,u=t.memoizedState;return e===null?(wl(t),u!==null?(Dt(t),Kh(t,u)):(Dt(t),Ac(t,s,null,a,n))):u?u!==e.memoizedState?(wl(t),Dt(t),Kh(t,u)):(Dt(t),t.flags&=-16777217):(e=e.memoizedProps,e!==a&&wl(t),Dt(t),Ac(t,s,e,a,n)),null;case 27:if(we(t),n=P.current,s=t.type,e!==null&&t.stateNode!=null)e.memoizedProps!==a&&wl(t);else{if(!a){if(t.stateNode===null)throw Error(r(166));return Dt(t),null}e=Q.current,oi(t)?Rm(t):(e=Pg(s,a,n),t.stateNode=e,wl(t))}return Dt(t),null;case 5:if(we(t),s=t.type,e!==null&&t.stateNode!=null)e.memoizedProps!==a&&wl(t);else{if(!a){if(t.stateNode===null)throw Error(r(166));return Dt(t),null}if(u=Q.current,oi(t))Rm(t);else{var g=ns(P.current);switch(u){case 1:u=g.createElementNS("http://www.w3.org/2000/svg",s);break;case 2:u=g.createElementNS("http://www.w3.org/1998/Math/MathML",s);break;default:switch(s){case"svg":u=g.createElementNS("http://www.w3.org/2000/svg",s);break;case"math":u=g.createElementNS("http://www.w3.org/1998/Math/MathML",s);break;case"script":u=g.createElement("div"),u.innerHTML="<script><\/script>",u=u.removeChild(u.firstChild);break;case"select":u=typeof a.is=="string"?g.createElement("select",{is:a.is}):g.createElement("select"),a.multiple?u.multiple=!0:a.size&&(u.size=a.size);break;default:u=typeof a.is=="string"?g.createElement(s,{is:a.is}):g.createElement(s)}}u[Wt]=t,u[on]=a;e:for(g=t.child;g!==null;){if(g.tag===5||g.tag===6)u.appendChild(g.stateNode);else if(g.tag!==4&&g.tag!==27&&g.child!==null){g.child.return=g,g=g.child;continue}if(g===t)break e;for(;g.sibling===null;){if(g.return===null||g.return===t)break e;g=g.return}g.sibling.return=g.return,g=g.sibling}t.stateNode=u;e:switch(cn(u,s,a),s){case"button":case"input":case"select":case"textarea":a=!!a.autoFocus;break e;case"img":a=!0;break e;default:a=!1}a&&wl(t)}}return Dt(t),Ac(t,t.type,e===null?null:e.memoizedProps,t.pendingProps,n),null;case 6:if(e&&t.stateNode!=null)e.memoizedProps!==a&&wl(t);else{if(typeof a!="string"&&t.stateNode===null)throw Error(r(166));if(e=P.current,oi(t)){if(e=t.stateNode,n=t.memoizedProps,a=null,s=rn,s!==null)switch(s.tag){case 27:case 5:a=s.memoizedProps}e[Wt]=t,e=!!(e.nodeValue===n||a!==null&&a.suppressHydrationWarning===!0||qg(e.nodeValue,n)),e||ql(t,!0)}else e=ns(e).createTextNode(a),e[Wt]=t,t.stateNode=e}return Dt(t),null;case 31:if(n=t.memoizedState,e===null||e.memoizedState!==null){if(a=oi(t),n!==null){if(e===null){if(!a)throw Error(r(318));if(e=t.memoizedState,e=e!==null?e.dehydrated:null,!e)throw Error(r(557));e[Wt]=t}else wa(),(t.flags&128)===0&&(t.memoizedState=null),t.flags|=4;Dt(t),e=!1}else n=Lu(),e!==null&&e.memoizedState!==null&&(e.memoizedState.hydrationErrors=n),e=!0;if(!e)return t.flags&256?(On(t),t):(On(t),null);if((t.flags&128)!==0)throw Error(r(558))}return Dt(t),null;case 13:if(a=t.memoizedState,e===null||e.memoizedState!==null&&e.memoizedState.dehydrated!==null){if(s=oi(t),a!==null&&a.dehydrated!==null){if(e===null){if(!s)throw Error(r(318));if(s=t.memoizedState,s=s!==null?s.dehydrated:null,!s)throw Error(r(317));s[Wt]=t}else wa(),(t.flags&128)===0&&(t.memoizedState=null),t.flags|=4;Dt(t),s=!1}else s=Lu(),e!==null&&e.memoizedState!==null&&(e.memoizedState.hydrationErrors=s),s=!0;if(!s)return t.flags&256?(On(t),t):(On(t),null)}return On(t),(t.flags&128)!==0?(t.lanes=n,t):(n=a!==null,e=e!==null&&e.memoizedState!==null,n&&(a=t.child,s=null,a.alternate!==null&&a.alternate.memoizedState!==null&&a.alternate.memoizedState.cachePool!==null&&(s=a.alternate.memoizedState.cachePool.pool),u=null,a.memoizedState!==null&&a.memoizedState.cachePool!==null&&(u=a.memoizedState.cachePool.pool),u!==s&&(a.flags|=2048)),n!==e&&n&&(t.child.flags|=8192),Yr(t,t.updateQueue),Dt(t),null);case 4:return be(),e===null&&Ic(t.stateNode.containerInfo),Dt(t),null;case 10:return bl(t.type),Dt(t),null;case 19:if(O(Gt),a=t.memoizedState,a===null)return Dt(t),null;if(s=(t.flags&128)!==0,u=a.rendering,u===null)if(s)co(a,!1);else{if(qt!==0||e!==null&&(e.flags&128)!==0)for(e=t.child;e!==null;){if(u=Mr(e),u!==null){for(t.flags|=128,co(a,!1),e=u.updateQueue,t.updateQueue=e,Yr(t,e),t.subtreeFlags=0,e=n,n=t.child;n!==null;)xm(n,e),n=n.sibling;return W(Gt,Gt.current&1|2),at&&vl(t,a.treeForkCount),t.child}e=e.sibling}a.tail!==null&&Se()>Kr&&(t.flags|=128,s=!0,co(a,!1),t.lanes=4194304)}else{if(!s)if(e=Mr(u),e!==null){if(t.flags|=128,s=!0,e=e.updateQueue,t.updateQueue=e,Yr(t,e),co(a,!0),a.tail===null&&a.tailMode==="hidden"&&!u.alternate&&!at)return Dt(t),null}else 2*Se()-a.renderingStartTime>Kr&&n!==536870912&&(t.flags|=128,s=!0,co(a,!1),t.lanes=4194304);a.isBackwards?(u.sibling=t.child,t.child=u):(e=a.last,e!==null?e.sibling=u:t.child=u,a.last=u)}return a.tail!==null?(e=a.tail,a.rendering=e,a.tail=e.sibling,a.renderingStartTime=Se(),e.sibling=null,n=Gt.current,W(Gt,s?n&1|2:n&1),at&&vl(t,a.treeForkCount),e):(Dt(t),null);case 22:case 23:return On(t),Fu(),a=t.memoizedState!==null,e!==null?e.memoizedState!==null!==a&&(t.flags|=8192):a&&(t.flags|=8192),a?(n&536870912)!==0&&(t.flags&128)===0&&(Dt(t),t.subtreeFlags&6&&(t.flags|=8192)):Dt(t),n=t.updateQueue,n!==null&&Yr(t,n.retryQueue),n=null,e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(n=e.memoizedState.cachePool.pool),a=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(a=t.memoizedState.cachePool.pool),a!==n&&(t.flags|=2048),e!==null&&O(Aa),null;case 24:return n=null,e!==null&&(n=e.memoizedState.cache),t.memoizedState.cache!==n&&(t.flags|=2048),bl(Qt),Dt(t),null;case 25:return null;case 30:return null}throw Error(r(156,t.tag))}function Kb(e,t){switch(ku(t),t.tag){case 1:return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 3:return bl(Qt),be(),e=t.flags,(e&65536)!==0&&(e&128)===0?(t.flags=e&-65537|128,t):null;case 26:case 27:case 5:return we(t),null;case 31:if(t.memoizedState!==null){if(On(t),t.alternate===null)throw Error(r(340));wa()}return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 13:if(On(t),e=t.memoizedState,e!==null&&e.dehydrated!==null){if(t.alternate===null)throw Error(r(340));wa()}return e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 19:return O(Gt),null;case 4:return be(),null;case 10:return bl(t.type),null;case 22:case 23:return On(t),Fu(),e!==null&&O(Aa),e=t.flags,e&65536?(t.flags=e&-65537|128,t):null;case 24:return bl(Qt),null;case 25:return null;default:return null}}function Zh(e,t){switch(ku(t),t.tag){case 3:bl(Qt),be();break;case 26:case 27:case 5:we(t);break;case 4:be();break;case 31:t.memoizedState!==null&&On(t);break;case 13:On(t);break;case 19:O(Gt);break;case 10:bl(t.type);break;case 22:case 23:On(t),Fu(),e!==null&&O(Aa);break;case 24:bl(Qt)}}function fo(e,t){try{var n=t.updateQueue,a=n!==null?n.lastEffect:null;if(a!==null){var s=a.next;n=s;do{if((n.tag&e)===e){a=void 0;var u=n.create,g=n.inst;a=u(),g.destroy=a}n=n.next}while(n!==s)}}catch(b){wt(t,t.return,b)}}function Fl(e,t,n){try{var a=t.updateQueue,s=a!==null?a.lastEffect:null;if(s!==null){var u=s.next;a=u;do{if((a.tag&e)===e){var g=a.inst,b=g.destroy;if(b!==void 0){g.destroy=void 0,s=t;var z=n,Z=b;try{Z()}catch(ne){wt(s,z,ne)}}}a=a.next}while(a!==u)}}catch(ne){wt(t,t.return,ne)}}function Ih(e){var t=e.updateQueue;if(t!==null){var n=e.stateNode;try{Bm(t,n)}catch(a){wt(e,e.return,a)}}}function Fh(e,t,n){n.props=_a(e.type,e.memoizedProps),n.state=e.memoizedState;try{n.componentWillUnmount()}catch(a){wt(e,t,a)}}function mo(e,t){try{var n=e.ref;if(n!==null){switch(e.tag){case 26:case 27:case 5:var a=e.stateNode;break;case 30:a=e.stateNode;break;default:a=e.stateNode}typeof n=="function"?e.refCleanup=n(a):n.current=a}}catch(s){wt(e,t,s)}}function il(e,t){var n=e.ref,a=e.refCleanup;if(n!==null)if(typeof a=="function")try{a()}catch(s){wt(e,t,s)}finally{e.refCleanup=null,e=e.alternate,e!=null&&(e.refCleanup=null)}else if(typeof n=="function")try{n(null)}catch(s){wt(e,t,s)}else n.current=null}function Jh(e){var t=e.type,n=e.memoizedProps,a=e.stateNode;try{e:switch(t){case"button":case"input":case"select":case"textarea":n.autoFocus&&a.focus();break e;case"img":n.src?a.src=n.src:n.srcSet&&(a.srcset=n.srcSet)}}catch(s){wt(e,e.return,s)}}function Cc(e,t,n){try{var a=e.stateNode;h1(a,e.type,n,t),a[on]=t}catch(s){wt(e,e.return,s)}}function Wh(e){return e.tag===5||e.tag===3||e.tag===26||e.tag===27&&na(e.type)||e.tag===4}function Mc(e){e:for(;;){for(;e.sibling===null;){if(e.return===null||Wh(e.return))return null;e=e.return}for(e.sibling.return=e.return,e=e.sibling;e.tag!==5&&e.tag!==6&&e.tag!==18;){if(e.tag===27&&na(e.type)||e.flags&2||e.child===null||e.tag===4)continue e;e.child.return=e,e=e.child}if(!(e.flags&2))return e.stateNode}}function Oc(e,t,n){var a=e.tag;if(a===5||a===6)e=e.stateNode,t?(n.nodeType===9?n.body:n.nodeName==="HTML"?n.ownerDocument.body:n).insertBefore(e,t):(t=n.nodeType===9?n.body:n.nodeName==="HTML"?n.ownerDocument.body:n,t.appendChild(e),n=n._reactRootContainer,n!=null||t.onclick!==null||(t.onclick=hl));else if(a!==4&&(a===27&&na(e.type)&&(n=e.stateNode,t=null),e=e.child,e!==null))for(Oc(e,t,n),e=e.sibling;e!==null;)Oc(e,t,n),e=e.sibling}function qr(e,t,n){var a=e.tag;if(a===5||a===6)e=e.stateNode,t?n.insertBefore(e,t):n.appendChild(e);else if(a!==4&&(a===27&&na(e.type)&&(n=e.stateNode),e=e.child,e!==null))for(qr(e,t,n),e=e.sibling;e!==null;)qr(e,t,n),e=e.sibling}function $h(e){var t=e.stateNode,n=e.memoizedProps;try{for(var a=e.type,s=t.attributes;s.length;)t.removeAttributeNode(s[0]);cn(t,a,n),t[Wt]=e,t[on]=n}catch(u){wt(e,e.return,u)}}var Tl=!1,It=!1,zc=!1,Ph=typeof WeakSet=="function"?WeakSet:Set,ln=null;function Zb(e,t){if(e=e.containerInfo,Wc=us,e=fm(e),wu(e)){if("selectionStart"in e)var n={start:e.selectionStart,end:e.selectionEnd};else e:{n=(n=e.ownerDocument)&&n.defaultView||window;var a=n.getSelection&&n.getSelection();if(a&&a.rangeCount!==0){n=a.anchorNode;var s=a.anchorOffset,u=a.focusNode;a=a.focusOffset;try{n.nodeType,u.nodeType}catch{n=null;break e}var g=0,b=-1,z=-1,Z=0,ne=0,se=e,J=null;t:for(;;){for(var ee;se!==n||s!==0&&se.nodeType!==3||(b=g+s),se!==u||a!==0&&se.nodeType!==3||(z=g+a),se.nodeType===3&&(g+=se.nodeValue.length),(ee=se.firstChild)!==null;)J=se,se=ee;for(;;){if(se===e)break t;if(J===n&&++Z===s&&(b=g),J===u&&++ne===a&&(z=g),(ee=se.nextSibling)!==null)break;se=J,J=se.parentNode}se=ee}n=b===-1||z===-1?null:{start:b,end:z}}else n=null}n=n||{start:0,end:0}}else n=null;for($c={focusedElem:e,selectionRange:n},us=!1,ln=t;ln!==null;)if(t=ln,e=t.child,(t.subtreeFlags&1028)!==0&&e!==null)e.return=t,ln=e;else for(;ln!==null;){switch(t=ln,u=t.alternate,e=t.flags,t.tag){case 0:if((e&4)!==0&&(e=t.updateQueue,e=e!==null?e.events:null,e!==null))for(n=0;n<e.length;n++)s=e[n],s.ref.impl=s.nextImpl;break;case 11:case 15:break;case 1:if((e&1024)!==0&&u!==null){e=void 0,n=t,s=u.memoizedProps,u=u.memoizedState,a=n.stateNode;try{var Ae=_a(n.type,s);e=a.getSnapshotBeforeUpdate(Ae,u),a.__reactInternalSnapshotBeforeUpdate=e}catch(Ue){wt(n,n.return,Ue)}}break;case 3:if((e&1024)!==0){if(e=t.stateNode.containerInfo,n=e.nodeType,n===9)tf(e);else if(n===1)switch(e.nodeName){case"HEAD":case"HTML":case"BODY":tf(e);break;default:e.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if((e&1024)!==0)throw Error(r(163))}if(e=t.sibling,e!==null){e.return=t.return,ln=e;break}ln=t.return}}function eg(e,t,n){var a=n.flags;switch(n.tag){case 0:case 11:case 15:Al(e,n),a&4&&fo(5,n);break;case 1:if(Al(e,n),a&4)if(e=n.stateNode,t===null)try{e.componentDidMount()}catch(g){wt(n,n.return,g)}else{var s=_a(n.type,t.memoizedProps);t=t.memoizedState;try{e.componentDidUpdate(s,t,e.__reactInternalSnapshotBeforeUpdate)}catch(g){wt(n,n.return,g)}}a&64&&Ih(n),a&512&&mo(n,n.return);break;case 3:if(Al(e,n),a&64&&(e=n.updateQueue,e!==null)){if(t=null,n.child!==null)switch(n.child.tag){case 27:case 5:t=n.child.stateNode;break;case 1:t=n.child.stateNode}try{Bm(e,t)}catch(g){wt(n,n.return,g)}}break;case 27:t===null&&a&4&&$h(n);case 26:case 5:Al(e,n),t===null&&a&4&&Jh(n),a&512&&mo(n,n.return);break;case 12:Al(e,n);break;case 31:Al(e,n),a&4&&lg(e,n);break;case 13:Al(e,n),a&4&&ag(e,n),a&64&&(e=n.memoizedState,e!==null&&(e=e.dehydrated,e!==null&&(n=n1.bind(null,n),E1(e,n))));break;case 22:if(a=n.memoizedState!==null||Tl,!a){t=t!==null&&t.memoizedState!==null||It,s=Tl;var u=It;Tl=a,(It=t)&&!u?Cl(e,n,(n.subtreeFlags&8772)!==0):Al(e,n),Tl=s,It=u}break;case 30:break;default:Al(e,n)}}function tg(e){var t=e.alternate;t!==null&&(e.alternate=null,tg(t)),e.child=null,e.deletions=null,e.sibling=null,e.tag===5&&(t=e.stateNode,t!==null&&ou(t)),e.stateNode=null,e.return=null,e.dependencies=null,e.memoizedProps=null,e.memoizedState=null,e.pendingProps=null,e.stateNode=null,e.updateQueue=null}var jt=null,yn=!1;function Rl(e,t,n){for(n=n.child;n!==null;)ng(e,t,n),n=n.sibling}function ng(e,t,n){if(pt&&typeof pt.onCommitFiberUnmount=="function")try{pt.onCommitFiberUnmount(gt,n)}catch{}switch(n.tag){case 26:It||il(n,t),Rl(e,t,n),n.memoizedState?n.memoizedState.count--:n.stateNode&&(n=n.stateNode,n.parentNode.removeChild(n));break;case 27:It||il(n,t);var a=jt,s=yn;na(n.type)&&(jt=n.stateNode,yn=!1),Rl(e,t,n),Eo(n.stateNode),jt=a,yn=s;break;case 5:It||il(n,t);case 6:if(a=jt,s=yn,jt=null,Rl(e,t,n),jt=a,yn=s,jt!==null)if(yn)try{(jt.nodeType===9?jt.body:jt.nodeName==="HTML"?jt.ownerDocument.body:jt).removeChild(n.stateNode)}catch(u){wt(n,t,u)}else try{jt.removeChild(n.stateNode)}catch(u){wt(n,t,u)}break;case 18:jt!==null&&(yn?(e=jt,Ig(e.nodeType===9?e.body:e.nodeName==="HTML"?e.ownerDocument.body:e,n.stateNode),Ci(e)):Ig(jt,n.stateNode));break;case 4:a=jt,s=yn,jt=n.stateNode.containerInfo,yn=!0,Rl(e,t,n),jt=a,yn=s;break;case 0:case 11:case 14:case 15:Fl(2,n,t),It||Fl(4,n,t),Rl(e,t,n);break;case 1:It||(il(n,t),a=n.stateNode,typeof a.componentWillUnmount=="function"&&Fh(n,t,a)),Rl(e,t,n);break;case 21:Rl(e,t,n);break;case 22:It=(a=It)||n.memoizedState!==null,Rl(e,t,n),It=a;break;default:Rl(e,t,n)}}function lg(e,t){if(t.memoizedState===null&&(e=t.alternate,e!==null&&(e=e.memoizedState,e!==null))){e=e.dehydrated;try{Ci(e)}catch(n){wt(t,t.return,n)}}}function ag(e,t){if(t.memoizedState===null&&(e=t.alternate,e!==null&&(e=e.memoizedState,e!==null&&(e=e.dehydrated,e!==null))))try{Ci(e)}catch(n){wt(t,t.return,n)}}function Ib(e){switch(e.tag){case 31:case 13:case 19:var t=e.stateNode;return t===null&&(t=e.stateNode=new Ph),t;case 22:return e=e.stateNode,t=e._retryCache,t===null&&(t=e._retryCache=new Ph),t;default:throw Error(r(435,e.tag))}}function Gr(e,t){var n=Ib(e);t.forEach(function(a){if(!n.has(a)){n.add(a);var s=l1.bind(null,e,a);a.then(s,s)}})}function bn(e,t){var n=t.deletions;if(n!==null)for(var a=0;a<n.length;a++){var s=n[a],u=e,g=t,b=g;e:for(;b!==null;){switch(b.tag){case 27:if(na(b.type)){jt=b.stateNode,yn=!1;break e}break;case 5:jt=b.stateNode,yn=!1;break e;case 3:case 4:jt=b.stateNode.containerInfo,yn=!0;break e}b=b.return}if(jt===null)throw Error(r(160));ng(u,g,s),jt=null,yn=!1,u=s.alternate,u!==null&&(u.return=null),s.return=null}if(t.subtreeFlags&13886)for(t=t.child;t!==null;)ig(t,e),t=t.sibling}var Wn=null;function ig(e,t){var n=e.alternate,a=e.flags;switch(e.tag){case 0:case 11:case 14:case 15:bn(t,e),xn(e),a&4&&(Fl(3,e,e.return),fo(3,e),Fl(5,e,e.return));break;case 1:bn(t,e),xn(e),a&512&&(It||n===null||il(n,n.return)),a&64&&Tl&&(e=e.updateQueue,e!==null&&(a=e.callbacks,a!==null&&(n=e.shared.hiddenCallbacks,e.shared.hiddenCallbacks=n===null?a:n.concat(a))));break;case 26:var s=Wn;if(bn(t,e),xn(e),a&512&&(It||n===null||il(n,n.return)),a&4){var u=n!==null?n.memoizedState:null;if(a=e.memoizedState,n===null)if(a===null)if(e.stateNode===null){e:{a=e.type,n=e.memoizedProps,s=s.ownerDocument||s;t:switch(a){case"title":u=s.getElementsByTagName("title")[0],(!u||u[Hi]||u[Wt]||u.namespaceURI==="http://www.w3.org/2000/svg"||u.hasAttribute("itemprop"))&&(u=s.createElement(a),s.head.insertBefore(u,s.querySelector("head > title"))),cn(u,a,n),u[Wt]=e,nn(u),a=u;break e;case"link":var g=ip("link","href",s).get(a+(n.href||""));if(g){for(var b=0;b<g.length;b++)if(u=g[b],u.getAttribute("href")===(n.href==null||n.href===""?null:n.href)&&u.getAttribute("rel")===(n.rel==null?null:n.rel)&&u.getAttribute("title")===(n.title==null?null:n.title)&&u.getAttribute("crossorigin")===(n.crossOrigin==null?null:n.crossOrigin)){g.splice(b,1);break t}}u=s.createElement(a),cn(u,a,n),s.head.appendChild(u);break;case"meta":if(g=ip("meta","content",s).get(a+(n.content||""))){for(b=0;b<g.length;b++)if(u=g[b],u.getAttribute("content")===(n.content==null?null:""+n.content)&&u.getAttribute("name")===(n.name==null?null:n.name)&&u.getAttribute("property")===(n.property==null?null:n.property)&&u.getAttribute("http-equiv")===(n.httpEquiv==null?null:n.httpEquiv)&&u.getAttribute("charset")===(n.charSet==null?null:n.charSet)){g.splice(b,1);break t}}u=s.createElement(a),cn(u,a,n),s.head.appendChild(u);break;default:throw Error(r(468,a))}u[Wt]=e,nn(u),a=u}e.stateNode=a}else op(s,e.type,e.stateNode);else e.stateNode=ap(s,a,e.memoizedProps);else u!==a?(u===null?n.stateNode!==null&&(n=n.stateNode,n.parentNode.removeChild(n)):u.count--,a===null?op(s,e.type,e.stateNode):ap(s,a,e.memoizedProps)):a===null&&e.stateNode!==null&&Cc(e,e.memoizedProps,n.memoizedProps)}break;case 27:bn(t,e),xn(e),a&512&&(It||n===null||il(n,n.return)),n!==null&&a&4&&Cc(e,e.memoizedProps,n.memoizedProps);break;case 5:if(bn(t,e),xn(e),a&512&&(It||n===null||il(n,n.return)),e.flags&32){s=e.stateNode;try{Ja(s,"")}catch(Ae){wt(e,e.return,Ae)}}a&4&&e.stateNode!=null&&(s=e.memoizedProps,Cc(e,s,n!==null?n.memoizedProps:s)),a&1024&&(zc=!0);break;case 6:if(bn(t,e),xn(e),a&4){if(e.stateNode===null)throw Error(r(162));a=e.memoizedProps,n=e.stateNode;try{n.nodeValue=a}catch(Ae){wt(e,e.return,Ae)}}break;case 3:if(is=null,s=Wn,Wn=ls(t.containerInfo),bn(t,e),Wn=s,xn(e),a&4&&n!==null&&n.memoizedState.isDehydrated)try{Ci(t.containerInfo)}catch(Ae){wt(e,e.return,Ae)}zc&&(zc=!1,og(e));break;case 4:a=Wn,Wn=ls(e.stateNode.containerInfo),bn(t,e),xn(e),Wn=a;break;case 12:bn(t,e),xn(e);break;case 31:bn(t,e),xn(e),a&4&&(a=e.updateQueue,a!==null&&(e.updateQueue=null,Gr(e,a)));break;case 13:bn(t,e),xn(e),e.child.flags&8192&&e.memoizedState!==null!=(n!==null&&n.memoizedState!==null)&&(Qr=Se()),a&4&&(a=e.updateQueue,a!==null&&(e.updateQueue=null,Gr(e,a)));break;case 22:s=e.memoizedState!==null;var z=n!==null&&n.memoizedState!==null,Z=Tl,ne=It;if(Tl=Z||s,It=ne||z,bn(t,e),It=ne,Tl=Z,xn(e),a&8192)e:for(t=e.stateNode,t._visibility=s?t._visibility&-2:t._visibility|1,s&&(n===null||z||Tl||It||Na(e)),n=null,t=e;;){if(t.tag===5||t.tag===26){if(n===null){z=n=t;try{if(u=z.stateNode,s)g=u.style,typeof g.setProperty=="function"?g.setProperty("display","none","important"):g.display="none";else{b=z.stateNode;var se=z.memoizedProps.style,J=se!=null&&se.hasOwnProperty("display")?se.display:null;b.style.display=J==null||typeof J=="boolean"?"":(""+J).trim()}}catch(Ae){wt(z,z.return,Ae)}}}else if(t.tag===6){if(n===null){z=t;try{z.stateNode.nodeValue=s?"":z.memoizedProps}catch(Ae){wt(z,z.return,Ae)}}}else if(t.tag===18){if(n===null){z=t;try{var ee=z.stateNode;s?Fg(ee,!0):Fg(z.stateNode,!1)}catch(Ae){wt(z,z.return,Ae)}}}else if((t.tag!==22&&t.tag!==23||t.memoizedState===null||t===e)&&t.child!==null){t.child.return=t,t=t.child;continue}if(t===e)break e;for(;t.sibling===null;){if(t.return===null||t.return===e)break e;n===t&&(n=null),t=t.return}n===t&&(n=null),t.sibling.return=t.return,t=t.sibling}a&4&&(a=e.updateQueue,a!==null&&(n=a.retryQueue,n!==null&&(a.retryQueue=null,Gr(e,n))));break;case 19:bn(t,e),xn(e),a&4&&(a=e.updateQueue,a!==null&&(e.updateQueue=null,Gr(e,a)));break;case 30:break;case 21:break;default:bn(t,e),xn(e)}}function xn(e){var t=e.flags;if(t&2){try{for(var n,a=e.return;a!==null;){if(Wh(a)){n=a;break}a=a.return}if(n==null)throw Error(r(160));switch(n.tag){case 27:var s=n.stateNode,u=Mc(e);qr(e,u,s);break;case 5:var g=n.stateNode;n.flags&32&&(Ja(g,""),n.flags&=-33);var b=Mc(e);qr(e,b,g);break;case 3:case 4:var z=n.stateNode.containerInfo,Z=Mc(e);Oc(e,Z,z);break;default:throw Error(r(161))}}catch(ne){wt(e,e.return,ne)}e.flags&=-3}t&4096&&(e.flags&=-4097)}function og(e){if(e.subtreeFlags&1024)for(e=e.child;e!==null;){var t=e;og(t),t.tag===5&&t.flags&1024&&t.stateNode.reset(),e=e.sibling}}function Al(e,t){if(t.subtreeFlags&8772)for(t=t.child;t!==null;)eg(e,t.alternate,t),t=t.sibling}function Na(e){for(e=e.child;e!==null;){var t=e;switch(t.tag){case 0:case 11:case 14:case 15:Fl(4,t,t.return),Na(t);break;case 1:il(t,t.return);var n=t.stateNode;typeof n.componentWillUnmount=="function"&&Fh(t,t.return,n),Na(t);break;case 27:Eo(t.stateNode);case 26:case 5:il(t,t.return),Na(t);break;case 22:t.memoizedState===null&&Na(t);break;case 30:Na(t);break;default:Na(t)}e=e.sibling}}function Cl(e,t,n){for(n=n&&(t.subtreeFlags&8772)!==0,t=t.child;t!==null;){var a=t.alternate,s=e,u=t,g=u.flags;switch(u.tag){case 0:case 11:case 15:Cl(s,u,n),fo(4,u);break;case 1:if(Cl(s,u,n),a=u,s=a.stateNode,typeof s.componentDidMount=="function")try{s.componentDidMount()}catch(Z){wt(a,a.return,Z)}if(a=u,s=a.updateQueue,s!==null){var b=a.stateNode;try{var z=s.shared.hiddenCallbacks;if(z!==null)for(s.shared.hiddenCallbacks=null,s=0;s<z.length;s++)Um(z[s],b)}catch(Z){wt(a,a.return,Z)}}n&&g&64&&Ih(u),mo(u,u.return);break;case 27:$h(u);case 26:case 5:Cl(s,u,n),n&&a===null&&g&4&&Jh(u),mo(u,u.return);break;case 12:Cl(s,u,n);break;case 31:Cl(s,u,n),n&&g&4&&lg(s,u);break;case 13:Cl(s,u,n),n&&g&4&&ag(s,u);break;case 22:u.memoizedState===null&&Cl(s,u,n),mo(u,u.return);break;case 30:break;default:Cl(s,u,n)}t=t.sibling}}function _c(e,t){var n=null;e!==null&&e.memoizedState!==null&&e.memoizedState.cachePool!==null&&(n=e.memoizedState.cachePool.pool),e=null,t.memoizedState!==null&&t.memoizedState.cachePool!==null&&(e=t.memoizedState.cachePool.pool),e!==n&&(e!=null&&e.refCount++,n!=null&&$i(n))}function Nc(e,t){e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&$i(e))}function $n(e,t,n,a){if(t.subtreeFlags&10256)for(t=t.child;t!==null;)rg(e,t,n,a),t=t.sibling}function rg(e,t,n,a){var s=t.flags;switch(t.tag){case 0:case 11:case 15:$n(e,t,n,a),s&2048&&fo(9,t);break;case 1:$n(e,t,n,a);break;case 3:$n(e,t,n,a),s&2048&&(e=null,t.alternate!==null&&(e=t.alternate.memoizedState.cache),t=t.memoizedState.cache,t!==e&&(t.refCount++,e!=null&&$i(e)));break;case 12:if(s&2048){$n(e,t,n,a),e=t.stateNode;try{var u=t.memoizedProps,g=u.id,b=u.onPostCommit;typeof b=="function"&&b(g,t.alternate===null?"mount":"update",e.passiveEffectDuration,-0)}catch(z){wt(t,t.return,z)}}else $n(e,t,n,a);break;case 31:$n(e,t,n,a);break;case 13:$n(e,t,n,a);break;case 23:break;case 22:u=t.stateNode,g=t.alternate,t.memoizedState!==null?u._visibility&2?$n(e,t,n,a):ho(e,t):u._visibility&2?$n(e,t,n,a):(u._visibility|=2,pi(e,t,n,a,(t.subtreeFlags&10256)!==0||!1)),s&2048&&_c(g,t);break;case 24:$n(e,t,n,a),s&2048&&Nc(t.alternate,t);break;default:$n(e,t,n,a)}}function pi(e,t,n,a,s){for(s=s&&((t.subtreeFlags&10256)!==0||!1),t=t.child;t!==null;){var u=e,g=t,b=n,z=a,Z=g.flags;switch(g.tag){case 0:case 11:case 15:pi(u,g,b,z,s),fo(8,g);break;case 23:break;case 22:var ne=g.stateNode;g.memoizedState!==null?ne._visibility&2?pi(u,g,b,z,s):ho(u,g):(ne._visibility|=2,pi(u,g,b,z,s)),s&&Z&2048&&_c(g.alternate,g);break;case 24:pi(u,g,b,z,s),s&&Z&2048&&Nc(g.alternate,g);break;default:pi(u,g,b,z,s)}t=t.sibling}}function ho(e,t){if(t.subtreeFlags&10256)for(t=t.child;t!==null;){var n=e,a=t,s=a.flags;switch(a.tag){case 22:ho(n,a),s&2048&&_c(a.alternate,a);break;case 24:ho(n,a),s&2048&&Nc(a.alternate,a);break;default:ho(n,a)}t=t.sibling}}var go=8192;function vi(e,t,n){if(e.subtreeFlags&go)for(e=e.child;e!==null;)sg(e,t,n),e=e.sibling}function sg(e,t,n){switch(e.tag){case 26:vi(e,t,n),e.flags&go&&e.memoizedState!==null&&k1(n,Wn,e.memoizedState,e.memoizedProps);break;case 5:vi(e,t,n);break;case 3:case 4:var a=Wn;Wn=ls(e.stateNode.containerInfo),vi(e,t,n),Wn=a;break;case 22:e.memoizedState===null&&(a=e.alternate,a!==null&&a.memoizedState!==null?(a=go,go=16777216,vi(e,t,n),go=a):vi(e,t,n));break;default:vi(e,t,n)}}function ug(e){var t=e.alternate;if(t!==null&&(e=t.child,e!==null)){t.child=null;do t=e.sibling,e.sibling=null,e=t;while(e!==null)}}function po(e){var t=e.deletions;if((e.flags&16)!==0){if(t!==null)for(var n=0;n<t.length;n++){var a=t[n];ln=a,fg(a,e)}ug(e)}if(e.subtreeFlags&10256)for(e=e.child;e!==null;)cg(e),e=e.sibling}function cg(e){switch(e.tag){case 0:case 11:case 15:po(e),e.flags&2048&&Fl(9,e,e.return);break;case 3:po(e);break;case 12:po(e);break;case 22:var t=e.stateNode;e.memoizedState!==null&&t._visibility&2&&(e.return===null||e.return.tag!==13)?(t._visibility&=-3,Xr(e)):po(e);break;default:po(e)}}function Xr(e){var t=e.deletions;if((e.flags&16)!==0){if(t!==null)for(var n=0;n<t.length;n++){var a=t[n];ln=a,fg(a,e)}ug(e)}for(e=e.child;e!==null;){switch(t=e,t.tag){case 0:case 11:case 15:Fl(8,t,t.return),Xr(t);break;case 22:n=t.stateNode,n._visibility&2&&(n._visibility&=-3,Xr(t));break;default:Xr(t)}e=e.sibling}}function fg(e,t){for(;ln!==null;){var n=ln;switch(n.tag){case 0:case 11:case 15:Fl(8,n,t);break;case 23:case 22:if(n.memoizedState!==null&&n.memoizedState.cachePool!==null){var a=n.memoizedState.cachePool.pool;a!=null&&a.refCount++}break;case 24:$i(n.memoizedState.cache)}if(a=n.child,a!==null)a.return=n,ln=a;else e:for(n=e;ln!==null;){a=ln;var s=a.sibling,u=a.return;if(tg(a),a===n){ln=null;break e}if(s!==null){s.return=u,ln=s;break e}ln=u}}}var Fb={getCacheForType:function(e){var t=sn(Qt),n=t.data.get(e);return n===void 0&&(n=e(),t.data.set(e,n)),n},cacheSignal:function(){return sn(Qt).controller.signal}},Jb=typeof WeakMap=="function"?WeakMap:Map,yt=0,Mt=null,et=null,nt=0,Et=0,zn=null,Jl=!1,yi=!1,Dc=!1,Ml=0,qt=0,Wl=0,Da=0,kc=0,_n=0,bi=0,vo=null,Sn=null,jc=!1,Qr=0,dg=0,Kr=1/0,Zr=null,$l=null,$t=0,Pl=null,xi=null,Ol=0,Lc=0,Uc=null,mg=null,yo=0,Bc=null;function Nn(){return(yt&2)!==0&&nt!==0?nt&-nt:C.T!==null?Xc():pn()}function hg(){if(_n===0)if((nt&536870912)===0||at){var e=Ot;Ot<<=1,(Ot&3932160)===0&&(Ot=262144),_n=e}else _n=536870912;return e=Mn.current,e!==null&&(e.flags|=32),_n}function En(e,t,n){(e===Mt&&(Et===2||Et===9)||e.cancelPendingCommit!==null)&&(Si(e,0),ea(e,nt,_n,!1)),ft(e,n),((yt&2)===0||e!==Mt)&&(e===Mt&&((yt&2)===0&&(Da|=n),qt===4&&ea(e,nt,_n,!1)),ol(e))}function gg(e,t,n){if((yt&6)!==0)throw Error(r(327));var a=!n&&(t&127)===0&&(t&e.expiredLanes)===0||$e(e,t),s=a?Pb(e,t):Vc(e,t,!0),u=a;do{if(s===0){yi&&!a&&ea(e,t,0,!1);break}else{if(n=e.current.alternate,u&&!Wb(n)){s=Vc(e,t,!1),u=!1;continue}if(s===2){if(u=t,e.errorRecoveryDisabledLanes&u)var g=0;else g=e.pendingLanes&-536870913,g=g!==0?g:g&536870912?536870912:0;if(g!==0){t=g;e:{var b=e;s=vo;var z=b.current.memoizedState.isDehydrated;if(z&&(Si(b,g).flags|=256),g=Vc(b,g,!1),g!==2){if(Dc&&!z){b.errorRecoveryDisabledLanes|=u,Da|=u,s=4;break e}u=Sn,Sn=s,u!==null&&(Sn===null?Sn=u:Sn.push.apply(Sn,u))}s=g}if(u=!1,s!==2)continue}}if(s===1){Si(e,0),ea(e,t,0,!0);break}e:{switch(a=e,u=s,u){case 0:case 1:throw Error(r(345));case 4:if((t&4194048)!==t)break;case 6:ea(a,t,_n,!Jl);break e;case 2:Sn=null;break;case 3:case 5:break;default:throw Error(r(329))}if((t&62914560)===t&&(s=Qr+300-Se(),10<s)){if(ea(a,t,_n,!Jl),Te(a,0,!0)!==0)break e;Ol=t,a.timeoutHandle=Kg(pg.bind(null,a,n,Sn,Zr,jc,t,_n,Da,bi,Jl,u,"Throttled",-0,0),s);break e}pg(a,n,Sn,Zr,jc,t,_n,Da,bi,Jl,u,null,-0,0)}}break}while(!0);ol(e)}function pg(e,t,n,a,s,u,g,b,z,Z,ne,se,J,ee){if(e.timeoutHandle=-1,se=t.subtreeFlags,se&8192||(se&16785408)===16785408){se={stylesheets:null,count:0,imgCount:0,imgBytes:0,suspenseyImages:[],waitingForImages:!0,waitingForViewTransition:!1,unsuspend:hl},sg(t,u,se);var Ae=(u&62914560)===u?Qr-Se():(u&4194048)===u?dg-Se():0;if(Ae=j1(se,Ae),Ae!==null){Ol=u,e.cancelPendingCommit=Ae(Tg.bind(null,e,t,u,n,a,s,g,b,z,ne,se,null,J,ee)),ea(e,u,g,!Z);return}}Tg(e,t,u,n,a,s,g,b,z)}function Wb(e){for(var t=e;;){var n=t.tag;if((n===0||n===11||n===15)&&t.flags&16384&&(n=t.updateQueue,n!==null&&(n=n.stores,n!==null)))for(var a=0;a<n.length;a++){var s=n[a],u=s.getSnapshot;s=s.value;try{if(!An(u(),s))return!1}catch{return!1}}if(n=t.child,t.subtreeFlags&16384&&n!==null)n.return=t,t=n;else{if(t===e)break;for(;t.sibling===null;){if(t.return===null||t.return===e)return!0;t=t.return}t.sibling.return=t.return,t=t.sibling}}return!0}function ea(e,t,n,a){t&=~kc,t&=~Da,e.suspendedLanes|=t,e.pingedLanes&=~t,a&&(e.warmLanes|=t),a=e.expirationTimes;for(var s=t;0<s;){var u=31-it(s),g=1<<u;a[u]=-1,s&=~g}n!==0&&St(e,n,t)}function Ir(){return(yt&6)===0?(bo(0),!1):!0}function Hc(){if(et!==null){if(Et===0)var e=et.return;else e=et,yl=Ta=null,tc(e),fi=null,eo=0,e=et;for(;e!==null;)Zh(e.alternate,e),e=e.return;et=null}}function Si(e,t){var n=e.timeoutHandle;n!==-1&&(e.timeoutHandle=-1,v1(n)),n=e.cancelPendingCommit,n!==null&&(e.cancelPendingCommit=null,n()),Ol=0,Hc(),Mt=e,et=n=pl(e.current,null),nt=t,Et=0,zn=null,Jl=!1,yi=$e(e,t),Dc=!1,bi=_n=kc=Da=Wl=qt=0,Sn=vo=null,jc=!1,(t&8)!==0&&(t|=t&32);var a=e.entangledLanes;if(a!==0)for(e=e.entanglements,a&=t;0<a;){var s=31-it(a),u=1<<s;t|=e[s],a&=~u}return Ml=t,gr(),n}function vg(e,t){Ie=null,C.H=so,t===ci||t===wr?(t=Dm(),Et=3):t===Gu?(t=Dm(),Et=4):Et=t===vc?8:t!==null&&typeof t=="object"&&typeof t.then=="function"?6:1,zn=t,et===null&&(qt=1,Ur(e,Yn(t,e.current)))}function yg(){var e=Mn.current;return e===null?!0:(nt&4194048)===nt?Qn===null:(nt&62914560)===nt||(nt&536870912)!==0?e===Qn:!1}function bg(){var e=C.H;return C.H=so,e===null?so:e}function xg(){var e=C.A;return C.A=Fb,e}function Fr(){qt=4,Jl||(nt&4194048)!==nt&&Mn.current!==null||(yi=!0),(Wl&134217727)===0&&(Da&134217727)===0||Mt===null||ea(Mt,nt,_n,!1)}function Vc(e,t,n){var a=yt;yt|=2;var s=bg(),u=xg();(Mt!==e||nt!==t)&&(Zr=null,Si(e,t)),t=!1;var g=qt;e:do try{if(Et!==0&&et!==null){var b=et,z=zn;switch(Et){case 8:Hc(),g=6;break e;case 3:case 2:case 9:case 6:Mn.current===null&&(t=!0);var Z=Et;if(Et=0,zn=null,Ei(e,b,z,Z),n&&yi){g=0;break e}break;default:Z=Et,Et=0,zn=null,Ei(e,b,z,Z)}}$b(),g=qt;break}catch(ne){vg(e,ne)}while(!0);return t&&e.shellSuspendCounter++,yl=Ta=null,yt=a,C.H=s,C.A=u,et===null&&(Mt=null,nt=0,gr()),g}function $b(){for(;et!==null;)Sg(et)}function Pb(e,t){var n=yt;yt|=2;var a=bg(),s=xg();Mt!==e||nt!==t?(Zr=null,Kr=Se()+500,Si(e,t)):yi=$e(e,t);e:do try{if(Et!==0&&et!==null){t=et;var u=zn;t:switch(Et){case 1:Et=0,zn=null,Ei(e,t,u,1);break;case 2:case 9:if(_m(u)){Et=0,zn=null,Eg(t);break}t=function(){Et!==2&&Et!==9||Mt!==e||(Et=7),ol(e)},u.then(t,t);break e;case 3:Et=7;break e;case 4:Et=5;break e;case 7:_m(u)?(Et=0,zn=null,Eg(t)):(Et=0,zn=null,Ei(e,t,u,7));break;case 5:var g=null;switch(et.tag){case 26:g=et.memoizedState;case 5:case 27:var b=et;if(g?rp(g):b.stateNode.complete){Et=0,zn=null;var z=b.sibling;if(z!==null)et=z;else{var Z=b.return;Z!==null?(et=Z,Jr(Z)):et=null}break t}}Et=0,zn=null,Ei(e,t,u,5);break;case 6:Et=0,zn=null,Ei(e,t,u,6);break;case 8:Hc(),qt=6;break e;default:throw Error(r(462))}}e1();break}catch(ne){vg(e,ne)}while(!0);return yl=Ta=null,C.H=a,C.A=s,yt=n,et!==null?0:(Mt=null,nt=0,gr(),qt)}function e1(){for(;et!==null&&!$();)Sg(et)}function Sg(e){var t=Qh(e.alternate,e,Ml);e.memoizedProps=e.pendingProps,t===null?Jr(e):et=t}function Eg(e){var t=e,n=t.alternate;switch(t.tag){case 15:case 0:t=Hh(n,t,t.pendingProps,t.type,void 0,nt);break;case 11:t=Hh(n,t,t.pendingProps,t.type.render,t.ref,nt);break;case 5:tc(t);default:Zh(n,t),t=et=xm(t,Ml),t=Qh(n,t,Ml)}e.memoizedProps=e.pendingProps,t===null?Jr(e):et=t}function Ei(e,t,n,a){yl=Ta=null,tc(t),fi=null,eo=0;var s=t.return;try{if(qb(e,s,t,n,nt)){qt=1,Ur(e,Yn(n,e.current)),et=null;return}}catch(u){if(s!==null)throw et=s,u;qt=1,Ur(e,Yn(n,e.current)),et=null;return}t.flags&32768?(at||a===1?e=!0:yi||(nt&536870912)!==0?e=!1:(Jl=e=!0,(a===2||a===9||a===3||a===6)&&(a=Mn.current,a!==null&&a.tag===13&&(a.flags|=16384))),wg(t,e)):Jr(t)}function Jr(e){var t=e;do{if((t.flags&32768)!==0){wg(t,Jl);return}e=t.return;var n=Qb(t.alternate,t,Ml);if(n!==null){et=n;return}if(t=t.sibling,t!==null){et=t;return}et=t=e}while(t!==null);qt===0&&(qt=5)}function wg(e,t){do{var n=Kb(e.alternate,e);if(n!==null){n.flags&=32767,et=n;return}if(n=e.return,n!==null&&(n.flags|=32768,n.subtreeFlags=0,n.deletions=null),!t&&(e=e.sibling,e!==null)){et=e;return}et=e=n}while(e!==null);qt=6,et=null}function Tg(e,t,n,a,s,u,g,b,z){e.cancelPendingCommit=null;do Wr();while($t!==0);if((yt&6)!==0)throw Error(r(327));if(t!==null){if(t===e.current)throw Error(r(177));if(u=t.lanes|t.childLanes,u|=Mu,ke(e,n,u,g,b,z),e===Mt&&(et=Mt=null,nt=0),xi=t,Pl=e,Ol=n,Lc=u,Uc=s,mg=a,(t.subtreeFlags&10256)!==0||(t.flags&10256)!==0?(e.callbackNode=null,e.callbackPriority=0,a1(Je,function(){return Og(),null})):(e.callbackNode=null,e.callbackPriority=0),a=(t.flags&13878)!==0,(t.subtreeFlags&13878)!==0||a){a=C.T,C.T=null,s=k.p,k.p=2,g=yt,yt|=4;try{Zb(e,t,n)}finally{yt=g,k.p=s,C.T=a}}$t=1,Rg(),Ag(),Cg()}}function Rg(){if($t===1){$t=0;var e=Pl,t=xi,n=(t.flags&13878)!==0;if((t.subtreeFlags&13878)!==0||n){n=C.T,C.T=null;var a=k.p;k.p=2;var s=yt;yt|=4;try{ig(t,e);var u=$c,g=fm(e.containerInfo),b=u.focusedElem,z=u.selectionRange;if(g!==b&&b&&b.ownerDocument&&cm(b.ownerDocument.documentElement,b)){if(z!==null&&wu(b)){var Z=z.start,ne=z.end;if(ne===void 0&&(ne=Z),"selectionStart"in b)b.selectionStart=Z,b.selectionEnd=Math.min(ne,b.value.length);else{var se=b.ownerDocument||document,J=se&&se.defaultView||window;if(J.getSelection){var ee=J.getSelection(),Ae=b.textContent.length,Ue=Math.min(z.start,Ae),Ct=z.end===void 0?Ue:Math.min(z.end,Ae);!ee.extend&&Ue>Ct&&(g=Ct,Ct=Ue,Ue=g);var V=um(b,Ue),j=um(b,Ct);if(V&&j&&(ee.rangeCount!==1||ee.anchorNode!==V.node||ee.anchorOffset!==V.offset||ee.focusNode!==j.node||ee.focusOffset!==j.offset)){var K=se.createRange();K.setStart(V.node,V.offset),ee.removeAllRanges(),Ue>Ct?(ee.addRange(K),ee.extend(j.node,j.offset)):(K.setEnd(j.node,j.offset),ee.addRange(K))}}}}for(se=[],ee=b;ee=ee.parentNode;)ee.nodeType===1&&se.push({element:ee,left:ee.scrollLeft,top:ee.scrollTop});for(typeof b.focus=="function"&&b.focus(),b=0;b<se.length;b++){var ie=se[b];ie.element.scrollLeft=ie.left,ie.element.scrollTop=ie.top}}us=!!Wc,$c=Wc=null}finally{yt=s,k.p=a,C.T=n}}e.current=t,$t=2}}function Ag(){if($t===2){$t=0;var e=Pl,t=xi,n=(t.flags&8772)!==0;if((t.subtreeFlags&8772)!==0||n){n=C.T,C.T=null;var a=k.p;k.p=2;var s=yt;yt|=4;try{eg(e,t.alternate,t)}finally{yt=s,k.p=a,C.T=n}}$t=3}}function Cg(){if($t===4||$t===3){$t=0,re();var e=Pl,t=xi,n=Ol,a=mg;(t.subtreeFlags&10256)!==0||(t.flags&10256)!==0?$t=5:($t=0,xi=Pl=null,Mg(e,e.pendingLanes));var s=e.pendingLanes;if(s===0&&($l=null),Vt(n),t=t.stateNode,pt&&typeof pt.onCommitFiberRoot=="function")try{pt.onCommitFiberRoot(gt,t,void 0,(t.current.flags&128)===128)}catch{}if(a!==null){t=C.T,s=k.p,k.p=2,C.T=null;try{for(var u=e.onRecoverableError,g=0;g<a.length;g++){var b=a[g];u(b.value,{componentStack:b.stack})}}finally{C.T=t,k.p=s}}(Ol&3)!==0&&Wr(),ol(e),s=e.pendingLanes,(n&261930)!==0&&(s&42)!==0?e===Bc?yo++:(yo=0,Bc=e):yo=0,bo(0)}}function Mg(e,t){(e.pooledCacheLanes&=t)===0&&(t=e.pooledCache,t!=null&&(e.pooledCache=null,$i(t)))}function Wr(){return Rg(),Ag(),Cg(),Og()}function Og(){if($t!==5)return!1;var e=Pl,t=Lc;Lc=0;var n=Vt(Ol),a=C.T,s=k.p;try{k.p=32>n?32:n,C.T=null,n=Uc,Uc=null;var u=Pl,g=Ol;if($t=0,xi=Pl=null,Ol=0,(yt&6)!==0)throw Error(r(331));var b=yt;if(yt|=4,cg(u.current),rg(u,u.current,g,n),yt=b,bo(0,!1),pt&&typeof pt.onPostCommitFiberRoot=="function")try{pt.onPostCommitFiberRoot(gt,u)}catch{}return!0}finally{k.p=s,C.T=a,Mg(e,t)}}function zg(e,t,n){t=Yn(n,t),t=pc(e.stateNode,t,2),e=Kl(e,t,2),e!==null&&(ft(e,2),ol(e))}function wt(e,t,n){if(e.tag===3)zg(e,e,n);else for(;t!==null;){if(t.tag===3){zg(t,e,n);break}else if(t.tag===1){var a=t.stateNode;if(typeof t.type.getDerivedStateFromError=="function"||typeof a.componentDidCatch=="function"&&($l===null||!$l.has(a))){e=Yn(n,e),n=_h(2),a=Kl(t,n,2),a!==null&&(Nh(n,a,t,e),ft(a,2),ol(a));break}}t=t.return}}function Yc(e,t,n){var a=e.pingCache;if(a===null){a=e.pingCache=new Jb;var s=new Set;a.set(t,s)}else s=a.get(t),s===void 0&&(s=new Set,a.set(t,s));s.has(n)||(Dc=!0,s.add(n),e=t1.bind(null,e,t,n),t.then(e,e))}function t1(e,t,n){var a=e.pingCache;a!==null&&a.delete(t),e.pingedLanes|=e.suspendedLanes&n,e.warmLanes&=~n,Mt===e&&(nt&n)===n&&(qt===4||qt===3&&(nt&62914560)===nt&&300>Se()-Qr?(yt&2)===0&&Si(e,0):kc|=n,bi===nt&&(bi=0)),ol(e)}function _g(e,t){t===0&&(t=xt()),e=Sa(e,t),e!==null&&(ft(e,t),ol(e))}function n1(e){var t=e.memoizedState,n=0;t!==null&&(n=t.retryLane),_g(e,n)}function l1(e,t){var n=0;switch(e.tag){case 31:case 13:var a=e.stateNode,s=e.memoizedState;s!==null&&(n=s.retryLane);break;case 19:a=e.stateNode;break;case 22:a=e.stateNode._retryCache;break;default:throw Error(r(314))}a!==null&&a.delete(t),_g(e,n)}function a1(e,t){return oe(e,t)}var $r=null,wi=null,qc=!1,Pr=!1,Gc=!1,ta=0;function ol(e){e!==wi&&e.next===null&&(wi===null?$r=wi=e:wi=wi.next=e),Pr=!0,qc||(qc=!0,o1())}function bo(e,t){if(!Gc&&Pr){Gc=!0;do for(var n=!1,a=$r;a!==null;){if(e!==0){var s=a.pendingLanes;if(s===0)var u=0;else{var g=a.suspendedLanes,b=a.pingedLanes;u=(1<<31-it(42|e)+1)-1,u&=s&~(g&~b),u=u&201326741?u&201326741|1:u?u|2:0}u!==0&&(n=!0,jg(a,u))}else u=nt,u=Te(a,a===Mt?u:0,a.cancelPendingCommit!==null||a.timeoutHandle!==-1),(u&3)===0||$e(a,u)||(n=!0,jg(a,u));a=a.next}while(n);Gc=!1}}function i1(){Ng()}function Ng(){Pr=qc=!1;var e=0;ta!==0&&p1()&&(e=ta);for(var t=Se(),n=null,a=$r;a!==null;){var s=a.next,u=Dg(a,t);u===0?(a.next=null,n===null?$r=s:n.next=s,s===null&&(wi=n)):(n=a,(e!==0||(u&3)!==0)&&(Pr=!0)),a=s}$t!==0&&$t!==5||bo(e),ta!==0&&(ta=0)}function Dg(e,t){for(var n=e.suspendedLanes,a=e.pingedLanes,s=e.expirationTimes,u=e.pendingLanes&-62914561;0<u;){var g=31-it(u),b=1<<g,z=s[g];z===-1?((b&n)===0||(b&a)!==0)&&(s[g]=bt(b,t)):z<=t&&(e.expiredLanes|=b),u&=~b}if(t=Mt,n=nt,n=Te(e,e===t?n:0,e.cancelPendingCommit!==null||e.timeoutHandle!==-1),a=e.callbackNode,n===0||e===t&&(Et===2||Et===9)||e.cancelPendingCommit!==null)return a!==null&&a!==null&&H(a),e.callbackNode=null,e.callbackPriority=0;if((n&3)===0||$e(e,n)){if(t=n&-n,t===e.callbackPriority)return t;switch(a!==null&&H(a),Vt(n)){case 2:case 8:n=ut;break;case 32:n=Je;break;case 268435456:n=Qe;break;default:n=Je}return a=kg.bind(null,e),n=oe(n,a),e.callbackPriority=t,e.callbackNode=n,t}return a!==null&&a!==null&&H(a),e.callbackPriority=2,e.callbackNode=null,2}function kg(e,t){if($t!==0&&$t!==5)return e.callbackNode=null,e.callbackPriority=0,null;var n=e.callbackNode;if(Wr()&&e.callbackNode!==n)return null;var a=nt;return a=Te(e,e===Mt?a:0,e.cancelPendingCommit!==null||e.timeoutHandle!==-1),a===0?null:(gg(e,a,t),Dg(e,Se()),e.callbackNode!=null&&e.callbackNode===n?kg.bind(null,e):null)}function jg(e,t){if(Wr())return null;gg(e,t,!0)}function o1(){y1(function(){(yt&6)!==0?oe(Xe,i1):Ng()})}function Xc(){if(ta===0){var e=si;e===0&&(e=ct,ct<<=1,(ct&261888)===0&&(ct=256)),ta=e}return ta}function Lg(e){return e==null||typeof e=="symbol"||typeof e=="boolean"?null:typeof e=="function"?e:rr(""+e)}function Ug(e,t){var n=t.ownerDocument.createElement("input");return n.name=t.name,n.value=t.value,e.id&&n.setAttribute("form",e.id),t.parentNode.insertBefore(n,t),e=new FormData(e),n.parentNode.removeChild(n),e}function r1(e,t,n,a,s){if(t==="submit"&&n&&n.stateNode===s){var u=Lg((s[on]||null).action),g=a.submitter;g&&(t=(t=g[on]||null)?Lg(t.formAction):g.getAttribute("formAction"),t!==null&&(u=t,g=null));var b=new fr("action","action",null,a,s);e.push({event:b,listeners:[{instance:null,listener:function(){if(a.defaultPrevented){if(ta!==0){var z=g?Ug(s,g):new FormData(s);cc(n,{pending:!0,data:z,method:s.method,action:u},null,z)}}else typeof u=="function"&&(b.preventDefault(),z=g?Ug(s,g):new FormData(s),cc(n,{pending:!0,data:z,method:s.method,action:u},u,z))},currentTarget:s}]})}}for(var Qc=0;Qc<Cu.length;Qc++){var Kc=Cu[Qc],s1=Kc.toLowerCase(),u1=Kc[0].toUpperCase()+Kc.slice(1);Jn(s1,"on"+u1)}Jn(hm,"onAnimationEnd"),Jn(gm,"onAnimationIteration"),Jn(pm,"onAnimationStart"),Jn("dblclick","onDoubleClick"),Jn("focusin","onFocus"),Jn("focusout","onBlur"),Jn(Rb,"onTransitionRun"),Jn(Ab,"onTransitionStart"),Jn(Cb,"onTransitionCancel"),Jn(vm,"onTransitionEnd"),Ia("onMouseEnter",["mouseout","mouseover"]),Ia("onMouseLeave",["mouseout","mouseover"]),Ia("onPointerEnter",["pointerout","pointerover"]),Ia("onPointerLeave",["pointerout","pointerover"]),va("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),va("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),va("onBeforeInput",["compositionend","keypress","textInput","paste"]),va("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),va("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),va("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var xo="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),c1=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(xo));function Bg(e,t){t=(t&4)!==0;for(var n=0;n<e.length;n++){var a=e[n],s=a.event;a=a.listeners;e:{var u=void 0;if(t)for(var g=a.length-1;0<=g;g--){var b=a[g],z=b.instance,Z=b.currentTarget;if(b=b.listener,z!==u&&s.isPropagationStopped())break e;u=b,s.currentTarget=Z;try{u(s)}catch(ne){hr(ne)}s.currentTarget=null,u=z}else for(g=0;g<a.length;g++){if(b=a[g],z=b.instance,Z=b.currentTarget,b=b.listener,z!==u&&s.isPropagationStopped())break e;u=b,s.currentTarget=Z;try{u(s)}catch(ne){hr(ne)}s.currentTarget=null,u=z}}}}function tt(e,t){var n=t[Xa];n===void 0&&(n=t[Xa]=new Set);var a=e+"__bubble";n.has(a)||(Hg(t,e,2,!1),n.add(a))}function Zc(e,t,n){var a=0;t&&(a|=4),Hg(n,e,a,t)}var es="_reactListening"+Math.random().toString(36).slice(2);function Ic(e){if(!e[es]){e[es]=!0,Nd.forEach(function(n){n!=="selectionchange"&&(c1.has(n)||Zc(n,!1,e),Zc(n,!0,e))});var t=e.nodeType===9?e:e.ownerDocument;t===null||t[es]||(t[es]=!0,Zc("selectionchange",!1,t))}}function Hg(e,t,n,a){switch(hp(t)){case 2:var s=B1;break;case 8:s=H1;break;default:s=cf}n=s.bind(null,t,n,e),s=void 0,!hu||t!=="touchstart"&&t!=="touchmove"&&t!=="wheel"||(s=!0),a?s!==void 0?e.addEventListener(t,n,{capture:!0,passive:s}):e.addEventListener(t,n,!0):s!==void 0?e.addEventListener(t,n,{passive:s}):e.addEventListener(t,n,!1)}function Fc(e,t,n,a,s){var u=a;if((t&1)===0&&(t&2)===0&&a!==null)e:for(;;){if(a===null)return;var g=a.tag;if(g===3||g===4){var b=a.stateNode.containerInfo;if(b===s)break;if(g===4)for(g=a.return;g!==null;){var z=g.tag;if((z===3||z===4)&&g.stateNode.containerInfo===s)return;g=g.return}for(;b!==null;){if(g=Qa(b),g===null)return;if(z=g.tag,z===5||z===6||z===26||z===27){a=u=g;continue e}b=b.parentNode}}a=a.return}Xd(function(){var Z=u,ne=du(n),se=[];e:{var J=ym.get(e);if(J!==void 0){var ee=fr,Ae=e;switch(e){case"keypress":if(ur(n)===0)break e;case"keydown":case"keyup":ee=lb;break;case"focusin":Ae="focus",ee=yu;break;case"focusout":Ae="blur",ee=yu;break;case"beforeblur":case"afterblur":ee=yu;break;case"click":if(n.button===2)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":ee=Zd;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":ee=Q0;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":ee=ob;break;case hm:case gm:case pm:ee=I0;break;case vm:ee=sb;break;case"scroll":case"scrollend":ee=G0;break;case"wheel":ee=cb;break;case"copy":case"cut":case"paste":ee=J0;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":ee=Fd;break;case"toggle":case"beforetoggle":ee=db}var Ue=(t&4)!==0,Ct=!Ue&&(e==="scroll"||e==="scrollend"),V=Ue?J!==null?J+"Capture":null:J;Ue=[];for(var j=Z,K;j!==null;){var ie=j;if(K=ie.stateNode,ie=ie.tag,ie!==5&&ie!==26&&ie!==27||K===null||V===null||(ie=Yi(j,V),ie!=null&&Ue.push(So(j,ie,K))),Ct)break;j=j.return}0<Ue.length&&(J=new ee(J,Ae,null,n,ne),se.push({event:J,listeners:Ue}))}}if((t&7)===0){e:{if(J=e==="mouseover"||e==="pointerover",ee=e==="mouseout"||e==="pointerout",J&&n!==fu&&(Ae=n.relatedTarget||n.fromElement)&&(Qa(Ae)||Ae[dl]))break e;if((ee||J)&&(J=ne.window===ne?ne:(J=ne.ownerDocument)?J.defaultView||J.parentWindow:window,ee?(Ae=n.relatedTarget||n.toElement,ee=Z,Ae=Ae?Qa(Ae):null,Ae!==null&&(Ct=f(Ae),Ue=Ae.tag,Ae!==Ct||Ue!==5&&Ue!==27&&Ue!==6)&&(Ae=null)):(ee=null,Ae=Z),ee!==Ae)){if(Ue=Zd,ie="onMouseLeave",V="onMouseEnter",j="mouse",(e==="pointerout"||e==="pointerover")&&(Ue=Fd,ie="onPointerLeave",V="onPointerEnter",j="pointer"),Ct=ee==null?J:Vi(ee),K=Ae==null?J:Vi(Ae),J=new Ue(ie,j+"leave",ee,n,ne),J.target=Ct,J.relatedTarget=K,ie=null,Qa(ne)===Z&&(Ue=new Ue(V,j+"enter",Ae,n,ne),Ue.target=K,Ue.relatedTarget=Ct,ie=Ue),Ct=ie,ee&&Ae)t:{for(Ue=f1,V=ee,j=Ae,K=0,ie=V;ie;ie=Ue(ie))K++;ie=0;for(var Ne=j;Ne;Ne=Ue(Ne))ie++;for(;0<K-ie;)V=Ue(V),K--;for(;0<ie-K;)j=Ue(j),ie--;for(;K--;){if(V===j||j!==null&&V===j.alternate){Ue=V;break t}V=Ue(V),j=Ue(j)}Ue=null}else Ue=null;ee!==null&&Vg(se,J,ee,Ue,!1),Ae!==null&&Ct!==null&&Vg(se,Ct,Ae,Ue,!0)}}e:{if(J=Z?Vi(Z):window,ee=J.nodeName&&J.nodeName.toLowerCase(),ee==="select"||ee==="input"&&J.type==="file")var dt=lm;else if(tm(J))if(am)dt=Eb;else{dt=xb;var ze=bb}else ee=J.nodeName,!ee||ee.toLowerCase()!=="input"||J.type!=="checkbox"&&J.type!=="radio"?Z&&cu(Z.elementType)&&(dt=lm):dt=Sb;if(dt&&(dt=dt(e,Z))){nm(se,dt,n,ne);break e}ze&&ze(e,J,Z),e==="focusout"&&Z&&J.type==="number"&&Z.memoizedProps.value!=null&&uu(J,"number",J.value)}switch(ze=Z?Vi(Z):window,e){case"focusin":(tm(ze)||ze.contentEditable==="true")&&(ei=ze,Tu=Z,Fi=null);break;case"focusout":Fi=Tu=ei=null;break;case"mousedown":Ru=!0;break;case"contextmenu":case"mouseup":case"dragend":Ru=!1,dm(se,n,ne);break;case"selectionchange":if(Tb)break;case"keydown":case"keyup":dm(se,n,ne)}var Fe;if(xu)e:{switch(e){case"compositionstart":var lt="onCompositionStart";break e;case"compositionend":lt="onCompositionEnd";break e;case"compositionupdate":lt="onCompositionUpdate";break e}lt=void 0}else Pa?Pd(e,n)&&(lt="onCompositionEnd"):e==="keydown"&&n.keyCode===229&&(lt="onCompositionStart");lt&&(Jd&&n.locale!=="ko"&&(Pa||lt!=="onCompositionStart"?lt==="onCompositionEnd"&&Pa&&(Fe=Qd()):(Hl=ne,gu="value"in Hl?Hl.value:Hl.textContent,Pa=!0)),ze=ts(Z,lt),0<ze.length&&(lt=new Id(lt,e,null,n,ne),se.push({event:lt,listeners:ze}),Fe?lt.data=Fe:(Fe=em(n),Fe!==null&&(lt.data=Fe)))),(Fe=hb?gb(e,n):pb(e,n))&&(lt=ts(Z,"onBeforeInput"),0<lt.length&&(ze=new Id("onBeforeInput","beforeinput",null,n,ne),se.push({event:ze,listeners:lt}),ze.data=Fe)),r1(se,e,Z,n,ne)}Bg(se,t)})}function So(e,t,n){return{instance:e,listener:t,currentTarget:n}}function ts(e,t){for(var n=t+"Capture",a=[];e!==null;){var s=e,u=s.stateNode;if(s=s.tag,s!==5&&s!==26&&s!==27||u===null||(s=Yi(e,n),s!=null&&a.unshift(So(e,s,u)),s=Yi(e,t),s!=null&&a.push(So(e,s,u))),e.tag===3)return a;e=e.return}return[]}function f1(e){if(e===null)return null;do e=e.return;while(e&&e.tag!==5&&e.tag!==27);return e||null}function Vg(e,t,n,a,s){for(var u=t._reactName,g=[];n!==null&&n!==a;){var b=n,z=b.alternate,Z=b.stateNode;if(b=b.tag,z!==null&&z===a)break;b!==5&&b!==26&&b!==27||Z===null||(z=Z,s?(Z=Yi(n,u),Z!=null&&g.unshift(So(n,Z,z))):s||(Z=Yi(n,u),Z!=null&&g.push(So(n,Z,z)))),n=n.return}g.length!==0&&e.push({event:t,listeners:g})}var d1=/\r\n?/g,m1=/\u0000|\uFFFD/g;function Yg(e){return(typeof e=="string"?e:""+e).replace(d1,`
`).replace(m1,"")}function qg(e,t){return t=Yg(t),Yg(e)===t}function At(e,t,n,a,s,u){switch(n){case"children":typeof a=="string"?t==="body"||t==="textarea"&&a===""||Ja(e,a):(typeof a=="number"||typeof a=="bigint")&&t!=="body"&&Ja(e,""+a);break;case"className":ir(e,"class",a);break;case"tabIndex":ir(e,"tabindex",a);break;case"dir":case"role":case"viewBox":case"width":case"height":ir(e,n,a);break;case"style":qd(e,a,u);break;case"data":if(t!=="object"){ir(e,"data",a);break}case"src":case"href":if(a===""&&(t!=="a"||n!=="href")){e.removeAttribute(n);break}if(a==null||typeof a=="function"||typeof a=="symbol"||typeof a=="boolean"){e.removeAttribute(n);break}a=rr(""+a),e.setAttribute(n,a);break;case"action":case"formAction":if(typeof a=="function"){e.setAttribute(n,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof u=="function"&&(n==="formAction"?(t!=="input"&&At(e,t,"name",s.name,s,null),At(e,t,"formEncType",s.formEncType,s,null),At(e,t,"formMethod",s.formMethod,s,null),At(e,t,"formTarget",s.formTarget,s,null)):(At(e,t,"encType",s.encType,s,null),At(e,t,"method",s.method,s,null),At(e,t,"target",s.target,s,null)));if(a==null||typeof a=="symbol"||typeof a=="boolean"){e.removeAttribute(n);break}a=rr(""+a),e.setAttribute(n,a);break;case"onClick":a!=null&&(e.onclick=hl);break;case"onScroll":a!=null&&tt("scroll",e);break;case"onScrollEnd":a!=null&&tt("scrollend",e);break;case"dangerouslySetInnerHTML":if(a!=null){if(typeof a!="object"||!("__html"in a))throw Error(r(61));if(n=a.__html,n!=null){if(s.children!=null)throw Error(r(60));e.innerHTML=n}}break;case"multiple":e.multiple=a&&typeof a!="function"&&typeof a!="symbol";break;case"muted":e.muted=a&&typeof a!="function"&&typeof a!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(a==null||typeof a=="function"||typeof a=="boolean"||typeof a=="symbol"){e.removeAttribute("xlink:href");break}n=rr(""+a),e.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",n);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":a!=null&&typeof a!="function"&&typeof a!="symbol"?e.setAttribute(n,""+a):e.removeAttribute(n);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":a&&typeof a!="function"&&typeof a!="symbol"?e.setAttribute(n,""):e.removeAttribute(n);break;case"capture":case"download":a===!0?e.setAttribute(n,""):a!==!1&&a!=null&&typeof a!="function"&&typeof a!="symbol"?e.setAttribute(n,a):e.removeAttribute(n);break;case"cols":case"rows":case"size":case"span":a!=null&&typeof a!="function"&&typeof a!="symbol"&&!isNaN(a)&&1<=a?e.setAttribute(n,a):e.removeAttribute(n);break;case"rowSpan":case"start":a==null||typeof a=="function"||typeof a=="symbol"||isNaN(a)?e.removeAttribute(n):e.setAttribute(n,a);break;case"popover":tt("beforetoggle",e),tt("toggle",e),ar(e,"popover",a);break;case"xlinkActuate":ml(e,"http://www.w3.org/1999/xlink","xlink:actuate",a);break;case"xlinkArcrole":ml(e,"http://www.w3.org/1999/xlink","xlink:arcrole",a);break;case"xlinkRole":ml(e,"http://www.w3.org/1999/xlink","xlink:role",a);break;case"xlinkShow":ml(e,"http://www.w3.org/1999/xlink","xlink:show",a);break;case"xlinkTitle":ml(e,"http://www.w3.org/1999/xlink","xlink:title",a);break;case"xlinkType":ml(e,"http://www.w3.org/1999/xlink","xlink:type",a);break;case"xmlBase":ml(e,"http://www.w3.org/XML/1998/namespace","xml:base",a);break;case"xmlLang":ml(e,"http://www.w3.org/XML/1998/namespace","xml:lang",a);break;case"xmlSpace":ml(e,"http://www.w3.org/XML/1998/namespace","xml:space",a);break;case"is":ar(e,"is",a);break;case"innerText":case"textContent":break;default:(!(2<n.length)||n[0]!=="o"&&n[0]!=="O"||n[1]!=="n"&&n[1]!=="N")&&(n=Y0.get(n)||n,ar(e,n,a))}}function Jc(e,t,n,a,s,u){switch(n){case"style":qd(e,a,u);break;case"dangerouslySetInnerHTML":if(a!=null){if(typeof a!="object"||!("__html"in a))throw Error(r(61));if(n=a.__html,n!=null){if(s.children!=null)throw Error(r(60));e.innerHTML=n}}break;case"children":typeof a=="string"?Ja(e,a):(typeof a=="number"||typeof a=="bigint")&&Ja(e,""+a);break;case"onScroll":a!=null&&tt("scroll",e);break;case"onScrollEnd":a!=null&&tt("scrollend",e);break;case"onClick":a!=null&&(e.onclick=hl);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!Dd.hasOwnProperty(n))e:{if(n[0]==="o"&&n[1]==="n"&&(s=n.endsWith("Capture"),t=n.slice(2,s?n.length-7:void 0),u=e[on]||null,u=u!=null?u[n]:null,typeof u=="function"&&e.removeEventListener(t,u,s),typeof a=="function")){typeof u!="function"&&u!==null&&(n in e?e[n]=null:e.hasAttribute(n)&&e.removeAttribute(n)),e.addEventListener(t,a,s);break e}n in e?e[n]=a:a===!0?e.setAttribute(n,""):ar(e,n,a)}}}function cn(e,t,n){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":tt("error",e),tt("load",e);var a=!1,s=!1,u;for(u in n)if(n.hasOwnProperty(u)){var g=n[u];if(g!=null)switch(u){case"src":a=!0;break;case"srcSet":s=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(r(137,t));default:At(e,t,u,g,n,null)}}s&&At(e,t,"srcSet",n.srcSet,n,null),a&&At(e,t,"src",n.src,n,null);return;case"input":tt("invalid",e);var b=u=g=s=null,z=null,Z=null;for(a in n)if(n.hasOwnProperty(a)){var ne=n[a];if(ne!=null)switch(a){case"name":s=ne;break;case"type":g=ne;break;case"checked":z=ne;break;case"defaultChecked":Z=ne;break;case"value":u=ne;break;case"defaultValue":b=ne;break;case"children":case"dangerouslySetInnerHTML":if(ne!=null)throw Error(r(137,t));break;default:At(e,t,a,ne,n,null)}}Bd(e,u,b,z,Z,g,s,!1);return;case"select":tt("invalid",e),a=g=u=null;for(s in n)if(n.hasOwnProperty(s)&&(b=n[s],b!=null))switch(s){case"value":u=b;break;case"defaultValue":g=b;break;case"multiple":a=b;default:At(e,t,s,b,n,null)}t=u,n=g,e.multiple=!!a,t!=null?Fa(e,!!a,t,!1):n!=null&&Fa(e,!!a,n,!0);return;case"textarea":tt("invalid",e),u=s=a=null;for(g in n)if(n.hasOwnProperty(g)&&(b=n[g],b!=null))switch(g){case"value":a=b;break;case"defaultValue":s=b;break;case"children":u=b;break;case"dangerouslySetInnerHTML":if(b!=null)throw Error(r(91));break;default:At(e,t,g,b,n,null)}Vd(e,a,s,u);return;case"option":for(z in n)if(n.hasOwnProperty(z)&&(a=n[z],a!=null))switch(z){case"selected":e.selected=a&&typeof a!="function"&&typeof a!="symbol";break;default:At(e,t,z,a,n,null)}return;case"dialog":tt("beforetoggle",e),tt("toggle",e),tt("cancel",e),tt("close",e);break;case"iframe":case"object":tt("load",e);break;case"video":case"audio":for(a=0;a<xo.length;a++)tt(xo[a],e);break;case"image":tt("error",e),tt("load",e);break;case"details":tt("toggle",e);break;case"embed":case"source":case"link":tt("error",e),tt("load",e);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(Z in n)if(n.hasOwnProperty(Z)&&(a=n[Z],a!=null))switch(Z){case"children":case"dangerouslySetInnerHTML":throw Error(r(137,t));default:At(e,t,Z,a,n,null)}return;default:if(cu(t)){for(ne in n)n.hasOwnProperty(ne)&&(a=n[ne],a!==void 0&&Jc(e,t,ne,a,n,void 0));return}}for(b in n)n.hasOwnProperty(b)&&(a=n[b],a!=null&&At(e,t,b,a,n,null))}function h1(e,t,n,a){switch(t){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var s=null,u=null,g=null,b=null,z=null,Z=null,ne=null;for(ee in n){var se=n[ee];if(n.hasOwnProperty(ee)&&se!=null)switch(ee){case"checked":break;case"value":break;case"defaultValue":z=se;default:a.hasOwnProperty(ee)||At(e,t,ee,null,a,se)}}for(var J in a){var ee=a[J];if(se=n[J],a.hasOwnProperty(J)&&(ee!=null||se!=null))switch(J){case"type":u=ee;break;case"name":s=ee;break;case"checked":Z=ee;break;case"defaultChecked":ne=ee;break;case"value":g=ee;break;case"defaultValue":b=ee;break;case"children":case"dangerouslySetInnerHTML":if(ee!=null)throw Error(r(137,t));break;default:ee!==se&&At(e,t,J,ee,a,se)}}su(e,g,b,z,Z,ne,u,s);return;case"select":ee=g=b=J=null;for(u in n)if(z=n[u],n.hasOwnProperty(u)&&z!=null)switch(u){case"value":break;case"multiple":ee=z;default:a.hasOwnProperty(u)||At(e,t,u,null,a,z)}for(s in a)if(u=a[s],z=n[s],a.hasOwnProperty(s)&&(u!=null||z!=null))switch(s){case"value":J=u;break;case"defaultValue":b=u;break;case"multiple":g=u;default:u!==z&&At(e,t,s,u,a,z)}t=b,n=g,a=ee,J!=null?Fa(e,!!n,J,!1):!!a!=!!n&&(t!=null?Fa(e,!!n,t,!0):Fa(e,!!n,n?[]:"",!1));return;case"textarea":ee=J=null;for(b in n)if(s=n[b],n.hasOwnProperty(b)&&s!=null&&!a.hasOwnProperty(b))switch(b){case"value":break;case"children":break;default:At(e,t,b,null,a,s)}for(g in a)if(s=a[g],u=n[g],a.hasOwnProperty(g)&&(s!=null||u!=null))switch(g){case"value":J=s;break;case"defaultValue":ee=s;break;case"children":break;case"dangerouslySetInnerHTML":if(s!=null)throw Error(r(91));break;default:s!==u&&At(e,t,g,s,a,u)}Hd(e,J,ee);return;case"option":for(var Ae in n)if(J=n[Ae],n.hasOwnProperty(Ae)&&J!=null&&!a.hasOwnProperty(Ae))switch(Ae){case"selected":e.selected=!1;break;default:At(e,t,Ae,null,a,J)}for(z in a)if(J=a[z],ee=n[z],a.hasOwnProperty(z)&&J!==ee&&(J!=null||ee!=null))switch(z){case"selected":e.selected=J&&typeof J!="function"&&typeof J!="symbol";break;default:At(e,t,z,J,a,ee)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var Ue in n)J=n[Ue],n.hasOwnProperty(Ue)&&J!=null&&!a.hasOwnProperty(Ue)&&At(e,t,Ue,null,a,J);for(Z in a)if(J=a[Z],ee=n[Z],a.hasOwnProperty(Z)&&J!==ee&&(J!=null||ee!=null))switch(Z){case"children":case"dangerouslySetInnerHTML":if(J!=null)throw Error(r(137,t));break;default:At(e,t,Z,J,a,ee)}return;default:if(cu(t)){for(var Ct in n)J=n[Ct],n.hasOwnProperty(Ct)&&J!==void 0&&!a.hasOwnProperty(Ct)&&Jc(e,t,Ct,void 0,a,J);for(ne in a)J=a[ne],ee=n[ne],!a.hasOwnProperty(ne)||J===ee||J===void 0&&ee===void 0||Jc(e,t,ne,J,a,ee);return}}for(var V in n)J=n[V],n.hasOwnProperty(V)&&J!=null&&!a.hasOwnProperty(V)&&At(e,t,V,null,a,J);for(se in a)J=a[se],ee=n[se],!a.hasOwnProperty(se)||J===ee||J==null&&ee==null||At(e,t,se,J,a,ee)}function Gg(e){switch(e){case"css":case"script":case"font":case"img":case"image":case"input":case"link":return!0;default:return!1}}function g1(){if(typeof performance.getEntriesByType=="function"){for(var e=0,t=0,n=performance.getEntriesByType("resource"),a=0;a<n.length;a++){var s=n[a],u=s.transferSize,g=s.initiatorType,b=s.duration;if(u&&b&&Gg(g)){for(g=0,b=s.responseEnd,a+=1;a<n.length;a++){var z=n[a],Z=z.startTime;if(Z>b)break;var ne=z.transferSize,se=z.initiatorType;ne&&Gg(se)&&(z=z.responseEnd,g+=ne*(z<b?1:(b-Z)/(z-Z)))}if(--a,t+=8*(u+g)/(s.duration/1e3),e++,10<e)break}}if(0<e)return t/e/1e6}return navigator.connection&&(e=navigator.connection.downlink,typeof e=="number")?e:5}var Wc=null,$c=null;function ns(e){return e.nodeType===9?e:e.ownerDocument}function Xg(e){switch(e){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function Qg(e,t){if(e===0)switch(t){case"svg":return 1;case"math":return 2;default:return 0}return e===1&&t==="foreignObject"?0:e}function Pc(e,t){return e==="textarea"||e==="noscript"||typeof t.children=="string"||typeof t.children=="number"||typeof t.children=="bigint"||typeof t.dangerouslySetInnerHTML=="object"&&t.dangerouslySetInnerHTML!==null&&t.dangerouslySetInnerHTML.__html!=null}var ef=null;function p1(){var e=window.event;return e&&e.type==="popstate"?e===ef?!1:(ef=e,!0):(ef=null,!1)}var Kg=typeof setTimeout=="function"?setTimeout:void 0,v1=typeof clearTimeout=="function"?clearTimeout:void 0,Zg=typeof Promise=="function"?Promise:void 0,y1=typeof queueMicrotask=="function"?queueMicrotask:typeof Zg<"u"?function(e){return Zg.resolve(null).then(e).catch(b1)}:Kg;function b1(e){setTimeout(function(){throw e})}function na(e){return e==="head"}function Ig(e,t){var n=t,a=0;do{var s=n.nextSibling;if(e.removeChild(n),s&&s.nodeType===8)if(n=s.data,n==="/$"||n==="/&"){if(a===0){e.removeChild(s),Ci(t);return}a--}else if(n==="$"||n==="$?"||n==="$~"||n==="$!"||n==="&")a++;else if(n==="html")Eo(e.ownerDocument.documentElement);else if(n==="head"){n=e.ownerDocument.head,Eo(n);for(var u=n.firstChild;u;){var g=u.nextSibling,b=u.nodeName;u[Hi]||b==="SCRIPT"||b==="STYLE"||b==="LINK"&&u.rel.toLowerCase()==="stylesheet"||n.removeChild(u),u=g}}else n==="body"&&Eo(e.ownerDocument.body);n=s}while(n);Ci(t)}function Fg(e,t){var n=e;e=0;do{var a=n.nextSibling;if(n.nodeType===1?t?(n._stashedDisplay=n.style.display,n.style.display="none"):(n.style.display=n._stashedDisplay||"",n.getAttribute("style")===""&&n.removeAttribute("style")):n.nodeType===3&&(t?(n._stashedText=n.nodeValue,n.nodeValue=""):n.nodeValue=n._stashedText||""),a&&a.nodeType===8)if(n=a.data,n==="/$"){if(e===0)break;e--}else n!=="$"&&n!=="$?"&&n!=="$~"&&n!=="$!"||e++;n=a}while(n)}function tf(e){var t=e.firstChild;for(t&&t.nodeType===10&&(t=t.nextSibling);t;){var n=t;switch(t=t.nextSibling,n.nodeName){case"HTML":case"HEAD":case"BODY":tf(n),ou(n);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(n.rel.toLowerCase()==="stylesheet")continue}e.removeChild(n)}}function x1(e,t,n,a){for(;e.nodeType===1;){var s=n;if(e.nodeName.toLowerCase()!==t.toLowerCase()){if(!a&&(e.nodeName!=="INPUT"||e.type!=="hidden"))break}else if(a){if(!e[Hi])switch(t){case"meta":if(!e.hasAttribute("itemprop"))break;return e;case"link":if(u=e.getAttribute("rel"),u==="stylesheet"&&e.hasAttribute("data-precedence"))break;if(u!==s.rel||e.getAttribute("href")!==(s.href==null||s.href===""?null:s.href)||e.getAttribute("crossorigin")!==(s.crossOrigin==null?null:s.crossOrigin)||e.getAttribute("title")!==(s.title==null?null:s.title))break;return e;case"style":if(e.hasAttribute("data-precedence"))break;return e;case"script":if(u=e.getAttribute("src"),(u!==(s.src==null?null:s.src)||e.getAttribute("type")!==(s.type==null?null:s.type)||e.getAttribute("crossorigin")!==(s.crossOrigin==null?null:s.crossOrigin))&&u&&e.hasAttribute("async")&&!e.hasAttribute("itemprop"))break;return e;default:return e}}else if(t==="input"&&e.type==="hidden"){var u=s.name==null?null:""+s.name;if(s.type==="hidden"&&e.getAttribute("name")===u)return e}else return e;if(e=Kn(e.nextSibling),e===null)break}return null}function S1(e,t,n){if(t==="")return null;for(;e.nodeType!==3;)if((e.nodeType!==1||e.nodeName!=="INPUT"||e.type!=="hidden")&&!n||(e=Kn(e.nextSibling),e===null))return null;return e}function Jg(e,t){for(;e.nodeType!==8;)if((e.nodeType!==1||e.nodeName!=="INPUT"||e.type!=="hidden")&&!t||(e=Kn(e.nextSibling),e===null))return null;return e}function nf(e){return e.data==="$?"||e.data==="$~"}function lf(e){return e.data==="$!"||e.data==="$?"&&e.ownerDocument.readyState!=="loading"}function E1(e,t){var n=e.ownerDocument;if(e.data==="$~")e._reactRetry=t;else if(e.data!=="$?"||n.readyState!=="loading")t();else{var a=function(){t(),n.removeEventListener("DOMContentLoaded",a)};n.addEventListener("DOMContentLoaded",a),e._reactRetry=a}}function Kn(e){for(;e!=null;e=e.nextSibling){var t=e.nodeType;if(t===1||t===3)break;if(t===8){if(t=e.data,t==="$"||t==="$!"||t==="$?"||t==="$~"||t==="&"||t==="F!"||t==="F")break;if(t==="/$"||t==="/&")return null}}return e}var af=null;function Wg(e){e=e.nextSibling;for(var t=0;e;){if(e.nodeType===8){var n=e.data;if(n==="/$"||n==="/&"){if(t===0)return Kn(e.nextSibling);t--}else n!=="$"&&n!=="$!"&&n!=="$?"&&n!=="$~"&&n!=="&"||t++}e=e.nextSibling}return null}function $g(e){e=e.previousSibling;for(var t=0;e;){if(e.nodeType===8){var n=e.data;if(n==="$"||n==="$!"||n==="$?"||n==="$~"||n==="&"){if(t===0)return e;t--}else n!=="/$"&&n!=="/&"||t++}e=e.previousSibling}return null}function Pg(e,t,n){switch(t=ns(n),e){case"html":if(e=t.documentElement,!e)throw Error(r(452));return e;case"head":if(e=t.head,!e)throw Error(r(453));return e;case"body":if(e=t.body,!e)throw Error(r(454));return e;default:throw Error(r(451))}}function Eo(e){for(var t=e.attributes;t.length;)e.removeAttributeNode(t[0]);ou(e)}var Zn=new Map,ep=new Set;function ls(e){return typeof e.getRootNode=="function"?e.getRootNode():e.nodeType===9?e:e.ownerDocument}var zl=k.d;k.d={f:w1,r:T1,D:R1,C:A1,L:C1,m:M1,X:z1,S:O1,M:_1};function w1(){var e=zl.f(),t=Ir();return e||t}function T1(e){var t=Ka(e);t!==null&&t.tag===5&&t.type==="form"?vh(t):zl.r(e)}var Ti=typeof document>"u"?null:document;function tp(e,t,n){var a=Ti;if(a&&typeof t=="string"&&t){var s=Hn(t);s='link[rel="'+e+'"][href="'+s+'"]',typeof n=="string"&&(s+='[crossorigin="'+n+'"]'),ep.has(s)||(ep.add(s),e={rel:e,crossOrigin:n,href:t},a.querySelector(s)===null&&(t=a.createElement("link"),cn(t,"link",e),nn(t),a.head.appendChild(t)))}}function R1(e){zl.D(e),tp("dns-prefetch",e,null)}function A1(e,t){zl.C(e,t),tp("preconnect",e,t)}function C1(e,t,n){zl.L(e,t,n);var a=Ti;if(a&&e&&t){var s='link[rel="preload"][as="'+Hn(t)+'"]';t==="image"&&n&&n.imageSrcSet?(s+='[imagesrcset="'+Hn(n.imageSrcSet)+'"]',typeof n.imageSizes=="string"&&(s+='[imagesizes="'+Hn(n.imageSizes)+'"]')):s+='[href="'+Hn(e)+'"]';var u=s;switch(t){case"style":u=Ri(e);break;case"script":u=Ai(e)}Zn.has(u)||(e=h({rel:"preload",href:t==="image"&&n&&n.imageSrcSet?void 0:e,as:t},n),Zn.set(u,e),a.querySelector(s)!==null||t==="style"&&a.querySelector(wo(u))||t==="script"&&a.querySelector(To(u))||(t=a.createElement("link"),cn(t,"link",e),nn(t),a.head.appendChild(t)))}}function M1(e,t){zl.m(e,t);var n=Ti;if(n&&e){var a=t&&typeof t.as=="string"?t.as:"script",s='link[rel="modulepreload"][as="'+Hn(a)+'"][href="'+Hn(e)+'"]',u=s;switch(a){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":u=Ai(e)}if(!Zn.has(u)&&(e=h({rel:"modulepreload",href:e},t),Zn.set(u,e),n.querySelector(s)===null)){switch(a){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(n.querySelector(To(u)))return}a=n.createElement("link"),cn(a,"link",e),nn(a),n.head.appendChild(a)}}}function O1(e,t,n){zl.S(e,t,n);var a=Ti;if(a&&e){var s=Za(a).hoistableStyles,u=Ri(e);t=t||"default";var g=s.get(u);if(!g){var b={loading:0,preload:null};if(g=a.querySelector(wo(u)))b.loading=5;else{e=h({rel:"stylesheet",href:e,"data-precedence":t},n),(n=Zn.get(u))&&of(e,n);var z=g=a.createElement("link");nn(z),cn(z,"link",e),z._p=new Promise(function(Z,ne){z.onload=Z,z.onerror=ne}),z.addEventListener("load",function(){b.loading|=1}),z.addEventListener("error",function(){b.loading|=2}),b.loading|=4,as(g,t,a)}g={type:"stylesheet",instance:g,count:1,state:b},s.set(u,g)}}}function z1(e,t){zl.X(e,t);var n=Ti;if(n&&e){var a=Za(n).hoistableScripts,s=Ai(e),u=a.get(s);u||(u=n.querySelector(To(s)),u||(e=h({src:e,async:!0},t),(t=Zn.get(s))&&rf(e,t),u=n.createElement("script"),nn(u),cn(u,"link",e),n.head.appendChild(u)),u={type:"script",instance:u,count:1,state:null},a.set(s,u))}}function _1(e,t){zl.M(e,t);var n=Ti;if(n&&e){var a=Za(n).hoistableScripts,s=Ai(e),u=a.get(s);u||(u=n.querySelector(To(s)),u||(e=h({src:e,async:!0,type:"module"},t),(t=Zn.get(s))&&rf(e,t),u=n.createElement("script"),nn(u),cn(u,"link",e),n.head.appendChild(u)),u={type:"script",instance:u,count:1,state:null},a.set(s,u))}}function np(e,t,n,a){var s=(s=P.current)?ls(s):null;if(!s)throw Error(r(446));switch(e){case"meta":case"title":return null;case"style":return typeof n.precedence=="string"&&typeof n.href=="string"?(t=Ri(n.href),n=Za(s).hoistableStyles,a=n.get(t),a||(a={type:"style",instance:null,count:0,state:null},n.set(t,a)),a):{type:"void",instance:null,count:0,state:null};case"link":if(n.rel==="stylesheet"&&typeof n.href=="string"&&typeof n.precedence=="string"){e=Ri(n.href);var u=Za(s).hoistableStyles,g=u.get(e);if(g||(s=s.ownerDocument||s,g={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},u.set(e,g),(u=s.querySelector(wo(e)))&&!u._p&&(g.instance=u,g.state.loading=5),Zn.has(e)||(n={rel:"preload",as:"style",href:n.href,crossOrigin:n.crossOrigin,integrity:n.integrity,media:n.media,hrefLang:n.hrefLang,referrerPolicy:n.referrerPolicy},Zn.set(e,n),u||N1(s,e,n,g.state))),t&&a===null)throw Error(r(528,""));return g}if(t&&a!==null)throw Error(r(529,""));return null;case"script":return t=n.async,n=n.src,typeof n=="string"&&t&&typeof t!="function"&&typeof t!="symbol"?(t=Ai(n),n=Za(s).hoistableScripts,a=n.get(t),a||(a={type:"script",instance:null,count:0,state:null},n.set(t,a)),a):{type:"void",instance:null,count:0,state:null};default:throw Error(r(444,e))}}function Ri(e){return'href="'+Hn(e)+'"'}function wo(e){return'link[rel="stylesheet"]['+e+"]"}function lp(e){return h({},e,{"data-precedence":e.precedence,precedence:null})}function N1(e,t,n,a){e.querySelector('link[rel="preload"][as="style"]['+t+"]")?a.loading=1:(t=e.createElement("link"),a.preload=t,t.addEventListener("load",function(){return a.loading|=1}),t.addEventListener("error",function(){return a.loading|=2}),cn(t,"link",n),nn(t),e.head.appendChild(t))}function Ai(e){return'[src="'+Hn(e)+'"]'}function To(e){return"script[async]"+e}function ap(e,t,n){if(t.count++,t.instance===null)switch(t.type){case"style":var a=e.querySelector('style[data-href~="'+Hn(n.href)+'"]');if(a)return t.instance=a,nn(a),a;var s=h({},n,{"data-href":n.href,"data-precedence":n.precedence,href:null,precedence:null});return a=(e.ownerDocument||e).createElement("style"),nn(a),cn(a,"style",s),as(a,n.precedence,e),t.instance=a;case"stylesheet":s=Ri(n.href);var u=e.querySelector(wo(s));if(u)return t.state.loading|=4,t.instance=u,nn(u),u;a=lp(n),(s=Zn.get(s))&&of(a,s),u=(e.ownerDocument||e).createElement("link"),nn(u);var g=u;return g._p=new Promise(function(b,z){g.onload=b,g.onerror=z}),cn(u,"link",a),t.state.loading|=4,as(u,n.precedence,e),t.instance=u;case"script":return u=Ai(n.src),(s=e.querySelector(To(u)))?(t.instance=s,nn(s),s):(a=n,(s=Zn.get(u))&&(a=h({},n),rf(a,s)),e=e.ownerDocument||e,s=e.createElement("script"),nn(s),cn(s,"link",a),e.head.appendChild(s),t.instance=s);case"void":return null;default:throw Error(r(443,t.type))}else t.type==="stylesheet"&&(t.state.loading&4)===0&&(a=t.instance,t.state.loading|=4,as(a,n.precedence,e));return t.instance}function as(e,t,n){for(var a=n.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),s=a.length?a[a.length-1]:null,u=s,g=0;g<a.length;g++){var b=a[g];if(b.dataset.precedence===t)u=b;else if(u!==s)break}u?u.parentNode.insertBefore(e,u.nextSibling):(t=n.nodeType===9?n.head:n,t.insertBefore(e,t.firstChild))}function of(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.title==null&&(e.title=t.title)}function rf(e,t){e.crossOrigin==null&&(e.crossOrigin=t.crossOrigin),e.referrerPolicy==null&&(e.referrerPolicy=t.referrerPolicy),e.integrity==null&&(e.integrity=t.integrity)}var is=null;function ip(e,t,n){if(is===null){var a=new Map,s=is=new Map;s.set(n,a)}else s=is,a=s.get(n),a||(a=new Map,s.set(n,a));if(a.has(e))return a;for(a.set(e,null),n=n.getElementsByTagName(e),s=0;s<n.length;s++){var u=n[s];if(!(u[Hi]||u[Wt]||e==="link"&&u.getAttribute("rel")==="stylesheet")&&u.namespaceURI!=="http://www.w3.org/2000/svg"){var g=u.getAttribute(t)||"";g=e+g;var b=a.get(g);b?b.push(u):a.set(g,[u])}}return a}function op(e,t,n){e=e.ownerDocument||e,e.head.insertBefore(n,t==="title"?e.querySelector("head > title"):null)}function D1(e,t,n){if(n===1||t.itemProp!=null)return!1;switch(e){case"meta":case"title":return!0;case"style":if(typeof t.precedence!="string"||typeof t.href!="string"||t.href==="")break;return!0;case"link":if(typeof t.rel!="string"||typeof t.href!="string"||t.href===""||t.onLoad||t.onError)break;switch(t.rel){case"stylesheet":return e=t.disabled,typeof t.precedence=="string"&&e==null;default:return!0}case"script":if(t.async&&typeof t.async!="function"&&typeof t.async!="symbol"&&!t.onLoad&&!t.onError&&t.src&&typeof t.src=="string")return!0}return!1}function rp(e){return!(e.type==="stylesheet"&&(e.state.loading&3)===0)}function k1(e,t,n,a){if(n.type==="stylesheet"&&(typeof a.media!="string"||matchMedia(a.media).matches!==!1)&&(n.state.loading&4)===0){if(n.instance===null){var s=Ri(a.href),u=t.querySelector(wo(s));if(u){t=u._p,t!==null&&typeof t=="object"&&typeof t.then=="function"&&(e.count++,e=os.bind(e),t.then(e,e)),n.state.loading|=4,n.instance=u,nn(u);return}u=t.ownerDocument||t,a=lp(a),(s=Zn.get(s))&&of(a,s),u=u.createElement("link"),nn(u);var g=u;g._p=new Promise(function(b,z){g.onload=b,g.onerror=z}),cn(u,"link",a),n.instance=u}e.stylesheets===null&&(e.stylesheets=new Map),e.stylesheets.set(n,t),(t=n.state.preload)&&(n.state.loading&3)===0&&(e.count++,n=os.bind(e),t.addEventListener("load",n),t.addEventListener("error",n))}}var sf=0;function j1(e,t){return e.stylesheets&&e.count===0&&ss(e,e.stylesheets),0<e.count||0<e.imgCount?function(n){var a=setTimeout(function(){if(e.stylesheets&&ss(e,e.stylesheets),e.unsuspend){var u=e.unsuspend;e.unsuspend=null,u()}},6e4+t);0<e.imgBytes&&sf===0&&(sf=62500*g1());var s=setTimeout(function(){if(e.waitingForImages=!1,e.count===0&&(e.stylesheets&&ss(e,e.stylesheets),e.unsuspend)){var u=e.unsuspend;e.unsuspend=null,u()}},(e.imgBytes>sf?50:800)+t);return e.unsuspend=n,function(){e.unsuspend=null,clearTimeout(a),clearTimeout(s)}}:null}function os(){if(this.count--,this.count===0&&(this.imgCount===0||!this.waitingForImages)){if(this.stylesheets)ss(this,this.stylesheets);else if(this.unsuspend){var e=this.unsuspend;this.unsuspend=null,e()}}}var rs=null;function ss(e,t){e.stylesheets=null,e.unsuspend!==null&&(e.count++,rs=new Map,t.forEach(L1,e),rs=null,os.call(e))}function L1(e,t){if(!(t.state.loading&4)){var n=rs.get(e);if(n)var a=n.get(null);else{n=new Map,rs.set(e,n);for(var s=e.querySelectorAll("link[data-precedence],style[data-precedence]"),u=0;u<s.length;u++){var g=s[u];(g.nodeName==="LINK"||g.getAttribute("media")!=="not all")&&(n.set(g.dataset.precedence,g),a=g)}a&&n.set(null,a)}s=t.instance,g=s.getAttribute("data-precedence"),u=n.get(g)||a,u===a&&n.set(null,s),n.set(g,s),this.count++,a=os.bind(this),s.addEventListener("load",a),s.addEventListener("error",a),u?u.parentNode.insertBefore(s,u.nextSibling):(e=e.nodeType===9?e.head:e,e.insertBefore(s,e.firstChild)),t.state.loading|=4}}var Ro={$$typeof:_,Provider:null,Consumer:null,_currentValue:X,_currentValue2:X,_threadCount:0};function U1(e,t,n,a,s,u,g,b,z){this.tag=1,this.containerInfo=e,this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=tn(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=tn(0),this.hiddenUpdates=tn(null),this.identifierPrefix=a,this.onUncaughtError=s,this.onCaughtError=u,this.onRecoverableError=g,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=z,this.incompleteTransitions=new Map}function sp(e,t,n,a,s,u,g,b,z,Z,ne,se){return e=new U1(e,t,n,g,z,Z,ne,se,b),t=1,u===!0&&(t|=24),u=Cn(3,null,null,t),e.current=u,u.stateNode=e,t=Vu(),t.refCount++,e.pooledCache=t,t.refCount++,u.memoizedState={element:a,isDehydrated:n,cache:t},Xu(u),e}function up(e){return e?(e=li,e):li}function cp(e,t,n,a,s,u){s=up(s),a.context===null?a.context=s:a.pendingContext=s,a=Ql(t),a.payload={element:n},u=u===void 0?null:u,u!==null&&(a.callback=u),n=Kl(e,a,t),n!==null&&(En(n,e,t),no(n,e,t))}function fp(e,t){if(e=e.memoizedState,e!==null&&e.dehydrated!==null){var n=e.retryLane;e.retryLane=n!==0&&n<t?n:t}}function uf(e,t){fp(e,t),(e=e.alternate)&&fp(e,t)}function dp(e){if(e.tag===13||e.tag===31){var t=Sa(e,67108864);t!==null&&En(t,e,67108864),uf(e,67108864)}}function mp(e){if(e.tag===13||e.tag===31){var t=Nn();t=Ht(t);var n=Sa(e,t);n!==null&&En(n,e,t),uf(e,t)}}var us=!0;function B1(e,t,n,a){var s=C.T;C.T=null;var u=k.p;try{k.p=2,cf(e,t,n,a)}finally{k.p=u,C.T=s}}function H1(e,t,n,a){var s=C.T;C.T=null;var u=k.p;try{k.p=8,cf(e,t,n,a)}finally{k.p=u,C.T=s}}function cf(e,t,n,a){if(us){var s=ff(a);if(s===null)Fc(e,t,a,cs,n),gp(e,a);else if(Y1(s,e,t,n,a))a.stopPropagation();else if(gp(e,a),t&4&&-1<V1.indexOf(e)){for(;s!==null;){var u=Ka(s);if(u!==null)switch(u.tag){case 3:if(u=u.stateNode,u.current.memoizedState.isDehydrated){var g=_t(u.pendingLanes);if(g!==0){var b=u;for(b.pendingLanes|=2,b.entangledLanes|=2;g;){var z=1<<31-it(g);b.entanglements[1]|=z,g&=~z}ol(u),(yt&6)===0&&(Kr=Se()+500,bo(0))}}break;case 31:case 13:b=Sa(u,2),b!==null&&En(b,u,2),Ir(),uf(u,2)}if(u=ff(a),u===null&&Fc(e,t,a,cs,n),u===s)break;s=u}s!==null&&a.stopPropagation()}else Fc(e,t,a,null,n)}}function ff(e){return e=du(e),df(e)}var cs=null;function df(e){if(cs=null,e=Qa(e),e!==null){var t=f(e);if(t===null)e=null;else{var n=t.tag;if(n===13){if(e=d(t),e!==null)return e;e=null}else if(n===31){if(e=p(t),e!==null)return e;e=null}else if(n===3){if(t.stateNode.current.memoizedState.isDehydrated)return t.tag===3?t.stateNode.containerInfo:null;e=null}else t!==e&&(e=null)}}return cs=e,null}function hp(e){switch(e){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch(Le()){case Xe:return 2;case ut:return 8;case Je:case je:return 32;case Qe:return 268435456;default:return 32}default:return 32}}var mf=!1,la=null,aa=null,ia=null,Ao=new Map,Co=new Map,oa=[],V1="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function gp(e,t){switch(e){case"focusin":case"focusout":la=null;break;case"dragenter":case"dragleave":aa=null;break;case"mouseover":case"mouseout":ia=null;break;case"pointerover":case"pointerout":Ao.delete(t.pointerId);break;case"gotpointercapture":case"lostpointercapture":Co.delete(t.pointerId)}}function Mo(e,t,n,a,s,u){return e===null||e.nativeEvent!==u?(e={blockedOn:t,domEventName:n,eventSystemFlags:a,nativeEvent:u,targetContainers:[s]},t!==null&&(t=Ka(t),t!==null&&dp(t)),e):(e.eventSystemFlags|=a,t=e.targetContainers,s!==null&&t.indexOf(s)===-1&&t.push(s),e)}function Y1(e,t,n,a,s){switch(t){case"focusin":return la=Mo(la,e,t,n,a,s),!0;case"dragenter":return aa=Mo(aa,e,t,n,a,s),!0;case"mouseover":return ia=Mo(ia,e,t,n,a,s),!0;case"pointerover":var u=s.pointerId;return Ao.set(u,Mo(Ao.get(u)||null,e,t,n,a,s)),!0;case"gotpointercapture":return u=s.pointerId,Co.set(u,Mo(Co.get(u)||null,e,t,n,a,s)),!0}return!1}function pp(e){var t=Qa(e.target);if(t!==null){var n=f(t);if(n!==null){if(t=n.tag,t===13){if(t=d(n),t!==null){e.blockedOn=t,lr(e.priority,function(){mp(n)});return}}else if(t===31){if(t=p(n),t!==null){e.blockedOn=t,lr(e.priority,function(){mp(n)});return}}else if(t===3&&n.stateNode.current.memoizedState.isDehydrated){e.blockedOn=n.tag===3?n.stateNode.containerInfo:null;return}}}e.blockedOn=null}function fs(e){if(e.blockedOn!==null)return!1;for(var t=e.targetContainers;0<t.length;){var n=ff(e.nativeEvent);if(n===null){n=e.nativeEvent;var a=new n.constructor(n.type,n);fu=a,n.target.dispatchEvent(a),fu=null}else return t=Ka(n),t!==null&&dp(t),e.blockedOn=n,!1;t.shift()}return!0}function vp(e,t,n){fs(e)&&n.delete(t)}function q1(){mf=!1,la!==null&&fs(la)&&(la=null),aa!==null&&fs(aa)&&(aa=null),ia!==null&&fs(ia)&&(ia=null),Ao.forEach(vp),Co.forEach(vp)}function ds(e,t){e.blockedOn===t&&(e.blockedOn=null,mf||(mf=!0,l.unstable_scheduleCallback(l.unstable_NormalPriority,q1)))}var ms=null;function yp(e){ms!==e&&(ms=e,l.unstable_scheduleCallback(l.unstable_NormalPriority,function(){ms===e&&(ms=null);for(var t=0;t<e.length;t+=3){var n=e[t],a=e[t+1],s=e[t+2];if(typeof a!="function"){if(df(a||n)===null)continue;break}var u=Ka(n);u!==null&&(e.splice(t,3),t-=3,cc(u,{pending:!0,data:s,method:n.method,action:a},a,s))}}))}function Ci(e){function t(z){return ds(z,e)}la!==null&&ds(la,e),aa!==null&&ds(aa,e),ia!==null&&ds(ia,e),Ao.forEach(t),Co.forEach(t);for(var n=0;n<oa.length;n++){var a=oa[n];a.blockedOn===e&&(a.blockedOn=null)}for(;0<oa.length&&(n=oa[0],n.blockedOn===null);)pp(n),n.blockedOn===null&&oa.shift();if(n=(e.ownerDocument||e).$$reactFormReplay,n!=null)for(a=0;a<n.length;a+=3){var s=n[a],u=n[a+1],g=s[on]||null;if(typeof u=="function")g||yp(n);else if(g){var b=null;if(u&&u.hasAttribute("formAction")){if(s=u,g=u[on]||null)b=g.formAction;else if(df(s)!==null)continue}else b=g.action;typeof b=="function"?n[a+1]=b:(n.splice(a,3),a-=3),yp(n)}}}function bp(){function e(u){u.canIntercept&&u.info==="react-transition"&&u.intercept({handler:function(){return new Promise(function(g){return s=g})},focusReset:"manual",scroll:"manual"})}function t(){s!==null&&(s(),s=null),a||setTimeout(n,20)}function n(){if(!a&&!navigation.transition){var u=navigation.currentEntry;u&&u.url!=null&&navigation.navigate(u.url,{state:u.getState(),info:"react-transition",history:"replace"})}}if(typeof navigation=="object"){var a=!1,s=null;return navigation.addEventListener("navigate",e),navigation.addEventListener("navigatesuccess",t),navigation.addEventListener("navigateerror",t),setTimeout(n,100),function(){a=!0,navigation.removeEventListener("navigate",e),navigation.removeEventListener("navigatesuccess",t),navigation.removeEventListener("navigateerror",t),s!==null&&(s(),s=null)}}}function hf(e){this._internalRoot=e}hs.prototype.render=hf.prototype.render=function(e){var t=this._internalRoot;if(t===null)throw Error(r(409));var n=t.current,a=Nn();cp(n,a,e,t,null,null)},hs.prototype.unmount=hf.prototype.unmount=function(){var e=this._internalRoot;if(e!==null){this._internalRoot=null;var t=e.containerInfo;cp(e.current,2,null,e,null,null),Ir(),t[dl]=null}};function hs(e){this._internalRoot=e}hs.prototype.unstable_scheduleHydration=function(e){if(e){var t=pn();e={blockedOn:null,target:e,priority:t};for(var n=0;n<oa.length&&t!==0&&t<oa[n].priority;n++);oa.splice(n,0,e),n===0&&pp(e)}};var xp=i.version;if(xp!=="19.2.4")throw Error(r(527,xp,"19.2.4"));k.findDOMNode=function(e){var t=e._reactInternals;if(t===void 0)throw typeof e.render=="function"?Error(r(188)):(e=Object.keys(e).join(","),Error(r(268,e)));return e=m(t),e=e!==null?y(e):null,e=e===null?null:e.stateNode,e};var G1={bundleType:0,version:"19.2.4",rendererPackageName:"react-dom",currentDispatcherRef:C,reconcilerVersion:"19.2.4"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var gs=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!gs.isDisabled&&gs.supportsFiber)try{gt=gs.inject(G1),pt=gs}catch{}}return zo.createRoot=function(e,t){if(!c(e))throw Error(r(299));var n=!1,a="",s=Ch,u=Mh,g=Oh;return t!=null&&(t.unstable_strictMode===!0&&(n=!0),t.identifierPrefix!==void 0&&(a=t.identifierPrefix),t.onUncaughtError!==void 0&&(s=t.onUncaughtError),t.onCaughtError!==void 0&&(u=t.onCaughtError),t.onRecoverableError!==void 0&&(g=t.onRecoverableError)),t=sp(e,1,!1,null,null,n,a,null,s,u,g,bp),e[dl]=t.current,Ic(e),new hf(t)},zo.hydrateRoot=function(e,t,n){if(!c(e))throw Error(r(299));var a=!1,s="",u=Ch,g=Mh,b=Oh,z=null;return n!=null&&(n.unstable_strictMode===!0&&(a=!0),n.identifierPrefix!==void 0&&(s=n.identifierPrefix),n.onUncaughtError!==void 0&&(u=n.onUncaughtError),n.onCaughtError!==void 0&&(g=n.onCaughtError),n.onRecoverableError!==void 0&&(b=n.onRecoverableError),n.formState!==void 0&&(z=n.formState)),t=sp(e,1,!0,t,n??null,a,s,z,u,g,b,bp),t.context=up(null),n=t.current,a=Nn(),a=Ht(a),s=Ql(a),s.callback=null,Kl(n,s,a),n=a,t.current.lanes=n,ft(t,n),ol(t),e[dl]=t.current,Ic(e),new hs(t)},zo.version="19.2.4",zo}var zp;function tx(){if(zp)return vf.exports;zp=1;function l(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(l)}catch(i){console.error(i)}}return l(),vf.exports=ex(),vf.exports}var nx=tx(),lx=(l,i,o,r,c,f,d,p)=>{let v=document.documentElement,m=["light","dark"];function y(w){(Array.isArray(l)?l:[l]).forEach(S=>{let R=S==="class",M=R&&f?c.map(T=>f[T]||T):c;R?(v.classList.remove(...M),v.classList.add(f&&f[w]?f[w]:w)):v.setAttribute(S,w)}),h(w)}function h(w){p&&m.includes(w)&&(v.style.colorScheme=w)}function E(){return window.matchMedia("(prefers-color-scheme: dark)").matches?"dark":"light"}if(r)y(r);else try{let w=localStorage.getItem(i)||o,S=d&&w==="system"?E():w;y(S)}catch{}},ax=x.createContext(void 0),ix={setTheme:l=>{},themes:[]},ox=()=>{var l;return(l=x.useContext(ax))!=null?l:ix};x.memo(({forcedTheme:l,storageKey:i,attribute:o,enableSystem:r,enableColorScheme:c,defaultTheme:f,value:d,themes:p,nonce:v,scriptProps:m})=>{let y=JSON.stringify([o,i,f,l,p,d,r,c]).slice(1,-1);return x.createElement("script",{...m,suppressHydrationWarning:!0,nonce:typeof window>"u"?v:"",dangerouslySetInnerHTML:{__html:`(${lx.toString()})(${y})`}})});var Ya=Vv();const rx=Bv(Ya);function sx(l){if(typeof document>"u")return;let i=document.head||document.getElementsByTagName("head")[0],o=document.createElement("style");o.type="text/css",i.appendChild(o),o.styleSheet?o.styleSheet.cssText=l:o.appendChild(document.createTextNode(l))}const ux=l=>{switch(l){case"success":return dx;case"info":return hx;case"warning":return mx;case"error":return gx;default:return null}},cx=Array(12).fill(0),fx=({visible:l,className:i})=>ye.createElement("div",{className:["sonner-loading-wrapper",i].filter(Boolean).join(" "),"data-visible":l},ye.createElement("div",{className:"sonner-spinner"},cx.map((o,r)=>ye.createElement("div",{className:"sonner-loading-bar",key:`spinner-bar-${r}`})))),dx=ye.createElement("svg",{xmlns:"http://www.w3.org/2000/svg",viewBox:"0 0 20 20",fill:"currentColor",height:"20",width:"20"},ye.createElement("path",{fillRule:"evenodd",d:"M10 18a8 8 0 100-16 8 8 0 000 16zm3.857-9.809a.75.75 0 00-1.214-.882l-3.483 4.79-1.88-1.88a.75.75 0 10-1.06 1.061l2.5 2.5a.75.75 0 001.137-.089l4-5.5z",clipRule:"evenodd"})),mx=ye.createElement("svg",{xmlns:"http://www.w3.org/2000/svg",viewBox:"0 0 24 24",fill:"currentColor",height:"20",width:"20"},ye.createElement("path",{fillRule:"evenodd",d:"M9.401 3.003c1.155-2 4.043-2 5.197 0l7.355 12.748c1.154 2-.29 4.5-2.599 4.5H4.645c-2.309 0-3.752-2.5-2.598-4.5L9.4 3.003zM12 8.25a.75.75 0 01.75.75v3.75a.75.75 0 01-1.5 0V9a.75.75 0 01.75-.75zm0 8.25a.75.75 0 100-1.5.75.75 0 000 1.5z",clipRule:"evenodd"})),hx=ye.createElement("svg",{xmlns:"http://www.w3.org/2000/svg",viewBox:"0 0 20 20",fill:"currentColor",height:"20",width:"20"},ye.createElement("path",{fillRule:"evenodd",d:"M18 10a8 8 0 11-16 0 8 8 0 0116 0zm-7-4a1 1 0 11-2 0 1 1 0 012 0zM9 9a.75.75 0 000 1.5h.253a.25.25 0 01.244.304l-.459 2.066A1.75 1.75 0 0010.747 15H11a.75.75 0 000-1.5h-.253a.25.25 0 01-.244-.304l.459-2.066A1.75 1.75 0 009.253 9H9z",clipRule:"evenodd"})),gx=ye.createElement("svg",{xmlns:"http://www.w3.org/2000/svg",viewBox:"0 0 20 20",fill:"currentColor",height:"20",width:"20"},ye.createElement("path",{fillRule:"evenodd",d:"M18 10a8 8 0 11-16 0 8 8 0 0116 0zm-8-5a.75.75 0 01.75.75v4.5a.75.75 0 01-1.5 0v-4.5A.75.75 0 0110 5zm0 10a1 1 0 100-2 1 1 0 000 2z",clipRule:"evenodd"})),px=ye.createElement("svg",{xmlns:"http://www.w3.org/2000/svg",width:"12",height:"12",viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:"1.5",strokeLinecap:"round",strokeLinejoin:"round"},ye.createElement("line",{x1:"18",y1:"6",x2:"6",y2:"18"}),ye.createElement("line",{x1:"6",y1:"6",x2:"18",y2:"18"})),vx=()=>{const[l,i]=ye.useState(document.hidden);return ye.useEffect(()=>{const o=()=>{i(document.hidden)};return document.addEventListener("visibilitychange",o),()=>window.removeEventListener("visibilitychange",o)},[]),l};let Gf=1;class yx{constructor(){this.subscribe=i=>(this.subscribers.push(i),()=>{const o=this.subscribers.indexOf(i);this.subscribers.splice(o,1)}),this.publish=i=>{this.subscribers.forEach(o=>o(i))},this.addToast=i=>{this.publish(i),this.toasts=[...this.toasts,i]},this.create=i=>{var o;const{message:r,...c}=i,f=typeof(i==null?void 0:i.id)=="number"||((o=i.id)==null?void 0:o.length)>0?i.id:Gf++,d=this.toasts.find(v=>v.id===f),p=i.dismissible===void 0?!0:i.dismissible;return this.dismissedToasts.has(f)&&this.dismissedToasts.delete(f),d?this.toasts=this.toasts.map(v=>v.id===f?(this.publish({...v,...i,id:f,title:r}),{...v,...i,id:f,dismissible:p,title:r}):v):this.addToast({title:r,...c,dismissible:p,id:f}),f},this.dismiss=i=>(i?(this.dismissedToasts.add(i),requestAnimationFrame(()=>this.subscribers.forEach(o=>o({id:i,dismiss:!0})))):this.toasts.forEach(o=>{this.subscribers.forEach(r=>r({id:o.id,dismiss:!0}))}),i),this.message=(i,o)=>this.create({...o,message:i}),this.error=(i,o)=>this.create({...o,message:i,type:"error"}),this.success=(i,o)=>this.create({...o,type:"success",message:i}),this.info=(i,o)=>this.create({...o,type:"info",message:i}),this.warning=(i,o)=>this.create({...o,type:"warning",message:i}),this.loading=(i,o)=>this.create({...o,type:"loading",message:i}),this.promise=(i,o)=>{if(!o)return;let r;o.loading!==void 0&&(r=this.create({...o,promise:i,type:"loading",message:o.loading,description:typeof o.description!="function"?o.description:void 0}));const c=Promise.resolve(i instanceof Function?i():i);let f=r!==void 0,d;const p=c.then(async m=>{if(d=["resolve",m],ye.isValidElement(m))f=!1,this.create({id:r,type:"default",message:m});else if(xx(m)&&!m.ok){f=!1;const h=typeof o.error=="function"?await o.error(`HTTP error! status: ${m.status}`):o.error,E=typeof o.description=="function"?await o.description(`HTTP error! status: ${m.status}`):o.description,S=typeof h=="object"&&!ye.isValidElement(h)?h:{message:h};this.create({id:r,type:"error",description:E,...S})}else if(m instanceof Error){f=!1;const h=typeof o.error=="function"?await o.error(m):o.error,E=typeof o.description=="function"?await o.description(m):o.description,S=typeof h=="object"&&!ye.isValidElement(h)?h:{message:h};this.create({id:r,type:"error",description:E,...S})}else if(o.success!==void 0){f=!1;const h=typeof o.success=="function"?await o.success(m):o.success,E=typeof o.description=="function"?await o.description(m):o.description,S=typeof h=="object"&&!ye.isValidElement(h)?h:{message:h};this.create({id:r,type:"success",description:E,...S})}}).catch(async m=>{if(d=["reject",m],o.error!==void 0){f=!1;const y=typeof o.error=="function"?await o.error(m):o.error,h=typeof o.description=="function"?await o.description(m):o.description,w=typeof y=="object"&&!ye.isValidElement(y)?y:{message:y};this.create({id:r,type:"error",description:h,...w})}}).finally(()=>{f&&(this.dismiss(r),r=void 0),o.finally==null||o.finally.call(o)}),v=()=>new Promise((m,y)=>p.then(()=>d[0]==="reject"?y(d[1]):m(d[1])).catch(y));return typeof r!="string"&&typeof r!="number"?{unwrap:v}:Object.assign(r,{unwrap:v})},this.custom=(i,o)=>{const r=(o==null?void 0:o.id)||Gf++;return this.create({jsx:i(r),id:r,...o}),r},this.getActiveToasts=()=>this.toasts.filter(i=>!this.dismissedToasts.has(i.id)),this.subscribers=[],this.toasts=[],this.dismissedToasts=new Set}}const wn=new yx,bx=(l,i)=>{const o=(i==null?void 0:i.id)||Gf++;return wn.addToast({title:l,...i,id:o}),o},xx=l=>l&&typeof l=="object"&&"ok"in l&&typeof l.ok=="boolean"&&"status"in l&&typeof l.status=="number",Sx=bx,Ex=()=>wn.toasts,wx=()=>wn.getActiveToasts(),Oi=Object.assign(Sx,{success:wn.success,info:wn.info,warning:wn.warning,error:wn.error,custom:wn.custom,message:wn.message,promise:wn.promise,dismiss:wn.dismiss,loading:wn.loading},{getHistory:Ex,getToasts:wx});sx("[data-sonner-toaster][dir=ltr],html[dir=ltr]{--toast-icon-margin-start:-3px;--toast-icon-margin-end:4px;--toast-svg-margin-start:-1px;--toast-svg-margin-end:0px;--toast-button-margin-start:auto;--toast-button-margin-end:0;--toast-close-button-start:0;--toast-close-button-end:unset;--toast-close-button-transform:translate(-35%, -35%)}[data-sonner-toaster][dir=rtl],html[dir=rtl]{--toast-icon-margin-start:4px;--toast-icon-margin-end:-3px;--toast-svg-margin-start:0px;--toast-svg-margin-end:-1px;--toast-button-margin-start:0;--toast-button-margin-end:auto;--toast-close-button-start:unset;--toast-close-button-end:0;--toast-close-button-transform:translate(35%, -35%)}[data-sonner-toaster]{position:fixed;width:var(--width);font-family:ui-sans-serif,system-ui,-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Helvetica Neue,Arial,Noto Sans,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol,Noto Color Emoji;--gray1:hsl(0, 0%, 99%);--gray2:hsl(0, 0%, 97.3%);--gray3:hsl(0, 0%, 95.1%);--gray4:hsl(0, 0%, 93%);--gray5:hsl(0, 0%, 90.9%);--gray6:hsl(0, 0%, 88.7%);--gray7:hsl(0, 0%, 85.8%);--gray8:hsl(0, 0%, 78%);--gray9:hsl(0, 0%, 56.1%);--gray10:hsl(0, 0%, 52.3%);--gray11:hsl(0, 0%, 43.5%);--gray12:hsl(0, 0%, 9%);--border-radius:8px;box-sizing:border-box;padding:0;margin:0;list-style:none;outline:0;z-index:999999999;transition:transform .4s ease}@media (hover:none) and (pointer:coarse){[data-sonner-toaster][data-lifted=true]{transform:none}}[data-sonner-toaster][data-x-position=right]{right:var(--offset-right)}[data-sonner-toaster][data-x-position=left]{left:var(--offset-left)}[data-sonner-toaster][data-x-position=center]{left:50%;transform:translateX(-50%)}[data-sonner-toaster][data-y-position=top]{top:var(--offset-top)}[data-sonner-toaster][data-y-position=bottom]{bottom:var(--offset-bottom)}[data-sonner-toast]{--y:translateY(100%);--lift-amount:calc(var(--lift) * var(--gap));z-index:var(--z-index);position:absolute;opacity:0;transform:var(--y);touch-action:none;transition:transform .4s,opacity .4s,height .4s,box-shadow .2s;box-sizing:border-box;outline:0;overflow-wrap:anywhere}[data-sonner-toast][data-styled=true]{padding:16px;background:var(--normal-bg);border:1px solid var(--normal-border);color:var(--normal-text);border-radius:var(--border-radius);box-shadow:0 4px 12px rgba(0,0,0,.1);width:var(--width);font-size:13px;display:flex;align-items:center;gap:6px}[data-sonner-toast]:focus-visible{box-shadow:0 4px 12px rgba(0,0,0,.1),0 0 0 2px rgba(0,0,0,.2)}[data-sonner-toast][data-y-position=top]{top:0;--y:translateY(-100%);--lift:1;--lift-amount:calc(1 * var(--gap))}[data-sonner-toast][data-y-position=bottom]{bottom:0;--y:translateY(100%);--lift:-1;--lift-amount:calc(var(--lift) * var(--gap))}[data-sonner-toast][data-styled=true] [data-description]{font-weight:400;line-height:1.4;color:#3f3f3f}[data-rich-colors=true][data-sonner-toast][data-styled=true] [data-description]{color:inherit}[data-sonner-toaster][data-sonner-theme=dark] [data-description]{color:#e8e8e8}[data-sonner-toast][data-styled=true] [data-title]{font-weight:500;line-height:1.5;color:inherit}[data-sonner-toast][data-styled=true] [data-icon]{display:flex;height:16px;width:16px;position:relative;justify-content:flex-start;align-items:center;flex-shrink:0;margin-left:var(--toast-icon-margin-start);margin-right:var(--toast-icon-margin-end)}[data-sonner-toast][data-promise=true] [data-icon]>svg{opacity:0;transform:scale(.8);transform-origin:center;animation:sonner-fade-in .3s ease forwards}[data-sonner-toast][data-styled=true] [data-icon]>*{flex-shrink:0}[data-sonner-toast][data-styled=true] [data-icon] svg{margin-left:var(--toast-svg-margin-start);margin-right:var(--toast-svg-margin-end)}[data-sonner-toast][data-styled=true] [data-content]{display:flex;flex-direction:column;gap:2px}[data-sonner-toast][data-styled=true] [data-button]{border-radius:4px;padding-left:8px;padding-right:8px;height:24px;font-size:12px;color:var(--normal-bg);background:var(--normal-text);margin-left:var(--toast-button-margin-start);margin-right:var(--toast-button-margin-end);border:none;font-weight:500;cursor:pointer;outline:0;display:flex;align-items:center;flex-shrink:0;transition:opacity .4s,box-shadow .2s}[data-sonner-toast][data-styled=true] [data-button]:focus-visible{box-shadow:0 0 0 2px rgba(0,0,0,.4)}[data-sonner-toast][data-styled=true] [data-button]:first-of-type{margin-left:var(--toast-button-margin-start);margin-right:var(--toast-button-margin-end)}[data-sonner-toast][data-styled=true] [data-cancel]{color:var(--normal-text);background:rgba(0,0,0,.08)}[data-sonner-toaster][data-sonner-theme=dark] [data-sonner-toast][data-styled=true] [data-cancel]{background:rgba(255,255,255,.3)}[data-sonner-toast][data-styled=true] [data-close-button]{position:absolute;left:var(--toast-close-button-start);right:var(--toast-close-button-end);top:0;height:20px;width:20px;display:flex;justify-content:center;align-items:center;padding:0;color:var(--gray12);background:var(--normal-bg);border:1px solid var(--gray4);transform:var(--toast-close-button-transform);border-radius:50%;cursor:pointer;z-index:1;transition:opacity .1s,background .2s,border-color .2s}[data-sonner-toast][data-styled=true] [data-close-button]:focus-visible{box-shadow:0 4px 12px rgba(0,0,0,.1),0 0 0 2px rgba(0,0,0,.2)}[data-sonner-toast][data-styled=true] [data-disabled=true]{cursor:not-allowed}[data-sonner-toast][data-styled=true]:hover [data-close-button]:hover{background:var(--gray2);border-color:var(--gray5)}[data-sonner-toast][data-swiping=true]::before{content:'';position:absolute;left:-100%;right:-100%;height:100%;z-index:-1}[data-sonner-toast][data-y-position=top][data-swiping=true]::before{bottom:50%;transform:scaleY(3) translateY(50%)}[data-sonner-toast][data-y-position=bottom][data-swiping=true]::before{top:50%;transform:scaleY(3) translateY(-50%)}[data-sonner-toast][data-swiping=false][data-removed=true]::before{content:'';position:absolute;inset:0;transform:scaleY(2)}[data-sonner-toast][data-expanded=true]::after{content:'';position:absolute;left:0;height:calc(var(--gap) + 1px);bottom:100%;width:100%}[data-sonner-toast][data-mounted=true]{--y:translateY(0);opacity:1}[data-sonner-toast][data-expanded=false][data-front=false]{--scale:var(--toasts-before) * 0.05 + 1;--y:translateY(calc(var(--lift-amount) * var(--toasts-before))) scale(calc(-1 * var(--scale)));height:var(--front-toast-height)}[data-sonner-toast]>*{transition:opacity .4s}[data-sonner-toast][data-x-position=right]{right:0}[data-sonner-toast][data-x-position=left]{left:0}[data-sonner-toast][data-expanded=false][data-front=false][data-styled=true]>*{opacity:0}[data-sonner-toast][data-visible=false]{opacity:0;pointer-events:none}[data-sonner-toast][data-mounted=true][data-expanded=true]{--y:translateY(calc(var(--lift) * var(--offset)));height:var(--initial-height)}[data-sonner-toast][data-removed=true][data-front=true][data-swipe-out=false]{--y:translateY(calc(var(--lift) * -100%));opacity:0}[data-sonner-toast][data-removed=true][data-front=false][data-swipe-out=false][data-expanded=true]{--y:translateY(calc(var(--lift) * var(--offset) + var(--lift) * -100%));opacity:0}[data-sonner-toast][data-removed=true][data-front=false][data-swipe-out=false][data-expanded=false]{--y:translateY(40%);opacity:0;transition:transform .5s,opacity .2s}[data-sonner-toast][data-removed=true][data-front=false]::before{height:calc(var(--initial-height) + 20%)}[data-sonner-toast][data-swiping=true]{transform:var(--y) translateY(var(--swipe-amount-y,0)) translateX(var(--swipe-amount-x,0));transition:none}[data-sonner-toast][data-swiped=true]{user-select:none}[data-sonner-toast][data-swipe-out=true][data-y-position=bottom],[data-sonner-toast][data-swipe-out=true][data-y-position=top]{animation-duration:.2s;animation-timing-function:ease-out;animation-fill-mode:forwards}[data-sonner-toast][data-swipe-out=true][data-swipe-direction=left]{animation-name:swipe-out-left}[data-sonner-toast][data-swipe-out=true][data-swipe-direction=right]{animation-name:swipe-out-right}[data-sonner-toast][data-swipe-out=true][data-swipe-direction=up]{animation-name:swipe-out-up}[data-sonner-toast][data-swipe-out=true][data-swipe-direction=down]{animation-name:swipe-out-down}@keyframes swipe-out-left{from{transform:var(--y) translateX(var(--swipe-amount-x));opacity:1}to{transform:var(--y) translateX(calc(var(--swipe-amount-x) - 100%));opacity:0}}@keyframes swipe-out-right{from{transform:var(--y) translateX(var(--swipe-amount-x));opacity:1}to{transform:var(--y) translateX(calc(var(--swipe-amount-x) + 100%));opacity:0}}@keyframes swipe-out-up{from{transform:var(--y) translateY(var(--swipe-amount-y));opacity:1}to{transform:var(--y) translateY(calc(var(--swipe-amount-y) - 100%));opacity:0}}@keyframes swipe-out-down{from{transform:var(--y) translateY(var(--swipe-amount-y));opacity:1}to{transform:var(--y) translateY(calc(var(--swipe-amount-y) + 100%));opacity:0}}@media (max-width:600px){[data-sonner-toaster]{position:fixed;right:var(--mobile-offset-right);left:var(--mobile-offset-left);width:100%}[data-sonner-toaster][dir=rtl]{left:calc(var(--mobile-offset-left) * -1)}[data-sonner-toaster] [data-sonner-toast]{left:0;right:0;width:calc(100% - var(--mobile-offset-left) * 2)}[data-sonner-toaster][data-x-position=left]{left:var(--mobile-offset-left)}[data-sonner-toaster][data-y-position=bottom]{bottom:var(--mobile-offset-bottom)}[data-sonner-toaster][data-y-position=top]{top:var(--mobile-offset-top)}[data-sonner-toaster][data-x-position=center]{left:var(--mobile-offset-left);right:var(--mobile-offset-right);transform:none}}[data-sonner-toaster][data-sonner-theme=light]{--normal-bg:#fff;--normal-border:var(--gray4);--normal-text:var(--gray12);--success-bg:hsl(143, 85%, 96%);--success-border:hsl(145, 92%, 87%);--success-text:hsl(140, 100%, 27%);--info-bg:hsl(208, 100%, 97%);--info-border:hsl(221, 91%, 93%);--info-text:hsl(210, 92%, 45%);--warning-bg:hsl(49, 100%, 97%);--warning-border:hsl(49, 91%, 84%);--warning-text:hsl(31, 92%, 45%);--error-bg:hsl(359, 100%, 97%);--error-border:hsl(359, 100%, 94%);--error-text:hsl(360, 100%, 45%)}[data-sonner-toaster][data-sonner-theme=light] [data-sonner-toast][data-invert=true]{--normal-bg:#000;--normal-border:hsl(0, 0%, 20%);--normal-text:var(--gray1)}[data-sonner-toaster][data-sonner-theme=dark] [data-sonner-toast][data-invert=true]{--normal-bg:#fff;--normal-border:var(--gray3);--normal-text:var(--gray12)}[data-sonner-toaster][data-sonner-theme=dark]{--normal-bg:#000;--normal-bg-hover:hsl(0, 0%, 12%);--normal-border:hsl(0, 0%, 20%);--normal-border-hover:hsl(0, 0%, 25%);--normal-text:var(--gray1);--success-bg:hsl(150, 100%, 6%);--success-border:hsl(147, 100%, 12%);--success-text:hsl(150, 86%, 65%);--info-bg:hsl(215, 100%, 6%);--info-border:hsl(223, 43%, 17%);--info-text:hsl(216, 87%, 65%);--warning-bg:hsl(64, 100%, 6%);--warning-border:hsl(60, 100%, 9%);--warning-text:hsl(46, 87%, 65%);--error-bg:hsl(358, 76%, 10%);--error-border:hsl(357, 89%, 16%);--error-text:hsl(358, 100%, 81%)}[data-sonner-toaster][data-sonner-theme=dark] [data-sonner-toast] [data-close-button]{background:var(--normal-bg);border-color:var(--normal-border);color:var(--normal-text)}[data-sonner-toaster][data-sonner-theme=dark] [data-sonner-toast] [data-close-button]:hover{background:var(--normal-bg-hover);border-color:var(--normal-border-hover)}[data-rich-colors=true][data-sonner-toast][data-type=success]{background:var(--success-bg);border-color:var(--success-border);color:var(--success-text)}[data-rich-colors=true][data-sonner-toast][data-type=success] [data-close-button]{background:var(--success-bg);border-color:var(--success-border);color:var(--success-text)}[data-rich-colors=true][data-sonner-toast][data-type=info]{background:var(--info-bg);border-color:var(--info-border);color:var(--info-text)}[data-rich-colors=true][data-sonner-toast][data-type=info] [data-close-button]{background:var(--info-bg);border-color:var(--info-border);color:var(--info-text)}[data-rich-colors=true][data-sonner-toast][data-type=warning]{background:var(--warning-bg);border-color:var(--warning-border);color:var(--warning-text)}[data-rich-colors=true][data-sonner-toast][data-type=warning] [data-close-button]{background:var(--warning-bg);border-color:var(--warning-border);color:var(--warning-text)}[data-rich-colors=true][data-sonner-toast][data-type=error]{background:var(--error-bg);border-color:var(--error-border);color:var(--error-text)}[data-rich-colors=true][data-sonner-toast][data-type=error] [data-close-button]{background:var(--error-bg);border-color:var(--error-border);color:var(--error-text)}.sonner-loading-wrapper{--size:16px;height:var(--size);width:var(--size);position:absolute;inset:0;z-index:10}.sonner-loading-wrapper[data-visible=false]{transform-origin:center;animation:sonner-fade-out .2s ease forwards}.sonner-spinner{position:relative;top:50%;left:50%;height:var(--size);width:var(--size)}.sonner-loading-bar{animation:sonner-spin 1.2s linear infinite;background:var(--gray11);border-radius:6px;height:8%;left:-10%;position:absolute;top:-3.9%;width:24%}.sonner-loading-bar:first-child{animation-delay:-1.2s;transform:rotate(.0001deg) translate(146%)}.sonner-loading-bar:nth-child(2){animation-delay:-1.1s;transform:rotate(30deg) translate(146%)}.sonner-loading-bar:nth-child(3){animation-delay:-1s;transform:rotate(60deg) translate(146%)}.sonner-loading-bar:nth-child(4){animation-delay:-.9s;transform:rotate(90deg) translate(146%)}.sonner-loading-bar:nth-child(5){animation-delay:-.8s;transform:rotate(120deg) translate(146%)}.sonner-loading-bar:nth-child(6){animation-delay:-.7s;transform:rotate(150deg) translate(146%)}.sonner-loading-bar:nth-child(7){animation-delay:-.6s;transform:rotate(180deg) translate(146%)}.sonner-loading-bar:nth-child(8){animation-delay:-.5s;transform:rotate(210deg) translate(146%)}.sonner-loading-bar:nth-child(9){animation-delay:-.4s;transform:rotate(240deg) translate(146%)}.sonner-loading-bar:nth-child(10){animation-delay:-.3s;transform:rotate(270deg) translate(146%)}.sonner-loading-bar:nth-child(11){animation-delay:-.2s;transform:rotate(300deg) translate(146%)}.sonner-loading-bar:nth-child(12){animation-delay:-.1s;transform:rotate(330deg) translate(146%)}@keyframes sonner-fade-in{0%{opacity:0;transform:scale(.8)}100%{opacity:1;transform:scale(1)}}@keyframes sonner-fade-out{0%{opacity:1;transform:scale(1)}100%{opacity:0;transform:scale(.8)}}@keyframes sonner-spin{0%{opacity:1}100%{opacity:.15}}@media (prefers-reduced-motion){.sonner-loading-bar,[data-sonner-toast],[data-sonner-toast]>*{transition:none!important;animation:none!important}}.sonner-loader{position:absolute;top:50%;left:50%;transform:translate(-50%,-50%);transform-origin:center;transition:opacity .2s,transform .2s}.sonner-loader[data-visible=false]{opacity:0;transform:scale(.8) translate(-50%,-50%)}");function ps(l){return l.label!==void 0}const Tx=3,Rx="24px",Ax="16px",_p=4e3,Cx=356,Mx=14,Ox=45,zx=200;function rl(...l){return l.filter(Boolean).join(" ")}function _x(l){const[i,o]=l.split("-"),r=[];return i&&r.push(i),o&&r.push(o),r}const Nx=l=>{var i,o,r,c,f,d,p,v,m;const{invert:y,toast:h,unstyled:E,interacting:w,setHeights:S,visibleToasts:R,heights:M,index:T,toasts:B,expanded:_,removeToast:N,defaultRichColors:U,closeButton:G,style:Y,cancelButtonStyle:D,actionButtonStyle:le,className:ge="",descriptionClassName:me="",duration:ue,position:F,gap:ae,expandByDefault:he,classNames:C,icons:k,closeButtonAriaLabel:X="Close toast"}=l,[I,q]=ye.useState(null),[A,O]=ye.useState(null),[W,Q]=ye.useState(!1),[te,P]=ye.useState(!1),[de,ce]=ye.useState(!1),[be,De]=ye.useState(!1),[we,Ye]=ye.useState(!1),[xe,pe]=ye.useState(0),[Re,Ge]=ye.useState(0),ve=ye.useRef(h.duration||ue||_p),Ee=ye.useRef(null),fe=ye.useRef(null),oe=T===0,H=T+1<=R,$=h.type,re=h.dismissible!==!1,Se=h.className||"",Le=h.descriptionClassName||"",Xe=ye.useMemo(()=>M.findIndex(Te=>Te.toastId===h.id)||0,[M,h.id]),ut=ye.useMemo(()=>{var Te;return(Te=h.closeButton)!=null?Te:G},[h.closeButton,G]),Je=ye.useMemo(()=>h.duration||ue||_p,[h.duration,ue]),je=ye.useRef(0),Qe=ye.useRef(0),He=ye.useRef(0),rt=ye.useRef(null),[gt,pt]=F.split("-"),Lt=ye.useMemo(()=>M.reduce((Te,$e,bt)=>bt>=Xe?Te:Te+$e.height,0),[M,Xe]),it=vx(),ot=h.invert||y,Tt=$==="loading";Qe.current=ye.useMemo(()=>Xe*ae+Lt,[Xe,Lt]),ye.useEffect(()=>{ve.current=Je},[Je]),ye.useEffect(()=>{Q(!0)},[]),ye.useEffect(()=>{const Te=fe.current;if(Te){const $e=Te.getBoundingClientRect().height;return Ge($e),S(bt=>[{toastId:h.id,height:$e,position:h.position},...bt]),()=>S(bt=>bt.filter(xt=>xt.toastId!==h.id))}},[S,h.id]),ye.useLayoutEffect(()=>{if(!W)return;const Te=fe.current,$e=Te.style.height;Te.style.height="auto";const bt=Te.getBoundingClientRect().height;Te.style.height=$e,Ge(bt),S(xt=>xt.find(ft=>ft.toastId===h.id)?xt.map(ft=>ft.toastId===h.id?{...ft,height:bt}:ft):[{toastId:h.id,height:bt,position:h.position},...xt])},[W,h.title,h.description,S,h.id,h.jsx,h.action,h.cancel]);const qe=ye.useCallback(()=>{P(!0),pe(Qe.current),S(Te=>Te.filter($e=>$e.toastId!==h.id)),setTimeout(()=>{N(h)},zx)},[h,N,S,Qe]);ye.useEffect(()=>{if(h.promise&&$==="loading"||h.duration===1/0||h.type==="loading")return;let Te;return _||w||it?(()=>{if(He.current<je.current){const xt=new Date().getTime()-je.current;ve.current=ve.current-xt}He.current=new Date().getTime()})():(()=>{ve.current!==1/0&&(je.current=new Date().getTime(),Te=setTimeout(()=>{h.onAutoClose==null||h.onAutoClose.call(h,h),qe()},ve.current))})(),()=>clearTimeout(Te)},[_,w,h,$,it,qe]),ye.useEffect(()=>{h.delete&&(qe(),h.onDismiss==null||h.onDismiss.call(h,h))},[qe,h.delete]);function ct(){var Te;if(k!=null&&k.loading){var $e;return ye.createElement("div",{className:rl(C==null?void 0:C.loader,h==null||($e=h.classNames)==null?void 0:$e.loader,"sonner-loader"),"data-visible":$==="loading"},k.loading)}return ye.createElement(fx,{className:rl(C==null?void 0:C.loader,h==null||(Te=h.classNames)==null?void 0:Te.loader),visible:$==="loading"})}const Ot=h.icon||(k==null?void 0:k[$])||ux($);var st,_t;return ye.createElement("li",{tabIndex:0,ref:fe,className:rl(ge,Se,C==null?void 0:C.toast,h==null||(i=h.classNames)==null?void 0:i.toast,C==null?void 0:C.default,C==null?void 0:C[$],h==null||(o=h.classNames)==null?void 0:o[$]),"data-sonner-toast":"","data-rich-colors":(st=h.richColors)!=null?st:U,"data-styled":!(h.jsx||h.unstyled||E),"data-mounted":W,"data-promise":!!h.promise,"data-swiped":we,"data-removed":te,"data-visible":H,"data-y-position":gt,"data-x-position":pt,"data-index":T,"data-front":oe,"data-swiping":de,"data-dismissible":re,"data-type":$,"data-invert":ot,"data-swipe-out":be,"data-swipe-direction":A,"data-expanded":!!(_||he&&W),"data-testid":h.testId,style:{"--index":T,"--toasts-before":T,"--z-index":B.length-T,"--offset":`${te?xe:Qe.current}px`,"--initial-height":he?"auto":`${Re}px`,...Y,...h.style},onDragEnd:()=>{ce(!1),q(null),rt.current=null},onPointerDown:Te=>{Te.button!==2&&(Tt||!re||(Ee.current=new Date,pe(Qe.current),Te.target.setPointerCapture(Te.pointerId),Te.target.tagName!=="BUTTON"&&(ce(!0),rt.current={x:Te.clientX,y:Te.clientY})))},onPointerUp:()=>{var Te,$e,bt;if(be||!re)return;rt.current=null;const xt=Number(((Te=fe.current)==null?void 0:Te.style.getPropertyValue("--swipe-amount-x").replace("px",""))||0),tn=Number((($e=fe.current)==null?void 0:$e.style.getPropertyValue("--swipe-amount-y").replace("px",""))||0),ft=new Date().getTime()-((bt=Ee.current)==null?void 0:bt.getTime()),ke=I==="x"?xt:tn,St=Math.abs(ke)/ft;if(Math.abs(ke)>=Ox||St>.11){pe(Qe.current),h.onDismiss==null||h.onDismiss.call(h,h),O(I==="x"?xt>0?"right":"left":tn>0?"down":"up"),qe(),De(!0);return}else{var Pe,vt;(Pe=fe.current)==null||Pe.style.setProperty("--swipe-amount-x","0px"),(vt=fe.current)==null||vt.style.setProperty("--swipe-amount-y","0px")}Ye(!1),ce(!1),q(null)},onPointerMove:Te=>{var $e,bt,xt;if(!rt.current||!re||(($e=window.getSelection())==null?void 0:$e.toString().length)>0)return;const ft=Te.clientY-rt.current.y,ke=Te.clientX-rt.current.x;var St;const Pe=(St=l.swipeDirections)!=null?St:_x(F);!I&&(Math.abs(ke)>1||Math.abs(ft)>1)&&q(Math.abs(ke)>Math.abs(ft)?"x":"y");let vt={x:0,y:0};const Ht=Vt=>1/(1.5+Math.abs(Vt)/20);if(I==="y"){if(Pe.includes("top")||Pe.includes("bottom"))if(Pe.includes("top")&&ft<0||Pe.includes("bottom")&&ft>0)vt.y=ft;else{const Vt=ft*Ht(ft);vt.y=Math.abs(Vt)<Math.abs(ft)?Vt:ft}}else if(I==="x"&&(Pe.includes("left")||Pe.includes("right")))if(Pe.includes("left")&&ke<0||Pe.includes("right")&&ke>0)vt.x=ke;else{const Vt=ke*Ht(ke);vt.x=Math.abs(Vt)<Math.abs(ke)?Vt:ke}(Math.abs(vt.x)>0||Math.abs(vt.y)>0)&&Ye(!0),(bt=fe.current)==null||bt.style.setProperty("--swipe-amount-x",`${vt.x}px`),(xt=fe.current)==null||xt.style.setProperty("--swipe-amount-y",`${vt.y}px`)}},ut&&!h.jsx&&$!=="loading"?ye.createElement("button",{"aria-label":X,"data-disabled":Tt,"data-close-button":!0,onClick:Tt||!re?()=>{}:()=>{qe(),h.onDismiss==null||h.onDismiss.call(h,h)},className:rl(C==null?void 0:C.closeButton,h==null||(r=h.classNames)==null?void 0:r.closeButton)},(_t=k==null?void 0:k.close)!=null?_t:px):null,($||h.icon||h.promise)&&h.icon!==null&&((k==null?void 0:k[$])!==null||h.icon)?ye.createElement("div",{"data-icon":"",className:rl(C==null?void 0:C.icon,h==null||(c=h.classNames)==null?void 0:c.icon)},h.promise||h.type==="loading"&&!h.icon?h.icon||ct():null,h.type!=="loading"?Ot:null):null,ye.createElement("div",{"data-content":"",className:rl(C==null?void 0:C.content,h==null||(f=h.classNames)==null?void 0:f.content)},ye.createElement("div",{"data-title":"",className:rl(C==null?void 0:C.title,h==null||(d=h.classNames)==null?void 0:d.title)},h.jsx?h.jsx:typeof h.title=="function"?h.title():h.title),h.description?ye.createElement("div",{"data-description":"",className:rl(me,Le,C==null?void 0:C.description,h==null||(p=h.classNames)==null?void 0:p.description)},typeof h.description=="function"?h.description():h.description):null),ye.isValidElement(h.cancel)?h.cancel:h.cancel&&ps(h.cancel)?ye.createElement("button",{"data-button":!0,"data-cancel":!0,style:h.cancelButtonStyle||D,onClick:Te=>{ps(h.cancel)&&re&&(h.cancel.onClick==null||h.cancel.onClick.call(h.cancel,Te),qe())},className:rl(C==null?void 0:C.cancelButton,h==null||(v=h.classNames)==null?void 0:v.cancelButton)},h.cancel.label):null,ye.isValidElement(h.action)?h.action:h.action&&ps(h.action)?ye.createElement("button",{"data-button":!0,"data-action":!0,style:h.actionButtonStyle||le,onClick:Te=>{ps(h.action)&&(h.action.onClick==null||h.action.onClick.call(h.action,Te),!Te.defaultPrevented&&qe())},className:rl(C==null?void 0:C.actionButton,h==null||(m=h.classNames)==null?void 0:m.actionButton)},h.action.label):null)};function Np(){if(typeof window>"u"||typeof document>"u")return"ltr";const l=document.documentElement.getAttribute("dir");return l==="auto"||!l?window.getComputedStyle(document.documentElement).direction:l}function Dx(l,i){const o={};return[l,i].forEach((r,c)=>{const f=c===1,d=f?"--mobile-offset":"--offset",p=f?Ax:Rx;function v(m){["top","right","bottom","left"].forEach(y=>{o[`${d}-${y}`]=typeof m=="number"?`${m}px`:m})}typeof r=="number"||typeof r=="string"?v(r):typeof r=="object"?["top","right","bottom","left"].forEach(m=>{r[m]===void 0?o[`${d}-${m}`]=p:o[`${d}-${m}`]=typeof r[m]=="number"?`${r[m]}px`:r[m]}):v(p)}),o}const kx=ye.forwardRef(function(i,o){const{id:r,invert:c,position:f="bottom-right",hotkey:d=["altKey","KeyT"],expand:p,closeButton:v,className:m,offset:y,mobileOffset:h,theme:E="light",richColors:w,duration:S,style:R,visibleToasts:M=Tx,toastOptions:T,dir:B=Np(),gap:_=Mx,icons:N,containerAriaLabel:U="Notifications"}=i,[G,Y]=ye.useState([]),D=ye.useMemo(()=>r?G.filter(W=>W.toasterId===r):G.filter(W=>!W.toasterId),[G,r]),le=ye.useMemo(()=>Array.from(new Set([f].concat(D.filter(W=>W.position).map(W=>W.position)))),[D,f]),[ge,me]=ye.useState([]),[ue,F]=ye.useState(!1),[ae,he]=ye.useState(!1),[C,k]=ye.useState(E!=="system"?E:typeof window<"u"&&window.matchMedia&&window.matchMedia("(prefers-color-scheme: dark)").matches?"dark":"light"),X=ye.useRef(null),I=d.join("+").replace(/Key/g,"").replace(/Digit/g,""),q=ye.useRef(null),A=ye.useRef(!1),O=ye.useCallback(W=>{Y(Q=>{var te;return(te=Q.find(P=>P.id===W.id))!=null&&te.delete||wn.dismiss(W.id),Q.filter(({id:P})=>P!==W.id)})},[]);return ye.useEffect(()=>wn.subscribe(W=>{if(W.dismiss){requestAnimationFrame(()=>{Y(Q=>Q.map(te=>te.id===W.id?{...te,delete:!0}:te))});return}setTimeout(()=>{rx.flushSync(()=>{Y(Q=>{const te=Q.findIndex(P=>P.id===W.id);return te!==-1?[...Q.slice(0,te),{...Q[te],...W},...Q.slice(te+1)]:[W,...Q]})})})}),[G]),ye.useEffect(()=>{if(E!=="system"){k(E);return}if(E==="system"&&(window.matchMedia&&window.matchMedia("(prefers-color-scheme: dark)").matches?k("dark"):k("light")),typeof window>"u")return;const W=window.matchMedia("(prefers-color-scheme: dark)");try{W.addEventListener("change",({matches:Q})=>{k(Q?"dark":"light")})}catch{W.addListener(({matches:te})=>{try{k(te?"dark":"light")}catch(P){console.error(P)}})}},[E]),ye.useEffect(()=>{G.length<=1&&F(!1)},[G]),ye.useEffect(()=>{const W=Q=>{var te;if(d.every(ce=>Q[ce]||Q.code===ce)){var de;F(!0),(de=X.current)==null||de.focus()}Q.code==="Escape"&&(document.activeElement===X.current||(te=X.current)!=null&&te.contains(document.activeElement))&&F(!1)};return document.addEventListener("keydown",W),()=>document.removeEventListener("keydown",W)},[d]),ye.useEffect(()=>{if(X.current)return()=>{q.current&&(q.current.focus({preventScroll:!0}),q.current=null,A.current=!1)}},[X.current]),ye.createElement("section",{ref:o,"aria-label":`${U} ${I}`,tabIndex:-1,"aria-live":"polite","aria-relevant":"additions text","aria-atomic":"false",suppressHydrationWarning:!0},le.map((W,Q)=>{var te;const[P,de]=W.split("-");return D.length?ye.createElement("ol",{key:W,dir:B==="auto"?Np():B,tabIndex:-1,ref:X,className:m,"data-sonner-toaster":!0,"data-sonner-theme":C,"data-y-position":P,"data-x-position":de,style:{"--front-toast-height":`${((te=ge[0])==null?void 0:te.height)||0}px`,"--width":`${Cx}px`,"--gap":`${_}px`,...R,...Dx(y,h)},onBlur:ce=>{A.current&&!ce.currentTarget.contains(ce.relatedTarget)&&(A.current=!1,q.current&&(q.current.focus({preventScroll:!0}),q.current=null))},onFocus:ce=>{ce.target instanceof HTMLElement&&ce.target.dataset.dismissible==="false"||A.current||(A.current=!0,q.current=ce.relatedTarget)},onMouseEnter:()=>F(!0),onMouseMove:()=>F(!0),onMouseLeave:()=>{ae||F(!1)},onDragEnd:()=>F(!1),onPointerDown:ce=>{ce.target instanceof HTMLElement&&ce.target.dataset.dismissible==="false"||he(!0)},onPointerUp:()=>he(!1)},D.filter(ce=>!ce.position&&Q===0||ce.position===W).map((ce,be)=>{var De,we;return ye.createElement(Nx,{key:ce.id,icons:N,index:be,toast:ce,defaultRichColors:w,duration:(De=T==null?void 0:T.duration)!=null?De:S,className:T==null?void 0:T.className,descriptionClassName:T==null?void 0:T.descriptionClassName,invert:c,visibleToasts:M,closeButton:(we=T==null?void 0:T.closeButton)!=null?we:v,interacting:ae,position:W,style:T==null?void 0:T.style,unstyled:T==null?void 0:T.unstyled,classNames:T==null?void 0:T.classNames,cancelButtonStyle:T==null?void 0:T.cancelButtonStyle,actionButtonStyle:T==null?void 0:T.actionButtonStyle,closeButtonAriaLabel:T==null?void 0:T.closeButtonAriaLabel,removeToast:O,toasts:D.filter(Ye=>Ye.position==ce.position),heights:ge.filter(Ye=>Ye.position==ce.position),setHeights:me,expandByDefault:p,gap:_,expanded:ue,swipeDirections:i.swipeDirections})})):null}))});/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const Yv=(...l)=>l.filter((i,o,r)=>!!i&&i.trim()!==""&&r.indexOf(i)===o).join(" ").trim();/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const jx=l=>l.replace(/([a-z0-9])([A-Z])/g,"$1-$2").toLowerCase();/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const Lx=l=>l.replace(/^([A-Z])|[\s-_]+(\w)/g,(i,o,r)=>r?r.toUpperCase():o.toLowerCase());/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const Dp=l=>{const i=Lx(l);return i.charAt(0).toUpperCase()+i.slice(1)};/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */var Ux={xmlns:"http://www.w3.org/2000/svg",width:24,height:24,viewBox:"0 0 24 24",fill:"none",stroke:"currentColor",strokeWidth:2,strokeLinecap:"round",strokeLinejoin:"round"};/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const Bx=l=>{for(const i in l)if(i.startsWith("aria-")||i==="role"||i==="title")return!0;return!1};/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const Hx=x.forwardRef(({color:l="currentColor",size:i=24,strokeWidth:o=2,absoluteStrokeWidth:r,className:c="",children:f,iconNode:d,...p},v)=>x.createElement("svg",{ref:v,...Ux,width:i,height:i,stroke:l,strokeWidth:r?Number(o)*24/Number(i):o,className:Yv("lucide",c),...!f&&!Bx(p)&&{"aria-hidden":"true"},...p},[...d.map(([m,y])=>x.createElement(m,y)),...Array.isArray(f)?f:[f]]));/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const Bl=(l,i)=>{const o=x.forwardRef(({className:r,...c},f)=>x.createElement(Hx,{ref:f,iconNode:i,className:Yv(`lucide-${jx(Dp(l))}`,`lucide-${l}`,r),...c}));return o.displayName=Dp(l),o};/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const Vx=[["path",{d:"M20 6 9 17l-5-5",key:"1gmf2c"}]],qv=Bl("check",Vx);/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const Yx=[["path",{d:"m6 9 6 6 6-6",key:"qrunsl"}]],Gv=Bl("chevron-down",Yx);/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const qx=[["path",{d:"m18 15-6-6-6 6",key:"153udz"}]],Gx=Bl("chevron-up",qx);/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const Xx=[["circle",{cx:"12",cy:"12",r:"10",key:"1mglay"}],["path",{d:"m9 12 2 2 4-4",key:"dzmm74"}]],Qx=Bl("circle-check",Xx);/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const Kx=[["circle",{cx:"12",cy:"12",r:"10",key:"1mglay"}],["path",{d:"M12 16v-4",key:"1dtifu"}],["path",{d:"M12 8h.01",key:"e9boi3"}]],Zx=Bl("info",Kx);/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const Ix=[["path",{d:"M21 12a9 9 0 1 1-6.219-8.56",key:"13zald"}]],Fx=Bl("loader-circle",Ix);/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const Jx=[["path",{d:"m15 9-6 6",key:"1uzhvr"}],["path",{d:"M2.586 16.726A2 2 0 0 1 2 15.312V8.688a2 2 0 0 1 .586-1.414l4.688-4.688A2 2 0 0 1 8.688 2h6.624a2 2 0 0 1 1.414.586l4.688 4.688A2 2 0 0 1 22 8.688v6.624a2 2 0 0 1-.586 1.414l-4.688 4.688a2 2 0 0 1-1.414.586H8.688a2 2 0 0 1-1.414-.586z",key:"2d38gg"}],["path",{d:"m9 9 6 6",key:"z0biqf"}]],Wx=Bl("octagon-x",Jx);/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const $x=[["path",{d:"m21.73 18-8-14a2 2 0 0 0-3.48 0l-8 14A2 2 0 0 0 4 21h16a2 2 0 0 0 1.73-3",key:"wmoenq"}],["path",{d:"M12 9v4",key:"juzpu7"}],["path",{d:"M12 17h.01",key:"p32p05"}]],Px=Bl("triangle-alert",$x);/**
 * @license lucide-react v0.577.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */const eS=[["path",{d:"M18 6 6 18",key:"1bl5f8"}],["path",{d:"m6 6 12 12",key:"d8bk6v"}]],tS=Bl("x",eS),nS=({...l})=>{const{theme:i="system"}=ox();return L.jsx(kx,{theme:i,className:"toaster group",icons:{success:L.jsx(Qx,{className:"size-4"}),info:L.jsx(Zx,{className:"size-4"}),warning:L.jsx(Px,{className:"size-4"}),error:L.jsx(Wx,{className:"size-4"}),loading:L.jsx(Fx,{className:"size-4 animate-spin"})},style:{"--normal-bg":"var(--popover)","--normal-text":"var(--popover-foreground)","--normal-border":"var(--border)","--border-radius":"var(--radius)"},toastOptions:{classNames:{toast:"cn-toast"}},...l})};function Ks(){return typeof window<"u"}function ma(l){return rd(l)?(l.nodeName||"").toLowerCase():"#document"}function en(l){var i;return(l==null||(i=l.ownerDocument)==null?void 0:i.defaultView)||window}function cl(l){var i;return(i=(rd(l)?l.ownerDocument:l.document)||window.document)==null?void 0:i.documentElement}function rd(l){return Ks()?l instanceof Node||l instanceof en(l).Node:!1}function Bt(l){return Ks()?l instanceof Element||l instanceof en(l).Element:!1}function Ft(l){return Ks()?l instanceof HTMLElement||l instanceof en(l).HTMLElement:!1}function Yo(l){return!Ks()||typeof ShadowRoot>"u"?!1:l instanceof ShadowRoot||l instanceof en(l).ShadowRoot}function ha(l){const{overflow:i,overflowX:o,overflowY:r,display:c}=Un(l);return/auto|scroll|overlay|hidden|clip/.test(i+r+o)&&c!=="inline"&&c!=="contents"}function lS(l){return/^(table|td|th)$/.test(ma(l))}function Zs(l){try{if(l.matches(":popover-open"))return!0}catch{}try{return l.matches(":modal")}catch{return!1}}const aS=/transform|translate|scale|rotate|perspective|filter/,iS=/paint|layout|strict|content/,ka=l=>!!l&&l!=="none";let Sf;function sd(l){const i=Bt(l)?Un(l):l;return ka(i.transform)||ka(i.translate)||ka(i.scale)||ka(i.rotate)||ka(i.perspective)||!Is()&&(ka(i.backdropFilter)||ka(i.filter))||aS.test(i.willChange||"")||iS.test(i.contain||"")}function oS(l){let i=Ll(l);for(;Ft(i)&&!jl(i);){if(sd(i))return i;if(Zs(i))return null;i=Ll(i)}return null}function Is(){return Sf==null&&(Sf=typeof CSS<"u"&&CSS.supports&&CSS.supports("-webkit-backdrop-filter","none")),Sf}function jl(l){return/^(html|body|#document)$/.test(ma(l))}function Un(l){return en(l).getComputedStyle(l)}function Fs(l){return Bt(l)?{scrollLeft:l.scrollLeft,scrollTop:l.scrollTop}:{scrollLeft:l.scrollX,scrollTop:l.scrollY}}function Ll(l){if(ma(l)==="html")return l;const i=l.assignedSlot||l.parentNode||Yo(l)&&l.host||cl(l);return Yo(i)?i.host:i}function Xv(l){const i=Ll(l);return jl(i)?l.ownerDocument?l.ownerDocument.body:l.body:Ft(i)&&ha(i)?i:Xv(i)}function qo(l,i,o){var r;i===void 0&&(i=[]),o===void 0&&(o=!0);const c=Xv(l),f=c===((r=l.ownerDocument)==null?void 0:r.body),d=en(c);if(f){const p=Xf(d);return i.concat(d,d.visualViewport||[],ha(c)?c:[],p&&o?qo(p):[])}else return i.concat(c,qo(c,[],o))}function Xf(l){return l.parent&&Object.getPrototypeOf(l.parent)?l.frameElement:null}const kp={};function Rn(l,i){const o=x.useRef(kp);return o.current===kp&&(o.current=l(i)),o}const Ef=Hv[`useInsertionEffect${Math.random().toFixed(1)}`.slice(0,-3)],rS=Ef&&Ef!==x.useLayoutEffect?Ef:l=>l();function Ze(l){const i=Rn(sS).current;return i.next=l,rS(i.effect),i.trampoline}function sS(){const l={next:void 0,callback:uS,trampoline:(...i)=>{var o;return(o=l.callback)==null?void 0:o.call(l,...i)},effect:()=>{l.callback=l.next}};return l}function uS(){}const cS={...Hv},fS=()=>{},Oe=typeof document<"u"?x.useLayoutEffect:fS;function Qf(l,i){if(l&&!i)return l;if(!l&&i)return i;if(l||i)return{...l,...i}}const Uo={};function Li(l,i,o,r,c){let f={...Kf(l,Uo)};return i&&(f=jo(f,i)),o&&(f=jo(f,o)),r&&(f=jo(f,r)),c&&(f=jo(f,c)),f}function dS(l){if(l.length===0)return Uo;if(l.length===1)return Kf(l[0],Uo);let i={...Kf(l[0],Uo)};for(let o=1;o<l.length;o+=1)i=jo(i,l[o]);return i}function jo(l,i){return Qv(i)?i(l):mS(l,i)}function mS(l,i){if(!i)return l;for(const o in i){const r=i[o];switch(o){case"style":{l[o]=Qf(l.style,r);break}case"className":{l[o]=Kv(l.className,r);break}default:hS(o,r)?l[o]=gS(l[o],r):l[o]=r}}return l}function hS(l,i){const o=l.charCodeAt(0),r=l.charCodeAt(1),c=l.charCodeAt(2);return o===111&&r===110&&c>=65&&c<=90&&(typeof i=="function"||typeof i>"u")}function Qv(l){return typeof l=="function"}function Kf(l,i){return Qv(l)?l(i):l??Uo}function gS(l,i){return i?l?o=>{if(pS(o)){const c=o;Zf(c);const f=i(c);return c.baseUIHandlerPrevented||l==null||l(c),f}const r=i(o);return l==null||l(o),r}:i:l}function Zf(l){return l.preventBaseUIHandler=()=>{l.baseUIHandlerPrevented=!0},l}function Kv(l,i){return i?l?i+" "+l:i:l}function pS(l){return l!=null&&typeof l=="object"&&"nativeEvent"in l}function vS(l,i){return function(r,...c){const f=new URL(l);return f.searchParams.set("code",r.toString()),c.forEach(d=>f.searchParams.append("args[]",d)),`${i} error #${r}; visit ${f} for the full message.`}}const Fn=vS("https://base-ui.com/production-error","Base UI"),yS=x.createContext(void 0);function bS(l=!1){const i=x.useContext(yS);if(i===void 0&&!l)throw new Error(Fn(16));return i}function xS(l){const{focusableWhenDisabled:i,disabled:o,composite:r=!1,tabIndex:c=0,isNativeButton:f}=l,d=r&&i!==!1,p=r&&i===!1;return{props:x.useMemo(()=>{const m={onKeyDown(y){o&&i&&y.key!=="Tab"&&y.preventDefault()}};return r||(m.tabIndex=c,!f&&o&&(m.tabIndex=i?c:-1)),(f&&(i||d)||!f&&o)&&(m["aria-disabled"]=o),f&&(!i||p)&&(m.disabled=o),m},[r,o,i,d,p,f,c])}}function Fo(l={}){const{disabled:i=!1,focusableWhenDisabled:o,tabIndex:r=0,native:c=!0,composite:f}=l,d=x.useRef(null),p=bS(!0),v=f??p!==void 0,{props:m}=xS({focusableWhenDisabled:o,disabled:i,composite:v,tabIndex:r,isNativeButton:c}),y=x.useCallback(()=>{const w=d.current;wf(w)&&v&&i&&m.disabled===void 0&&w.disabled&&(w.disabled=!1)},[i,m.disabled,v]);Oe(y,[y]);const h=x.useCallback((w={})=>{const{onClick:S,onMouseDown:R,onKeyUp:M,onKeyDown:T,onPointerDown:B,..._}=w;return Li({type:c?"button":void 0,onClick(U){if(i){U.preventDefault();return}S==null||S(U)},onMouseDown(U){i||R==null||R(U)},onKeyDown(U){if(i||(Zf(U),T==null||T(U),U.baseUIHandlerPrevented))return;const G=U.target===U.currentTarget,Y=U.currentTarget,D=wf(Y),le=!c&&SS(Y),ge=G&&(c?D:!le),me=U.key==="Enter",ue=U.key===" ",F=Y.getAttribute("role"),ae=(F==null?void 0:F.startsWith("menuitem"))||F==="option"||F==="gridcell";if(G&&v&&ue){if(U.defaultPrevented&&ae)return;U.preventDefault(),le||c&&D?(Y.click(),U.preventBaseUIHandler()):ge&&(S==null||S(U),U.preventBaseUIHandler());return}ge&&(!c&&(ue||me)&&U.preventDefault(),!c&&me&&(S==null||S(U)))},onKeyUp(U){if(!i){if(Zf(U),M==null||M(U),U.target===U.currentTarget&&c&&v&&wf(U.currentTarget)&&U.key===" "){U.preventDefault();return}U.baseUIHandlerPrevented||U.target===U.currentTarget&&!c&&!v&&U.key===" "&&(S==null||S(U))}},onPointerDown(U){if(i){U.preventDefault();return}B==null||B(U)}},c?void 0:{role:"button"},m,_)},[i,m,v,c]),E=Ze(w=>{d.current=w,y()});return{getButtonProps:h,buttonRef:E}}function wf(l){return Ft(l)&&l.tagName==="BUTTON"}function SS(l){return!!((l==null?void 0:l.tagName)==="A"&&(l!=null&&l.href))}function Ba(l,i,o,r){const c=Rn(Zv).current;return wS(c,l,i,o,r)&&Iv(c,[l,i,o,r]),c.callback}function ES(l){const i=Rn(Zv).current;return TS(i,l)&&Iv(i,l),i.callback}function Zv(){return{callback:null,cleanup:null,refs:[]}}function wS(l,i,o,r,c){return l.refs[0]!==i||l.refs[1]!==o||l.refs[2]!==r||l.refs[3]!==c}function TS(l,i){return l.refs.length!==i.length||l.refs.some((o,r)=>o!==i[r])}function Iv(l,i){if(l.refs=i,i.every(o=>o==null)){l.callback=null;return}l.callback=o=>{if(l.cleanup&&(l.cleanup(),l.cleanup=null),o!=null){const r=Array(i.length).fill(null);for(let c=0;c<i.length;c+=1){const f=i[c];if(f!=null)switch(typeof f){case"function":{const d=f(o);typeof d=="function"&&(r[c]=d);break}case"object":{f.current=o;break}}}l.cleanup=()=>{for(let c=0;c<i.length;c+=1){const f=i[c];if(f!=null)switch(typeof f){case"function":{const d=r[c];typeof d=="function"?d():f(null);break}case"object":{f.current=null;break}}}}}}}const RS=parseInt(x.version,10);function ud(l){return RS>=l}function jp(l){if(!x.isValidElement(l))return null;const i=l,o=i.props;return(ud(19)?o==null?void 0:o.ref:i.ref)??null}function AS(l,i){const o={};for(const r in l){const c=l[r];if(i!=null&&i.hasOwnProperty(r)){const f=i[r](c);f!=null&&Object.assign(o,f);continue}c===!0?o[`data-${r.toLowerCase()}`]="":c&&(o[`data-${r.toLowerCase()}`]=c.toString())}return o}function CS(l,i){return typeof l=="function"?l(i):l}function MS(l,i){return typeof l=="function"?l(i):l}function dn(){}const Lp=Object.freeze([]),an=Object.freeze({}),OS={style:{transition:"none"}},zS="data-base-ui-click-trigger",_S={fallbackAxisSide:"none"},NS={clipPath:"inset(50%)",position:"fixed",top:0,left:0};function Jt(l,i,o={}){const r=i.render,c=DS(i,o);if(o.enabled===!1)return null;const f=o.state??an;return jS(l,r,c,f)}function DS(l,i={}){const{className:o,style:r,render:c}=l,{state:f=an,ref:d,props:p,stateAttributesMapping:v,enabled:m=!0}=i,y=m?CS(o,f):void 0,h=m?MS(r,f):void 0,E=m?AS(f,v):an,w=m?Qf(E,Array.isArray(p)?dS(p):p)??an:an;return typeof document<"u"&&(m?Array.isArray(d)?w.ref=ES([w.ref,jp(c),...d]):w.ref=Ba(w.ref,jp(c),d):Ba(null,null)),m?(y!==void 0&&(w.className=Kv(w.className,y)),h!==void 0&&(w.style=Qf(w.style,h)),w):an}const kS=Symbol.for("react.lazy");function jS(l,i,o,r){if(i){if(typeof i=="function")return i(o,r);const c=Li(o,i.props);c.ref=o.ref;let f=i;return(f==null?void 0:f.$$typeof)===kS&&(f=x.Children.toArray(i)[0]),x.cloneElement(f,c)}if(l&&typeof l=="string")return LS(l,o);throw new Error(Fn(8))}function LS(l,i){return l==="button"?x.createElement("button",{type:"button",...i,key:i.key}):l==="img"?x.createElement("img",{alt:"",...i,key:i.key}):x.createElement(l,i)}const US=x.forwardRef(function(i,o){const{render:r,className:c,disabled:f=!1,focusableWhenDisabled:d=!1,nativeButton:p=!0,...v}=i,{getButtonProps:m,buttonRef:y}=Fo({disabled:f,focusableWhenDisabled:d,native:p});return Jt("button",i,{state:{disabled:f},ref:[o,y],props:[v,m]})});function Fv(l){var i,o,r="";if(typeof l=="string"||typeof l=="number")r+=l;else if(typeof l=="object")if(Array.isArray(l)){var c=l.length;for(i=0;i<c;i++)l[i]&&(o=Fv(l[i]))&&(r&&(r+=" "),r+=o)}else for(o in l)l[o]&&(r&&(r+=" "),r+=o);return r}function Jv(){for(var l,i,o=0,r="",c=arguments.length;o<c;o++)(l=arguments[o])&&(i=Fv(l))&&(r&&(r+=" "),r+=i);return r}const Up=l=>typeof l=="boolean"?`${l}`:l===0?"0":l,Bp=Jv,Wv=(l,i)=>o=>{var r;if((i==null?void 0:i.variants)==null)return Bp(l,o==null?void 0:o.class,o==null?void 0:o.className);const{variants:c,defaultVariants:f}=i,d=Object.keys(c).map(m=>{const y=o==null?void 0:o[m],h=f==null?void 0:f[m];if(y===null)return null;const E=Up(y)||Up(h);return c[m][E]}),p=o&&Object.entries(o).reduce((m,y)=>{let[h,E]=y;return E===void 0||(m[h]=E),m},{}),v=i==null||(r=i.compoundVariants)===null||r===void 0?void 0:r.reduce((m,y)=>{let{class:h,className:E,...w}=y;return Object.entries(w).every(S=>{let[R,M]=S;return Array.isArray(M)?M.includes({...f,...p}[R]):{...f,...p}[R]===M})?[...m,h,E]:m},[]);return Bp(l,d,v,o==null?void 0:o.class,o==null?void 0:o.className)},BS=(l,i)=>{const o=new Array(l.length+i.length);for(let r=0;r<l.length;r++)o[r]=l[r];for(let r=0;r<i.length;r++)o[l.length+r]=i[r];return o},HS=(l,i)=>({classGroupId:l,validator:i}),$v=(l=new Map,i=null,o)=>({nextPart:l,validators:i,classGroupId:o}),Ns="-",Hp=[],VS="arbitrary..",YS=l=>{const i=GS(l),{conflictingClassGroups:o,conflictingClassGroupModifiers:r}=l;return{getClassGroupId:d=>{if(d.startsWith("[")&&d.endsWith("]"))return qS(d);const p=d.split(Ns),v=p[0]===""&&p.length>1?1:0;return Pv(p,v,i)},getConflictingClassGroupIds:(d,p)=>{if(p){const v=r[d],m=o[d];return v?m?BS(m,v):v:m||Hp}return o[d]||Hp}}},Pv=(l,i,o)=>{if(l.length-i===0)return o.classGroupId;const c=l[i],f=o.nextPart.get(c);if(f){const m=Pv(l,i+1,f);if(m)return m}const d=o.validators;if(d===null)return;const p=i===0?l.join(Ns):l.slice(i).join(Ns),v=d.length;for(let m=0;m<v;m++){const y=d[m];if(y.validator(p))return y.classGroupId}},qS=l=>l.slice(1,-1).indexOf(":")===-1?void 0:(()=>{const i=l.slice(1,-1),o=i.indexOf(":"),r=i.slice(0,o);return r?VS+r:void 0})(),GS=l=>{const{theme:i,classGroups:o}=l;return XS(o,i)},XS=(l,i)=>{const o=$v();for(const r in l){const c=l[r];cd(c,o,r,i)}return o},cd=(l,i,o,r)=>{const c=l.length;for(let f=0;f<c;f++){const d=l[f];QS(d,i,o,r)}},QS=(l,i,o,r)=>{if(typeof l=="string"){KS(l,i,o);return}if(typeof l=="function"){ZS(l,i,o,r);return}IS(l,i,o,r)},KS=(l,i,o)=>{const r=l===""?i:ey(i,l);r.classGroupId=o},ZS=(l,i,o,r)=>{if(FS(l)){cd(l(r),i,o,r);return}i.validators===null&&(i.validators=[]),i.validators.push(HS(o,l))},IS=(l,i,o,r)=>{const c=Object.entries(l),f=c.length;for(let d=0;d<f;d++){const[p,v]=c[d];cd(v,ey(i,p),o,r)}},ey=(l,i)=>{let o=l;const r=i.split(Ns),c=r.length;for(let f=0;f<c;f++){const d=r[f];let p=o.nextPart.get(d);p||(p=$v(),o.nextPart.set(d,p)),o=p}return o},FS=l=>"isThemeGetter"in l&&l.isThemeGetter===!0,JS=l=>{if(l<1)return{get:()=>{},set:()=>{}};let i=0,o=Object.create(null),r=Object.create(null);const c=(f,d)=>{o[f]=d,i++,i>l&&(i=0,r=o,o=Object.create(null))};return{get(f){let d=o[f];if(d!==void 0)return d;if((d=r[f])!==void 0)return c(f,d),d},set(f,d){f in o?o[f]=d:c(f,d)}}},If="!",Vp=":",WS=[],Yp=(l,i,o,r,c)=>({modifiers:l,hasImportantModifier:i,baseClassName:o,maybePostfixModifierPosition:r,isExternal:c}),$S=l=>{const{prefix:i,experimentalParseClassName:o}=l;let r=c=>{const f=[];let d=0,p=0,v=0,m;const y=c.length;for(let R=0;R<y;R++){const M=c[R];if(d===0&&p===0){if(M===Vp){f.push(c.slice(v,R)),v=R+1;continue}if(M==="/"){m=R;continue}}M==="["?d++:M==="]"?d--:M==="("?p++:M===")"&&p--}const h=f.length===0?c:c.slice(v);let E=h,w=!1;h.endsWith(If)?(E=h.slice(0,-1),w=!0):h.startsWith(If)&&(E=h.slice(1),w=!0);const S=m&&m>v?m-v:void 0;return Yp(f,w,E,S)};if(i){const c=i+Vp,f=r;r=d=>d.startsWith(c)?f(d.slice(c.length)):Yp(WS,!1,d,void 0,!0)}if(o){const c=r;r=f=>o({className:f,parseClassName:c})}return r},PS=l=>{const i=new Map;return l.orderSensitiveModifiers.forEach((o,r)=>{i.set(o,1e6+r)}),o=>{const r=[];let c=[];for(let f=0;f<o.length;f++){const d=o[f],p=d[0]==="[",v=i.has(d);p||v?(c.length>0&&(c.sort(),r.push(...c),c=[]),r.push(d)):c.push(d)}return c.length>0&&(c.sort(),r.push(...c)),r}},eE=l=>({cache:JS(l.cacheSize),parseClassName:$S(l),sortModifiers:PS(l),...YS(l)}),tE=/\s+/,nE=(l,i)=>{const{parseClassName:o,getClassGroupId:r,getConflictingClassGroupIds:c,sortModifiers:f}=i,d=[],p=l.trim().split(tE);let v="";for(let m=p.length-1;m>=0;m-=1){const y=p[m],{isExternal:h,modifiers:E,hasImportantModifier:w,baseClassName:S,maybePostfixModifierPosition:R}=o(y);if(h){v=y+(v.length>0?" "+v:v);continue}let M=!!R,T=r(M?S.substring(0,R):S);if(!T){if(!M){v=y+(v.length>0?" "+v:v);continue}if(T=r(S),!T){v=y+(v.length>0?" "+v:v);continue}M=!1}const B=E.length===0?"":E.length===1?E[0]:f(E).join(":"),_=w?B+If:B,N=_+T;if(d.indexOf(N)>-1)continue;d.push(N);const U=c(T,M);for(let G=0;G<U.length;++G){const Y=U[G];d.push(_+Y)}v=y+(v.length>0?" "+v:v)}return v},lE=(...l)=>{let i=0,o,r,c="";for(;i<l.length;)(o=l[i++])&&(r=ty(o))&&(c&&(c+=" "),c+=r);return c},ty=l=>{if(typeof l=="string")return l;let i,o="";for(let r=0;r<l.length;r++)l[r]&&(i=ty(l[r]))&&(o&&(o+=" "),o+=i);return o},aE=(l,...i)=>{let o,r,c,f;const d=v=>{const m=i.reduce((y,h)=>h(y),l());return o=eE(m),r=o.cache.get,c=o.cache.set,f=p,p(v)},p=v=>{const m=r(v);if(m)return m;const y=nE(v,o);return c(v,y),y};return f=d,(...v)=>f(lE(...v))},iE=[],Pt=l=>{const i=o=>o[l]||iE;return i.isThemeGetter=!0,i},ny=/^\[(?:(\w[\w-]*):)?(.+)\]$/i,ly=/^\((?:(\w[\w-]*):)?(.+)\)$/i,oE=/^\d+(?:\.\d+)?\/\d+(?:\.\d+)?$/,rE=/^(\d+(\.\d+)?)?(xs|sm|md|lg|xl)$/,sE=/\d+(%|px|r?em|[sdl]?v([hwib]|min|max)|pt|pc|in|cm|mm|cap|ch|ex|r?lh|cq(w|h|i|b|min|max))|\b(calc|min|max|clamp)\(.+\)|^0$/,uE=/^(rgba?|hsla?|hwb|(ok)?(lab|lch)|color-mix)\(.+\)$/,cE=/^(inset_)?-?((\d+)?\.?(\d+)[a-z]+|0)_-?((\d+)?\.?(\d+)[a-z]+|0)/,fE=/^(url|image|image-set|cross-fade|element|(repeating-)?(linear|radial|conic)-gradient)\(.+\)$/,sa=l=>oE.test(l),We=l=>!!l&&!Number.isNaN(Number(l)),ua=l=>!!l&&Number.isInteger(Number(l)),Tf=l=>l.endsWith("%")&&We(l.slice(0,-1)),_l=l=>rE.test(l),ay=()=>!0,dE=l=>sE.test(l)&&!uE.test(l),fd=()=>!1,mE=l=>cE.test(l),hE=l=>fE.test(l),gE=l=>!Ce(l)&&!Me(l),pE=l=>ga(l,ry,fd),Ce=l=>ny.test(l),ja=l=>ga(l,sy,dE),qp=l=>ga(l,TE,We),vE=l=>ga(l,cy,ay),yE=l=>ga(l,uy,fd),Gp=l=>ga(l,iy,fd),bE=l=>ga(l,oy,hE),vs=l=>ga(l,fy,mE),Me=l=>ly.test(l),_o=l=>qa(l,sy),xE=l=>qa(l,uy),Xp=l=>qa(l,iy),SE=l=>qa(l,ry),EE=l=>qa(l,oy),ys=l=>qa(l,fy,!0),wE=l=>qa(l,cy,!0),ga=(l,i,o)=>{const r=ny.exec(l);return r?r[1]?i(r[1]):o(r[2]):!1},qa=(l,i,o=!1)=>{const r=ly.exec(l);return r?r[1]?i(r[1]):o:!1},iy=l=>l==="position"||l==="percentage",oy=l=>l==="image"||l==="url",ry=l=>l==="length"||l==="size"||l==="bg-size",sy=l=>l==="length",TE=l=>l==="number",uy=l=>l==="family-name",cy=l=>l==="number"||l==="weight",fy=l=>l==="shadow",RE=()=>{const l=Pt("color"),i=Pt("font"),o=Pt("text"),r=Pt("font-weight"),c=Pt("tracking"),f=Pt("leading"),d=Pt("breakpoint"),p=Pt("container"),v=Pt("spacing"),m=Pt("radius"),y=Pt("shadow"),h=Pt("inset-shadow"),E=Pt("text-shadow"),w=Pt("drop-shadow"),S=Pt("blur"),R=Pt("perspective"),M=Pt("aspect"),T=Pt("ease"),B=Pt("animate"),_=()=>["auto","avoid","all","avoid-page","page","left","right","column"],N=()=>["center","top","bottom","left","right","top-left","left-top","top-right","right-top","bottom-right","right-bottom","bottom-left","left-bottom"],U=()=>[...N(),Me,Ce],G=()=>["auto","hidden","clip","visible","scroll"],Y=()=>["auto","contain","none"],D=()=>[Me,Ce,v],le=()=>[sa,"full","auto",...D()],ge=()=>[ua,"none","subgrid",Me,Ce],me=()=>["auto",{span:["full",ua,Me,Ce]},ua,Me,Ce],ue=()=>[ua,"auto",Me,Ce],F=()=>["auto","min","max","fr",Me,Ce],ae=()=>["start","end","center","between","around","evenly","stretch","baseline","center-safe","end-safe"],he=()=>["start","end","center","stretch","center-safe","end-safe"],C=()=>["auto",...D()],k=()=>[sa,"auto","full","dvw","dvh","lvw","lvh","svw","svh","min","max","fit",...D()],X=()=>[sa,"screen","full","dvw","lvw","svw","min","max","fit",...D()],I=()=>[sa,"screen","full","lh","dvh","lvh","svh","min","max","fit",...D()],q=()=>[l,Me,Ce],A=()=>[...N(),Xp,Gp,{position:[Me,Ce]}],O=()=>["no-repeat",{repeat:["","x","y","space","round"]}],W=()=>["auto","cover","contain",SE,pE,{size:[Me,Ce]}],Q=()=>[Tf,_o,ja],te=()=>["","none","full",m,Me,Ce],P=()=>["",We,_o,ja],de=()=>["solid","dashed","dotted","double"],ce=()=>["normal","multiply","screen","overlay","darken","lighten","color-dodge","color-burn","hard-light","soft-light","difference","exclusion","hue","saturation","color","luminosity"],be=()=>[We,Tf,Xp,Gp],De=()=>["","none",S,Me,Ce],we=()=>["none",We,Me,Ce],Ye=()=>["none",We,Me,Ce],xe=()=>[We,Me,Ce],pe=()=>[sa,"full",...D()];return{cacheSize:500,theme:{animate:["spin","ping","pulse","bounce"],aspect:["video"],blur:[_l],breakpoint:[_l],color:[ay],container:[_l],"drop-shadow":[_l],ease:["in","out","in-out"],font:[gE],"font-weight":["thin","extralight","light","normal","medium","semibold","bold","extrabold","black"],"inset-shadow":[_l],leading:["none","tight","snug","normal","relaxed","loose"],perspective:["dramatic","near","normal","midrange","distant","none"],radius:[_l],shadow:[_l],spacing:["px",We],text:[_l],"text-shadow":[_l],tracking:["tighter","tight","normal","wide","wider","widest"]},classGroups:{aspect:[{aspect:["auto","square",sa,Ce,Me,M]}],container:["container"],columns:[{columns:[We,Ce,Me,p]}],"break-after":[{"break-after":_()}],"break-before":[{"break-before":_()}],"break-inside":[{"break-inside":["auto","avoid","avoid-page","avoid-column"]}],"box-decoration":[{"box-decoration":["slice","clone"]}],box:[{box:["border","content"]}],display:["block","inline-block","inline","flex","inline-flex","table","inline-table","table-caption","table-cell","table-column","table-column-group","table-footer-group","table-header-group","table-row-group","table-row","flow-root","grid","inline-grid","contents","list-item","hidden"],sr:["sr-only","not-sr-only"],float:[{float:["right","left","none","start","end"]}],clear:[{clear:["left","right","both","none","start","end"]}],isolation:["isolate","isolation-auto"],"object-fit":[{object:["contain","cover","fill","none","scale-down"]}],"object-position":[{object:U()}],overflow:[{overflow:G()}],"overflow-x":[{"overflow-x":G()}],"overflow-y":[{"overflow-y":G()}],overscroll:[{overscroll:Y()}],"overscroll-x":[{"overscroll-x":Y()}],"overscroll-y":[{"overscroll-y":Y()}],position:["static","fixed","absolute","relative","sticky"],inset:[{inset:le()}],"inset-x":[{"inset-x":le()}],"inset-y":[{"inset-y":le()}],start:[{"inset-s":le(),start:le()}],end:[{"inset-e":le(),end:le()}],"inset-bs":[{"inset-bs":le()}],"inset-be":[{"inset-be":le()}],top:[{top:le()}],right:[{right:le()}],bottom:[{bottom:le()}],left:[{left:le()}],visibility:["visible","invisible","collapse"],z:[{z:[ua,"auto",Me,Ce]}],basis:[{basis:[sa,"full","auto",p,...D()]}],"flex-direction":[{flex:["row","row-reverse","col","col-reverse"]}],"flex-wrap":[{flex:["nowrap","wrap","wrap-reverse"]}],flex:[{flex:[We,sa,"auto","initial","none",Ce]}],grow:[{grow:["",We,Me,Ce]}],shrink:[{shrink:["",We,Me,Ce]}],order:[{order:[ua,"first","last","none",Me,Ce]}],"grid-cols":[{"grid-cols":ge()}],"col-start-end":[{col:me()}],"col-start":[{"col-start":ue()}],"col-end":[{"col-end":ue()}],"grid-rows":[{"grid-rows":ge()}],"row-start-end":[{row:me()}],"row-start":[{"row-start":ue()}],"row-end":[{"row-end":ue()}],"grid-flow":[{"grid-flow":["row","col","dense","row-dense","col-dense"]}],"auto-cols":[{"auto-cols":F()}],"auto-rows":[{"auto-rows":F()}],gap:[{gap:D()}],"gap-x":[{"gap-x":D()}],"gap-y":[{"gap-y":D()}],"justify-content":[{justify:[...ae(),"normal"]}],"justify-items":[{"justify-items":[...he(),"normal"]}],"justify-self":[{"justify-self":["auto",...he()]}],"align-content":[{content:["normal",...ae()]}],"align-items":[{items:[...he(),{baseline:["","last"]}]}],"align-self":[{self:["auto",...he(),{baseline:["","last"]}]}],"place-content":[{"place-content":ae()}],"place-items":[{"place-items":[...he(),"baseline"]}],"place-self":[{"place-self":["auto",...he()]}],p:[{p:D()}],px:[{px:D()}],py:[{py:D()}],ps:[{ps:D()}],pe:[{pe:D()}],pbs:[{pbs:D()}],pbe:[{pbe:D()}],pt:[{pt:D()}],pr:[{pr:D()}],pb:[{pb:D()}],pl:[{pl:D()}],m:[{m:C()}],mx:[{mx:C()}],my:[{my:C()}],ms:[{ms:C()}],me:[{me:C()}],mbs:[{mbs:C()}],mbe:[{mbe:C()}],mt:[{mt:C()}],mr:[{mr:C()}],mb:[{mb:C()}],ml:[{ml:C()}],"space-x":[{"space-x":D()}],"space-x-reverse":["space-x-reverse"],"space-y":[{"space-y":D()}],"space-y-reverse":["space-y-reverse"],size:[{size:k()}],"inline-size":[{inline:["auto",...X()]}],"min-inline-size":[{"min-inline":["auto",...X()]}],"max-inline-size":[{"max-inline":["none",...X()]}],"block-size":[{block:["auto",...I()]}],"min-block-size":[{"min-block":["auto",...I()]}],"max-block-size":[{"max-block":["none",...I()]}],w:[{w:[p,"screen",...k()]}],"min-w":[{"min-w":[p,"screen","none",...k()]}],"max-w":[{"max-w":[p,"screen","none","prose",{screen:[d]},...k()]}],h:[{h:["screen","lh",...k()]}],"min-h":[{"min-h":["screen","lh","none",...k()]}],"max-h":[{"max-h":["screen","lh",...k()]}],"font-size":[{text:["base",o,_o,ja]}],"font-smoothing":["antialiased","subpixel-antialiased"],"font-style":["italic","not-italic"],"font-weight":[{font:[r,wE,vE]}],"font-stretch":[{"font-stretch":["ultra-condensed","extra-condensed","condensed","semi-condensed","normal","semi-expanded","expanded","extra-expanded","ultra-expanded",Tf,Ce]}],"font-family":[{font:[xE,yE,i]}],"font-features":[{"font-features":[Ce]}],"fvn-normal":["normal-nums"],"fvn-ordinal":["ordinal"],"fvn-slashed-zero":["slashed-zero"],"fvn-figure":["lining-nums","oldstyle-nums"],"fvn-spacing":["proportional-nums","tabular-nums"],"fvn-fraction":["diagonal-fractions","stacked-fractions"],tracking:[{tracking:[c,Me,Ce]}],"line-clamp":[{"line-clamp":[We,"none",Me,qp]}],leading:[{leading:[f,...D()]}],"list-image":[{"list-image":["none",Me,Ce]}],"list-style-position":[{list:["inside","outside"]}],"list-style-type":[{list:["disc","decimal","none",Me,Ce]}],"text-alignment":[{text:["left","center","right","justify","start","end"]}],"placeholder-color":[{placeholder:q()}],"text-color":[{text:q()}],"text-decoration":["underline","overline","line-through","no-underline"],"text-decoration-style":[{decoration:[...de(),"wavy"]}],"text-decoration-thickness":[{decoration:[We,"from-font","auto",Me,ja]}],"text-decoration-color":[{decoration:q()}],"underline-offset":[{"underline-offset":[We,"auto",Me,Ce]}],"text-transform":["uppercase","lowercase","capitalize","normal-case"],"text-overflow":["truncate","text-ellipsis","text-clip"],"text-wrap":[{text:["wrap","nowrap","balance","pretty"]}],indent:[{indent:D()}],"vertical-align":[{align:["baseline","top","middle","bottom","text-top","text-bottom","sub","super",Me,Ce]}],whitespace:[{whitespace:["normal","nowrap","pre","pre-line","pre-wrap","break-spaces"]}],break:[{break:["normal","words","all","keep"]}],wrap:[{wrap:["break-word","anywhere","normal"]}],hyphens:[{hyphens:["none","manual","auto"]}],content:[{content:["none",Me,Ce]}],"bg-attachment":[{bg:["fixed","local","scroll"]}],"bg-clip":[{"bg-clip":["border","padding","content","text"]}],"bg-origin":[{"bg-origin":["border","padding","content"]}],"bg-position":[{bg:A()}],"bg-repeat":[{bg:O()}],"bg-size":[{bg:W()}],"bg-image":[{bg:["none",{linear:[{to:["t","tr","r","br","b","bl","l","tl"]},ua,Me,Ce],radial:["",Me,Ce],conic:[ua,Me,Ce]},EE,bE]}],"bg-color":[{bg:q()}],"gradient-from-pos":[{from:Q()}],"gradient-via-pos":[{via:Q()}],"gradient-to-pos":[{to:Q()}],"gradient-from":[{from:q()}],"gradient-via":[{via:q()}],"gradient-to":[{to:q()}],rounded:[{rounded:te()}],"rounded-s":[{"rounded-s":te()}],"rounded-e":[{"rounded-e":te()}],"rounded-t":[{"rounded-t":te()}],"rounded-r":[{"rounded-r":te()}],"rounded-b":[{"rounded-b":te()}],"rounded-l":[{"rounded-l":te()}],"rounded-ss":[{"rounded-ss":te()}],"rounded-se":[{"rounded-se":te()}],"rounded-ee":[{"rounded-ee":te()}],"rounded-es":[{"rounded-es":te()}],"rounded-tl":[{"rounded-tl":te()}],"rounded-tr":[{"rounded-tr":te()}],"rounded-br":[{"rounded-br":te()}],"rounded-bl":[{"rounded-bl":te()}],"border-w":[{border:P()}],"border-w-x":[{"border-x":P()}],"border-w-y":[{"border-y":P()}],"border-w-s":[{"border-s":P()}],"border-w-e":[{"border-e":P()}],"border-w-bs":[{"border-bs":P()}],"border-w-be":[{"border-be":P()}],"border-w-t":[{"border-t":P()}],"border-w-r":[{"border-r":P()}],"border-w-b":[{"border-b":P()}],"border-w-l":[{"border-l":P()}],"divide-x":[{"divide-x":P()}],"divide-x-reverse":["divide-x-reverse"],"divide-y":[{"divide-y":P()}],"divide-y-reverse":["divide-y-reverse"],"border-style":[{border:[...de(),"hidden","none"]}],"divide-style":[{divide:[...de(),"hidden","none"]}],"border-color":[{border:q()}],"border-color-x":[{"border-x":q()}],"border-color-y":[{"border-y":q()}],"border-color-s":[{"border-s":q()}],"border-color-e":[{"border-e":q()}],"border-color-bs":[{"border-bs":q()}],"border-color-be":[{"border-be":q()}],"border-color-t":[{"border-t":q()}],"border-color-r":[{"border-r":q()}],"border-color-b":[{"border-b":q()}],"border-color-l":[{"border-l":q()}],"divide-color":[{divide:q()}],"outline-style":[{outline:[...de(),"none","hidden"]}],"outline-offset":[{"outline-offset":[We,Me,Ce]}],"outline-w":[{outline:["",We,_o,ja]}],"outline-color":[{outline:q()}],shadow:[{shadow:["","none",y,ys,vs]}],"shadow-color":[{shadow:q()}],"inset-shadow":[{"inset-shadow":["none",h,ys,vs]}],"inset-shadow-color":[{"inset-shadow":q()}],"ring-w":[{ring:P()}],"ring-w-inset":["ring-inset"],"ring-color":[{ring:q()}],"ring-offset-w":[{"ring-offset":[We,ja]}],"ring-offset-color":[{"ring-offset":q()}],"inset-ring-w":[{"inset-ring":P()}],"inset-ring-color":[{"inset-ring":q()}],"text-shadow":[{"text-shadow":["none",E,ys,vs]}],"text-shadow-color":[{"text-shadow":q()}],opacity:[{opacity:[We,Me,Ce]}],"mix-blend":[{"mix-blend":[...ce(),"plus-darker","plus-lighter"]}],"bg-blend":[{"bg-blend":ce()}],"mask-clip":[{"mask-clip":["border","padding","content","fill","stroke","view"]},"mask-no-clip"],"mask-composite":[{mask:["add","subtract","intersect","exclude"]}],"mask-image-linear-pos":[{"mask-linear":[We]}],"mask-image-linear-from-pos":[{"mask-linear-from":be()}],"mask-image-linear-to-pos":[{"mask-linear-to":be()}],"mask-image-linear-from-color":[{"mask-linear-from":q()}],"mask-image-linear-to-color":[{"mask-linear-to":q()}],"mask-image-t-from-pos":[{"mask-t-from":be()}],"mask-image-t-to-pos":[{"mask-t-to":be()}],"mask-image-t-from-color":[{"mask-t-from":q()}],"mask-image-t-to-color":[{"mask-t-to":q()}],"mask-image-r-from-pos":[{"mask-r-from":be()}],"mask-image-r-to-pos":[{"mask-r-to":be()}],"mask-image-r-from-color":[{"mask-r-from":q()}],"mask-image-r-to-color":[{"mask-r-to":q()}],"mask-image-b-from-pos":[{"mask-b-from":be()}],"mask-image-b-to-pos":[{"mask-b-to":be()}],"mask-image-b-from-color":[{"mask-b-from":q()}],"mask-image-b-to-color":[{"mask-b-to":q()}],"mask-image-l-from-pos":[{"mask-l-from":be()}],"mask-image-l-to-pos":[{"mask-l-to":be()}],"mask-image-l-from-color":[{"mask-l-from":q()}],"mask-image-l-to-color":[{"mask-l-to":q()}],"mask-image-x-from-pos":[{"mask-x-from":be()}],"mask-image-x-to-pos":[{"mask-x-to":be()}],"mask-image-x-from-color":[{"mask-x-from":q()}],"mask-image-x-to-color":[{"mask-x-to":q()}],"mask-image-y-from-pos":[{"mask-y-from":be()}],"mask-image-y-to-pos":[{"mask-y-to":be()}],"mask-image-y-from-color":[{"mask-y-from":q()}],"mask-image-y-to-color":[{"mask-y-to":q()}],"mask-image-radial":[{"mask-radial":[Me,Ce]}],"mask-image-radial-from-pos":[{"mask-radial-from":be()}],"mask-image-radial-to-pos":[{"mask-radial-to":be()}],"mask-image-radial-from-color":[{"mask-radial-from":q()}],"mask-image-radial-to-color":[{"mask-radial-to":q()}],"mask-image-radial-shape":[{"mask-radial":["circle","ellipse"]}],"mask-image-radial-size":[{"mask-radial":[{closest:["side","corner"],farthest:["side","corner"]}]}],"mask-image-radial-pos":[{"mask-radial-at":N()}],"mask-image-conic-pos":[{"mask-conic":[We]}],"mask-image-conic-from-pos":[{"mask-conic-from":be()}],"mask-image-conic-to-pos":[{"mask-conic-to":be()}],"mask-image-conic-from-color":[{"mask-conic-from":q()}],"mask-image-conic-to-color":[{"mask-conic-to":q()}],"mask-mode":[{mask:["alpha","luminance","match"]}],"mask-origin":[{"mask-origin":["border","padding","content","fill","stroke","view"]}],"mask-position":[{mask:A()}],"mask-repeat":[{mask:O()}],"mask-size":[{mask:W()}],"mask-type":[{"mask-type":["alpha","luminance"]}],"mask-image":[{mask:["none",Me,Ce]}],filter:[{filter:["","none",Me,Ce]}],blur:[{blur:De()}],brightness:[{brightness:[We,Me,Ce]}],contrast:[{contrast:[We,Me,Ce]}],"drop-shadow":[{"drop-shadow":["","none",w,ys,vs]}],"drop-shadow-color":[{"drop-shadow":q()}],grayscale:[{grayscale:["",We,Me,Ce]}],"hue-rotate":[{"hue-rotate":[We,Me,Ce]}],invert:[{invert:["",We,Me,Ce]}],saturate:[{saturate:[We,Me,Ce]}],sepia:[{sepia:["",We,Me,Ce]}],"backdrop-filter":[{"backdrop-filter":["","none",Me,Ce]}],"backdrop-blur":[{"backdrop-blur":De()}],"backdrop-brightness":[{"backdrop-brightness":[We,Me,Ce]}],"backdrop-contrast":[{"backdrop-contrast":[We,Me,Ce]}],"backdrop-grayscale":[{"backdrop-grayscale":["",We,Me,Ce]}],"backdrop-hue-rotate":[{"backdrop-hue-rotate":[We,Me,Ce]}],"backdrop-invert":[{"backdrop-invert":["",We,Me,Ce]}],"backdrop-opacity":[{"backdrop-opacity":[We,Me,Ce]}],"backdrop-saturate":[{"backdrop-saturate":[We,Me,Ce]}],"backdrop-sepia":[{"backdrop-sepia":["",We,Me,Ce]}],"border-collapse":[{border:["collapse","separate"]}],"border-spacing":[{"border-spacing":D()}],"border-spacing-x":[{"border-spacing-x":D()}],"border-spacing-y":[{"border-spacing-y":D()}],"table-layout":[{table:["auto","fixed"]}],caption:[{caption:["top","bottom"]}],transition:[{transition:["","all","colors","opacity","shadow","transform","none",Me,Ce]}],"transition-behavior":[{transition:["normal","discrete"]}],duration:[{duration:[We,"initial",Me,Ce]}],ease:[{ease:["linear","initial",T,Me,Ce]}],delay:[{delay:[We,Me,Ce]}],animate:[{animate:["none",B,Me,Ce]}],backface:[{backface:["hidden","visible"]}],perspective:[{perspective:[R,Me,Ce]}],"perspective-origin":[{"perspective-origin":U()}],rotate:[{rotate:we()}],"rotate-x":[{"rotate-x":we()}],"rotate-y":[{"rotate-y":we()}],"rotate-z":[{"rotate-z":we()}],scale:[{scale:Ye()}],"scale-x":[{"scale-x":Ye()}],"scale-y":[{"scale-y":Ye()}],"scale-z":[{"scale-z":Ye()}],"scale-3d":["scale-3d"],skew:[{skew:xe()}],"skew-x":[{"skew-x":xe()}],"skew-y":[{"skew-y":xe()}],transform:[{transform:[Me,Ce,"","none","gpu","cpu"]}],"transform-origin":[{origin:U()}],"transform-style":[{transform:["3d","flat"]}],translate:[{translate:pe()}],"translate-x":[{"translate-x":pe()}],"translate-y":[{"translate-y":pe()}],"translate-z":[{"translate-z":pe()}],"translate-none":["translate-none"],accent:[{accent:q()}],appearance:[{appearance:["none","auto"]}],"caret-color":[{caret:q()}],"color-scheme":[{scheme:["normal","dark","light","light-dark","only-dark","only-light"]}],cursor:[{cursor:["auto","default","pointer","wait","text","move","help","not-allowed","none","context-menu","progress","cell","crosshair","vertical-text","alias","copy","no-drop","grab","grabbing","all-scroll","col-resize","row-resize","n-resize","e-resize","s-resize","w-resize","ne-resize","nw-resize","se-resize","sw-resize","ew-resize","ns-resize","nesw-resize","nwse-resize","zoom-in","zoom-out",Me,Ce]}],"field-sizing":[{"field-sizing":["fixed","content"]}],"pointer-events":[{"pointer-events":["auto","none"]}],resize:[{resize:["none","","y","x"]}],"scroll-behavior":[{scroll:["auto","smooth"]}],"scroll-m":[{"scroll-m":D()}],"scroll-mx":[{"scroll-mx":D()}],"scroll-my":[{"scroll-my":D()}],"scroll-ms":[{"scroll-ms":D()}],"scroll-me":[{"scroll-me":D()}],"scroll-mbs":[{"scroll-mbs":D()}],"scroll-mbe":[{"scroll-mbe":D()}],"scroll-mt":[{"scroll-mt":D()}],"scroll-mr":[{"scroll-mr":D()}],"scroll-mb":[{"scroll-mb":D()}],"scroll-ml":[{"scroll-ml":D()}],"scroll-p":[{"scroll-p":D()}],"scroll-px":[{"scroll-px":D()}],"scroll-py":[{"scroll-py":D()}],"scroll-ps":[{"scroll-ps":D()}],"scroll-pe":[{"scroll-pe":D()}],"scroll-pbs":[{"scroll-pbs":D()}],"scroll-pbe":[{"scroll-pbe":D()}],"scroll-pt":[{"scroll-pt":D()}],"scroll-pr":[{"scroll-pr":D()}],"scroll-pb":[{"scroll-pb":D()}],"scroll-pl":[{"scroll-pl":D()}],"snap-align":[{snap:["start","end","center","align-none"]}],"snap-stop":[{snap:["normal","always"]}],"snap-type":[{snap:["none","x","y","both"]}],"snap-strictness":[{snap:["mandatory","proximity"]}],touch:[{touch:["auto","none","manipulation"]}],"touch-x":[{"touch-pan":["x","left","right"]}],"touch-y":[{"touch-pan":["y","up","down"]}],"touch-pz":["touch-pinch-zoom"],select:[{select:["none","text","all","auto"]}],"will-change":[{"will-change":["auto","scroll","contents","transform",Me,Ce]}],fill:[{fill:["none",...q()]}],"stroke-w":[{stroke:[We,_o,ja,qp]}],stroke:[{stroke:["none",...q()]}],"forced-color-adjust":[{"forced-color-adjust":["auto","none"]}]},conflictingClassGroups:{overflow:["overflow-x","overflow-y"],overscroll:["overscroll-x","overscroll-y"],inset:["inset-x","inset-y","inset-bs","inset-be","start","end","top","right","bottom","left"],"inset-x":["right","left"],"inset-y":["top","bottom"],flex:["basis","grow","shrink"],gap:["gap-x","gap-y"],p:["px","py","ps","pe","pbs","pbe","pt","pr","pb","pl"],px:["pr","pl"],py:["pt","pb"],m:["mx","my","ms","me","mbs","mbe","mt","mr","mb","ml"],mx:["mr","ml"],my:["mt","mb"],size:["w","h"],"font-size":["leading"],"fvn-normal":["fvn-ordinal","fvn-slashed-zero","fvn-figure","fvn-spacing","fvn-fraction"],"fvn-ordinal":["fvn-normal"],"fvn-slashed-zero":["fvn-normal"],"fvn-figure":["fvn-normal"],"fvn-spacing":["fvn-normal"],"fvn-fraction":["fvn-normal"],"line-clamp":["display","overflow"],rounded:["rounded-s","rounded-e","rounded-t","rounded-r","rounded-b","rounded-l","rounded-ss","rounded-se","rounded-ee","rounded-es","rounded-tl","rounded-tr","rounded-br","rounded-bl"],"rounded-s":["rounded-ss","rounded-es"],"rounded-e":["rounded-se","rounded-ee"],"rounded-t":["rounded-tl","rounded-tr"],"rounded-r":["rounded-tr","rounded-br"],"rounded-b":["rounded-br","rounded-bl"],"rounded-l":["rounded-tl","rounded-bl"],"border-spacing":["border-spacing-x","border-spacing-y"],"border-w":["border-w-x","border-w-y","border-w-s","border-w-e","border-w-bs","border-w-be","border-w-t","border-w-r","border-w-b","border-w-l"],"border-w-x":["border-w-r","border-w-l"],"border-w-y":["border-w-t","border-w-b"],"border-color":["border-color-x","border-color-y","border-color-s","border-color-e","border-color-bs","border-color-be","border-color-t","border-color-r","border-color-b","border-color-l"],"border-color-x":["border-color-r","border-color-l"],"border-color-y":["border-color-t","border-color-b"],translate:["translate-x","translate-y","translate-none"],"translate-none":["translate","translate-x","translate-y","translate-z"],"scroll-m":["scroll-mx","scroll-my","scroll-ms","scroll-me","scroll-mbs","scroll-mbe","scroll-mt","scroll-mr","scroll-mb","scroll-ml"],"scroll-mx":["scroll-mr","scroll-ml"],"scroll-my":["scroll-mt","scroll-mb"],"scroll-p":["scroll-px","scroll-py","scroll-ps","scroll-pe","scroll-pbs","scroll-pbe","scroll-pt","scroll-pr","scroll-pb","scroll-pl"],"scroll-px":["scroll-pr","scroll-pl"],"scroll-py":["scroll-pt","scroll-pb"],touch:["touch-x","touch-y","touch-pz"],"touch-x":["touch"],"touch-y":["touch"],"touch-pz":["touch"]},conflictingClassGroupModifiers:{"font-size":["leading"]},orderSensitiveModifiers:["*","**","after","backdrop","before","details-content","file","first-letter","first-line","marker","placeholder","selection"]}},AE=aE(RE);function mn(...l){return AE(Jv(l))}const CE=Wv("group/button inline-flex shrink-0 items-center justify-center rounded-lg border border-transparent bg-clip-padding text-sm font-medium whitespace-nowrap transition-all outline-none select-none focus-visible:border-ring focus-visible:ring-3 focus-visible:ring-ring/50 active:translate-y-px disabled:pointer-events-none disabled:opacity-50 aria-invalid:border-destructive aria-invalid:ring-3 aria-invalid:ring-destructive/20 dark:aria-invalid:border-destructive/50 dark:aria-invalid:ring-destructive/40 [&_svg]:pointer-events-none [&_svg]:shrink-0 [&_svg:not([class*='size-'])]:size-4",{variants:{variant:{default:"bg-primary text-primary-foreground [a]:hover:bg-primary/80",outline:"border-border bg-background hover:bg-muted hover:text-foreground aria-expanded:bg-muted aria-expanded:text-foreground dark:border-input dark:bg-input/30 dark:hover:bg-input/50",secondary:"bg-secondary text-secondary-foreground hover:bg-secondary/80 aria-expanded:bg-secondary aria-expanded:text-secondary-foreground",ghost:"hover:bg-muted hover:text-foreground aria-expanded:bg-muted aria-expanded:text-foreground dark:hover:bg-muted/50",destructive:"bg-destructive/10 text-destructive hover:bg-destructive/20 focus-visible:border-destructive/40 focus-visible:ring-destructive/20 dark:bg-destructive/20 dark:hover:bg-destructive/30 dark:focus-visible:ring-destructive/40",link:"text-primary underline-offset-4 hover:underline"},size:{default:"h-8 gap-1.5 px-2.5 has-data-[icon=inline-end]:pr-2 has-data-[icon=inline-start]:pl-2",xs:"h-6 gap-1 rounded-[min(var(--radius-md),10px)] px-2 text-xs in-data-[slot=button-group]:rounded-lg has-data-[icon=inline-end]:pr-1.5 has-data-[icon=inline-start]:pl-1.5 [&_svg:not([class*='size-'])]:size-3",sm:"h-7 gap-1 rounded-[min(var(--radius-md),12px)] px-2.5 text-[0.8rem] in-data-[slot=button-group]:rounded-lg has-data-[icon=inline-end]:pr-1.5 has-data-[icon=inline-start]:pl-1.5 [&_svg:not([class*='size-'])]:size-3.5",lg:"h-9 gap-1.5 px-2.5 has-data-[icon=inline-end]:pr-3 has-data-[icon=inline-start]:pl-3",icon:"size-8","icon-xs":"size-6 rounded-[min(var(--radius-md),10px)] in-data-[slot=button-group]:rounded-lg [&_svg:not([class*='size-'])]:size-3","icon-sm":"size-7 rounded-[min(var(--radius-md),12px)] in-data-[slot=button-group]:rounded-lg","icon-lg":"size-9"}},defaultVariants:{variant:"default",size:"default"}});function Go({className:l,variant:i="default",size:o="default",...r}){return L.jsx(US,{"data-slot":"button",className:mn(CE({variant:i,size:o,className:l})),...r})}let Qp=(function(l){return l.disabled="data-disabled",l.valid="data-valid",l.invalid="data-invalid",l.touched="data-touched",l.dirty="data-dirty",l.filled="data-filled",l.focused="data-focused",l})({});const ME={badInput:!1,customError:!1,patternMismatch:!1,rangeOverflow:!1,rangeUnderflow:!1,stepMismatch:!1,tooLong:!1,tooShort:!1,typeMismatch:!1,valid:null,valueMissing:!1},Lo={valid:null,touched:!1,dirty:!1,filled:!1,focused:!1},OE={disabled:!1,...Lo},Js={valid(l){return l===null?null:l?{[Qp.valid]:""}:{[Qp.invalid]:""}}},zE=x.createContext({invalid:void 0,name:void 0,validityData:{state:ME,errors:[],error:"",value:"",initialValue:null},setValidityData:dn,disabled:void 0,touched:Lo.touched,setTouched:dn,dirty:Lo.dirty,setDirty:dn,filled:Lo.filled,setFilled:dn,focused:Lo.focused,setFocused:dn,validate:()=>null,validationMode:"onSubmit",validationDebounceTime:0,shouldValidateOnChange:()=>!1,state:OE,markedDirtyRef:{current:!1},validation:{getValidationProps:(l=an)=>l,getInputValidationProps:(l=an)=>l,inputRef:{current:null},commit:async()=>{}}});function Jo(l=!0){const i=x.useContext(zE);if(i.setValidityData===dn&&!l)throw new Error(Fn(28));return i}const _E=x.createContext({formRef:{current:{fields:new Map}},errors:{},clearErrors:dn,validationMode:"onSubmit",submitAttemptedRef:{current:!1}});function dd(){return x.useContext(_E)}let Kp=0;function NE(l,i="mui"){const[o,r]=x.useState(l),c=l||o;return x.useEffect(()=>{o==null&&(Kp+=1,r(`${i}-${Kp}`))},[o,i]),c}const Zp=cS.useId;function Wo(l,i){if(Zp!==void 0){const o=Zp();return l??(i?`${i}-${o}`:o)}return NE(l,i)}function Xo(l){return Wo(l,"base-ui")}const DE=x.createContext({controlId:void 0,registerControlId:dn,labelId:void 0,setLabelId:dn,messageIds:[],setMessageIds:dn,getDescriptionProps:l=>l});function Ws(){return x.useContext(DE)}const kE=[];function dy(l){x.useEffect(l,kE)}const No=0;class Ha{constructor(){Ut(this,"currentId",No);Ut(this,"clear",()=>{this.currentId!==No&&(clearTimeout(this.currentId),this.currentId=No)});Ut(this,"disposeEffect",()=>this.clear)}static create(){return new Ha}start(i,o){this.clear(),this.currentId=setTimeout(()=>{this.currentId=No,o()},i)}isStarted(){return this.currentId!==No}}function jn(){const l=Rn(Ha.create).current;return dy(l.disposeEffect),l}function jE(l,i){return{...l,state:{...l.state,valid:!i&&l.state.valid}}}function kt(l){return(l==null?void 0:l.ownerDocument)||document}const Ui=typeof navigator<"u",Rf=BE(),my=VE(),hy=HE(),md=typeof CSS>"u"||!CSS.supports?!1:CSS.supports("-webkit-backdrop-filter:none"),gy=Rf.platform==="MacIntel"&&Rf.maxTouchPoints>1?!0:/iP(hone|ad|od)|iOS/.test(Rf.platform),LE=Ui&&/apple/i.test(navigator.vendor),Ff=Ui&&/android/i.test(my)||/android/i.test(hy);Ui&&my.toLowerCase().startsWith("mac")&&navigator.maxTouchPoints;const UE=hy.includes("jsdom/");function BE(){if(!Ui)return{platform:"",maxTouchPoints:-1};const l=navigator.userAgentData;return l!=null&&l.platform?{platform:l.platform,maxTouchPoints:navigator.maxTouchPoints}:{platform:navigator.platform??"",maxTouchPoints:navigator.maxTouchPoints??-1}}function HE(){if(!Ui)return"";const l=navigator.userAgentData;return l&&Array.isArray(l.brands)?l.brands.map(({brand:i,version:o})=>`${i}/${o}`).join(" "):navigator.userAgent}function VE(){if(!Ui)return"";const l=navigator.userAgentData;return l!=null&&l.platform?l.platform:navigator.platform??""}const Jf="data-base-ui-focusable",py="active",vy="selected",YE="input:not([type='hidden']):not([disabled]),[contenteditable]:not([contenteditable='false']),textarea:not([disabled])",fa="ArrowLeft",da="ArrowRight",hd="ArrowUp",$o="ArrowDown";function el(l){var o;let i=l.activeElement;for(;((o=i==null?void 0:i.shadowRoot)==null?void 0:o.activeElement)!=null;)i=i.shadowRoot.activeElement;return i}function ht(l,i){var r;if(!l||!i)return!1;const o=(r=i.getRootNode)==null?void 0:r.call(i);if(l.contains(i))return!0;if(o&&Yo(o)){let c=i;for(;c;){if(l===c)return!0;c=c.parentNode||c.host}}return!1}function tl(l){return"composedPath"in l?l.composedPath()[0]:l.target}function Pn(l,i){if(i==null)return!1;if("composedPath"in l)return l.composedPath().includes(i);const o=l;return o.target!=null&&i.contains(o.target)}function qE(l){return l.matches("html,body")}function gd(l){return Ft(l)&&l.matches(YE)}function Wf(l){return l?l.getAttribute("role")==="combobox"&&gd(l):!1}function Qo(l){return l?l.hasAttribute(Jf)?l:l.querySelector(`[${Jf}]`)||l:null}function zi(l,i,o=!0){return l.filter(c=>{var f;return c.parentId===i&&(!o||((f=c.context)==null?void 0:f.open))}).flatMap(c=>[c,...zi(l,c.id,o)])}function Ip(l,i){var c;let o=[],r=(c=l.find(f=>f.id===i))==null?void 0:c.parentId;for(;r;){const f=l.find(d=>d.id===r);r=f==null?void 0:f.parentId,f&&(o=o.concat(f))}return o}function Dn(l){l.preventDefault(),l.stopPropagation()}function GE(l){return"nativeEvent"in l}function yy(l){return l.pointerType===""&&l.isTrusted?!0:Ff&&l.pointerType?l.type==="click"&&l.buttons===1:l.detail===0&&!l.pointerType}function by(l){return UE?!1:!Ff&&l.width===0&&l.height===0||Ff&&l.width===1&&l.height===1&&l.pressure===0&&l.detail===0&&l.pointerType==="mouse"||l.width<1&&l.height<1&&l.pressure===0&&l.detail===0&&l.pointerType==="touch"}function Fp(l,i){return["mouse","pen"].includes(l)}function xy(l){const i=l.type;return i==="click"||i==="mousedown"||i==="keydown"||i==="keyup"}const XE=["top","right","bottom","left"],Di=Math.min,kn=Math.max,Ds=Math.round,La=Math.floor,ul=l=>({x:l,y:l}),QE={left:"right",right:"left",bottom:"top",top:"bottom"};function $f(l,i,o){return kn(l,Di(i,o))}function Ul(l,i){return typeof l=="function"?l(i):l}function Ln(l){return l.split("-")[0]}function pa(l){return l.split("-")[1]}function pd(l){return l==="x"?"y":"x"}function vd(l){return l==="y"?"height":"width"}function In(l){const i=l[0];return i==="t"||i==="b"?"y":"x"}function yd(l){return pd(In(l))}function KE(l,i,o){o===void 0&&(o=!1);const r=pa(l),c=yd(l),f=vd(c);let d=c==="x"?r===(o?"end":"start")?"right":"left":r==="start"?"bottom":"top";return i.reference[f]>i.floating[f]&&(d=ks(d)),[d,ks(d)]}function ZE(l){const i=ks(l);return[Pf(l),i,Pf(i)]}function Pf(l){return l.includes("start")?l.replace("start","end"):l.replace("end","start")}const Jp=["left","right"],Wp=["right","left"],IE=["top","bottom"],FE=["bottom","top"];function JE(l,i,o){switch(l){case"top":case"bottom":return o?i?Wp:Jp:i?Jp:Wp;case"left":case"right":return i?IE:FE;default:return[]}}function WE(l,i,o,r){const c=pa(l);let f=JE(Ln(l),o==="start",r);return c&&(f=f.map(d=>d+"-"+c),i&&(f=f.concat(f.map(Pf)))),f}function ks(l){const i=Ln(l);return QE[i]+l.slice(i.length)}function $E(l){return{top:0,right:0,bottom:0,left:0,...l}}function Sy(l){return typeof l!="number"?$E(l):{top:l,right:l,bottom:l,left:l}}function Ko(l){const{x:i,y:o,width:r,height:c}=l;return{width:r,height:c,top:o,left:i,right:i+r,bottom:o+c,x:i,y:o}}function bs(l,i,o){return Math.floor(l/i)!==o}function js(l,i){return i<0||i>=l.current.length}function Af(l,i){return gn(l,{disabledIndices:i})}function $p(l,i){return gn(l,{decrement:!0,startingIndex:l.current.length,disabledIndices:i})}function gn(l,{startingIndex:i=-1,decrement:o=!1,disabledIndices:r,amount:c=1}={}){let f=i;do f+=o?-c:c;while(f>=0&&f<=l.current.length-1&&_i(l,f,r));return f}function PE(l,{event:i,orientation:o,loopFocus:r,rtl:c,cols:f,disabledIndices:d,minIndex:p,maxIndex:v,prevIndex:m,stopEvent:y=!1}){let h=m,E;if(i.key===hd?E="up":i.key===$o&&(E="down"),E){const w=[],S=[];let R=!1,M=0;{let D=null,le=-1;l.current.forEach((ge,me)=>{if(ge==null)return;M+=1;const ue=ge.closest('[role="row"]');ue&&(R=!0),(ue!==D||le===-1)&&(D=ue,le+=1,w[le]=[]),w[le].push(me),S[me]=le})}let T=!1,B=0;if(R)for(const D of w){const le=D.length;le>B&&(B=le),le!==f&&(T=!0)}const _=T&&M<l.current.length,N=B||f,U=D=>{if(!T||m===-1)return;const le=S[m];if(le==null)return;const ge=w[le].indexOf(m),me=D==="up"?-1:1;for(let ue=le+me,F=0;F<w.length;F+=1,ue+=me){if(ue<0||ue>=w.length){if(!r||_)return;ue=ue<0?w.length-1:0}const ae=w[ue];for(let he=Math.min(ge,ae.length-1);he>=0;he-=1){const C=ae[he];if(!_i(l,C,d))return C}}},G=D=>{if(!_||m===-1)return;const le=m%N,ge=D==="up"?-N:N,me=v-v%N,ue=La(v/N)+1;for(let F=m-le+ge,ae=0;ae<ue;ae+=1,F+=ge){if(F<0||F>v){if(!r)return;F=F<0?me:0}const he=Math.min(F+N-1,v);for(let C=Math.min(F+le,he);C>=F;C-=1)if(!_i(l,C,d))return C}};y&&Dn(i);const Y=U(E)??G(E);if(Y!==void 0)h=Y;else if(m===-1)h=E==="up"?v:p;else if(h=gn(l,{startingIndex:m,amount:N,decrement:E==="up",disabledIndices:d}),r){if(E==="up"&&(m-N<p||h<0)){const D=m%N,le=v%N,ge=v-(le-D);le===D?h=v:h=le>D?ge:ge-N}E==="down"&&m+N>v&&(h=gn(l,{startingIndex:m%N-N,amount:N,disabledIndices:d}))}js(l,h)&&(h=m)}if(o==="both"){const w=La(m/f);i.key===(c?fa:da)&&(y&&Dn(i),m%f!==f-1?(h=gn(l,{startingIndex:m,disabledIndices:d}),r&&bs(h,f,w)&&(h=gn(l,{startingIndex:m-m%f-1,disabledIndices:d}))):r&&(h=gn(l,{startingIndex:m-m%f-1,disabledIndices:d})),bs(h,f,w)&&(h=m)),i.key===(c?da:fa)&&(y&&Dn(i),m%f!==0?(h=gn(l,{startingIndex:m,decrement:!0,disabledIndices:d}),r&&bs(h,f,w)&&(h=gn(l,{startingIndex:m+(f-m%f),decrement:!0,disabledIndices:d}))):r&&(h=gn(l,{startingIndex:m+(f-m%f),decrement:!0,disabledIndices:d})),bs(h,f,w)&&(h=m));const S=La(v/f)===w;js(l,h)&&(r&&S?h=i.key===(c?da:fa)?v:gn(l,{startingIndex:m-m%f-1,disabledIndices:d}):h=m)}return h}function e2(l,i,o){const r=[];let c=0;return l.forEach(({width:f,height:d},p)=>{let v=!1;for(;!v;){const m=[];for(let y=0;y<f;y+=1)for(let h=0;h<d;h+=1)m.push(c+y+h*i);c%i+f<=i&&m.every(y=>r[y]==null)?(m.forEach(y=>{r[y]=p}),v=!0):c+=1}}),[...r]}function t2(l,i,o,r,c){if(l===-1)return-1;const f=o.indexOf(l),d=i[l];switch(c){case"tl":return f;case"tr":return d?f+d.width-1:f;case"bl":return d?f+(d.height-1)*r:f;case"br":return o.lastIndexOf(l);default:return-1}}function n2(l,i){return i.flatMap((o,r)=>l.includes(o)?[r]:[])}function _i(l,i,o){if(typeof o=="function"?o(i):(o==null?void 0:o.includes(i))??!1)return!0;const c=l.current[i];return c?bd(c)?!o&&(c.hasAttribute("disabled")||c.getAttribute("aria-disabled")==="true"):!0:!1}function bd(l){return Un(l).display!=="none"}/*!
* tabbable 6.4.0
* @license MIT, https://github.com/focus-trap/tabbable/blob/master/LICENSE
*/var l2=["input:not([inert]):not([inert] *)","select:not([inert]):not([inert] *)","textarea:not([inert]):not([inert] *)","a[href]:not([inert]):not([inert] *)","button:not([inert]):not([inert] *)","[tabindex]:not(slot):not([inert]):not([inert] *)","audio[controls]:not([inert]):not([inert] *)","video[controls]:not([inert]):not([inert] *)",'[contenteditable]:not([contenteditable="false"]):not([inert]):not([inert] *)',"details>summary:first-of-type:not([inert]):not([inert] *)","details:not([inert]):not([inert] *)"],Ls=l2.join(","),Ey=typeof Element>"u",ki=Ey?function(){}:Element.prototype.matches||Element.prototype.msMatchesSelector||Element.prototype.webkitMatchesSelector,Us=!Ey&&Element.prototype.getRootNode?function(l){var i;return l==null||(i=l.getRootNode)===null||i===void 0?void 0:i.call(l)}:function(l){return l==null?void 0:l.ownerDocument},Bs=function(i,o){var r;o===void 0&&(o=!0);var c=i==null||(r=i.getAttribute)===null||r===void 0?void 0:r.call(i,"inert"),f=c===""||c==="true",d=f||o&&i&&(typeof i.closest=="function"?i.closest("[inert]"):Bs(i.parentNode));return d},a2=function(i){var o,r=i==null||(o=i.getAttribute)===null||o===void 0?void 0:o.call(i,"contenteditable");return r===""||r==="true"},wy=function(i,o,r){if(Bs(i))return[];var c=Array.prototype.slice.apply(i.querySelectorAll(Ls));return o&&ki.call(i,Ls)&&c.unshift(i),c=c.filter(r),c},Hs=function(i,o,r){for(var c=[],f=Array.from(i);f.length;){var d=f.shift();if(!Bs(d,!1))if(d.tagName==="SLOT"){var p=d.assignedElements(),v=p.length?p:d.children,m=Hs(v,!0,r);r.flatten?c.push.apply(c,m):c.push({scopeParent:d,candidates:m})}else{var y=ki.call(d,Ls);y&&r.filter(d)&&(o||!i.includes(d))&&c.push(d);var h=d.shadowRoot||typeof r.getShadowRoot=="function"&&r.getShadowRoot(d),E=!Bs(h,!1)&&(!r.shadowRootFilter||r.shadowRootFilter(d));if(h&&E){var w=Hs(h===!0?d.children:h.children,!0,r);r.flatten?c.push.apply(c,w):c.push({scopeParent:d,candidates:w})}else f.unshift.apply(f,d.children)}}return c},Ty=function(i){return!isNaN(parseInt(i.getAttribute("tabindex"),10))},Ry=function(i){if(!i)throw new Error("No node provided");return i.tabIndex<0&&(/^(AUDIO|VIDEO|DETAILS)$/.test(i.tagName)||a2(i))&&!Ty(i)?0:i.tabIndex},i2=function(i,o){var r=Ry(i);return r<0&&o&&!Ty(i)?0:r},o2=function(i,o){return i.tabIndex===o.tabIndex?i.documentOrder-o.documentOrder:i.tabIndex-o.tabIndex},Ay=function(i){return i.tagName==="INPUT"},r2=function(i){return Ay(i)&&i.type==="hidden"},s2=function(i){var o=i.tagName==="DETAILS"&&Array.prototype.slice.apply(i.children).some(function(r){return r.tagName==="SUMMARY"});return o},u2=function(i,o){for(var r=0;r<i.length;r++)if(i[r].checked&&i[r].form===o)return i[r]},c2=function(i){if(!i.name)return!0;var o=i.form||Us(i),r=function(p){return o.querySelectorAll('input[type="radio"][name="'+p+'"]')},c;if(typeof window<"u"&&typeof window.CSS<"u"&&typeof window.CSS.escape=="function")c=r(window.CSS.escape(i.name));else try{c=r(i.name)}catch(d){return console.error("Looks like you have a radio button with a name attribute containing invalid CSS selector characters and need the CSS.escape polyfill: %s",d.message),!1}var f=u2(c,i.form);return!f||f===i},f2=function(i){return Ay(i)&&i.type==="radio"},d2=function(i){return f2(i)&&!c2(i)},m2=function(i){var o,r=i&&Us(i),c=(o=r)===null||o===void 0?void 0:o.host,f=!1;if(r&&r!==i){var d,p,v;for(f=!!((d=c)!==null&&d!==void 0&&(p=d.ownerDocument)!==null&&p!==void 0&&p.contains(c)||i!=null&&(v=i.ownerDocument)!==null&&v!==void 0&&v.contains(i));!f&&c;){var m,y,h;r=Us(c),c=(m=r)===null||m===void 0?void 0:m.host,f=!!((y=c)!==null&&y!==void 0&&(h=y.ownerDocument)!==null&&h!==void 0&&h.contains(c))}}return f},Pp=function(i){var o=i.getBoundingClientRect(),r=o.width,c=o.height;return r===0&&c===0},h2=function(i,o){var r=o.displayCheck,c=o.getShadowRoot;if(r==="full-native"&&"checkVisibility"in i){var f=i.checkVisibility({checkOpacity:!1,opacityProperty:!1,contentVisibilityAuto:!0,visibilityProperty:!0,checkVisibilityCSS:!0});return!f}if(getComputedStyle(i).visibility==="hidden")return!0;var d=ki.call(i,"details>summary:first-of-type"),p=d?i.parentElement:i;if(ki.call(p,"details:not([open]) *"))return!0;if(!r||r==="full"||r==="full-native"||r==="legacy-full"){if(typeof c=="function"){for(var v=i;i;){var m=i.parentElement,y=Us(i);if(m&&!m.shadowRoot&&c(m)===!0)return Pp(i);i.assignedSlot?i=i.assignedSlot:!m&&y!==i.ownerDocument?i=y.host:i=m}i=v}if(m2(i))return!i.getClientRects().length;if(r!=="legacy-full")return!0}else if(r==="non-zero-area")return Pp(i);return!1},g2=function(i){if(/^(INPUT|BUTTON|SELECT|TEXTAREA)$/.test(i.tagName))for(var o=i.parentElement;o;){if(o.tagName==="FIELDSET"&&o.disabled){for(var r=0;r<o.children.length;r++){var c=o.children.item(r);if(c.tagName==="LEGEND")return ki.call(o,"fieldset[disabled] *")?!0:!c.contains(i)}return!0}o=o.parentElement}return!1},ed=function(i,o){return!(o.disabled||r2(o)||h2(o,i)||s2(o)||g2(o))},td=function(i,o){return!(d2(o)||Ry(o)<0||!ed(i,o))},p2=function(i){var o=parseInt(i.getAttribute("tabindex"),10);return!!(isNaN(o)||o>=0)},Cy=function(i){var o=[],r=[];return i.forEach(function(c,f){var d=!!c.scopeParent,p=d?c.scopeParent:c,v=i2(p,d),m=d?Cy(c.candidates):p;v===0?d?o.push.apply(o,m):o.push(p):r.push({documentOrder:f,tabIndex:v,item:c,isScope:d,content:m})}),r.sort(o2).reduce(function(c,f){return f.isScope?c.push.apply(c,f.content):c.push(f.content),c},[]).concat(o)},$s=function(i,o){o=o||{};var r;return o.getShadowRoot?r=Hs([i],o.includeContainer,{filter:td.bind(null,o),flatten:!1,getShadowRoot:o.getShadowRoot,shadowRootFilter:p2}):r=wy(i,o.includeContainer,td.bind(null,o)),Cy(r)},v2=function(i,o){o=o||{};var r;return o.getShadowRoot?r=Hs([i],o.includeContainer,{filter:ed.bind(null,o),flatten:!0,getShadowRoot:o.getShadowRoot}):r=wy(i,o.includeContainer,ed.bind(null,o)),r},My=function(i,o){if(o=o||{},!i)throw new Error("No node provided");return ki.call(i,Ls)===!1?!1:td(o,i)};const Po=()=>({getShadowRoot:!0,displayCheck:typeof ResizeObserver=="function"&&ResizeObserver.toString().includes("[native code]")?"full":"none"});function Oy(l,i){const o=$s(l,Po()),r=o.length;if(r===0)return;const c=el(kt(l)),f=o.indexOf(c),d=f===-1?i===1?0:r-1:f+i;return o[d]}function zy(l){return Oy(kt(l).body,1)||l}function _y(l){return Oy(kt(l).body,-1)||l}function Bo(l,i){const o=i||l.currentTarget,r=l.relatedTarget;return!r||!ht(o,r)}function y2(l){$s(l,Po()).forEach(o=>{o.dataset.tabindex=o.getAttribute("tabindex")||"",o.setAttribute("tabindex","-1")})}function ev(l){l.querySelectorAll("[data-tabindex]").forEach(o=>{const r=o.dataset.tabindex;delete o.dataset.tabindex,r?o.setAttribute("tabindex",r):o.removeAttribute("tabindex")})}const xs=null;class b2{constructor(){Ut(this,"callbacks",[]);Ut(this,"callbacksCount",0);Ut(this,"nextId",1);Ut(this,"startId",1);Ut(this,"isScheduled",!1);Ut(this,"tick",i=>{var c;this.isScheduled=!1;const o=this.callbacks,r=this.callbacksCount;if(this.callbacks=[],this.callbacksCount=0,this.startId=this.nextId,r>0)for(let f=0;f<o.length;f+=1)(c=o[f])==null||c.call(o,i)})}request(i){const o=this.nextId;return this.nextId+=1,this.callbacks.push(i),this.callbacksCount+=1,(!this.isScheduled||!1)&&(requestAnimationFrame(this.tick),this.isScheduled=!0),o}cancel(i){const o=i-this.startId;o<0||o>=this.callbacks.length||(this.callbacks[o]=null,this.callbacksCount-=1)}}const Ss=new b2;class sl{constructor(){Ut(this,"currentId",xs);Ut(this,"cancel",()=>{this.currentId!==xs&&(Ss.cancel(this.currentId),this.currentId=xs)});Ut(this,"disposeEffect",()=>this.cancel)}static create(){return new sl}static request(i){return Ss.request(i)}static cancel(i){return Ss.cancel(i)}request(i){this.cancel(),this.currentId=Ss.request(()=>{this.currentId=xs,i()})}}function Ps(){const l=Rn(sl.create).current;return dy(l.disposeEffect),l}function kl(l){return l==null?l:"current"in l?l.current:l}let Zo=(function(l){return l.startingStyle="data-starting-style",l.endingStyle="data-ending-style",l})({});const x2={[Zo.startingStyle]:""},S2={[Zo.endingStyle]:""},er={transitionStatus(l){return l==="starting"?x2:l==="ending"?S2:null}};function E2(l,i=!1,o=!0){const r=Ps();return Ze((c,f=null)=>{r.cancel();function d(){Ya.flushSync(c)}const p=kl(l);if(p==null)return;const v=p;if(typeof v.getAnimations!="function"||globalThis.BASE_UI_ANIMATIONS_DISABLED)c();else{let h=function(){const w=Zo.startingStyle;if(!v.hasAttribute(w)){r.request(E);return}const S=new MutationObserver(()=>{v.hasAttribute(w)||(S.disconnect(),E())});S.observe(v,{attributes:!0,attributeFilter:[w]}),f==null||f.addEventListener("abort",()=>S.disconnect(),{once:!0})},E=function(){Promise.all(v.getAnimations().map(w=>w.finished)).then(()=>{f!=null&&f.aborted||d()}).catch(()=>{const w=v.getAnimations();if(o){if(f!=null&&f.aborted)return;d()}else w.length>0&&w.some(S=>S.pending||S.playState!=="finished")&&E()})};var m=h,y=E;if(i){h();return}r.request(E)}})}function Ga(l){const{enabled:i=!0,open:o,ref:r,onComplete:c}=l,f=Ze(c),d=E2(r,o,!1);x.useEffect(()=>{if(!i)return;const p=new AbortController;return d(f,p.signal),()=>{p.abort()}},[i,o,f,d])}function tr(l,i=!1,o=!1){const[r,c]=x.useState(l&&i?"idle":void 0),[f,d]=x.useState(l);return l&&!f&&(d(!0),c("starting")),!l&&f&&r!=="ending"&&!o&&c("ending"),!l&&!f&&r==="ending"&&c(void 0),Oe(()=>{if(!l&&f&&r!=="ending"&&o){const p=sl.request(()=>{c("ending")});return()=>{sl.cancel(p)}}},[l,f,r,o]),Oe(()=>{if(!l||i)return;const p=sl.request(()=>{c(void 0)});return()=>{sl.cancel(p)}},[i,l]),Oe(()=>{if(!l||!i)return;l&&f&&r!=="idle"&&c("starting");const p=sl.request(()=>{c("idle")});return()=>{sl.cancel(p)}},[i,l,f,c,r]),x.useMemo(()=>({mounted:f,setMounted:d,transitionStatus:r}),[f,r])}function Vs({controlled:l,default:i,name:o,state:r="value"}){const{current:c}=x.useRef(l!==void 0),[f,d]=x.useState(i),p=c?l:f,v=x.useCallback(m=>{c||d(m)},[]);return[p,v]}function xd(l={}){const{id:i,implicit:o=!1,controlRef:r}=l,{controlId:c,registerControlId:f}=Ws(),d=Xo(i),p=o?c:void 0,v=Rn(()=>Symbol("labelable-control")),m=x.useRef(!1),y=x.useRef(i!=null),h=Ze(()=>{!m.current||f===dn||(m.current=!1,f(v.current,void 0))});return Oe(()=>{if(f===dn)return;let E;if(o){const w=r==null?void 0:r.current;Bt(w)&&w.closest("label")!=null?E=i??null:E=p??d}else if(i!=null)y.current=!0,E=i;else if(y.current)E=d;else{h();return}if(E===void 0){h();return}m.current=!0,f(v.current,E)},[i,r,p,f,o,d,v,h]),x.useEffect(()=>h,[h]),c??d}function Sd(l){const{enabled:i=!0,value:o,id:r,name:c,controlRef:f,commit:d}=l,{formRef:p}=dd(),{invalid:v,markedDirtyRef:m,validityData:y,setValidityData:h}=Jo(),E=Ze(l.getValue);Oe(()=>{if(!i)return;let w=o;w===void 0&&(w=E()),y.initialValue===null&&w!==null&&h(S=>({...S,initialValue:w}))},[i,h,o,y.initialValue,E]),Oe(()=>{!i||!r||p.current.fields.set(r,{getValue:E,name:c,controlRef:f,validityData:jE(y,v),validate(w=!0){let S=o;S===void 0&&(S=E()),m.current=!0,w?Ya.flushSync(()=>d(S)):d(S)}})},[d,f,i,p,E,r,v,m,c,y,o]),Oe(()=>{const w=p.current.fields;return()=>{r&&w.delete(r)}},[p,r])}const nr="none",nd="trigger-press",w2="trigger-hover",Ed="outside-press",Cf="item-press",T2="close-press",eu="focus-out",R2="escape-key",Mf="list-navigation",A2="cancel-open",C2="imperative-action",M2="window-resize";function zt(l,i,o,r){let c=!1,f=!1;const d=an;return{reason:l,event:i??new Event("base-ui"),cancel(){c=!0},allowPropagation(){f=!0},get isCanceled(){return c},get isPropagationAllowed(){return f},trigger:o,...d}}const O2=x.forwardRef(function(i,o){const{render:r,className:c,id:f,name:d,value:p,disabled:v=!1,onValueChange:m,defaultValue:y,autoFocus:h=!1,...E}=i,{state:w,name:S,disabled:R,setTouched:M,setDirty:T,validityData:B,setFocused:_,setFilled:N,validationMode:U,validation:G}=Jo(),Y=R||v,D=S??d,le={...w,disabled:Y},{labelId:ge}=Ws(),me=xd({id:f});Oe(()=>{var X;const k=p!=null;(X=G.inputRef.current)!=null&&X.value||k&&p!==""?N(!0):k&&p===""&&N(!1)},[G.inputRef,N,p]);const ue=x.useRef(null);Oe(()=>{h&&ue.current===el(kt(ue.current))&&_(!0)},[h,_]);const[F]=Vs({controlled:p,default:y,name:"FieldControl",state:"value"}),ae=p!==void 0,he=ae?F:void 0;return Sd({id:me,name:D,commit:G.commit,value:he,getValue:()=>{var k;return(k=G.inputRef.current)==null?void 0:k.value},controlRef:G.inputRef}),Jt("input",i,{ref:[o,ue],state:le,props:[{id:me,disabled:Y,name:D,ref:G.inputRef,"aria-labelledby":ge,autoFocus:h,...ae?{value:he}:{defaultValue:y},onChange(k){const X=k.currentTarget.value;m==null||m(X,zt(nr,k.nativeEvent)),T(X!==B.initialValue),N(X!=="")},onFocus(){_(!0)},onBlur(k){M(!0),_(!1),U==="onBlur"&&G.commit(k.currentTarget.value)},onKeyDown(k){k.currentTarget.tagName==="INPUT"&&k.key==="Enter"&&(M(!0),G.commit(k.currentTarget.value))}},G.getInputValidationProps(),E],stateAttributesMapping:Js})}),z2=x.createContext({disabled:!1});function _2(){return x.useContext(z2)}const N2=x.createContext(void 0);function D2(l=!0){const i=x.useContext(N2);if(i===void 0&&!l)throw new Error(Fn(3));return i}const k2=x.forwardRef(function(i,o){return L.jsx(O2,{ref:o,...i})});function Ny({className:l,type:i,...o}){return L.jsx(k2,{type:i,"data-slot":"input",className:mn("h-8 w-full min-w-0 rounded-lg border border-input bg-transparent px-2.5 py-1 text-base transition-colors outline-none file:inline-flex file:h-6 file:border-0 file:bg-transparent file:text-sm file:font-medium file:text-foreground placeholder:text-muted-foreground focus-visible:border-ring focus-visible:ring-3 focus-visible:ring-ring/50 disabled:pointer-events-none disabled:cursor-not-allowed disabled:bg-input/50 disabled:opacity-50 aria-invalid:border-destructive aria-invalid:ring-3 aria-invalid:ring-destructive/20 md:text-sm dark:bg-input/30 dark:disabled:bg-input/80 dark:aria-invalid:border-destructive/50 dark:aria-invalid:ring-destructive/40",l),...o})}function j2(l){let i=Math.floor(l);if(i<60)return`${i}s`;let o=Math.floor(i/60);if(i=i%60,o<60)return`${o}m ${i}s`;let r=Math.floor(o/60);if(o=o%60,r<24)return`${r}h ${o}m`;const c=Math.floor(r/24);return r=r%24,`${c}d ${r}h`}function Of(l){return l.toLocaleString()}function Es({value:l,label:i}){return L.jsxs("div",{className:"text-center",children:[L.jsx("div",{className:"text-2xl font-bold text-primary",children:l}),L.jsx("div",{className:"text-xs text-muted-foreground uppercase",children:i})]})}function L2({stats:l}){if(!l)return null;const i=l.total_seconds/60,o=i>0?Math.round(l.total_words/i):0;return L.jsxs("div",{className:"flex gap-8 mb-6",children:[L.jsx(Es,{value:Of(l.total_words),label:"Words"}),L.jsx(Es,{value:j2(l.total_seconds),label:"Recording"}),L.jsx(Es,{value:Of(o),label:"WPM"}),L.jsx(Es,{value:Of(l.total_transcriptions),label:"Sessions"})]})}function U2(l){return Jt(l.defaultTagName??"div",l,l)}const B2=Wv("group/badge inline-flex h-5 w-fit shrink-0 items-center justify-center gap-1 overflow-hidden rounded-4xl border border-transparent px-2 py-0.5 text-xs font-medium whitespace-nowrap transition-all focus-visible:border-ring focus-visible:ring-[3px] focus-visible:ring-ring/50 has-data-[icon=inline-end]:pr-1.5 has-data-[icon=inline-start]:pl-1.5 aria-invalid:border-destructive aria-invalid:ring-destructive/20 dark:aria-invalid:ring-destructive/40 [&>svg]:pointer-events-none [&>svg]:size-3!",{variants:{variant:{default:"bg-primary text-primary-foreground [a]:hover:bg-primary/80",secondary:"bg-secondary text-secondary-foreground [a]:hover:bg-secondary/80",destructive:"bg-destructive/10 text-destructive focus-visible:ring-destructive/20 dark:bg-destructive/20 dark:focus-visible:ring-destructive/40 [a]:hover:bg-destructive/20",outline:"border-border text-foreground [a]:hover:bg-muted [a]:hover:text-muted-foreground",ghost:"hover:bg-muted hover:text-muted-foreground dark:hover:bg-muted/50",link:"text-primary underline-offset-4 hover:underline"}},defaultVariants:{variant:"default"}});function H2({className:l,variant:i="default",render:o,...r}){return U2({defaultTagName:"span",props:Li({className:mn(B2({variant:i}),l)},r),render:o,state:{slot:"badge",variant:i}})}async function V2(){return(await fetch("/api/stats")).json()}async function Y2(l=50){return(await(await fetch(`/api/transcriptions?limit=${l}`)).json()).transcriptions}async function q2(){return(await fetch("/api/config")).json()}async function G2(l){return(await(await fetch("/api/config",{method:"PUT",headers:{"Content-Type":"application/json"},body:JSON.stringify(l)})).json()).ok===!0}async function X2(l){return(await(await fetch(`/api/transcriptions/${l}`,{method:"DELETE"})).json()).ok===!0}function Q2(l){const i=l.split("/");return i.length>=3&&i[1]==="Users"?"~/"+i.slice(3).join("/"):l}function K2(l){if(l<1)return"<1s";const i=Math.floor(l);if(i<60)return`${i}s`;const o=Math.floor(i/60),r=i%60;return r>0?`${o}m ${r}s`:`${o}m`}function Z2({t:l,onDelete:i}){const[o,r]=x.useState(!1),c=l.fixed_text||l.raw_text,f=l.fixed_text!=null&&l.fixed_text!==l.raw_text,d=l.fixed_text!=null;function p(){navigator.clipboard.writeText(c).then(()=>Oi("Copied to clipboard"),()=>Oi.error("Failed to copy"))}function v(){X2(l.id).then(m=>{m?(Oi("Deleted"),i==null||i()):Oi.error("Failed to delete")})}return L.jsxs("div",{className:"p-4 rounded-lg bg-card",children:[L.jsxs("div",{className:"flex justify-between items-start",children:[L.jsxs("div",{className:"flex gap-2 text-xs text-muted-foreground flex-wrap items-center mb-1",children:[L.jsx("span",{children:l.timestamp.slice(0,16).replace("T"," ")}),l.app_name&&L.jsx(H2,{variant:"secondary",children:l.app_name}),l.duration_seconds!=null&&L.jsx("span",{className:"tabular-nums",children:K2(l.duration_seconds)}),l.cwd&&L.jsx("span",{children:L.jsx("code",{className:"text-xs",children:Q2(l.cwd)})})]}),L.jsxs("div",{className:"flex gap-1",children:[L.jsx("button",{onClick:p,className:"text-muted-foreground hover:text-foreground p-1 rounded transition-colors",title:"Copy to clipboard",children:L.jsxs("svg",{className:"w-4 h-4",fill:"none",stroke:"currentColor",viewBox:"0 0 24 24",children:[L.jsx("rect",{x:"9",y:"9",width:"13",height:"13",rx:"2",ry:"2",strokeWidth:"2"}),L.jsx("path",{d:"M5 15H4a2 2 0 01-2-2V4a2 2 0 012-2h9a2 2 0 012 2v1",strokeWidth:"2"})]})}),L.jsx("button",{onClick:v,className:"text-muted-foreground hover:text-destructive p-1 rounded transition-colors",title:"Delete",children:L.jsx("svg",{className:"w-4 h-4",fill:"none",stroke:"currentColor",viewBox:"0 0 24 24",strokeWidth:"2",strokeLinecap:"round",children:L.jsx("path",{d:"M3 6h18M8 6V4a2 2 0 012-2h4a2 2 0 012 2v2M19 6l-1 14a2 2 0 01-2 2H8a2 2 0 01-2-2L5 6"})})})]})]}),L.jsx("div",{className:"break-words",children:c}),L.jsx("div",{className:"mt-1",children:f?L.jsxs(L.Fragment,{children:[L.jsxs("button",{onClick:()=>r(!o),className:"text-xs text-muted-foreground hover:text-foreground cursor-pointer",children:[o?"Hide":"Show"," original"]}),o&&L.jsx("div",{className:"text-sm text-muted-foreground mt-1",children:l.raw_text})]}):d?L.jsx("span",{className:"text-xs text-muted-foreground/50 italic",children:"No corrections needed"}):L.jsx("span",{className:"text-xs text-muted-foreground/50 italic",children:"Not post-processed"})}),l.audio_filename&&L.jsx("audio",{controls:!0,preload:"none",className:"mt-2 h-8 w-full",src:`/audio/${l.audio_filename}`})]})}function I2({transcriptions:l,onDelete:i}){return l.length===0?L.jsxs("div",{className:"text-center py-12 text-muted-foreground",children:[L.jsx("div",{className:"text-4xl mb-4",children:"🎙️"}),L.jsx("p",{children:"No transcriptions yet."}),L.jsx("p",{className:"text-sm",children:"Press your hotkey to start dictating."})]}):L.jsx("div",{className:"flex flex-col gap-2",children:l.map(o=>L.jsx(Z2,{t:o,onDelete:i},o.id))})}const Dy=x.createContext(void 0);function Bi(l){const i=x.useContext(Dy);if(l===!1&&i===void 0)throw new Error(Fn(27));return i}let Ua=(function(l){return l.open="data-open",l.closed="data-closed",l[l.startingStyle=Zo.startingStyle]="startingStyle",l[l.endingStyle=Zo.endingStyle]="endingStyle",l.anchorHidden="data-anchor-hidden",l.side="data-side",l.align="data-align",l})({}),ld=(function(l){return l.popupOpen="data-popup-open",l.pressed="data-pressed",l})({});const F2={[ld.popupOpen]:""},J2={[ld.popupOpen]:"",[ld.pressed]:""},W2={[Ua.open]:""},$2={[Ua.closed]:""},P2={[Ua.anchorHidden]:""},ew={open(l){return l?F2:null}},tw={open(l){return l?J2:null}},tu={open(l){return l?W2:$2},anchorHidden(l){return l?P2:null}},nw={...tu,...er},lw=x.forwardRef(function(i,o){const{render:r,className:c,forceRender:f=!1,...d}=i,{store:p}=Bi(),v=p.useState("open"),m=p.useState("nested"),y=p.useState("mounted"),h=p.useState("transitionStatus");return Jt("div",i,{state:{open:v,transitionStatus:h},ref:[p.context.backdropRef,o],stateAttributesMapping:nw,props:[{role:"presentation",hidden:!y,style:{userSelect:"none",WebkitUserSelect:"none"}},d],enabled:f||!m})}),ky=x.forwardRef(function(i,o){const{render:r,className:c,disabled:f=!1,nativeButton:d=!0,...p}=i,{store:v}=Bi(),m=v.useState("open");function y(S){m&&v.setOpen(!1,zt(T2,S.nativeEvent))}const{getButtonProps:h,buttonRef:E}=Fo({disabled:f,native:d});return Jt("button",i,{state:{disabled:f},ref:[o,E],props:[{onClick:y},p,h]})});function Tn(l){const i=Rn(aw,l).current;return i.next=l,Oe(i.effect),i}function aw(l){const i={current:l,next:l,effect:()=>{i.current=i.next}};return i}const jy={clipPath:"inset(50%)",overflow:"hidden",whiteSpace:"nowrap",border:0,padding:0,width:1,height:1,margin:-1},wd={...jy,position:"fixed",top:0,left:0},Ly={...jy,position:"absolute"},Ys=x.forwardRef(function(i,o){const[r,c]=x.useState();Oe(()=>{LE&&c("button")},[]);const f={tabIndex:0,role:r};return L.jsx("span",{...i,ref:o,style:wd,"aria-hidden":r?void 0:!0,...f,"data-base-ui-focus-guard":""})});function qs(l){return`data-base-ui-${l}`}let tv=0;function Ho(l,i={}){const{preventScroll:o=!1,cancelPrevious:r=!0,sync:c=!1}=i;r&&cancelAnimationFrame(tv);const f=()=>l==null?void 0:l.focus({preventScroll:o});c?f():tv=requestAnimationFrame(f)}const zf={inert:new WeakMap,"aria-hidden":new WeakMap},nv="data-base-ui-inert",ad={inert:new WeakSet,"aria-hidden":new WeakSet};let Do=new WeakMap,_f=0;function iw(l){return ad[l]}function Uy(l){return l?Yo(l)?l.host:Uy(l.parentNode):null}const Nf=(l,i)=>i.map(o=>{if(l.contains(o))return o;const r=Uy(o);return l.contains(r)?r:null}).filter(o=>o!=null),lv=l=>{const i=new Set;return l.forEach(o=>{let r=o;for(;r&&!i.has(r);)i.add(r),r=r.parentNode}),i},av=(l,i,o)=>{const r=[],c=f=>{!f||o.has(f)||Array.from(f.children).forEach(d=>{ma(d)!=="script"&&(i.has(d)?c(d):r.push(d))})};return c(l),r};function ow(l,i,o,r,{mark:c=!0,markerIgnoreElements:f=[]}){const d=r?"inert":o?"aria-hidden":null;let p=null,v=null;const m=Nf(i,l),y=c?Nf(i,f):[],h=new Set(y),E=c?av(i,lv(m),new Set(m)).filter(R=>!h.has(R)):[],w=[],S=[];if(d){const R=zf[d],M=iw(d);v=M,p=R;const T=Nf(i,Array.from(i.querySelectorAll("[aria-live]"))),B=m.concat(T);av(i,lv(B),new Set(B)).forEach(N=>{const U=N.getAttribute(d),G=U!==null&&U!=="false",Y=(R.get(N)||0)+1;R.set(N,Y),w.push(N),Y===1&&G&&M.add(N),G||N.setAttribute(d,d==="inert"?"":"true")})}return c&&E.forEach(R=>{const M=(Do.get(R)||0)+1;Do.set(R,M),S.push(R),M===1&&R.setAttribute(nv,"")}),_f+=1,()=>{p&&w.forEach(R=>{const T=(p.get(R)||0)-1;p.set(R,T),T||(!(v!=null&&v.has(R))&&d&&R.removeAttribute(d),v==null||v.delete(R))}),c&&S.forEach(R=>{const M=(Do.get(R)||0)-1;Do.set(R,M),M||R.removeAttribute(nv)}),_f-=1,_f||(zf.inert=new WeakMap,zf["aria-hidden"]=new WeakMap,ad.inert=new WeakSet,ad["aria-hidden"]=new WeakSet,Do=new WeakMap)}}function iv(l,i={}){const{ariaHidden:o=!1,inert:r=!1,mark:c=!0,markerIgnoreElements:f=[]}=i,d=kt(l[0]).body;return ow(l,d,o,r,{mark:c,markerIgnoreElements:f})}const By=x.createContext(null),Hy=()=>x.useContext(By),rw=qs("portal");function sw(l={}){const{ref:i,container:o,componentProps:r=an,elementProps:c}=l,f=Wo(),d=Hy(),p=d==null?void 0:d.portalNode,[v,m]=x.useState(null),[y,h]=x.useState(null),E=Ze(M=>{M!==null&&h(M)}),w=x.useRef(null);Oe(()=>{if(o===null){w.current&&(w.current=null,h(null),m(null));return}if(f==null)return;const M=(o&&(rd(o)?o:o.current))??p??document.body;if(M==null){w.current&&(w.current=null,h(null),m(null));return}w.current!==M&&(w.current=M,h(null),m(M))},[o,p,f]);const S=Jt("div",r,{ref:[i,E],props:[{id:f,[rw]:""},c]});return{portalNode:y,portalSubtree:v&&S?Ya.createPortal(S,v):null}}const Vy=x.forwardRef(function(i,o){const{children:r,container:c,className:f,render:d,renderGuards:p,...v}=i,{portalNode:m,portalSubtree:y}=sw({container:c,ref:o,componentProps:i,elementProps:v}),h=x.useRef(null),E=x.useRef(null),w=x.useRef(null),S=x.useRef(null),[R,M]=x.useState(null),T=R==null?void 0:R.modal,B=R==null?void 0:R.open,_=typeof p=="boolean"?p:!!R&&!R.modal&&R.open&&!!m;x.useEffect(()=>{if(!m||T)return;function U(G){m&&G.relatedTarget&&Bo(G)&&(G.type==="focusin"?ev:y2)(m)}return m.addEventListener("focusin",U,!0),m.addEventListener("focusout",U,!0),()=>{m.removeEventListener("focusin",U,!0),m.removeEventListener("focusout",U,!0)}},[m,T]),x.useEffect(()=>{!m||B||ev(m)},[B,m]);const N=x.useMemo(()=>({beforeOutsideRef:h,afterOutsideRef:E,beforeInsideRef:w,afterInsideRef:S,portalNode:m,setFocusManagerState:M}),[m]);return L.jsxs(x.Fragment,{children:[y,L.jsxs(By.Provider,{value:N,children:[_&&m&&L.jsx(Ys,{"data-type":"outside",ref:h,onFocus:U=>{var G;if(Bo(U,m))(G=w.current)==null||G.focus();else{const Y=R?R.domReference:null,D=_y(Y);D==null||D.focus()}}}),_&&m&&L.jsx("span",{"aria-owns":m.id,style:NS}),m&&Ya.createPortal(r,m),_&&m&&L.jsx(Ys,{"data-type":"outside",ref:E,onFocus:U=>{var G;if(Bo(U,m))(G=S.current)==null||G.focus();else{const Y=R?R.domReference:null,D=zy(Y);D==null||D.focus(),R!=null&&R.closeOnFocusOut&&(R==null||R.onOpenChange(!1,zt(eu,U.nativeEvent)))}}})]})]})});function uw(){const l=new Map;return{emit(i,o){var r;(r=l.get(i))==null||r.forEach(c=>c(o))},on(i,o){l.has(i)||l.set(i,new Set),l.get(i).add(o)},off(i,o){var r;(r=l.get(i))==null||r.delete(o)}}}const cw=x.createContext(null),fw=x.createContext(null),nu=()=>{var l;return((l=x.useContext(cw))==null?void 0:l.id)||null},lu=l=>{const i=x.useContext(fw);return l??i};function dw(l,i){const o=en(l.target);return l instanceof o.KeyboardEvent?"keyboard":l instanceof o.FocusEvent?i||"keyboard":"pointerType"in l?l.pointerType||"keyboard":"touches"in l?"touch":l instanceof o.MouseEvent?i||(l.detail===0?"keyboard":"mouse"):""}const ov=20;let ca=[];function Td(){ca=ca.filter(l=>{var i;return(i=l.deref())==null?void 0:i.isConnected})}function mw(l){Td(),l&&ma(l)!=="body"&&(ca.push(new WeakRef(l)),ca.length>ov&&(ca=ca.slice(-ov)))}function Df(){var l;return Td(),(l=ca[ca.length-1])==null?void 0:l.deref()}function hw(l){if(!l)return null;const i=Po();return My(l,i)?l:$s(l,i)[0]||l}function gw(l){return!l||!l.isConnected?!1:typeof l.checkVisibility=="function"?l.checkVisibility():bd(l)}function rv(l,i){var d;if(!i.current.includes("floating")&&!((d=l.getAttribute("role"))!=null&&d.includes("dialog")))return;const o=Po(),c=v2(l,o).filter(p=>{const v=p.getAttribute("data-tabindex")||"";return My(p,o)||p.hasAttribute("data-tabindex")&&!v.startsWith("-")}),f=l.getAttribute("tabindex");i.current.includes("floating")||c.length===0?f!=="0"&&l.setAttribute("tabindex","0"):(f!=="-1"||l.hasAttribute("data-tabindex")&&l.getAttribute("data-tabindex")!=="-1")&&(l.setAttribute("tabindex","-1"),l.setAttribute("data-tabindex","-1"))}function Yy(l){const{context:i,children:o,disabled:r=!1,initialFocus:c=!0,returnFocus:f=!0,restoreFocus:d=!1,modal:p=!0,closeOnFocusOut:v=!0,openInteractionType:m="",nextFocusableElement:y,previousFocusableElement:h,beforeContentFocusGuardRef:E,externalTree:w,getInsideElements:S}=l,R="rootStore"in i?i.rootStore:i,M=R.useState("open"),T=R.useState("domReferenceElement"),B=R.useState("floatingElement"),{events:_,dataRef:N}=R.context,U=Ze(()=>{var xe;return(xe=N.current.floatingContext)==null?void 0:xe.nodeId}),G=c===!1,Y=Wf(T)&&G,D=x.useRef(["content"]),le=Tn(c),ge=Tn(f),me=Tn(m),ue=lu(w),F=Hy(),ae=x.useRef(!1),he=x.useRef(!1),C=x.useRef(!1),k=x.useRef(-1),X=x.useRef(""),I=x.useRef(""),q=x.useRef(null),A=x.useRef(null),O=Ba(q,E,F==null?void 0:F.beforeInsideRef),W=Ba(A,F==null?void 0:F.afterInsideRef),Q=jn(),te=jn(),P=Ps(),de=F!=null,ce=Qo(B),be=Ze((xe=ce)=>xe?$s(xe,Po()):[]),De=Ze(()=>(S==null?void 0:S().filter(xe=>xe!=null))??[]),we=Ze(xe=>{const pe=be(xe);return D.current.map(()=>pe).filter(Boolean).flat()});x.useEffect(()=>{if(r||!p)return;function xe(Re){Re.key==="Tab"&&ht(ce,el(kt(ce)))&&be().length===0&&!Y&&Dn(Re)}const pe=kt(ce);return pe.addEventListener("keydown",xe),()=>{pe.removeEventListener("keydown",xe)}},[r,T,ce,p,D,Y,be,we]),x.useEffect(()=>{if(r||!M)return;const xe=kt(ce);function pe(){C.current=!1}function Re(ve){const Ee=tl(ve),fe=De(),oe=ht(B,Ee)||ht(T,Ee)||ht(F==null?void 0:F.portalNode,Ee)||fe.some(H=>H===Ee||ht(H,Ee));C.current=!oe,I.current=ve.pointerType||"keyboard",Ee!=null&&Ee.closest(`[${zS}]`)&&(he.current=!0)}function Ge(){I.current="keyboard"}return xe.addEventListener("pointerdown",Re,!0),xe.addEventListener("pointerup",pe,!0),xe.addEventListener("pointercancel",pe,!0),xe.addEventListener("keydown",Ge,!0),()=>{xe.removeEventListener("pointerdown",Re,!0),xe.removeEventListener("pointerup",pe,!0),xe.removeEventListener("pointercancel",pe,!0),xe.removeEventListener("keydown",Ge,!0)}},[r,B,T,ce,M,F,De]),x.useEffect(()=>{if(r||!v)return;const xe=kt(ce);function pe(){he.current=!0,te.start(0,()=>{he.current=!1})}function Re(oe){const H=tl(oe),re=be().indexOf(H);re!==-1&&(k.current=re)}function Ge(oe){const H=oe.relatedTarget,$=oe.currentTarget,re=tl(oe);queueMicrotask(()=>{const Se=U(),Le=R.context.triggerElements,Xe=De(),ut=(H==null?void 0:H.hasAttribute(qs("focus-guard")))&&[q.current,A.current,F==null?void 0:F.beforeInsideRef.current,F==null?void 0:F.afterInsideRef.current,F==null?void 0:F.beforeOutsideRef.current,F==null?void 0:F.afterOutsideRef.current,kl(h),kl(y)].includes(H),Je=!(ht(T,H)||ht(B,H)||ht(H,B)||ht(F==null?void 0:F.portalNode,H)||Xe.some(je=>je===H||ht(je,H))||H!=null&&Le.hasElement(H)||Le.hasMatchingElement(je=>ht(je,H))||ut||ue&&(zi(ue.nodesRef.current,Se).find(je=>{var Qe,He;return ht((Qe=je.context)==null?void 0:Qe.elements.floating,H)||ht((He=je.context)==null?void 0:He.elements.domReference,H)})||Ip(ue.nodesRef.current,Se).find(je=>{var Qe,He,rt;return[(Qe=je.context)==null?void 0:Qe.elements.floating,Qo((He=je.context)==null?void 0:He.elements.floating)].includes(H)||((rt=je.context)==null?void 0:rt.elements.domReference)===H})));if($===T&&ce&&rv(ce,D),d&&$!==T&&!gw(re)&&el(xe)===xe.body){if(Ft(ce)&&(ce.focus(),d==="popup")){P.request(()=>{ce.focus()});return}const je=k.current,Qe=be(),He=Qe[je]||Qe[Qe.length-1]||ce;Ft(He)&&He.focus()}if(N.current.insideReactTree){N.current.insideReactTree=!1;return}(Y||!p)&&H&&Je&&!he.current&&(Y||H!==Df())&&(ae.current=!0,R.setOpen(!1,zt(eu,oe)))})}function ve(){C.current||(N.current.insideReactTree=!0,Q.start(0,()=>{N.current.insideReactTree=!1}))}const Ee=Ft(T)?T:null,fe=[];if(!(!B&&!Ee))return Ee&&(Ee.addEventListener("focusout",Ge),Ee.addEventListener("pointerdown",pe),fe.push(()=>{Ee.removeEventListener("focusout",Ge),Ee.removeEventListener("pointerdown",pe)})),B&&(B.addEventListener("focusin",Re),B.addEventListener("focusout",Ge),F&&(B.addEventListener("focusout",ve,!0),fe.push(()=>{B.removeEventListener("focusout",ve,!0)})),fe.push(()=>{B.removeEventListener("focusin",Re),B.removeEventListener("focusout",Ge)})),()=>{fe.forEach(oe=>{oe()})}},[r,T,B,ce,p,ue,F,R,v,d,be,Y,U,D,N,Q,te,P,y,h,De]),x.useEffect(()=>{var H,$,re;if(r||!B||!M)return;const xe=Array.from(((H=F==null?void 0:F.portalNode)==null?void 0:H.querySelectorAll(`[${qs("portal")}]`))||[]),Re=(re=($=(ue?Ip(ue.nodesRef.current,U()):[]).find(Se=>{var Le;return Wf(((Le=Se.context)==null?void 0:Le.elements.domReference)||null)}))==null?void 0:$.context)==null?void 0:re.elements.domReference,ve=[...[B,...xe,q.current,A.current,F==null?void 0:F.beforeOutsideRef.current,F==null?void 0:F.afterOutsideRef.current,...De()],Re,kl(h),kl(y),Y?T:null].filter(Se=>Se!=null),Ee=iv(ve,{ariaHidden:p||Y,mark:!1}),fe=[B,...xe].filter(Se=>Se!=null),oe=iv(fe);return()=>{oe(),Ee()}},[M,r,T,B,p,D,F,Y,ue,U,y,h,De]),Oe(()=>{if(!M||r||!Ft(ce))return;const xe=kt(ce),pe=el(xe);queueMicrotask(()=>{const Re=we(ce),Ge=le.current,ve=typeof Ge=="function"?Ge(me.current||""):Ge;if(ve===void 0||ve===!1)return;let Ee;ve===!0||ve===null?Ee=Re[0]||ce:Ee=kl(ve),Ee=Ee||Re[0]||ce,!ht(ce,pe)&&Ho(Ee,{preventScroll:Ee===ce})})},[r,M,ce,G,we,le,me]),Oe(()=>{if(r||!ce)return;const xe=kt(ce),pe=el(xe);mw(pe);function Re(ve){if(ve.open||(X.current=dw(ve.nativeEvent,I.current)),ve.reason===w2&&ve.nativeEvent.type==="mouseleave"&&(ae.current=!0),ve.reason===Ed)if(ve.nested)ae.current=!1;else if(yy(ve.nativeEvent)||by(ve.nativeEvent))ae.current=!1;else{let Ee=!1;document.createElement("div").focus({get preventScroll(){return Ee=!0,!1}}),Ee?ae.current=!1:ae.current=!0}}_.on("openchange",Re);function Ge(){const ve=ge.current;let Ee=typeof ve=="function"?ve(X.current):ve;if(Ee===void 0||Ee===!1)return null;if(Ee===null&&(Ee=!0),typeof Ee=="boolean"){const oe=T||Df();return oe&&oe.isConnected?oe:null}const fe=T||Df();return kl(Ee)||fe||null}return()=>{_.off("openchange",Re);const ve=el(xe),Ee=De(),fe=ht(B,ve)||Ee.some(H=>H===ve||ht(H,ve))||ue&&zi(ue.nodesRef.current,U(),!1).some(H=>{var $;return ht(($=H.context)==null?void 0:$.elements.floating,ve)}),oe=Ge();queueMicrotask(()=>{const H=hw(oe),$=typeof ge.current!="boolean";ge.current&&!ae.current&&Ft(H)&&(!(!$&&H!==ve&&ve!==xe.body)||fe)&&H.focus({preventScroll:!0}),ae.current=!1})}},[r,B,ce,ge,N,_,ue,T,U,De]),Oe(()=>{if(!md||M||!B)return;const xe=el(kt(B));!Ft(xe)||!gd(xe)||ht(B,xe)&&xe.blur()},[M,B]),Oe(()=>{if(!(r||!F))return F.setFocusManagerState({modal:p,closeOnFocusOut:v,open:M,onOpenChange:R.setOpen,domReference:T}),()=>{F.setFocusManagerState(null)}},[r,F,p,M,R,v,T]),Oe(()=>{if(!(r||!ce))return rv(ce,D),()=>{queueMicrotask(Td)}},[r,ce,D]);const Ye=!r&&(p?!Y:!0)&&(de||p);return L.jsxs(x.Fragment,{children:[Ye&&L.jsx(Ys,{"data-type":"inside",ref:O,onFocus:xe=>{var pe;if(p){const Re=we();Ho(Re[Re.length-1])}else if(F!=null&&F.portalNode)if(ae.current=!1,Bo(xe,F.portalNode)){const Re=zy(T);Re==null||Re.focus()}else(pe=kl(h??F.beforeOutsideRef))==null||pe.focus()}}),o,Ye&&L.jsx(Ys,{"data-type":"inside",ref:W,onFocus:xe=>{var pe;if(p)Ho(we()[0]);else if(F!=null&&F.portalNode)if(v&&(ae.current=!0),Bo(xe,F.portalNode)){const Re=_y(T);Re==null||Re.focus()}else(pe=kl(y??F.afterOutsideRef))==null||pe.focus()}})]})}function pw(l,i={}){const o="rootStore"in l?l.rootStore:l,r=o.context.dataRef,{enabled:c=!0,event:f="click",toggle:d=!0,ignoreMouse:p=!1,stickIfOpen:v=!0,touchOpenDelay:m=0,reason:y=nd}=i,h=x.useRef(void 0),E=Ps(),w=jn(),S=x.useMemo(()=>({onPointerDown(R){h.current=R.pointerType},onMouseDown(R){const M=h.current,T=R.nativeEvent,B=o.select("open");if(R.button!==0||f==="click"||Fp(M)&&p)return;const _=r.current.openEvent,N=_==null?void 0:_.type,U=o.select("domReferenceElement")!==R.currentTarget,G=B&&U||!(B&&d&&(!(_&&v)||N==="click"||N==="mousedown"));if(gd(T.target)){const D=zt(y,T,T.target);G&&M==="touch"&&m>0?w.start(m,()=>{o.setOpen(!0,D)}):o.setOpen(G,D);return}const Y=R.currentTarget;E.request(()=>{const D=zt(y,T,Y);G&&M==="touch"&&m>0?w.start(m,()=>{o.setOpen(!0,D)}):o.setOpen(G,D)})},onClick(R){if(f==="mousedown-only")return;const M=h.current;if(f==="mousedown"&&M){h.current=void 0;return}if(Fp(M)&&p)return;const T=o.select("open"),B=r.current.openEvent,_=o.select("domReferenceElement")!==R.currentTarget,N=T&&_||!(T&&d&&(!(B&&v)||xy(B))),U=zt(y,R.nativeEvent,R.currentTarget);N&&M==="touch"&&m>0?w.start(m,()=>{o.setOpen(!0,U)}):o.setOpen(N,U)},onKeyDown(){h.current=void 0}}),[r,f,p,o,v,d,E,w,m,y]);return x.useMemo(()=>c?{reference:S}:an,[c,S])}const vw={intentional:"onClick",sloppy:"onPointerDown"};function yw(){return!1}function bw(l){return{escapeKey:typeof l=="boolean"?l:(l==null?void 0:l.escapeKey)??!1,outsidePress:typeof l=="boolean"?l:(l==null?void 0:l.outsidePress)??!0}}function qy(l,i={}){const o="rootStore"in l?l.rootStore:l,r=o.useState("open"),c=o.useState("floatingElement"),{dataRef:f}=o.context,{enabled:d=!0,escapeKey:p=!0,outsidePress:v=!0,outsidePressEvent:m="sloppy",referencePress:y=yw,referencePressEvent:h="sloppy",bubbles:E,externalTree:w}=i,S=lu(w),R=Ze(typeof v=="function"?v:()=>!1),M=typeof v=="function"?R:v,T=M!==!1,B=Ze(()=>m),_=x.useRef(!1),N=x.useRef(!1),U=x.useRef(!1),{escapeKey:G,outsidePress:Y}=bw(E),D=x.useRef(null),le=jn(),ge=jn(),me=Ze(()=>{ge.clear(),f.current.insideReactTree=!1}),ue=x.useRef(!1),F=x.useRef(""),ae=Ze(y),he=Ze(A=>{var P;if(!r||!d||!p||A.key!=="Escape"||ue.current)return;const O=(P=f.current.floatingContext)==null?void 0:P.nodeId,W=S?zi(S.nodesRef.current,O):[];if(!G&&W.length>0){let de=!0;if(W.forEach(ce=>{var be;(be=ce.context)!=null&&be.open&&!ce.context.dataRef.current.__escapeKeyBubbles&&(de=!1)}),!de)return}const Q=GE(A)?A.nativeEvent:A,te=zt(R2,Q);o.setOpen(!1,te),!G&&!te.isPropagationAllowed&&A.stopPropagation()}),C=Ze(()=>{f.current.insideReactTree=!0,ge.start(0,me)});x.useEffect(()=>{if(!r||!d)return;f.current.__escapeKeyBubbles=G,f.current.__outsidePressBubbles=Y;const A=new Ha,O=new Ha;function W(){A.clear(),ue.current=!0}function Q(){A.start(Is()?5:0,()=>{ue.current=!1})}function te(){U.current=!0,O.start(0,()=>{U.current=!1})}function P(){_.current=!1,N.current=!1}function de(){const H=F.current,$=H==="pen"||!H?"mouse":H,re=B(),Se=typeof re=="function"?re():re;return typeof Se=="string"?Se:Se[$]}function ce(H){const $=de();return $==="intentional"&&H.type!=="click"||$==="sloppy"&&H.type==="click"}function be(H){var Se;const $=(Se=f.current.floatingContext)==null?void 0:Se.nodeId,re=S&&zi(S.nodesRef.current,$).some(Le=>{var Xe;return Pn(H,(Xe=Le.context)==null?void 0:Xe.elements.floating)});return Pn(H,o.select("floatingElement"))||Pn(H,o.select("domReferenceElement"))||re}function De(H){var Qe;if(ce(H)){me();return}if(f.current.insideReactTree){me();return}const $=tl(H),re=`[${qs("inert")}]`;let Se=Array.from(kt(o.select("floatingElement")).querySelectorAll(re));const Le=Bt($)?$.getRootNode():null;Yo(Le)&&(Se=Se.concat(Array.from(Le.querySelectorAll(re))));const Xe=o.context.triggerElements;if($&&(Xe.hasElement($)||Xe.hasMatchingElement(He=>ht(He,$))))return;let ut=Bt($)?$:null;for(;ut&&!jl(ut);){const He=Ll(ut);if(jl(He)||!Bt(He))break;ut=He}if(Se.length&&Bt($)&&!qE($)&&!ht($,o.select("floatingElement"))&&Se.every(He=>!ht(ut,He)))return;if(Ft($)&&!("touches"in H)){const He=jl($),rt=Un($),gt=/auto|scroll/,pt=He||gt.test(rt.overflowX),Lt=He||gt.test(rt.overflowY),it=pt&&$.clientWidth>0&&$.scrollWidth>$.clientWidth,ot=Lt&&$.clientHeight>0&&$.scrollHeight>$.clientHeight,Tt=rt.direction==="rtl",qe=ot&&(Tt?H.offsetX<=$.offsetWidth-$.clientWidth:H.offsetX>$.clientWidth),ct=it&&H.offsetY>$.clientHeight;if(qe||ct)return}if(be(H))return;if(de()==="intentional"&&U.current){O.clear(),U.current=!1;return}if(typeof M=="function"&&!M(H))return;const Je=(Qe=f.current.floatingContext)==null?void 0:Qe.nodeId,je=S?zi(S.nodesRef.current,Je):[];if(je.length>0){let He=!0;if(je.forEach(rt=>{var gt;(gt=rt.context)!=null&&gt.open&&!rt.context.dataRef.current.__outsidePressBubbles&&(He=!1)}),!He)return}o.setOpen(!1,zt(Ed,H)),me()}function we(H){de()!=="sloppy"||H.pointerType==="touch"||!o.select("open")||!d||Pn(H,o.select("floatingElement"))||Pn(H,o.select("domReferenceElement"))||De(H)}function Ye(H){if(de()!=="sloppy"||!o.select("open")||!d||Pn(H,o.select("floatingElement"))||Pn(H,o.select("domReferenceElement")))return;const $=H.touches[0];$&&(D.current={startTime:Date.now(),startX:$.clientX,startY:$.clientY,dismissOnTouchEnd:!1,dismissOnMouseDown:!0},le.start(1e3,()=>{D.current&&(D.current.dismissOnTouchEnd=!1,D.current.dismissOnMouseDown=!1)}))}function xe(H){F.current="touch";const $=tl(H);function re(){Ye(H),$==null||$.removeEventListener(H.type,re)}$==null||$.addEventListener(H.type,re)}function pe(H){if(le.clear(),H.type==="pointerdown"&&(F.current=H.pointerType),H.type==="mousedown"&&D.current&&!D.current.dismissOnMouseDown)return;const $=tl(H);function re(){H.type==="pointerdown"?we(H):De(H),$==null||$.removeEventListener(H.type,re)}$==null||$.addEventListener(H.type,re)}function Re(H){if(!_.current)return;const $=N.current;if(P(),de()==="intentional"){if(H.type==="pointercancel"){$&&te();return}if(!be(H)){if($){te();return}typeof M=="function"&&!M(H)||(O.clear(),U.current=!0,me())}}}function Ge(H){if(de()!=="sloppy"||!D.current||Pn(H,o.select("floatingElement"))||Pn(H,o.select("domReferenceElement")))return;const $=H.touches[0];if(!$)return;const re=Math.abs($.clientX-D.current.startX),Se=Math.abs($.clientY-D.current.startY),Le=Math.sqrt(re*re+Se*Se);Le>5&&(D.current.dismissOnTouchEnd=!0),Le>10&&(De(H),le.clear(),D.current=null)}function ve(H){const $=tl(H);function re(){Ge(H),$==null||$.removeEventListener(H.type,re)}$==null||$.addEventListener(H.type,re)}function Ee(H){de()!=="sloppy"||!D.current||Pn(H,o.select("floatingElement"))||Pn(H,o.select("domReferenceElement"))||(D.current.dismissOnTouchEnd&&De(H),le.clear(),D.current=null)}function fe(H){const $=tl(H);function re(){Ee(H),$==null||$.removeEventListener(H.type,re)}$==null||$.addEventListener(H.type,re)}const oe=kt(c);return p&&(oe.addEventListener("keydown",he),oe.addEventListener("compositionstart",W),oe.addEventListener("compositionend",Q)),T&&(oe.addEventListener("click",pe,!0),oe.addEventListener("pointerdown",pe,!0),oe.addEventListener("pointerup",Re,!0),oe.addEventListener("pointercancel",Re,!0),oe.addEventListener("mousedown",pe,!0),oe.addEventListener("mouseup",Re,!0),oe.addEventListener("touchstart",xe,!0),oe.addEventListener("touchmove",ve,!0),oe.addEventListener("touchend",fe,!0)),()=>{p&&(oe.removeEventListener("keydown",he),oe.removeEventListener("compositionstart",W),oe.removeEventListener("compositionend",Q)),T&&(oe.removeEventListener("click",pe,!0),oe.removeEventListener("pointerdown",pe,!0),oe.removeEventListener("pointerup",Re,!0),oe.removeEventListener("pointercancel",Re,!0),oe.removeEventListener("mousedown",pe,!0),oe.removeEventListener("mouseup",Re,!0),oe.removeEventListener("touchstart",xe,!0),oe.removeEventListener("touchmove",ve,!0),oe.removeEventListener("touchend",fe,!0)),A.clear(),O.clear(),P(),U.current=!1}},[f,c,p,T,M,r,d,G,Y,he,me,B,S,o,le]),x.useEffect(me,[M,me]);const k=x.useMemo(()=>({onKeyDown:he,[vw[h]]:A=>{ae()&&o.setOpen(!1,zt(nd,A.nativeEvent))},...h!=="intentional"&&{onClick(A){ae()&&o.setOpen(!1,zt(nd,A.nativeEvent))}}}),[he,o,h,ae]),X=Ze(A=>{if(!r||!d||A.button!==0)return;const O=tl(A.nativeEvent);ht(o.select("floatingElement"),O)&&(_.current||(_.current=!0,N.current=!1))}),I=Ze(A=>{!r||!d||(A.defaultPrevented||A.nativeEvent.defaultPrevented)&&_.current&&(N.current=!0)}),q=x.useMemo(()=>({onKeyDown:he,onPointerDown:I,onMouseDown:I,onClickCapture:C,onMouseDownCapture(A){C(),X(A)},onPointerDownCapture(A){C(),X(A)},onMouseUpCapture:C,onTouchEndCapture:C,onTouchMoveCapture:C}),[he,C,X,I]);return x.useMemo(()=>d?{reference:k,floating:q,trigger:k}:{},[d,k,q])}function sv(l,i,o){let{reference:r,floating:c}=l;const f=In(i),d=yd(i),p=vd(d),v=Ln(i),m=f==="y",y=r.x+r.width/2-c.width/2,h=r.y+r.height/2-c.height/2,E=r[p]/2-c[p]/2;let w;switch(v){case"top":w={x:y,y:r.y-c.height};break;case"bottom":w={x:y,y:r.y+r.height};break;case"right":w={x:r.x+r.width,y:h};break;case"left":w={x:r.x-c.width,y:h};break;default:w={x:r.x,y:r.y}}switch(pa(i)){case"start":w[d]-=E*(o&&m?-1:1);break;case"end":w[d]+=E*(o&&m?-1:1);break}return w}async function xw(l,i){var o;i===void 0&&(i={});const{x:r,y:c,platform:f,rects:d,elements:p,strategy:v}=l,{boundary:m="clippingAncestors",rootBoundary:y="viewport",elementContext:h="floating",altBoundary:E=!1,padding:w=0}=Ul(i,l),S=Sy(w),M=p[E?h==="floating"?"reference":"floating":h],T=Ko(await f.getClippingRect({element:(o=await(f.isElement==null?void 0:f.isElement(M)))==null||o?M:M.contextElement||await(f.getDocumentElement==null?void 0:f.getDocumentElement(p.floating)),boundary:m,rootBoundary:y,strategy:v})),B=h==="floating"?{x:r,y:c,width:d.floating.width,height:d.floating.height}:d.reference,_=await(f.getOffsetParent==null?void 0:f.getOffsetParent(p.floating)),N=await(f.isElement==null?void 0:f.isElement(_))?await(f.getScale==null?void 0:f.getScale(_))||{x:1,y:1}:{x:1,y:1},U=Ko(f.convertOffsetParentRelativeRectToViewportRelativeRect?await f.convertOffsetParentRelativeRectToViewportRelativeRect({elements:p,rect:B,offsetParent:_,strategy:v}):B);return{top:(T.top-U.top+S.top)/N.y,bottom:(U.bottom-T.bottom+S.bottom)/N.y,left:(T.left-U.left+S.left)/N.x,right:(U.right-T.right+S.right)/N.x}}const Sw=50,Ew=async(l,i,o)=>{const{placement:r="bottom",strategy:c="absolute",middleware:f=[],platform:d}=o,p=d.detectOverflow?d:{...d,detectOverflow:xw},v=await(d.isRTL==null?void 0:d.isRTL(i));let m=await d.getElementRects({reference:l,floating:i,strategy:c}),{x:y,y:h}=sv(m,r,v),E=r,w=0;const S={};for(let R=0;R<f.length;R++){const M=f[R];if(!M)continue;const{name:T,fn:B}=M,{x:_,y:N,data:U,reset:G}=await B({x:y,y:h,initialPlacement:r,placement:E,strategy:c,middlewareData:S,rects:m,platform:p,elements:{reference:l,floating:i}});y=_??y,h=N??h,S[T]={...S[T],...U},G&&w<Sw&&(w++,typeof G=="object"&&(G.placement&&(E=G.placement),G.rects&&(m=G.rects===!0?await d.getElementRects({reference:l,floating:i,strategy:c}):G.rects),{x:y,y:h}=sv(m,E,v)),R=-1)}return{x:y,y:h,placement:E,strategy:c,middlewareData:S}},ww=function(l){return l===void 0&&(l={}),{name:"flip",options:l,async fn(i){var o,r;const{placement:c,middlewareData:f,rects:d,initialPlacement:p,platform:v,elements:m}=i,{mainAxis:y=!0,crossAxis:h=!0,fallbackPlacements:E,fallbackStrategy:w="bestFit",fallbackAxisSideDirection:S="none",flipAlignment:R=!0,...M}=Ul(l,i);if((o=f.arrow)!=null&&o.alignmentOffset)return{};const T=Ln(c),B=In(p),_=Ln(p)===p,N=await(v.isRTL==null?void 0:v.isRTL(m.floating)),U=E||(_||!R?[ks(p)]:ZE(p)),G=S!=="none";!E&&G&&U.push(...WE(p,R,S,N));const Y=[p,...U],D=await v.detectOverflow(i,M),le=[];let ge=((r=f.flip)==null?void 0:r.overflows)||[];if(y&&le.push(D[T]),h){const ae=KE(c,d,N);le.push(D[ae[0]],D[ae[1]])}if(ge=[...ge,{placement:c,overflows:le}],!le.every(ae=>ae<=0)){var me,ue;const ae=(((me=f.flip)==null?void 0:me.index)||0)+1,he=Y[ae];if(he&&(!(h==="alignment"?B!==In(he):!1)||ge.every(X=>In(X.placement)===B?X.overflows[0]>0:!0)))return{data:{index:ae,overflows:ge},reset:{placement:he}};let C=(ue=ge.filter(k=>k.overflows[0]<=0).sort((k,X)=>k.overflows[1]-X.overflows[1])[0])==null?void 0:ue.placement;if(!C)switch(w){case"bestFit":{var F;const k=(F=ge.filter(X=>{if(G){const I=In(X.placement);return I===B||I==="y"}return!0}).map(X=>[X.placement,X.overflows.filter(I=>I>0).reduce((I,q)=>I+q,0)]).sort((X,I)=>X[1]-I[1])[0])==null?void 0:F[0];k&&(C=k);break}case"initialPlacement":C=p;break}if(c!==C)return{reset:{placement:C}}}return{}}}};function uv(l,i){return{top:l.top-i.height,right:l.right-i.width,bottom:l.bottom-i.height,left:l.left-i.width}}function cv(l){return XE.some(i=>l[i]>=0)}const Tw=function(l){return l===void 0&&(l={}),{name:"hide",options:l,async fn(i){const{rects:o,platform:r}=i,{strategy:c="referenceHidden",...f}=Ul(l,i);switch(c){case"referenceHidden":{const d=await r.detectOverflow(i,{...f,elementContext:"reference"}),p=uv(d,o.reference);return{data:{referenceHiddenOffsets:p,referenceHidden:cv(p)}}}case"escaped":{const d=await r.detectOverflow(i,{...f,altBoundary:!0}),p=uv(d,o.floating);return{data:{escapedOffsets:p,escaped:cv(p)}}}default:return{}}}}},Gy=new Set(["left","top"]);async function Rw(l,i){const{placement:o,platform:r,elements:c}=l,f=await(r.isRTL==null?void 0:r.isRTL(c.floating)),d=Ln(o),p=pa(o),v=In(o)==="y",m=Gy.has(d)?-1:1,y=f&&v?-1:1,h=Ul(i,l);let{mainAxis:E,crossAxis:w,alignmentAxis:S}=typeof h=="number"?{mainAxis:h,crossAxis:0,alignmentAxis:null}:{mainAxis:h.mainAxis||0,crossAxis:h.crossAxis||0,alignmentAxis:h.alignmentAxis};return p&&typeof S=="number"&&(w=p==="end"?S*-1:S),v?{x:w*y,y:E*m}:{x:E*m,y:w*y}}const Aw=function(l){return l===void 0&&(l=0),{name:"offset",options:l,async fn(i){var o,r;const{x:c,y:f,placement:d,middlewareData:p}=i,v=await Rw(i,l);return d===((o=p.offset)==null?void 0:o.placement)&&(r=p.arrow)!=null&&r.alignmentOffset?{}:{x:c+v.x,y:f+v.y,data:{...v,placement:d}}}}},Cw=function(l){return l===void 0&&(l={}),{name:"shift",options:l,async fn(i){const{x:o,y:r,placement:c,platform:f}=i,{mainAxis:d=!0,crossAxis:p=!1,limiter:v={fn:T=>{let{x:B,y:_}=T;return{x:B,y:_}}},...m}=Ul(l,i),y={x:o,y:r},h=await f.detectOverflow(i,m),E=In(Ln(c)),w=pd(E);let S=y[w],R=y[E];if(d){const T=w==="y"?"top":"left",B=w==="y"?"bottom":"right",_=S+h[T],N=S-h[B];S=$f(_,S,N)}if(p){const T=E==="y"?"top":"left",B=E==="y"?"bottom":"right",_=R+h[T],N=R-h[B];R=$f(_,R,N)}const M=v.fn({...i,[w]:S,[E]:R});return{...M,data:{x:M.x-o,y:M.y-r,enabled:{[w]:d,[E]:p}}}}}},Mw=function(l){return l===void 0&&(l={}),{options:l,fn(i){const{x:o,y:r,placement:c,rects:f,middlewareData:d}=i,{offset:p=0,mainAxis:v=!0,crossAxis:m=!0}=Ul(l,i),y={x:o,y:r},h=In(c),E=pd(h);let w=y[E],S=y[h];const R=Ul(p,i),M=typeof R=="number"?{mainAxis:R,crossAxis:0}:{mainAxis:0,crossAxis:0,...R};if(v){const _=E==="y"?"height":"width",N=f.reference[E]-f.floating[_]+M.mainAxis,U=f.reference[E]+f.reference[_]-M.mainAxis;w<N?w=N:w>U&&(w=U)}if(m){var T,B;const _=E==="y"?"width":"height",N=Gy.has(Ln(c)),U=f.reference[h]-f.floating[_]+(N&&((T=d.offset)==null?void 0:T[h])||0)+(N?0:M.crossAxis),G=f.reference[h]+f.reference[_]+(N?0:((B=d.offset)==null?void 0:B[h])||0)-(N?M.crossAxis:0);S<U?S=U:S>G&&(S=G)}return{[E]:w,[h]:S}}}},Ow=function(l){return l===void 0&&(l={}),{name:"size",options:l,async fn(i){var o,r;const{placement:c,rects:f,platform:d,elements:p}=i,{apply:v=()=>{},...m}=Ul(l,i),y=await d.detectOverflow(i,m),h=Ln(c),E=pa(c),w=In(c)==="y",{width:S,height:R}=f.floating;let M,T;h==="top"||h==="bottom"?(M=h,T=E===(await(d.isRTL==null?void 0:d.isRTL(p.floating))?"start":"end")?"left":"right"):(T=h,M=E==="end"?"top":"bottom");const B=R-y.top-y.bottom,_=S-y.left-y.right,N=Di(R-y[M],B),U=Di(S-y[T],_),G=!i.middlewareData.shift;let Y=N,D=U;if((o=i.middlewareData.shift)!=null&&o.enabled.x&&(D=_),(r=i.middlewareData.shift)!=null&&r.enabled.y&&(Y=B),G&&!E){const ge=kn(y.left,0),me=kn(y.right,0),ue=kn(y.top,0),F=kn(y.bottom,0);w?D=S-2*(ge!==0||me!==0?ge+me:kn(y.left,y.right)):Y=R-2*(ue!==0||F!==0?ue+F:kn(y.top,y.bottom))}await v({...i,availableWidth:D,availableHeight:Y});const le=await d.getDimensions(p.floating);return S!==le.width||R!==le.height?{reset:{rects:!0}}:{}}}};function Xy(l){const i=Un(l);let o=parseFloat(i.width)||0,r=parseFloat(i.height)||0;const c=Ft(l),f=c?l.offsetWidth:o,d=c?l.offsetHeight:r,p=Ds(o)!==f||Ds(r)!==d;return p&&(o=f,r=d),{width:o,height:r,$:p}}function Rd(l){return Bt(l)?l:l.contextElement}function Ni(l){const i=Rd(l);if(!Ft(i))return ul(1);const o=i.getBoundingClientRect(),{width:r,height:c,$:f}=Xy(i);let d=(f?Ds(o.width):o.width)/r,p=(f?Ds(o.height):o.height)/c;return(!d||!Number.isFinite(d))&&(d=1),(!p||!Number.isFinite(p))&&(p=1),{x:d,y:p}}const zw=ul(0);function Qy(l){const i=en(l);return!Is()||!i.visualViewport?zw:{x:i.visualViewport.offsetLeft,y:i.visualViewport.offsetTop}}function _w(l,i,o){return i===void 0&&(i=!1),!o||i&&o!==en(l)?!1:i}function Va(l,i,o,r){i===void 0&&(i=!1),o===void 0&&(o=!1);const c=l.getBoundingClientRect(),f=Rd(l);let d=ul(1);i&&(r?Bt(r)&&(d=Ni(r)):d=Ni(l));const p=_w(f,o,r)?Qy(f):ul(0);let v=(c.left+p.x)/d.x,m=(c.top+p.y)/d.y,y=c.width/d.x,h=c.height/d.y;if(f){const E=en(f),w=r&&Bt(r)?en(r):r;let S=E,R=Xf(S);for(;R&&r&&w!==S;){const M=Ni(R),T=R.getBoundingClientRect(),B=Un(R),_=T.left+(R.clientLeft+parseFloat(B.paddingLeft))*M.x,N=T.top+(R.clientTop+parseFloat(B.paddingTop))*M.y;v*=M.x,m*=M.y,y*=M.x,h*=M.y,v+=_,m+=N,S=en(R),R=Xf(S)}}return Ko({width:y,height:h,x:v,y:m})}function au(l,i){const o=Fs(l).scrollLeft;return i?i.left+o:Va(cl(l)).left+o}function Ky(l,i){const o=l.getBoundingClientRect(),r=o.left+i.scrollLeft-au(l,o),c=o.top+i.scrollTop;return{x:r,y:c}}function Nw(l){let{elements:i,rect:o,offsetParent:r,strategy:c}=l;const f=c==="fixed",d=cl(r),p=i?Zs(i.floating):!1;if(r===d||p&&f)return o;let v={scrollLeft:0,scrollTop:0},m=ul(1);const y=ul(0),h=Ft(r);if((h||!h&&!f)&&((ma(r)!=="body"||ha(d))&&(v=Fs(r)),h)){const w=Va(r);m=Ni(r),y.x=w.x+r.clientLeft,y.y=w.y+r.clientTop}const E=d&&!h&&!f?Ky(d,v):ul(0);return{width:o.width*m.x,height:o.height*m.y,x:o.x*m.x-v.scrollLeft*m.x+y.x+E.x,y:o.y*m.y-v.scrollTop*m.y+y.y+E.y}}function Dw(l){return Array.from(l.getClientRects())}function kw(l){const i=cl(l),o=Fs(l),r=l.ownerDocument.body,c=kn(i.scrollWidth,i.clientWidth,r.scrollWidth,r.clientWidth),f=kn(i.scrollHeight,i.clientHeight,r.scrollHeight,r.clientHeight);let d=-o.scrollLeft+au(l);const p=-o.scrollTop;return Un(r).direction==="rtl"&&(d+=kn(i.clientWidth,r.clientWidth)-c),{width:c,height:f,x:d,y:p}}const fv=25;function jw(l,i){const o=en(l),r=cl(l),c=o.visualViewport;let f=r.clientWidth,d=r.clientHeight,p=0,v=0;if(c){f=c.width,d=c.height;const y=Is();(!y||y&&i==="fixed")&&(p=c.offsetLeft,v=c.offsetTop)}const m=au(r);if(m<=0){const y=r.ownerDocument,h=y.body,E=getComputedStyle(h),w=y.compatMode==="CSS1Compat"&&parseFloat(E.marginLeft)+parseFloat(E.marginRight)||0,S=Math.abs(r.clientWidth-h.clientWidth-w);S<=fv&&(f-=S)}else m<=fv&&(f+=m);return{width:f,height:d,x:p,y:v}}function Lw(l,i){const o=Va(l,!0,i==="fixed"),r=o.top+l.clientTop,c=o.left+l.clientLeft,f=Ft(l)?Ni(l):ul(1),d=l.clientWidth*f.x,p=l.clientHeight*f.y,v=c*f.x,m=r*f.y;return{width:d,height:p,x:v,y:m}}function dv(l,i,o){let r;if(i==="viewport")r=jw(l,o);else if(i==="document")r=kw(cl(l));else if(Bt(i))r=Lw(i,o);else{const c=Qy(l);r={x:i.x-c.x,y:i.y-c.y,width:i.width,height:i.height}}return Ko(r)}function Zy(l,i){const o=Ll(l);return o===i||!Bt(o)||jl(o)?!1:Un(o).position==="fixed"||Zy(o,i)}function Uw(l,i){const o=i.get(l);if(o)return o;let r=qo(l,[],!1).filter(p=>Bt(p)&&ma(p)!=="body"),c=null;const f=Un(l).position==="fixed";let d=f?Ll(l):l;for(;Bt(d)&&!jl(d);){const p=Un(d),v=sd(d);!v&&p.position==="fixed"&&(c=null),(f?!v&&!c:!v&&p.position==="static"&&!!c&&(c.position==="absolute"||c.position==="fixed")||ha(d)&&!v&&Zy(l,d))?r=r.filter(y=>y!==d):c=p,d=Ll(d)}return i.set(l,r),r}function Bw(l){let{element:i,boundary:o,rootBoundary:r,strategy:c}=l;const d=[...o==="clippingAncestors"?Zs(i)?[]:Uw(i,this._c):[].concat(o),r],p=dv(i,d[0],c);let v=p.top,m=p.right,y=p.bottom,h=p.left;for(let E=1;E<d.length;E++){const w=dv(i,d[E],c);v=kn(w.top,v),m=Di(w.right,m),y=Di(w.bottom,y),h=kn(w.left,h)}return{width:m-h,height:y-v,x:h,y:v}}function Hw(l){const{width:i,height:o}=Xy(l);return{width:i,height:o}}function Vw(l,i,o){const r=Ft(i),c=cl(i),f=o==="fixed",d=Va(l,!0,f,i);let p={scrollLeft:0,scrollTop:0};const v=ul(0);function m(){v.x=au(c)}if(r||!r&&!f)if((ma(i)!=="body"||ha(c))&&(p=Fs(i)),r){const w=Va(i,!0,f,i);v.x=w.x+i.clientLeft,v.y=w.y+i.clientTop}else c&&m();f&&!r&&c&&m();const y=c&&!r&&!f?Ky(c,p):ul(0),h=d.left+p.scrollLeft-v.x-y.x,E=d.top+p.scrollTop-v.y-y.y;return{x:h,y:E,width:d.width,height:d.height}}function kf(l){return Un(l).position==="static"}function mv(l,i){if(!Ft(l)||Un(l).position==="fixed")return null;if(i)return i(l);let o=l.offsetParent;return cl(l)===o&&(o=o.ownerDocument.body),o}function Iy(l,i){const o=en(l);if(Zs(l))return o;if(!Ft(l)){let c=Ll(l);for(;c&&!jl(c);){if(Bt(c)&&!kf(c))return c;c=Ll(c)}return o}let r=mv(l,i);for(;r&&lS(r)&&kf(r);)r=mv(r,i);return r&&jl(r)&&kf(r)&&!sd(r)?o:r||oS(l)||o}const Yw=async function(l){const i=this.getOffsetParent||Iy,o=this.getDimensions,r=await o(l.floating);return{reference:Vw(l.reference,await i(l.floating),l.strategy),floating:{x:0,y:0,width:r.width,height:r.height}}};function qw(l){return Un(l).direction==="rtl"}const Fy={convertOffsetParentRelativeRectToViewportRelativeRect:Nw,getDocumentElement:cl,getClippingRect:Bw,getOffsetParent:Iy,getElementRects:Yw,getClientRects:Dw,getDimensions:Hw,getScale:Ni,isElement:Bt,isRTL:qw};function Jy(l,i){return l.x===i.x&&l.y===i.y&&l.width===i.width&&l.height===i.height}function Gw(l,i){let o=null,r;const c=cl(l);function f(){var p;clearTimeout(r),(p=o)==null||p.disconnect(),o=null}function d(p,v){p===void 0&&(p=!1),v===void 0&&(v=1),f();const m=l.getBoundingClientRect(),{left:y,top:h,width:E,height:w}=m;if(p||i(),!E||!w)return;const S=La(h),R=La(c.clientWidth-(y+E)),M=La(c.clientHeight-(h+w)),T=La(y),_={rootMargin:-S+"px "+-R+"px "+-M+"px "+-T+"px",threshold:kn(0,Di(1,v))||1};let N=!0;function U(G){const Y=G[0].intersectionRatio;if(Y!==v){if(!N)return d();Y?d(!1,Y):r=setTimeout(()=>{d(!1,1e-7)},1e3)}Y===1&&!Jy(m,l.getBoundingClientRect())&&d(),N=!1}try{o=new IntersectionObserver(U,{..._,root:c.ownerDocument})}catch{o=new IntersectionObserver(U,_)}o.observe(l)}return d(!0),f}function hv(l,i,o,r){r===void 0&&(r={});const{ancestorScroll:c=!0,ancestorResize:f=!0,elementResize:d=typeof ResizeObserver=="function",layoutShift:p=typeof IntersectionObserver=="function",animationFrame:v=!1}=r,m=Rd(l),y=c||f?[...m?qo(m):[],...i?qo(i):[]]:[];y.forEach(T=>{c&&T.addEventListener("scroll",o,{passive:!0}),f&&T.addEventListener("resize",o)});const h=m&&p?Gw(m,o):null;let E=-1,w=null;d&&(w=new ResizeObserver(T=>{let[B]=T;B&&B.target===m&&w&&i&&(w.unobserve(i),cancelAnimationFrame(E),E=requestAnimationFrame(()=>{var _;(_=w)==null||_.observe(i)})),o()}),m&&!v&&w.observe(m),i&&w.observe(i));let S,R=v?Va(l):null;v&&M();function M(){const T=Va(l);R&&!Jy(R,T)&&o(),R=T,S=requestAnimationFrame(M)}return o(),()=>{var T;y.forEach(B=>{c&&B.removeEventListener("scroll",o),f&&B.removeEventListener("resize",o)}),h==null||h(),(T=w)==null||T.disconnect(),w=null,v&&cancelAnimationFrame(S)}}const Xw=Aw,Qw=Cw,Kw=ww,Zw=Ow,Iw=Tw,Fw=Mw,Jw=(l,i,o)=>{const r=new Map,c={platform:Fy,...o},f={...c.platform,_c:r};return Ew(l,i,{...c,platform:f})};var Ww=typeof document<"u",$w=function(){},_s=Ww?x.useLayoutEffect:$w;function Gs(l,i){if(l===i)return!0;if(typeof l!=typeof i)return!1;if(typeof l=="function"&&l.toString()===i.toString())return!0;let o,r,c;if(l&&i&&typeof l=="object"){if(Array.isArray(l)){if(o=l.length,o!==i.length)return!1;for(r=o;r--!==0;)if(!Gs(l[r],i[r]))return!1;return!0}if(c=Object.keys(l),o=c.length,o!==Object.keys(i).length)return!1;for(r=o;r--!==0;)if(!{}.hasOwnProperty.call(i,c[r]))return!1;for(r=o;r--!==0;){const f=c[r];if(!(f==="_owner"&&l.$$typeof)&&!Gs(l[f],i[f]))return!1}return!0}return l!==l&&i!==i}function Wy(l){return typeof window>"u"?1:(l.ownerDocument.defaultView||window).devicePixelRatio||1}function gv(l,i){const o=Wy(l);return Math.round(i*o)/o}function jf(l){const i=x.useRef(l);return _s(()=>{i.current=l}),i}function Pw(l){l===void 0&&(l={});const{placement:i="bottom",strategy:o="absolute",middleware:r=[],platform:c,elements:{reference:f,floating:d}={},transform:p=!0,whileElementsMounted:v,open:m}=l,[y,h]=x.useState({x:0,y:0,strategy:o,placement:i,middlewareData:{},isPositioned:!1}),[E,w]=x.useState(r);Gs(E,r)||w(r);const[S,R]=x.useState(null),[M,T]=x.useState(null),B=x.useCallback(X=>{X!==G.current&&(G.current=X,R(X))},[]),_=x.useCallback(X=>{X!==Y.current&&(Y.current=X,T(X))},[]),N=f||S,U=d||M,G=x.useRef(null),Y=x.useRef(null),D=x.useRef(y),le=v!=null,ge=jf(v),me=jf(c),ue=jf(m),F=x.useCallback(()=>{if(!G.current||!Y.current)return;const X={placement:i,strategy:o,middleware:E};me.current&&(X.platform=me.current),Jw(G.current,Y.current,X).then(I=>{const q={...I,isPositioned:ue.current!==!1};ae.current&&!Gs(D.current,q)&&(D.current=q,Ya.flushSync(()=>{h(q)}))})},[E,i,o,me,ue]);_s(()=>{m===!1&&D.current.isPositioned&&(D.current.isPositioned=!1,h(X=>({...X,isPositioned:!1})))},[m]);const ae=x.useRef(!1);_s(()=>(ae.current=!0,()=>{ae.current=!1}),[]),_s(()=>{if(N&&(G.current=N),U&&(Y.current=U),N&&U){if(ge.current)return ge.current(N,U,F);F()}},[N,U,F,ge,le]);const he=x.useMemo(()=>({reference:G,floating:Y,setReference:B,setFloating:_}),[B,_]),C=x.useMemo(()=>({reference:N,floating:U}),[N,U]),k=x.useMemo(()=>{const X={position:o,left:0,top:0};if(!C.floating)return X;const I=gv(C.floating,y.x),q=gv(C.floating,y.y);return p?{...X,transform:"translate("+I+"px, "+q+"px)",...Wy(C.floating)>=1.5&&{willChange:"transform"}}:{position:o,left:I,top:q}},[o,p,C.floating,y.x,y.y]);return x.useMemo(()=>({...y,update:F,refs:he,elements:C,floatingStyles:k}),[y,F,he,C,k])}const eT=(l,i)=>{const o=Xw(l);return{name:o.name,fn:o.fn,options:[l,i]}},tT=(l,i)=>{const o=Qw(l);return{name:o.name,fn:o.fn,options:[l,i]}},nT=(l,i)=>({fn:Fw(l).fn,options:[l,i]}),lT=(l,i)=>{const o=Kw(l);return{name:o.name,fn:o.fn,options:[l,i]}},aT=(l,i)=>{const o=Zw(l);return{name:o.name,fn:o.fn,options:[l,i]}},iT=(l,i)=>{const o=Iw(l);return{name:o.name,fn:o.fn,options:[l,i]}};var Xs=Symbol("NOT_FOUND");function oT(l,i=`expected a function, instead received ${typeof l}`){if(typeof l!="function")throw new TypeError(i)}function rT(l,i=`expected an object, instead received ${typeof l}`){if(typeof l!="object")throw new TypeError(i)}function sT(l,i="expected all items to be functions, instead received the following types: "){if(!l.every(o=>typeof o=="function")){const o=l.map(r=>typeof r=="function"?`function ${r.name||"unnamed"}()`:typeof r).join(", ");throw new TypeError(`${i}[${o}]`)}}var pv=l=>Array.isArray(l)?l:[l];function uT(l){const i=Array.isArray(l[0])?l[0]:l;return sT(i,"createSelector expects all input-selectors to be functions, but received the following types: "),i}function cT(l,i){const o=[],{length:r}=l;for(let c=0;c<r;c++)o.push(l[c].apply(null,i));return o}function fT(l){let i;return{get(o){return i&&l(i.key,o)?i.value:Xs},put(o,r){i={key:o,value:r}},getEntries(){return i?[i]:[]},clear(){i=void 0}}}function dT(l,i){let o=[];function r(p){const v=o.findIndex(m=>i(p,m.key));if(v>-1){const m=o[v];return v>0&&(o.splice(v,1),o.unshift(m)),m.value}return Xs}function c(p,v){r(p)===Xs&&(o.unshift({key:p,value:v}),o.length>l&&o.pop())}function f(){return o}function d(){o=[]}return{get:r,put:c,getEntries:f,clear:d}}var mT=(l,i)=>l===i;function hT(l){return function(o,r){if(o===null||r===null||o.length!==r.length)return!1;const{length:c}=o;for(let f=0;f<c;f++)if(!l(o[f],r[f]))return!1;return!0}}function gT(l,i){const o=typeof i=="object"?i:{equalityCheck:i},{equalityCheck:r=mT,maxSize:c=1,resultEqualityCheck:f}=o,d=hT(r);let p=0;const v=c<=1?fT(d):dT(c,d);function m(){let y=v.get(arguments);if(y===Xs){if(y=l.apply(null,arguments),p++,f){const E=v.getEntries().find(w=>f(w.value,y));E&&(y=E.value,p!==0&&p--)}v.put(arguments,y)}return y}return m.clearCache=()=>{v.clear(),m.resetResultsCount()},m.resultsCount=()=>p,m.resetResultsCount=()=>{p=0},m}var pT=class{constructor(l){this.value=l}deref(){return this.value}},vT=typeof WeakRef<"u"?WeakRef:pT,yT=0,vv=1;function ws(){return{s:yT,v:void 0,o:null,p:null}}function $y(l,i={}){let o=ws();const{resultEqualityCheck:r}=i;let c,f=0;function d(){var h;let p=o;const{length:v}=arguments;for(let E=0,w=v;E<w;E++){const S=arguments[E];if(typeof S=="function"||typeof S=="object"&&S!==null){let R=p.o;R===null&&(p.o=R=new WeakMap);const M=R.get(S);M===void 0?(p=ws(),R.set(S,p)):p=M}else{let R=p.p;R===null&&(p.p=R=new Map);const M=R.get(S);M===void 0?(p=ws(),R.set(S,p)):p=M}}const m=p;let y;if(p.s===vv)y=p.v;else if(y=l.apply(null,arguments),f++,r){const E=((h=c==null?void 0:c.deref)==null?void 0:h.call(c))??c;E!=null&&r(E,y)&&(y=E,f!==0&&f--),c=typeof y=="object"&&y!==null||typeof y=="function"?new vT(y):y}return m.s=vv,m.v=y,y}return d.clearCache=()=>{o=ws(),d.resetResultsCount()},d.resultsCount=()=>f,d.resetResultsCount=()=>{f=0},d}function Py(l,...i){const o=typeof l=="function"?{memoize:l,memoizeOptions:i}:l,r=(...c)=>{let f=0,d=0,p,v={},m=c.pop();typeof m=="object"&&(v=m,m=c.pop()),oT(m,`createSelector expects an output function after the inputs, but received: [${typeof m}]`);const y={...o,...v},{memoize:h,memoizeOptions:E=[],argsMemoize:w=$y,argsMemoizeOptions:S=[]}=y,R=pv(E),M=pv(S),T=uT(c),B=h(function(){return f++,m.apply(null,arguments)},...R),_=w(function(){d++;const U=cT(T,arguments);return p=B.apply(null,U),p},...M);return Object.assign(_,{resultFunc:m,memoizedResultFunc:B,dependencies:T,dependencyRecomputations:()=>d,resetDependencyRecomputations:()=>{d=0},lastResult:()=>p,recomputations:()=>f,resetRecomputations:()=>{f=0},memoize:h,argsMemoize:w})};return Object.assign(r,{withTypes:()=>r}),r}var bT=Py($y),xT=Object.assign((l,i=bT)=>{rT(l,`createStructuredSelector expects first argument to be an object where each property is a selector, instead received a ${typeof l}`);const o=Object.keys(l),r=o.map(f=>l[f]);return i(r,(...f)=>f.reduce((d,p,v)=>(d[o[v]]=p,d),{}))},{withTypes:()=>xT});Py({memoize:gT,memoizeOptions:{maxSize:1,equalityCheck:Object.is}});const _e=(l,i,o,r,c,f,...d)=>{if(d.length>0)throw new Error(Fn(1));let p;if(l)p=l;else throw new Error("Missing arguments");return p};var Lf={exports:{}},Uf={};/**
 * @license React
 * use-sync-external-store-shim.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var yv;function ST(){if(yv)return Uf;yv=1;var l=Io();function i(h,E){return h===E&&(h!==0||1/h===1/E)||h!==h&&E!==E}var o=typeof Object.is=="function"?Object.is:i,r=l.useState,c=l.useEffect,f=l.useLayoutEffect,d=l.useDebugValue;function p(h,E){var w=E(),S=r({inst:{value:w,getSnapshot:E}}),R=S[0].inst,M=S[1];return f(function(){R.value=w,R.getSnapshot=E,v(R)&&M({inst:R})},[h,w,E]),c(function(){return v(R)&&M({inst:R}),h(function(){v(R)&&M({inst:R})})},[h]),d(w),w}function v(h){var E=h.getSnapshot;h=h.value;try{var w=E();return!o(h,w)}catch{return!0}}function m(h,E){return E()}var y=typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"?m:p;return Uf.useSyncExternalStore=l.useSyncExternalStore!==void 0?l.useSyncExternalStore:y,Uf}var bv;function e0(){return bv||(bv=1,Lf.exports=ST()),Lf.exports}var ET=e0(),Bf={exports:{}},Hf={};/**
 * @license React
 * use-sync-external-store-shim/with-selector.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var xv;function wT(){if(xv)return Hf;xv=1;var l=Io(),i=e0();function o(m,y){return m===y&&(m!==0||1/m===1/y)||m!==m&&y!==y}var r=typeof Object.is=="function"?Object.is:o,c=i.useSyncExternalStore,f=l.useRef,d=l.useEffect,p=l.useMemo,v=l.useDebugValue;return Hf.useSyncExternalStoreWithSelector=function(m,y,h,E,w){var S=f(null);if(S.current===null){var R={hasValue:!1,value:null};S.current=R}else R=S.current;S=p(function(){function T(G){if(!B){if(B=!0,_=G,G=E(G),w!==void 0&&R.hasValue){var Y=R.value;if(w(Y,G))return N=Y}return N=G}if(Y=N,r(_,G))return Y;var D=E(G);return w!==void 0&&w(Y,D)?(_=G,Y):(_=G,N=D)}var B=!1,_,N,U=h===void 0?null:h;return[function(){return T(y())},U===null?void 0:function(){return T(U())}]},[y,h,E,w]);var M=c(m,S[0],S[1]);return d(function(){R.hasValue=!0,R.value=M},[M]),v(M),M},Hf}var Sv;function TT(){return Sv||(Sv=1,Bf.exports=wT()),Bf.exports}var RT=TT();const AT=ud(19),CT=AT?OT:zT;function Ve(l,i,o,r,c){return CT(l,i,o,r,c)}function MT(l,i,o,r,c){const f=x.useCallback(()=>i(l.getSnapshot(),o,r,c),[l,i,o,r,c]);return ET.useSyncExternalStore(l.subscribe,f,f)}function OT(l,i,o,r,c){return MT(l,i,o,r,c)}function zT(l,i,o,r,c){return RT.useSyncExternalStoreWithSelector(l.subscribe,l.getSnapshot,l.getSnapshot,f=>i(f,o,r,c))}class t0{constructor(i){Ut(this,"subscribe",i=>(this.listeners.add(i),()=>{this.listeners.delete(i)}));Ut(this,"getSnapshot",()=>this.state);this.state=i,this.listeners=new Set,this.updateTick=0}setState(i){if(this.state===i)return;this.state=i,this.updateTick+=1;const o=this.updateTick;for(const r of this.listeners){if(o!==this.updateTick)return;r(i)}}update(i){for(const o in i)if(!Object.is(this.state[o],i[o])){this.setState({...this.state,...i});return}}set(i,o){Object.is(this.state[i],o)||this.setState({...this.state,[i]:o})}notifyAll(){const i={...this.state};this.setState(i)}use(i,o,r,c){return Ve(this,i,o,r,c)}}class n0 extends t0{constructor(i,o={},r){super(i),this.context=o,this.selectors=r}useSyncedValue(i,o){x.useDebugValue(i),Oe(()=>{this.state[i]!==o&&this.set(i,o)},[i,o])}useSyncedValueWithCleanup(i,o){const r=this;Oe(()=>(r.state[i]!==o&&r.set(i,o),()=>{r.set(i,void 0)}),[r,i,o])}useSyncedValues(i){const o=this,r=Object.values(i);Oe(()=>{o.update(i)},[o,...r])}useControlledProp(i,o){x.useDebugValue(i);const r=o!==void 0;Oe(()=>{r&&!Object.is(this.state[i],o)&&super.setState({...this.state,[i]:o})},[i,o,r])}select(i,o,r,c){const f=this.selectors[i];return f(this.state,o,r,c)}useState(i,o,r,c){return x.useDebugValue(i),Ve(this,this.selectors[i],o,r,c)}useContextCallback(i,o){x.useDebugValue(i);const r=Ze(o??dn);this.context[i]=r}useStateSetter(i){const o=x.useRef(void 0);return o.current===void 0&&(o.current=r=>{this.set(i,r)}),o.current}observe(i,o){let r;typeof i=="function"?r=i:r=this.selectors[i];let c=r(this.state);return o(c,c,this),this.subscribe(f=>{const d=r(f);if(!Object.is(c,d)){const p=c;c=d,o(d,p,this)}})}}const _T={open:_e(l=>l.open),domReferenceElement:_e(l=>l.domReferenceElement),referenceElement:_e(l=>l.positionReference??l.referenceElement),floatingElement:_e(l=>l.floatingElement),floatingId:_e(l=>l.floatingId)};class Ad extends n0{constructor(o){const{nested:r,noEmit:c,onOpenChange:f,triggerElements:d,...p}=o;super({...p,positionReference:p.referenceElement,domReferenceElement:p.referenceElement},{onOpenChange:f,dataRef:{current:{}},events:uw(),nested:r,noEmit:c,triggerElements:d},_T);Ut(this,"setOpen",(o,r)=>{var c,f;if((!o||!this.state.open||xy(r.event))&&(this.context.dataRef.current.openEvent=o?r.event:void 0),!this.context.noEmit){const d={open:o,reason:r.reason,nativeEvent:r.event,nested:this.context.nested,triggerElement:r.trigger};this.context.events.emit("openchange",d)}(f=(c=this.context).onOpenChange)==null||f.call(c,o,r)})}}function NT(l){const i=l.useState("open");Oe(()=>{if(i&&!l.select("activeTriggerId")&&l.context.triggerElements.size===1){const o=l.context.triggerElements.entries().next();if(!o.done){const[r,c]=o.value;l.update({activeTriggerId:r,activeTriggerElement:c})}}},[i,l])}function DT(l,i,o){const{mounted:r,setMounted:c,transitionStatus:f}=tr(l);i.useSyncedValues({mounted:r,transitionStatus:f});const d=Ze(()=>{var v,m;c(!1),i.update({activeTriggerId:null,activeTriggerElement:null,mounted:!1}),(m=(v=i.context).onOpenChangeComplete)==null||m.call(v,!1)}),p=i.useState("preventUnmountingOnClose");return Ga({enabled:!p,open:l,ref:i.context.popupRef,onComplete(){l||d()}}),{forceUnmount:d,transitionStatus:f}}class Cd{constructor(){this.elementsSet=new Set,this.idMap=new Map}add(i,o){const r=this.idMap.get(i);r!==o&&(r!==void 0&&this.elementsSet.delete(r),this.elementsSet.add(o),this.idMap.set(i,o))}delete(i){const o=this.idMap.get(i);o&&(this.elementsSet.delete(o),this.idMap.delete(i))}hasElement(i){return this.elementsSet.has(i)}hasMatchingElement(i){for(const o of this.elementsSet)if(i(o))return!0;return!1}getById(i){return this.idMap.get(i)}entries(){return this.idMap.entries()}elements(){return this.elementsSet.values()}get size(){return this.idMap.size}}function kT(){return new Ad({open:!1,floatingElement:null,referenceElement:null,triggerElements:new Cd,floatingId:"",nested:!1,noEmit:!1,onOpenChange:void 0})}function jT(){return{open:!1,openProp:void 0,mounted:!1,transitionStatus:"idle",floatingRootContext:kT(),preventUnmountingOnClose:!1,payload:void 0,activeTriggerId:null,activeTriggerElement:null,triggerIdProp:void 0,popupElement:null,positionerElement:null,activeTriggerProps:an,inactiveTriggerProps:an,popupProps:an}}const Ts=_e(l=>l.triggerIdProp??l.activeTriggerId),LT={open:_e(l=>l.openProp??l.open),mounted:_e(l=>l.mounted),transitionStatus:_e(l=>l.transitionStatus),floatingRootContext:_e(l=>l.floatingRootContext),preventUnmountingOnClose:_e(l=>l.preventUnmountingOnClose),payload:_e(l=>l.payload),activeTriggerId:Ts,activeTriggerElement:_e(l=>l.mounted?l.activeTriggerElement:null),isTriggerActive:_e((l,i)=>i!==void 0&&Ts(l)===i),isOpenedByTrigger:_e((l,i)=>i!==void 0&&Ts(l)===i&&l.open),isMountedByTrigger:_e((l,i)=>i!==void 0&&Ts(l)===i&&l.mounted),triggerProps:_e((l,i)=>i?l.activeTriggerProps:l.inactiveTriggerProps),popupProps:_e(l=>l.popupProps),popupElement:_e(l=>l.popupElement),positionerElement:_e(l=>l.positionerElement)};function l0(l){const{open:i=!1,onOpenChange:o,elements:r={}}=l,c=Wo(),f=nu()!=null,d=Rn(()=>new Ad({open:i,onOpenChange:o,referenceElement:r.reference??null,floatingElement:r.floating??null,triggerElements:new Cd,floatingId:c,nested:f,noEmit:!1})).current;return Oe(()=>{const p={open:i,floatingId:c};r.reference!==void 0&&(p.referenceElement=r.reference,p.domReferenceElement=Bt(r.reference)?r.reference:null),r.floating!==void 0&&(p.floatingElement=r.floating),d.update(p)},[i,c,r.reference,r.floating,d]),d.context.onOpenChange=o,d.context.nested=f,d.context.noEmit=!1,d}function UT(l={}){const{nodeId:i,externalTree:o}=l,r=l0(l),c=l.rootContext||r,f={reference:c.useState("referenceElement"),floating:c.useState("floatingElement"),domReference:c.useState("domReferenceElement")},[d,p]=x.useState(null),v=x.useRef(null),m=lu(o);Oe(()=>{f.domReference&&(v.current=f.domReference)},[f.domReference]);const y=Pw({...l,elements:{...f,...d&&{reference:d}}}),h=x.useCallback(Y=>{const D=Bt(Y)?{getBoundingClientRect:()=>Y.getBoundingClientRect(),getClientRects:()=>Y.getClientRects(),contextElement:Y}:Y;p(D),y.refs.setReference(D)},[y.refs]),[E,w]=x.useState(null),[S,R]=x.useState(null);c.useSyncedValue("referenceElement",E),c.useSyncedValue("domReferenceElement",Bt(E)?E:null),c.useSyncedValue("floatingElement",S);const M=x.useCallback(Y=>{(Bt(Y)||Y===null)&&(v.current=Y,w(Y)),(Bt(y.refs.reference.current)||y.refs.reference.current===null||Y!==null&&!Bt(Y))&&y.refs.setReference(Y)},[y.refs,w]),T=x.useCallback(Y=>{R(Y),y.refs.setFloating(Y)},[y.refs]),B=x.useMemo(()=>({...y.refs,setReference:M,setFloating:T,setPositionReference:h,domReference:v}),[y.refs,M,T,h]),_=x.useMemo(()=>({...y.elements,domReference:f.domReference}),[y.elements,f.domReference]),N=c.useState("open"),U=c.useState("floatingId"),G=x.useMemo(()=>({...y,dataRef:c.context.dataRef,open:N,onOpenChange:c.setOpen,events:c.context.events,floatingId:U,refs:B,elements:_,nodeId:i,rootStore:c}),[y,B,_,i,c,N,U]);return Oe(()=>{c.context.dataRef.current.floatingContext=G;const Y=m==null?void 0:m.nodesRef.current.find(D=>D.id===i);Y&&(Y.context=G)}),x.useMemo(()=>({...y,context:G,refs:B,elements:_,rootStore:c}),[y,B,_,G,c])}function BT(l){const{popupStore:i,noEmit:o=!1,treatPopupAsFloatingElement:r=!1,onOpenChange:c}=l,f=Wo(),d=nu()!=null,p=i.useState("open"),v=i.useState("activeTriggerElement"),m=i.useState(r?"popupElement":"positionerElement"),y=i.context.triggerElements,h=Rn(()=>new Ad({open:p,referenceElement:v,floatingElement:m,triggerElements:y,onOpenChange:c,floatingId:f,nested:d,noEmit:o})).current;return Oe(()=>{const E={open:p,floatingId:f,referenceElement:v,floatingElement:m};Bt(v)&&(E.domReferenceElement=v),h.state.positionReference===h.state.referenceElement&&(E.positionReference=v),h.update(E)},[p,f,v,m,h]),h.context.onOpenChange=c,h.context.nested=d,h.context.noEmit=o,h}function a0(l=[]){const i=l.map(m=>m==null?void 0:m.reference),o=l.map(m=>m==null?void 0:m.floating),r=l.map(m=>m==null?void 0:m.item),c=l.map(m=>m==null?void 0:m.trigger),f=x.useCallback(m=>Rs(m,l,"reference"),i),d=x.useCallback(m=>Rs(m,l,"floating"),o),p=x.useCallback(m=>Rs(m,l,"item"),r),v=x.useCallback(m=>Rs(m,l,"trigger"),c);return x.useMemo(()=>({getReferenceProps:f,getFloatingProps:d,getItemProps:p,getTriggerProps:v}),[f,d,p,v])}function Rs(l,i,o){var d;const r=new Map,c=o==="item",f={};o==="floating"&&(f.tabIndex=-1,f[Jf]="");for(const p in l)c&&l&&(p===py||p===vy)||(f[p]=l[p]);for(let p=0;p<i.length;p+=1){let v;const m=(d=i[p])==null?void 0:d[o];typeof m=="function"?v=l?m(l):null:v=m,v&&Ev(f,v,c,r)}return Ev(f,l,c,r),f}function Ev(l,i,o,r){var c;for(const f in i){const d=i[f];o&&(f===py||f===vy)||(f.startsWith("on")?(r.has(f)||r.set(f,[]),typeof d=="function"&&((c=r.get(f))==null||c.push(d),l[f]=(...p)=>{var v;return(v=r.get(f))==null?void 0:v.map(m=>m(...p)).find(m=>m!==void 0)})):l[f]=d)}}const HT="Escape";function iu(l,i,o){switch(l){case"vertical":return i;case"horizontal":return o;default:return i||o}}function As(l,i){return iu(i,l===hd||l===$o,l===fa||l===da)}function Vf(l,i,o){return iu(i,l===$o,o?l===fa:l===da)||l==="Enter"||l===" "||l===""}function VT(l,i,o){return iu(i,o?l===fa:l===da,l===$o)}function YT(l,i,o,r){const c=o?l===da:l===fa,f=l===hd;return i==="both"||i==="horizontal"&&r&&r>1?l===HT:iu(i,c,f)}function qT(l,i){const o="rootStore"in l?l.rootStore:l,r=o.useState("open"),c=o.useState("floatingElement"),f=o.useState("domReferenceElement"),d=o.context.dataRef,{listRef:p,activeIndex:v,onNavigate:m=()=>{},enabled:y=!0,selectedIndex:h=null,allowEscape:E=!1,loopFocus:w=!1,nested:S=!1,rtl:R=!1,virtual:M=!1,focusItemOnOpen:T="auto",focusItemOnHover:B=!0,openOnArrowKeyDown:_=!0,disabledIndices:N=void 0,orientation:U="vertical",parentOrientation:G,cols:Y=1,id:D,resetOnPointerLeave:le=!0,externalTree:ge}=i,me=Qo(c),ue=Tn(me),F=nu(),ae=lu(ge);Oe(()=>{d.current.orientation=U},[d,U]);const he=Wf(f),C=x.useRef(T),k=x.useRef(h??-1),X=x.useRef(null),I=x.useRef(!0),q=Ze(fe=>{m(k.current===-1?null:k.current,fe)}),A=x.useRef(q),O=x.useRef(!!c),W=x.useRef(r),Q=x.useRef(!1),te=x.useRef(!1),P=Tn(N),de=Tn(r),ce=Tn(h),be=Tn(le),De=Ze(()=>{function fe(re){M?ae==null||ae.events.emit("virtualfocus",re):Ho(re,{sync:Q.current,preventScroll:!0})}const oe=p.current[k.current],H=te.current;oe&&fe(oe),(Q.current?re=>re():requestAnimationFrame)(()=>{var Le;const re=p.current[k.current]||oe;if(!re)return;oe||fe(re),Ye&&(H||!I.current)&&((Le=re.scrollIntoView)==null||Le.call(re,{block:"nearest",inline:"nearest"}))})});Oe(()=>{y&&(r&&c?(k.current=h??-1,C.current&&h!=null&&(te.current=!0,q())):O.current&&(k.current=-1,A.current()))},[y,r,c,h,q]),Oe(()=>{if(y){if(!r){Q.current=!1;return}if(c)if(v==null){if(Q.current=!1,ce.current!=null)return;if(O.current&&(k.current=-1,De()),(!W.current||!O.current)&&C.current&&(X.current!=null||C.current===!0&&X.current==null)){let fe=0;const oe=()=>{p.current[0]==null?(fe<2&&(fe?requestAnimationFrame:queueMicrotask)(oe),fe+=1):(k.current=X.current==null||Vf(X.current,U,R)||S?Af(p):$p(p),X.current=null,q())};oe()}}else js(p,v)||(k.current=v,De(),te.current=!1)}},[y,r,c,v,ce,S,p,U,R,q,De,P]),Oe(()=>{var re,Se;if(!y||c||!ae||M||!O.current)return;const fe=ae.nodesRef.current,oe=(Se=(re=fe.find(Le=>Le.id===F))==null?void 0:re.context)==null?void 0:Se.elements.floating,H=el(kt(c)),$=fe.some(Le=>Le.context&&ht(Le.context.elements.floating,H));oe&&!$&&I.current&&oe.focus({preventScroll:!0})},[y,c,ae,F,M]),Oe(()=>{A.current=q,W.current=r,O.current=!!c}),Oe(()=>{r||(X.current=null,C.current=T)},[r,T]);const we=v!=null,Ye=x.useMemo(()=>{function fe(H){if(!de.current)return;const $=p.current.indexOf(H.currentTarget);$!==-1&&k.current!==$&&(k.current=$,q(H))}return{onFocus(H){Q.current=!0,fe(H)},onClick:({currentTarget:H})=>H.focus({preventScroll:!0}),onMouseMove(H){Q.current=!0,te.current=!1,B&&fe(H)},onPointerLeave(H){if(!de.current||!I.current||H.pointerType==="touch")return;Q.current=!0;const $=H.relatedTarget;if(!(!B||p.current.includes($))&&be.current&&(Ho(null,{sync:!0}),k.current=-1,q(H),!M)){const re=ue.current,Se=el(kt(re));re&&ht(re,Se)&&re.focus({preventScroll:!0})}}}},[de,ue,B,p,q,be,M]),xe=x.useCallback(()=>{var fe,oe,H;return G??((H=(oe=(fe=ae==null?void 0:ae.nodesRef.current.find($=>$.id===F))==null?void 0:fe.context)==null?void 0:oe.dataRef)==null?void 0:H.current.orientation)},[F,ae,G]),pe=Ze(fe=>{if(I.current=!1,Q.current=!0,fe.which===229||!de.current&&fe.currentTarget===ue.current)return;if(S&&YT(fe.key,U,R,Y)){As(fe.key,xe())||Dn(fe),o.setOpen(!1,zt(Mf,fe.nativeEvent)),Ft(f)&&(M?ae==null||ae.events.emit("virtualfocus",f):f.focus());return}const oe=k.current,H=Af(p,N),$=$p(p,N);if(he||(fe.key==="Home"&&(Dn(fe),k.current=H,q(fe)),fe.key==="End"&&(Dn(fe),k.current=$,q(fe))),Y>1){const re=Array.from({length:p.current.length},()=>({width:1,height:1})),Se=e2(re,Y),Le=Se.findIndex(Je=>Je!=null&&!_i(p,Je,N)),Xe=Se.reduce((Je,je,Qe)=>je!=null&&!_i(p,je,N)?Qe:Je,-1),ut=Se[PE({current:Se.map(Je=>Je!=null?p.current[Je]:null)},{event:fe,orientation:U,loopFocus:w,rtl:R,cols:Y,disabledIndices:n2([...(typeof N!="function"?N:null)||p.current.map((Je,je)=>_i(p,je,N)?je:void 0),void 0],Se),minIndex:Le,maxIndex:Xe,prevIndex:t2(k.current>$?H:k.current,re,Se,Y,fe.key===$o?"bl":fe.key===(R?fa:da)?"tr":"tl"),stopEvent:!0})];if(ut!=null&&(k.current=ut,q(fe)),U==="both")return}if(As(fe.key,U)){if(Dn(fe),r&&!M&&el(fe.currentTarget.ownerDocument)===fe.currentTarget){k.current=Vf(fe.key,U,R)?H:$,q(fe);return}Vf(fe.key,U,R)?w?oe>=$?E&&oe!==p.current.length?k.current=-1:(Q.current=!1,k.current=H):k.current=gn(p,{startingIndex:oe,disabledIndices:N}):k.current=Math.min($,gn(p,{startingIndex:oe,disabledIndices:N})):w?oe<=H?E&&oe!==-1?k.current=p.current.length:(Q.current=!1,k.current=$):k.current=gn(p,{startingIndex:oe,decrement:!0,disabledIndices:N}):k.current=Math.max(H,gn(p,{startingIndex:oe,decrement:!0,disabledIndices:N})),js(p,k.current)&&(k.current=-1),q(fe)}}),Re=x.useMemo(()=>M&&r&&we&&{"aria-activedescendant":`${D}-${v}`},[M,r,we,D,v]),Ge=x.useMemo(()=>({"aria-orientation":U==="both"?void 0:U,...he?{}:Re,onKeyDown(fe){if(fe.key==="Tab"&&fe.shiftKey&&r&&!M){const oe=tl(fe.nativeEvent);if(oe&&!ht(ue.current,oe))return;Dn(fe),o.setOpen(!1,zt(eu,fe.nativeEvent)),Ft(f)&&f.focus();return}pe(fe)},onPointerMove(){I.current=!0}}),[Re,pe,ue,U,he,o,r,M,f]),ve=x.useMemo(()=>{function fe(H){T==="auto"&&yy(H.nativeEvent)&&(C.current=!M)}function oe(H){C.current=T,T==="auto"&&by(H.nativeEvent)&&(C.current=!0)}return{onKeyDown(H){const $=o.select("open");I.current=!1;const re=H.key.startsWith("Arrow"),Se=VT(H.key,xe(),R),Le=As(H.key,U),Xe=(S?Se:Le)||H.key==="Enter"||H.key.trim()==="";if(M&&$)return pe(H);if(!(!$&&!_&&re)){if(Xe){const ut=As(H.key,xe());X.current=S&&ut?null:H.key}if(S){Se&&(Dn(H),$?(k.current=Af(p,P.current),q(H)):o.setOpen(!0,zt(Mf,H.nativeEvent,H.currentTarget)));return}Le&&(ce.current!=null&&(k.current=ce.current),Dn(H),!$&&_?o.setOpen(!0,zt(Mf,H.nativeEvent,H.currentTarget)):pe(H),$&&q(H))}},onFocus(H){o.select("open")&&!M&&(k.current=-1,q(H))},onPointerDown:oe,onPointerEnter:oe,onMouseDown:fe,onClick:fe}},[pe,P,T,p,S,q,o,_,U,xe,R,ce,M]),Ee=x.useMemo(()=>({...Re,...ve}),[Re,ve]);return x.useMemo(()=>y?{reference:Ee,floating:Ge,item:Ye,trigger:ve}:{},[y,Ee,Ge,ve,Ye])}const GT=new Map([["select","listbox"],["combobox","listbox"],["label",!1]]);function XT(l,i={}){const o="rootStore"in l?l.rootStore:l,r=o.useState("open"),c=o.useState("floatingId"),f=o.useState("domReferenceElement"),d=o.useState("floatingElement"),{role:p="dialog"}=i,v=Wo(),m=(f==null?void 0:f.id)||v,y=x.useMemo(()=>{var B;return((B=Qo(d))==null?void 0:B.id)||c},[d,c]),h=GT.get(p)??p,w=nu()!=null,S=x.useMemo(()=>h==="tooltip"||p==="label"?an:{"aria-haspopup":h==="alertdialog"?"dialog":h,"aria-expanded":"false",...h==="listbox"&&{role:"combobox"},...h==="menu"&&w&&{role:"menuitem"},...p==="select"&&{"aria-autocomplete":"none"},...p==="combobox"&&{"aria-autocomplete":"list"}},[h,w,p]),R=x.useMemo(()=>h==="tooltip"||p==="label"?{[`aria-${p==="label"?"labelledby":"describedby"}`]:r?y:void 0}:{...S,"aria-expanded":r?"true":"false","aria-controls":r?y:void 0,...h==="menu"&&{id:m}},[h,y,r,m,p,S]),M=x.useMemo(()=>{const B={id:y,...h&&{role:h}};return h==="tooltip"||p==="label"?B:{...B,...h==="menu"&&{"aria-labelledby":m}}},[h,y,m,p]),T=x.useCallback(({active:B,selected:_})=>{const N={role:"option",...B&&{id:`${y}-fui-option`}};switch(p){case"select":case"combobox":return{...N,"aria-selected":_}}return{}},[y,p]);return x.useMemo(()=>({reference:R,floating:M,item:T,trigger:S}),[R,M,S,T])}function QT(l,i){const o="rootStore"in l?l.rootStore:l,r=o.context.dataRef,c=o.useState("open"),{listRef:f,elementsRef:d,activeIndex:p,onMatch:v,onTypingChange:m,enabled:y=!0,resetMs:h=750,selectedIndex:E=null}=i,w=jn(),S=x.useRef(""),R=x.useRef(E??p??-1),M=x.useRef(null);Oe(()=>{!c&&E!==null||(w.clear(),M.current=null,S.current!==""&&(S.current=""))},[c,E,w]),Oe(()=>{c&&S.current===""&&(R.current=E??p??-1)},[c,E,p]);const T=Ze(G=>{G?r.current.typing||(r.current.typing=G,m==null||m(G)):r.current.typing&&(r.current.typing=G,m==null||m(G))}),B=Ze(G=>{function Y(he){const C=d==null?void 0:d.current[he];return!C||bd(C)}function D(he,C,k=0){if(he.length===0)return-1;const X=(k%he.length+he.length)%he.length,I=C.toLocaleLowerCase();for(let q=0;q<he.length;q+=1){const A=(X+q)%he.length,O=he[A];if(!(!(O!=null&&O.toLocaleLowerCase().startsWith(I))||!Y(A)))return A}return-1}const le=f.current;if(S.current.length>0&&G.key===" "&&(Dn(G),T(!0)),S.current.length>0&&S.current[0]!==" "&&D(le,S.current)===-1&&G.key!==" "&&T(!1),le==null||G.key.length!==1||G.ctrlKey||G.metaKey||G.altKey)return;c&&G.key!==" "&&(Dn(G),T(!0));const ge=S.current==="";ge&&(R.current=E??p??-1),le.every(he=>{var C,k;return he?((C=he[0])==null?void 0:C.toLocaleLowerCase())!==((k=he[1])==null?void 0:k.toLocaleLowerCase()):!0})&&S.current===G.key&&(S.current="",R.current=M.current),S.current+=G.key,w.start(h,()=>{S.current="",R.current=M.current,T(!1)});const F=((ge?E??p??-1:R.current)??0)+1,ae=D(le,S.current,F);ae!==-1?(v==null||v(ae),M.current=ae):G.key!==" "&&(S.current="",T(!1))}),_=Ze(G=>{const Y=G.relatedTarget,D=o.select("domReferenceElement"),le=o.select("floatingElement"),ge=ht(D,Y),me=ht(le,Y);ge||me||(w.clear(),S.current="",R.current=M.current,T(!1))}),N=x.useMemo(()=>({onKeyDown:B,onBlur:_}),[B,_]),U=x.useMemo(()=>({onKeyDown:B,onBlur:_}),[B,_]);return x.useMemo(()=>y?{reference:N,floating:U}:{},[y,N,U])}let KT=(function(l){return l.nestedDialogs="--nested-dialogs",l})({}),ZT=(function(l){return l[l.open=Ua.open]="open",l[l.closed=Ua.closed]="closed",l[l.startingStyle=Ua.startingStyle]="startingStyle",l[l.endingStyle=Ua.endingStyle]="endingStyle",l.nested="data-nested",l.nestedDialogOpen="data-nested-dialog-open",l})({});const i0=x.createContext(void 0);function IT(){const l=x.useContext(i0);if(l===void 0)throw new Error(Fn(26));return l}const o0="ArrowUp",r0="ArrowDown",s0="ArrowLeft",u0="ArrowRight",c0="Home",f0="End",FT=new Set([s0,u0]),JT=new Set([o0,r0]),WT=new Set([...FT,...JT]);[...WT];const d0=new Set([o0,r0,s0,u0,c0,f0]),$T={...tu,...er,nestedDialogOpen(l){return l?{[ZT.nestedDialogOpen]:""}:null}},PT=x.forwardRef(function(i,o){const{className:r,finalFocus:c,initialFocus:f,render:d,...p}=i,{store:v}=Bi(),m=v.useState("descriptionElementId"),y=v.useState("disablePointerDismissal"),h=v.useState("floatingRootContext"),E=v.useState("popupProps"),w=v.useState("modal"),S=v.useState("mounted"),R=v.useState("nested"),M=v.useState("nestedOpenDialogCount"),T=v.useState("open"),B=v.useState("openMethod"),_=v.useState("titleElementId"),N=v.useState("transitionStatus"),U=v.useState("role");IT(),Ga({open:T,ref:v.context.popupRef,onComplete(){var me,ue;T&&((ue=(me=v.context).onOpenChangeComplete)==null||ue.call(me,!0))}});function G(me){return me==="touch"?v.context.popupRef.current:!0}const Y=f===void 0?G:f,D=M>0,ge=Jt("div",i,{state:{open:T,nested:R,transitionStatus:N,nestedDialogOpen:D},props:[E,{"aria-labelledby":_??void 0,"aria-describedby":m??void 0,role:U,tabIndex:-1,hidden:!S,onKeyDown(me){d0.has(me.key)&&me.stopPropagation()},style:{[KT.nestedDialogs]:M}},p],ref:[o,v.context.popupRef,v.useStateSetter("popupElement")],stateAttributesMapping:$T});return L.jsx(Yy,{context:h,openInteractionType:B,disabled:!S,closeOnFocusOut:!y,initialFocus:Y,returnFocus:c,modal:w!==!1,restoreFocus:"popup",children:ge})});function m0(l){return ud(19)?l:l?"true":void 0}const h0=x.forwardRef(function(i,o){const{cutout:r,...c}=i;let f;if(r){const d=r==null?void 0:r.getBoundingClientRect();f=`polygon(
      0% 0%,
      100% 0%,
      100% 100%,
      0% 100%,
      0% 0%,
      ${d.left}px ${d.top}px,
      ${d.left}px ${d.bottom}px,
      ${d.right}px ${d.bottom}px,
      ${d.right}px ${d.top}px,
      ${d.left}px ${d.top}px
    )`}return L.jsx("div",{ref:o,role:"presentation","data-base-ui-inert":"",...c,style:{position:"fixed",inset:0,userSelect:"none",WebkitUserSelect:"none",clipPath:f}})}),eR=x.forwardRef(function(i,o){const{keepMounted:r=!1,...c}=i,{store:f}=Bi(),d=f.useState("mounted"),p=f.useState("modal"),v=f.useState("open");return d||r?L.jsx(i0.Provider,{value:r,children:L.jsxs(Vy,{ref:o,...c,children:[d&&p===!0&&L.jsx(h0,{ref:f.context.internalBackdropRef,inert:m0(!v)}),i.children]})}):null});function g0(l){const i=x.useRef(!0);i.current&&(i.current=!1,l())}let wv={},Tv={},Rv="";function tR(l){if(typeof document>"u")return!1;const i=kt(l);return en(i).innerWidth-i.documentElement.clientWidth>0}function nR(l){if(!(typeof CSS<"u"&&CSS.supports&&CSS.supports("scrollbar-gutter","stable"))||typeof document>"u")return!1;const o=kt(l),r=o.documentElement,c=o.body,f=ha(r)?r:c,d=f.style.overflowY,p=r.style.scrollbarGutter;r.style.scrollbarGutter="stable",f.style.overflowY="scroll";const v=f.offsetWidth;f.style.overflowY="hidden";const m=f.offsetWidth;return f.style.overflowY=d,r.style.scrollbarGutter=p,v===m}function lR(l){const i=kt(l),o=i.documentElement,r=i.body,c=ha(o)?o:r,f={overflowY:c.style.overflowY,overflowX:c.style.overflowX};return Object.assign(c.style,{overflowY:"hidden",overflowX:"hidden"}),()=>{Object.assign(c.style,f)}}function aR(l){var E;const i=kt(l),o=i.documentElement,r=i.body,c=en(o);let f=0,d=0,p=!1;const v=sl.create();if(md&&(((E=c.visualViewport)==null?void 0:E.scale)??1)!==1)return()=>{};function m(){const w=c.getComputedStyle(o),S=c.getComputedStyle(r),T=(w.scrollbarGutter||"").includes("both-edges")?"stable both-edges":"stable";f=o.scrollTop,d=o.scrollLeft,wv={scrollbarGutter:o.style.scrollbarGutter,overflowY:o.style.overflowY,overflowX:o.style.overflowX},Rv=o.style.scrollBehavior,Tv={position:r.style.position,height:r.style.height,width:r.style.width,boxSizing:r.style.boxSizing,overflowY:r.style.overflowY,overflowX:r.style.overflowX,scrollBehavior:r.style.scrollBehavior};const B=o.scrollHeight>o.clientHeight,_=o.scrollWidth>o.clientWidth,N=w.overflowY==="scroll"||S.overflowY==="scroll",U=w.overflowX==="scroll"||S.overflowX==="scroll",G=Math.max(0,c.innerWidth-r.clientWidth),Y=Math.max(0,c.innerHeight-r.clientHeight),D=parseFloat(S.marginTop)+parseFloat(S.marginBottom),le=parseFloat(S.marginLeft)+parseFloat(S.marginRight),ge=ha(o)?o:r;if(p=nR(l),p){o.style.scrollbarGutter=T,ge.style.overflowY="hidden",ge.style.overflowX="hidden";return}Object.assign(o.style,{scrollbarGutter:T,overflowY:"hidden",overflowX:"hidden"}),(B||N)&&(o.style.overflowY="scroll"),(_||U)&&(o.style.overflowX="scroll"),Object.assign(r.style,{position:"relative",height:D||Y?`calc(100dvh - ${D+Y}px)`:"100dvh",width:le||G?`calc(100vw - ${le+G}px)`:"100vw",boxSizing:"border-box",overflow:"hidden",scrollBehavior:"unset"}),r.scrollTop=f,r.scrollLeft=d,o.setAttribute("data-base-ui-scroll-locked",""),o.style.scrollBehavior="unset"}function y(){Object.assign(o.style,wv),Object.assign(r.style,Tv),p||(o.scrollTop=f,o.scrollLeft=d,o.removeAttribute("data-base-ui-scroll-locked"),o.style.scrollBehavior=Rv)}function h(){y(),v.request(m)}return m(),c.addEventListener("resize",h),()=>{v.cancel(),y(),typeof c.removeEventListener=="function"&&c.removeEventListener("resize",h)}}class iR{constructor(){Ut(this,"lockCount",0);Ut(this,"restore",null);Ut(this,"timeoutLock",Ha.create());Ut(this,"timeoutUnlock",Ha.create());Ut(this,"release",()=>{this.lockCount-=1,this.lockCount===0&&this.restore&&this.timeoutUnlock.start(0,this.unlock)});Ut(this,"unlock",()=>{var i;this.lockCount===0&&this.restore&&((i=this.restore)==null||i.call(this),this.restore=null)})}acquire(i){return this.lockCount+=1,this.lockCount===1&&this.restore===null&&this.timeoutLock.start(0,()=>this.lock(i)),this.release}lock(i){if(this.lockCount===0||this.restore!==null)return;const r=kt(i).documentElement,c=en(r).getComputedStyle(r).overflowY;if(c==="hidden"||c==="clip"){this.restore=dn;return}const f=gy||!tR(i);this.restore=f?lR(i):aR(i)}}const oR=new iR;function p0(l=!0,i=null){Oe(()=>{if(l)return oR.acquire(i)},[l,i])}function rR(l){const i=x.useRef(""),o=x.useCallback(c=>{c.defaultPrevented||(i.current=c.pointerType,l(c,c.pointerType))},[l]);return{onClick:x.useCallback(c=>{if(c.detail===0){l(c,"keyboard");return}"pointerType"in c?l(c,c.pointerType):l(c,i.current),i.current=""},[l]),onPointerDown:o}}function Md(l,i){const o=x.useRef(l),r=Ze(i);Oe(()=>{o.current!==l&&r(o.current)},[l,r]),Oe(()=>{o.current=l},[l])}function v0(l){const[i,o]=x.useState(null),r=Ze((d,p)=>{l||o(p||(gy?"touch":""))});Md(l,d=>{d&&!l&&o(null)});const{onClick:c,onPointerDown:f}=rR(r);return x.useMemo(()=>({openMethod:i,triggerProps:{onClick:c,onPointerDown:f}}),[i,c,f])}function sR(l){const{store:i,parentContext:o,actionsRef:r}=l,c=i.useState("open"),f=i.useState("disablePointerDismissal"),d=i.useState("modal"),p=i.useState("popupElement"),{openMethod:v,triggerProps:m}=v0(c);NT(i);const{forceUnmount:y}=DT(c,i),h=Ze(le=>{const ge=zt(le);return ge.preventUnmountOnClose=()=>{i.set("preventUnmountingOnClose",!0)},ge}),E=x.useCallback(()=>{i.setOpen(!1,h(C2))},[i,h]);x.useImperativeHandle(r,()=>({unmount:y,close:E}),[y,E]);const w=BT({popupStore:i,onOpenChange:i.setOpen,treatPopupAsFloatingElement:!0,noEmit:!0}),[S,R]=x.useState(0),M=S===0,T=XT(w),B=qy(w,{outsidePressEvent(){return i.context.internalBackdropRef.current||i.context.backdropRef.current?"intentional":{mouse:d==="trap-focus"?"sloppy":"intentional",touch:"sloppy"}},outsidePress(le){if(!i.context.outsidePressEnabledRef.current||"button"in le&&le.button!==0||"touches"in le&&le.touches.length!==1)return!1;const ge=tl(le);if(M&&!f){const me=ge;return d&&(i.context.internalBackdropRef.current||i.context.backdropRef.current)?i.context.internalBackdropRef.current===me||i.context.backdropRef.current===me||ht(me,p)&&!(me!=null&&me.hasAttribute("data-base-ui-portal")):!0}return!1},escapeKey:M});p0(c&&d===!0,p);const{getReferenceProps:_,getFloatingProps:N,getTriggerProps:U}=a0([T,B]);i.useContextCallback("onNestedDialogOpen",le=>{R(le+1)}),i.useContextCallback("onNestedDialogClose",()=>{R(0)}),x.useEffect(()=>(o!=null&&o.onNestedDialogOpen&&c&&o.onNestedDialogOpen(S),o!=null&&o.onNestedDialogClose&&!c&&o.onNestedDialogClose(),()=>{o!=null&&o.onNestedDialogClose&&c&&o.onNestedDialogClose()}),[c,o,S]);const G=x.useMemo(()=>_(m),[_,m]),Y=x.useMemo(()=>U(m),[U,m]),D=x.useMemo(()=>N(),[N]);i.useSyncedValues({openMethod:v,activeTriggerProps:G,inactiveTriggerProps:Y,popupProps:D,floatingRootContext:w,nestedOpenDialogCount:S})}const uR={...LT,modal:_e(l=>l.modal),nested:_e(l=>l.nested),nestedOpenDialogCount:_e(l=>l.nestedOpenDialogCount),disablePointerDismissal:_e(l=>l.disablePointerDismissal),openMethod:_e(l=>l.openMethod),descriptionElementId:_e(l=>l.descriptionElementId),titleElementId:_e(l=>l.titleElementId),viewportElement:_e(l=>l.viewportElement),role:_e(l=>l.role)};class cR extends n0{constructor(o){super(fR(o),{popupRef:x.createRef(),backdropRef:x.createRef(),internalBackdropRef:x.createRef(),outsidePressEnabledRef:{current:!0},triggerElements:new Cd,onOpenChange:void 0,onOpenChangeComplete:void 0},uR);Ut(this,"setOpen",(o,r)=>{var p,v,m,y;if(r.preventUnmountOnClose=()=>{this.set("preventUnmountingOnClose",!0)},!o&&r.trigger==null&&this.state.activeTriggerId!=null&&(r.trigger=this.state.activeTriggerElement??void 0),(v=(p=this.context).onOpenChange)==null||v.call(p,o,r),r.isCanceled)return;const c={open:o,nativeEvent:r.event,reason:r.reason,nested:this.state.nested};(m=this.state.floatingRootContext.context.events)==null||m.emit("openchange",c);const f={open:o},d=((y=r.trigger)==null?void 0:y.id)??null;(d||o)&&(f.activeTriggerId=d,f.activeTriggerElement=r.trigger??null),this.update(f)})}}function fR(l={}){return{...jT(),modal:!0,disablePointerDismissal:!1,popupElement:null,viewportElement:null,descriptionElementId:void 0,titleElementId:void 0,openMethod:null,nested:!1,nestedOpenDialogCount:0,role:"dialog",...l}}function dR(l){const{children:i,open:o,defaultOpen:r=!1,onOpenChange:c,onOpenChangeComplete:f,disablePointerDismissal:d=!1,modal:p=!0,actionsRef:v,handle:m,triggerId:y,defaultTriggerId:h=null}=l,E=Bi(!0),w=!!E,S=Rn(()=>(m==null?void 0:m.store)??new cR({open:r,openProp:o,activeTriggerId:h,triggerIdProp:y,modal:p,disablePointerDismissal:d,nested:w})).current;g0(()=>{o===void 0&&S.state.open===!1&&r===!0&&S.update({open:!0,activeTriggerId:h})}),S.useControlledProp("openProp",o),S.useControlledProp("triggerIdProp",y),S.useSyncedValues({disablePointerDismissal:d,nested:w,modal:p}),S.useContextCallback("onOpenChange",c),S.useContextCallback("onOpenChangeComplete",f);const R=S.useState("payload");sR({store:S,actionsRef:v,parentContext:E==null?void 0:E.store.context});const M=x.useMemo(()=>({store:S}),[S]);return L.jsx(Dy.Provider,{value:M,children:typeof i=="function"?i({payload:R}):i})}const mR=x.forwardRef(function(i,o){const{render:r,className:c,id:f,...d}=i,{store:p}=Bi(),v=Xo(f);return p.useSyncedValueWithCleanup("titleElementId",v),Jt("h2",i,{ref:o,props:[{id:v},d]})});function hR({...l}){return L.jsx(dR,{"data-slot":"dialog",...l})}function gR({...l}){return L.jsx(eR,{"data-slot":"dialog-portal",...l})}function pR({className:l,...i}){return L.jsx(lw,{"data-slot":"dialog-overlay",className:mn("fixed inset-0 isolate z-50 bg-black/10 duration-100 supports-backdrop-filter:backdrop-blur-xs data-open:animate-in data-open:fade-in-0 data-closed:animate-out data-closed:fade-out-0",l),...i})}function vR({className:l,children:i,showCloseButton:o=!0,...r}){return L.jsxs(gR,{children:[L.jsx(pR,{}),L.jsxs(PT,{"data-slot":"dialog-content",className:mn("fixed top-1/2 left-1/2 z-50 grid w-full max-w-[calc(100%-2rem)] -translate-x-1/2 -translate-y-1/2 gap-4 rounded-xl bg-background p-4 text-sm ring-1 ring-foreground/10 duration-100 outline-none sm:max-w-md data-open:animate-in data-open:fade-in-0 data-open:zoom-in-95 data-closed:animate-out data-closed:fade-out-0 data-closed:zoom-out-95",l),...r,children:[i,o&&L.jsxs(ky,{"data-slot":"dialog-close",render:L.jsx(Go,{variant:"ghost",className:"absolute top-2 right-2",size:"icon-sm"}),children:[L.jsx(tS,{}),L.jsx("span",{className:"sr-only",children:"Close"})]})]})]})}function yR({className:l,...i}){return L.jsx("div",{"data-slot":"dialog-header",className:mn("flex flex-col gap-2",l),...i})}function bR({className:l,showCloseButton:i=!1,children:o,...r}){return L.jsxs("div",{"data-slot":"dialog-footer",className:mn("-mx-4 -mb-4 flex flex-col-reverse gap-2 rounded-b-xl border-t bg-muted/50 p-4 sm:flex-row sm:justify-end",l),...r,children:[o,i&&L.jsx(ky,{render:L.jsx(Go,{variant:"outline"}),children:"Close"})]})}function xR({className:l,...i}){return L.jsx(mR,{"data-slot":"dialog-title",className:mn("text-base leading-none font-medium",l),...i})}function Nl({className:l,...i}){return L.jsx("label",{"data-slot":"label",className:mn("flex items-center gap-2 text-sm leading-none font-medium select-none group-data-[disabled=true]:pointer-events-none group-data-[disabled=true]:opacity-50 peer-disabled:cursor-not-allowed peer-disabled:opacity-50",l),...i})}let Av=(function(l){return l.checked="data-checked",l.unchecked="data-unchecked",l.indeterminate="data-indeterminate",l.disabled="data-disabled",l.readonly="data-readonly",l.required="data-required",l.valid="data-valid",l.invalid="data-invalid",l.touched="data-touched",l.dirty="data-dirty",l.filled="data-filled",l.focused="data-focused",l})({});function y0(l){return x.useMemo(()=>({checked(i){return l.indeterminate?{}:i?{[Av.checked]:""}:{[Av.unchecked]:""}},...Js}),[l.indeterminate])}function SR(l,i,o,r=!0,c){const[f,d]=x.useState(),p=Xo(c?`${c}-label`:void 0),v=l??i??f;return Oe(()=>{const m=l||i||!r?void 0:ER(o.current,p);f!==m&&d(m)}),v}function ER(l,i){const o=wR(l);if(o)return!o.id&&i&&(o.id=i),o.id||void 0}function wR(l){if(!l)return;const i=l.parentElement;if(i&&i.tagName==="LABEL")return i;const o=l.id;if(o){const c=l.nextElementSibling;if(c&&c.htmlFor===o)return c}const r=l.labels;return r&&r[0]}const b0=x.createContext(void 0);function TR(){const l=x.useContext(b0);if(l===void 0)throw new Error(Fn(14));return l}const RR="data-parent",AR=x.forwardRef(function(i,o){const{checked:r,className:c,defaultChecked:f=!1,"aria-labelledby":d,disabled:p=!1,id:v,indeterminate:m=!1,inputRef:y,name:h,onCheckedChange:E,parent:w=!1,readOnly:S=!1,render:R,required:M=!1,uncheckedValue:T,value:B,nativeButton:_=!1,...N}=i,{clearErrors:U}=dd(),{disabled:G,name:Y,setDirty:D,setFilled:le,setFocused:ge,setTouched:me,state:ue,validationMode:F,validityData:ae,shouldValidateOnChange:he,validation:C}=Jo(),k=_2(),{labelId:X,controlId:I,registerControlId:q,getDescriptionProps:A}=Ws(),O=D2(),W=O==null?void 0:O.parent,Q=W&&O.allValues,te=G||k.disabled||(O==null?void 0:O.disabled)||p,P=Y??h,de=B??P,ce=Xo(),be=Xo();let De=I;Q?De=w?be:`${W.id}-${de}`:v&&(De=v);let we={};Q&&(w?we=O.parent.getParentProps():de&&(we=O.parent.getChildProps(de)));const Ye=Ze(E),{checked:xe=r,indeterminate:pe=m,onCheckedChange:Re,...Ge}=we,ve=O==null?void 0:O.value,Ee=O==null?void 0:O.setValue,fe=O==null?void 0:O.defaultValue,oe=x.useRef(null),H=Rn(()=>Symbol("checkbox-control")),$=x.useRef(!1),{getButtonProps:re,buttonRef:Se}=Fo({disabled:te,native:_}),Le=(O==null?void 0:O.validation)??C,[Xe,ut]=Vs({controlled:de&&ve&&!w?ve.includes(de):xe,default:de&&fe&&!w?fe.includes(de):f,name:"Checkbox",state:"checked"});Oe(()=>{q!==dn&&($.current=!0,q(H.current,De))},[De,O,q,w,H]),x.useEffect(()=>{const ot=H.current;return()=>{!$.current||q===dn||($.current=!1,q(ot,void 0))}},[q,H]),Sd({enabled:!O,id:ce,commit:Le.commit,value:Xe,controlRef:oe,name:P,getValue:()=>Xe});const Je=x.useRef(null),je=Ba(y,Je,Le.inputRef),Qe=SR(d,X,Je,!_,De??void 0);Oe(()=>{Je.current&&(Je.current.indeterminate=pe,Xe&&le(!0))},[Xe,pe,le]),Md(Xe,()=>{O&&!w||(U(P),le(Xe),D(Xe!==ae.initialValue),he()?Le.commit(Xe):Le.commit(Xe,!0))});const He=Li({checked:Xe,disabled:te,name:w?void 0:P,id:_?void 0:De??void 0,required:M,ref:je,style:P?Ly:wd,tabIndex:-1,type:"checkbox","aria-hidden":!0,onChange(ot){if(ot.nativeEvent.defaultPrevented)return;const Tt=ot.target.checked,qe=zt(nr,ot.nativeEvent);if(Re==null||Re(Tt,qe),Ye(Tt,qe),!qe.isCanceled&&(ut(Tt),de&&ve&&Ee&&!w)){const ct=Tt?[...ve,de]:ve.filter(Ot=>Ot!==de);Ee(ct,qe)}},onFocus(){var ot;(ot=oe.current)==null||ot.focus()}},B!==void 0?{value:(O?Xe&&B:B)||""}:an,A,O?Le.getValidationProps:Le.getInputValidationProps),rt=Q?!!xe:Xe,gt=Q&&pe||m;x.useEffect(()=>{if(!W||!de)return;const ot=W.disabledStatesRef.current;return ot.set(de,te),()=>{ot.delete(de)}},[W,te,de]);const pt=x.useMemo(()=>({...ue,checked:rt,disabled:te,readOnly:S,required:M,indeterminate:gt}),[ue,rt,te,S,M,gt]),Lt=y0(pt),it=Jt("span",i,{state:pt,ref:[Se,oe,o,O==null?void 0:O.registerControlRef],props:[{id:_?De??void 0:ce,role:"checkbox","aria-checked":pe?"mixed":Xe,"aria-readonly":S||void 0,"aria-required":M||void 0,"aria-labelledby":Qe,[RR]:w?"":void 0,onFocus(){ge(!0)},onBlur(){const ot=Je.current;ot&&(me(!0),ge(!1),F==="onBlur"&&Le.commit(O?ve:ot.checked))},onClick(ot){var Tt;S||te||(ot.preventDefault(),(Tt=Je.current)==null||Tt.dispatchEvent(new PointerEvent("click",{bubbles:!0,shiftKey:ot.shiftKey,ctrlKey:ot.ctrlKey,altKey:ot.altKey,metaKey:ot.metaKey})))}},A,Le.getValidationProps,N,Ge,re],stateAttributesMapping:Lt});return L.jsxs(b0.Provider,{value:pt,children:[it,!Xe&&!O&&P&&!w&&T!==void 0&&L.jsx("input",{type:"hidden",name:P,value:T}),L.jsx("input",{...He})]})}),CR=x.forwardRef(function(i,o){const{render:r,className:c,keepMounted:f=!1,...d}=i,p=TR(),v=p.checked||p.indeterminate,{mounted:m,transitionStatus:y,setMounted:h}=tr(v),E=x.useRef(null),w={...p,transitionStatus:y};Ga({open:v,ref:E,onComplete(){v||h(!1)}});const S=y0(p),R=x.useMemo(()=>({...S,...er,...Js}),[S]),M=f||m,T=Jt("span",i,{ref:[o,E],state:w,stateAttributesMapping:R,props:d});return M?T:null});function ko({className:l,...i}){return L.jsx(AR,{"data-slot":"checkbox",className:mn("peer relative flex size-4 shrink-0 items-center justify-center rounded-[4px] border border-input transition-colors outline-none group-has-disabled/field:opacity-50 after:absolute after:-inset-x-3 after:-inset-y-2 focus-visible:border-ring focus-visible:ring-3 focus-visible:ring-ring/50 disabled:cursor-not-allowed disabled:opacity-50 aria-invalid:border-destructive aria-invalid:ring-3 aria-invalid:ring-destructive/20 aria-invalid:aria-checked:border-primary dark:bg-input/30 dark:aria-invalid:border-destructive/50 dark:aria-invalid:ring-destructive/40 data-checked:border-primary data-checked:bg-primary data-checked:text-primary-foreground dark:data-checked:bg-primary",l),...i,children:L.jsx(CR,{"data-slot":"checkbox-indicator",className:"grid place-content-center text-current transition-none [&>svg]:size-3.5",children:L.jsx(qv,{})})})}const x0=x.createContext(null),S0=x.createContext(null);function fl(){const l=x.useContext(x0);if(l===null)throw new Error(Fn(60));return l}function E0(){const l=x.useContext(S0);if(l===null)throw new Error(Fn(61));return l}const MR=(l,i)=>Object.is(l,i);function ji(l,i,o){return l==null||i==null?Object.is(l,i):o(l,i)}function OR(l,i,o){return!l||l.length===0?!1:l.some(r=>r===void 0?!1:ji(i,r,o))}function Vo(l,i,o){return!l||l.length===0?-1:l.findIndex(r=>r===void 0?!1:ji(r,i,o))}function zR(l,i,o){return l.filter(r=>!ji(i,r,o))}function id(l){if(l==null)return"";if(typeof l=="string")return l;try{return JSON.stringify(l)}catch{return String(l)}}function w0(l){return l!=null&&l.length>0&&typeof l[0]=="object"&&l[0]!=null&&"items"in l[0]}function _R(l){if(!Array.isArray(l))return l!=null&&"null"in l;const i=l;if(w0(i)){for(const o of i)for(const r of o.items)if(r&&r.value==null&&r.label!=null)return!0;return!1}for(const o of i)if(o&&o.value==null&&o.label!=null)return!0;return!1}function NR(l,i){if(i&&l!=null)return i(l)??"";if(l&&typeof l=="object"){if("label"in l&&l.label!=null)return String(l.label);if("value"in l)return String(l.value)}return id(l)}function Mi(l,i){return i&&l!=null?i(l)??"":l&&typeof l=="object"&&"value"in l&&"label"in l?id(l.value):id(l)}function T0(l,i,o){function r(){return NR(l,o)}if(o&&l!=null)return o(l);if(l&&typeof l=="object"&&"label"in l&&l.label!=null)return l.label;if(i&&!Array.isArray(i))return i[l]??r();if(Array.isArray(i)){const c=i,f=w0(c)?c.flatMap(d=>d.items):c;if(l==null||typeof l!="object"){const d=f.find(p=>p.value===l);return d&&d.label!=null?d.label:r()}if("value"in l){const d=f.find(p=>p&&p.value===l.value);if(d&&d.label!=null)return d.label}}return r()}function DR(l,i,o){return l.reduce((r,c,f)=>(f>0&&r.push(", "),r.push(L.jsx(x.Fragment,{children:T0(c,i,o)},f)),r),[])}const Be={id:_e(l=>l.id),labelId:_e(l=>l.labelId),modal:_e(l=>l.modal),multiple:_e(l=>l.multiple),items:_e(l=>l.items),itemToStringLabel:_e(l=>l.itemToStringLabel),itemToStringValue:_e(l=>l.itemToStringValue),isItemEqualToValue:_e(l=>l.isItemEqualToValue),value:_e(l=>l.value),hasSelectedValue:_e(l=>{const{value:i,multiple:o,itemToStringValue:r}=l;return i==null?!1:o&&Array.isArray(i)?i.length>0:Mi(i,r)!==""}),hasNullItemLabel:_e((l,i)=>i?_R(l.items):!1),open:_e(l=>l.open),mounted:_e(l=>l.mounted),forceMount:_e(l=>l.forceMount),transitionStatus:_e(l=>l.transitionStatus),openMethod:_e(l=>l.openMethod),activeIndex:_e(l=>l.activeIndex),selectedIndex:_e(l=>l.selectedIndex),isActive:_e((l,i)=>l.activeIndex===i),isSelected:_e((l,i,o)=>{const r=l.isItemEqualToValue,c=l.value;return l.multiple?Array.isArray(c)&&c.some(f=>ji(o,f,r)):l.selectedIndex===i&&l.selectedIndex!==null?!0:ji(o,c,r)}),isSelectedByFocus:_e((l,i)=>l.selectedIndex===i),popupProps:_e(l=>l.popupProps),triggerProps:_e(l=>l.triggerProps),triggerElement:_e(l=>l.triggerElement),positionerElement:_e(l=>l.positionerElement),listElement:_e(l=>l.listElement),scrollUpArrowVisible:_e(l=>l.scrollUpArrowVisible),scrollDownArrowVisible:_e(l=>l.scrollDownArrowVisible),hasScrollArrows:_e(l=>l.hasScrollArrows)};function kR(l){const{id:i,value:o,defaultValue:r=null,onValueChange:c,open:f,defaultOpen:d=!1,onOpenChange:p,name:v,autoComplete:m,disabled:y=!1,readOnly:h=!1,required:E=!1,modal:w=!0,actionsRef:S,inputRef:R,onOpenChangeComplete:M,items:T,multiple:B=!1,itemToStringLabel:_,itemToStringValue:N,isItemEqualToValue:U=MR,highlightItemOnHover:G=!0,children:Y}=l,{clearErrors:D}=dd(),{setDirty:le,setTouched:ge,setFocused:me,shouldValidateOnChange:ue,validityData:F,setFilled:ae,name:he,disabled:C,validation:k,validationMode:X}=Jo(),I=xd({id:i}),q=C||y,A=he??v,[O,W]=Vs({controlled:o,default:B?r??Lp:r,name:"Select",state:"value"}),[Q,te]=Vs({controlled:f,default:d,name:"Select",state:"open"}),P=x.useRef([]),de=x.useRef([]),ce=x.useRef(null),be=x.useRef(null),De=x.useRef(0),we=x.useRef(null),Ye=x.useRef([]),xe=x.useRef(!1),pe=x.useRef(!1),Re=x.useRef(null),Ge=x.useRef({allowSelectedMouseUp:!1,allowUnselectedMouseUp:!1}),ve=x.useRef(!1),{mounted:Ee,setMounted:fe,transitionStatus:oe}=tr(Q),{openMethod:H,triggerProps:$}=v0(Q),re=Rn(()=>new t0({id:I,labelId:void 0,modal:w,multiple:B,itemToStringLabel:_,itemToStringValue:N,isItemEqualToValue:U,value:O,open:Q,mounted:Ee,transitionStatus:oe,items:T,forceMount:!1,openMethod:null,activeIndex:null,selectedIndex:null,popupProps:{},triggerProps:{},triggerElement:null,positionerElement:null,listElement:null,scrollUpArrowVisible:!1,scrollDownArrowVisible:!1,hasScrollArrows:!1})).current,Se=Ve(re,Be.activeIndex),Le=Ve(re,Be.selectedIndex),Xe=Ve(re,Be.triggerElement),ut=Ve(re,Be.positionerElement),Je=x.useMemo(()=>B&&Array.isArray(O)&&O.length===0?"":Mi(O,N),[B,O,N]),je=x.useMemo(()=>B&&Array.isArray(O)?O.map(ke=>Mi(ke,N)):Mi(O,N),[B,O,N]),Qe=Tn(re.state.triggerElement);Sd({id:I,commit:k.commit,value:O,controlRef:Qe,name:A,getValue:()=>je});const He=x.useRef(O);Oe(()=>{O!==He.current&&re.set("forceMount",!0)},[re,O]),Oe(()=>{ae(B?Array.isArray(O)&&O.length>0:O!=null)},[B,O,ae]),Oe(function(){if(Q)return;const St=Ye.current;if(B){const vt=Array.isArray(O)?O:[];if(vt.length===0){re.set("selectedIndex",null);return}const Ht=vt[vt.length-1],Vt=Vo(St,Ht,U);re.set("selectedIndex",Vt===-1?null:Vt);return}const Pe=Vo(St,O,U);re.set("selectedIndex",Pe===-1?null:Pe)},[B,Q,O,Ye,U,re]),Md(O,()=>{D(A),le(O!==F.initialValue),ue()?k.commit(O):k.commit(O,!0)});const rt=Ze((ke,St)=>{if(p==null||p(ke,St),!St.isCanceled&&(te(ke),!ke&&(St.reason===eu||St.reason===Ed)&&(ge(!0),me(!1),X==="onBlur"&&k.commit(O)),!ke&&re.state.activeIndex!==null)){const Pe=P.current[re.state.activeIndex];queueMicrotask(()=>{Pe==null||Pe.setAttribute("tabindex","-1")})}}),gt=Ze(()=>{fe(!1),re.set("activeIndex",null),M==null||M(!1)});Ga({enabled:!S,open:Q,ref:ce,onComplete(){Q||gt()}}),x.useImperativeHandle(S,()=>({unmount:gt}),[gt]);const pt=Ze((ke,St)=>{c==null||c(ke,St),!St.isCanceled&&W(ke)}),Lt=Ze(()=>{const ke=re.state.listElement||ce.current;if(!ke)return;const St=ke.scrollTop,Pe=ke.scrollTop+ke.clientHeight,vt=St>1,Ht=Pe<ke.scrollHeight-1;re.state.scrollUpArrowVisible!==vt&&re.set("scrollUpArrowVisible",vt),re.state.scrollDownArrowVisible!==Ht&&re.set("scrollDownArrowVisible",Ht)}),it=l0({open:Q,onOpenChange:rt,elements:{reference:Xe,floating:ut}}),ot=pw(it,{enabled:!h&&!q,event:"mousedown"}),Tt=qy(it,{bubbles:!1}),qe=qT(it,{enabled:!h&&!q,listRef:P,activeIndex:Se,selectedIndex:Le,disabledIndices:Lp,onNavigate(ke){ke===null&&!Q||re.set("activeIndex",ke)},focusItemOnHover:!1}),ct=QT(it,{enabled:!h&&!q&&(Q||!B),listRef:de,activeIndex:Se,selectedIndex:Le,onMatch(ke){Q?re.set("activeIndex",ke):pt(Ye.current[ke],zt("none"))},onTypingChange(ke){xe.current=ke}}),{getReferenceProps:Ot,getFloatingProps:st,getItemProps:_t}=a0([ot,Tt,qe,ct]),Te=x.useMemo(()=>Li(Ot(),$,I?{id:I}:an),[Ot,$,I]);g0(()=>{re.update({popupProps:st(),triggerProps:Te})}),Oe(()=>{re.update({id:I,modal:w,multiple:B,value:O,open:Q,mounted:Ee,transitionStatus:oe,popupProps:st(),triggerProps:Te,items:T,itemToStringLabel:_,itemToStringValue:N,isItemEqualToValue:U,openMethod:H})},[re,I,w,B,O,Q,Ee,oe,st,Te,T,_,N,U,H]);const $e=x.useMemo(()=>({store:re,name:A,required:E,disabled:q,readOnly:h,multiple:B,itemToStringLabel:_,itemToStringValue:N,highlightItemOnHover:G,setValue:pt,setOpen:rt,listRef:P,popupRef:ce,scrollHandlerRef:be,handleScrollArrowVisibility:Lt,scrollArrowsMountedCountRef:De,getItemProps:_t,events:it.context.events,valueRef:we,valuesRef:Ye,labelsRef:de,typingRef:xe,selectionRef:Ge,selectedItemTextRef:Re,validation:k,onOpenChangeComplete:M,keyboardActiveRef:pe,alignItemWithTriggerActiveRef:ve,initialValueRef:He}),[re,A,E,q,h,B,_,N,G,pt,rt,_t,it.context.events,k,M,Lt]),bt=Ba(R,k.inputRef),xt=B&&Array.isArray(O)&&O.length>0,tn=B?void 0:A,ft=x.useMemo(()=>!B||!Array.isArray(O)||!A?null:O.map(ke=>{const St=Mi(ke,N);return L.jsx("input",{type:"hidden",name:A,value:St},St)}),[B,O,A,N]);return L.jsx(x0.Provider,{value:$e,children:L.jsxs(S0.Provider,{value:it,children:[Y,L.jsx("input",{...k.getInputValidationProps({onFocus(){var ke;(ke=re.state.triggerElement)==null||ke.focus({focusVisible:!0})},onChange(ke){if(ke.nativeEvent.defaultPrevented)return;const St=ke.target.value,Pe=zt(nr,ke.nativeEvent);function vt(){if(B)return;const Ht=Ye.current.find(Vt=>Mi(Vt,N).toLowerCase()===St.toLowerCase());Ht!=null&&(le(Ht!==F.initialValue),pt(Ht,Pe),ue()&&k.commit(Ht))}re.set("forceMount",!0),queueMicrotask(vt)}}),id:I&&tn==null?`${I}-hidden-input`:void 0,name:tn,autoComplete:m,value:Je,disabled:q,required:E&&!xt,readOnly:h,ref:bt,style:A?Ly:wd,tabIndex:-1,"aria-hidden":!0}),ft]})})}function jR(l,i){return l??i}function LR(l){const i=l.getBoundingClientRect(),o=window.getComputedStyle(l,"::before"),r=window.getComputedStyle(l,"::after");if(!(o.content!=="none"||r.content!=="none"))return i;const f=parseFloat(o.width)||0,d=parseFloat(o.height)||0,p=parseFloat(r.width)||0,v=parseFloat(r.height)||0,m=Math.max(i.width,f,p),y=Math.max(i.height,d,v),h=m-i.width,E=y-i.height;return{left:i.left-h/2,right:i.right+h/2,top:i.top-E/2,bottom:i.bottom+E/2}}const Cs=2,UR=400,Cv=200,BR={...tw,...Js,value:()=>null},HR=x.forwardRef(function(i,o){const{render:r,className:c,id:f,disabled:d=!1,nativeButton:p=!0,...v}=i,{setTouched:m,setFocused:y,validationMode:h,state:E,disabled:w}=Jo(),{labelId:S}=Ws(),{store:R,setOpen:M,selectionRef:T,validation:B,readOnly:_,required:N,alignItemWithTriggerActiveRef:U,disabled:G,keyboardActiveRef:Y}=fl(),D=w||G||d,le=Ve(R,Be.open),ge=Ve(R,Be.value),me=Ve(R,Be.triggerProps),ue=Ve(R,Be.positionerElement),F=Ve(R,Be.listElement),ae=Ve(R,Be.id),he=Ve(R,Be.labelId),C=Ve(R,Be.hasSelectedValue),k=!C&&le,X=Ve(R,Be.hasNullItemLabel,k),I=f??ae,q=jR(S,he);xd({id:I});const A=Tn(ue),O=x.useRef(null),{getButtonProps:W,buttonRef:Q}=Fo({disabled:D,native:p}),te=Ze(pe=>{R.set("triggerElement",pe)}),P=Ba(o,O,Q,te),de=jn(),ce=jn(),be=jn(),De=jn();x.useEffect(()=>{if(le)return!(C||X)?be.start(UR,()=>{T.current.allowUnselectedMouseUp=!0,T.current.allowSelectedMouseUp=!0}):De.start(Cv,()=>{T.current.allowUnselectedMouseUp=!0,be.start(Cv,()=>{T.current.allowSelectedMouseUp=!0})}),()=>{be.clear(),De.clear()};T.current={allowSelectedMouseUp:!1,allowUnselectedMouseUp:!1},ce.clear()},[le,C,X,T,ce,be,De]);const we=x.useMemo(()=>{var pe;return(F==null?void 0:F.id)??((pe=Qo(ue))==null?void 0:pe.id)},[F,ue]),Ye=Li(me,{id:I,role:"combobox","aria-expanded":le?"true":"false","aria-haspopup":"listbox","aria-controls":le?we:void 0,"aria-labelledby":q,"aria-readonly":_||void 0,"aria-required":N||void 0,tabIndex:D?-1:0,ref:P,onFocus(pe){y(!0),le&&U.current&&M(!1,zt(nr,pe.nativeEvent)),de.start(0,()=>{R.set("forceMount",!0)})},onBlur(pe){ht(ue,pe.relatedTarget)||(m(!0),y(!1),h==="onBlur"&&B.commit(ge))},onPointerMove(){Y.current=!1},onKeyDown(){Y.current=!0},onMouseDown(pe){if(le)return;const Re=kt(pe.currentTarget);function Ge(ve){if(!O.current)return;const Ee=ve.target;if(ht(O.current,Ee)||ht(A.current,Ee)||Ee===O.current)return;const fe=LR(O.current);ve.clientX>=fe.left-Cs&&ve.clientX<=fe.right+Cs&&ve.clientY>=fe.top-Cs&&ve.clientY<=fe.bottom+Cs||M(!1,zt(A2,ve))}ce.start(0,()=>{Re.addEventListener("mouseup",Ge,{once:!0})})}},B.getValidationProps,v,W);Ye.role="combobox";const xe={...E,open:le,disabled:D,value:ge,readOnly:_,placeholder:!C};return Jt("button",i,{ref:[o,O],state:xe,stateAttributesMapping:BR,props:Ye})}),VR={value:()=>null},YR=x.forwardRef(function(i,o){const{className:r,render:c,children:f,placeholder:d,...p}=i,{store:v,valueRef:m}=fl(),y=Ve(v,Be.value),h=Ve(v,Be.items),E=Ve(v,Be.itemToStringLabel),w=Ve(v,Be.hasSelectedValue),S=!w&&d!=null&&f==null,R=Ve(v,Be.hasNullItemLabel,S),M={value:y,placeholder:!w};let T=null;return typeof f=="function"?T=f(y):f!=null?T=f:!w&&d!=null&&!R?T=d:Array.isArray(y)?T=DR(y,h,E):T=T0(y,h,E),Jt("span",i,{state:M,ref:[o,m],props:[{children:T},p],stateAttributesMapping:VR})}),qR=x.forwardRef(function(i,o){const{className:r,render:c,...f}=i,{store:d}=fl(),v={open:Ve(d,Be.open)};return Jt("span",i,{state:v,ref:o,props:[{"aria-hidden":!0,children:"▼"},f],stateAttributesMapping:ew})}),GR=x.createContext(void 0),XR=x.forwardRef(function(i,o){const{store:r}=fl(),c=Ve(r,Be.mounted),f=Ve(r,Be.forceMount);return c||f?L.jsx(GR.Provider,{value:!0,children:L.jsx(Vy,{ref:o,...i})}):null}),R0=x.createContext({register:()=>{},unregister:()=>{},subscribeMapChange:()=>()=>{},elementsRef:{current:[]},nextIndexRef:{current:0}});function QR(){return x.useContext(R0)}function KR(l){const{children:i,elementsRef:o,labelsRef:r,onMapChange:c}=l,f=Ze(c),d=x.useRef(0),p=Rn(IR).current,v=Rn(ZR).current,[m,y]=x.useState(0),h=x.useRef(m),E=Ze((T,B)=>{v.set(T,B??null),h.current+=1,y(h.current)}),w=Ze(T=>{v.delete(T),h.current+=1,y(h.current)}),S=x.useMemo(()=>{const T=new Map;return Array.from(v.keys()).filter(_=>_.isConnected).sort(FR).forEach((_,N)=>{const U=v.get(_)??{};T.set(_,{...U,index:N})}),T},[v,m]);Oe(()=>{if(typeof MutationObserver!="function"||S.size===0)return;const T=new MutationObserver(B=>{const _=new Set,N=U=>_.has(U)?_.delete(U):_.add(U);B.forEach(U=>{U.removedNodes.forEach(N),U.addedNodes.forEach(N)}),_.size===0&&(h.current+=1,y(h.current))});return S.forEach((B,_)=>{_.parentElement&&T.observe(_.parentElement,{childList:!0})}),()=>{T.disconnect()}},[S]),Oe(()=>{h.current===m&&(o.current.length!==S.size&&(o.current.length=S.size),r&&r.current.length!==S.size&&(r.current.length=S.size),d.current=S.size),f(S)},[f,S,o,r,m]),Oe(()=>()=>{o.current=[]},[o]),Oe(()=>()=>{r&&(r.current=[])},[r]);const R=Ze(T=>(p.add(T),()=>{p.delete(T)}));Oe(()=>{p.forEach(T=>T(S))},[p,S]);const M=x.useMemo(()=>({register:E,unregister:w,subscribeMapChange:R,elementsRef:o,labelsRef:r,nextIndexRef:d}),[E,w,R,o,r,d]);return L.jsx(R0.Provider,{value:M,children:i})}function ZR(){return new Map}function IR(){return new Set}function FR(l,i){const o=l.compareDocumentPosition(i);return o&Node.DOCUMENT_POSITION_FOLLOWING||o&Node.DOCUMENT_POSITION_CONTAINED_BY?-1:o&Node.DOCUMENT_POSITION_PRECEDING||o&Node.DOCUMENT_POSITION_CONTAINS?1:0}const JR=x.createContext(void 0);function WR(){const l=x.useContext(JR);return(l==null?void 0:l.direction)??"ltr"}const $R=l=>({name:"arrow",options:l,async fn(i){var I,q;const{x:o,y:r,placement:c,rects:f,platform:d,elements:p,middlewareData:v}=i,{element:m,padding:y=0,offsetParent:h="real"}=Ul(l,i)||{};if(m==null)return{};const E=Sy(y),w={x:o,y:r},S=yd(c),R=vd(S),M=await d.getDimensions(m),T=S==="y",B=T?"top":"left",_=T?"bottom":"right",N=T?"clientHeight":"clientWidth",U=f.reference[R]+f.reference[S]-w[S]-f.floating[R],G=w[S]-f.reference[S],Y=h==="real"?await((I=d.getOffsetParent)==null?void 0:I.call(d,m)):p.floating;let D=p.floating[N]||f.floating[R];(!D||!await((q=d.isElement)==null?void 0:q.call(d,Y)))&&(D=p.floating[N]||f.floating[R]);const le=U/2-G/2,ge=D/2-M[R]/2-1,me=Math.min(E[B],ge),ue=Math.min(E[_],ge),F=me,ae=D-M[R]-ue,he=D/2-M[R]/2+le,C=$f(F,he,ae),k=!v.arrow&&pa(c)!=null&&he!==C&&f.reference[R]/2-(he<F?me:ue)-M[R]/2<0,X=k?he<F?he-F:he-ae:0;return{[S]:w[S]+X,data:{[S]:C,centerOffset:he-C-X,...k&&{alignmentOffset:X}},reset:k}}}),PR=(l,i)=>({...$R(l),options:[l,i]}),eA={name:"hide",async fn(l){var p;const{width:i,height:o,x:r,y:c}=l.rects.reference,f=i===0&&o===0&&r===0&&c===0;return{data:{referenceHidden:((p=(await iT().fn(l)).data)==null?void 0:p.referenceHidden)||f}}}},tA={sideX:"left",sideY:"top"};function A0(l,i,o){const r=l==="inline-start"||l==="inline-end";return{top:"top",right:r?o?"inline-start":"inline-end":"right",bottom:"bottom",left:r?o?"inline-end":"inline-start":"left"}[i]}function Mv(l,i,o){const{rects:r,placement:c}=l;return{side:A0(i,Ln(c),o),align:pa(c)||"center",anchor:{width:r.reference.width,height:r.reference.height},positioner:{width:r.floating.width,height:r.floating.height}}}function nA(l){var ot,Tt;const{anchor:i,positionMethod:o="absolute",side:r="bottom",sideOffset:c=0,align:f="center",alignOffset:d=0,collisionBoundary:p,collisionPadding:v=5,sticky:m=!1,arrowPadding:y=5,disableAnchorTracking:h=!1,keepMounted:E=!1,floatingRootContext:w,mounted:S,collisionAvoidance:R,shiftCrossAxis:M=!1,nodeId:T,adaptiveOrigin:B,lazyFlip:_=!1,externalTree:N}=l,[U,G]=x.useState(null);!S&&U!==null&&G(null);const Y=R.side||"flip",D=R.align||"flip",le=R.fallbackAxisSide||"end",ge=typeof i=="function"?i:void 0,me=Ze(ge),ue=ge?me:i,F=Tn(i),he=WR()==="rtl",C=U||{top:"top",right:"right",bottom:"bottom",left:"left","inline-end":he?"left":"right","inline-start":he?"right":"left"}[r],k=f==="center"?C:`${C}-${f}`;let X=v;const I=1,q=r==="bottom"?I:0,A=r==="top"?I:0,O=r==="right"?I:0,W=r==="left"?I:0;typeof X=="number"?X={top:X+q,right:X+W,bottom:X+A,left:X+O}:X&&(X={top:(X.top||0)+q,right:(X.right||0)+W,bottom:(X.bottom||0)+A,left:(X.left||0)+O});const Q={boundary:p==="clipping-ancestors"?"clippingAncestors":p,padding:X},te=x.useRef(null),P=Tn(c),de=Tn(d),De=[eT(qe=>{const ct=Mv(qe,r,he),Ot=typeof P.current=="function"?P.current(ct):P.current,st=typeof de.current=="function"?de.current(ct):de.current;return{mainAxis:Ot,crossAxis:st,alignmentAxis:st}},[typeof c!="function"?c:0,typeof d!="function"?d:0,he,r])],we=D==="none"&&Y!=="shift",Ye=!we&&(m||M||Y==="shift"),xe=Y==="none"?null:lT({...Q,padding:{top:X.top+I,right:X.right+I,bottom:X.bottom+I,left:X.left+I},mainAxis:!M&&Y==="flip",crossAxis:D==="flip"?"alignment":!1,fallbackAxisSideDirection:le}),pe=we?null:tT(qe=>{const ct=kt(qe.elements.floating).documentElement;return{...Q,rootBoundary:M?{x:0,y:0,width:ct.clientWidth,height:ct.clientHeight}:void 0,mainAxis:D!=="none",crossAxis:Ye,limiter:m||M?void 0:nT(Ot=>{if(!te.current)return{};const{width:st,height:_t}=te.current.getBoundingClientRect(),Te=In(Ln(Ot.placement)),$e=Te==="y"?st:_t,bt=Te==="y"?X.left+X.right:X.top+X.bottom;return{offset:$e/2+bt/2}})}},[Q,m,M,X,D]);Y==="shift"||D==="shift"||f==="center"?De.push(pe,xe):De.push(xe,pe),De.push(aT({...Q,apply({elements:{floating:qe},availableWidth:ct,availableHeight:Ot,rects:st}){const _t=qe.style;_t.setProperty("--available-width",`${ct}px`),_t.setProperty("--available-height",`${Ot}px`);const Te=window.devicePixelRatio||1,{x:$e,y:bt,width:xt,height:tn}=st.reference,ft=(Math.round(($e+xt)*Te)-Math.round($e*Te))/Te,ke=(Math.round((bt+tn)*Te)-Math.round(bt*Te))/Te;_t.setProperty("--anchor-width",`${ft}px`),_t.setProperty("--anchor-height",`${ke}px`)}}),PR(()=>({element:te.current||document.createElement("div"),padding:y,offsetParent:"floating"}),[y]),{name:"transformOrigin",fn(qe){var on,dl,Xa;const{elements:ct,middlewareData:Ot,placement:st,rects:_t,y:Te}=qe,$e=Ln(st),bt=In($e),xt=te.current,tn=((on=Ot.arrow)==null?void 0:on.x)||0,ft=((dl=Ot.arrow)==null?void 0:dl.y)||0,ke=(xt==null?void 0:xt.clientWidth)||0,St=(xt==null?void 0:xt.clientHeight)||0,Pe=tn+ke/2,vt=ft+St/2,Ht=Math.abs(((Xa=Ot.shift)==null?void 0:Xa.y)||0),Vt=_t.reference.height/2,pn=typeof c=="function"?c(Mv(qe,r,he)):c,lr=Ht>pn,nl={top:`${Pe}px calc(100% + ${pn}px)`,bottom:`${Pe}px ${-pn}px`,left:`calc(100% + ${pn}px) ${vt}px`,right:`${-pn}px ${vt}px`}[$e],Wt=`${Pe}px ${_t.reference.y+Vt-Te}px`;return ct.floating.style.setProperty("--transform-origin",Ye&&bt==="y"&&lr?Wt:nl),{}}},eA,B),Oe(()=>{!S&&w&&w.update({referenceElement:null,floatingElement:null,domReferenceElement:null})},[S,w]);const Re=x.useMemo(()=>({elementResize:!h&&typeof ResizeObserver<"u",layoutShift:!h&&typeof IntersectionObserver<"u"}),[h]),{refs:Ge,elements:ve,x:Ee,y:fe,middlewareData:oe,update:H,placement:$,context:re,isPositioned:Se,floatingStyles:Le}=UT({rootContext:w,placement:k,middleware:De,strategy:o,whileElementsMounted:E?void 0:(...qe)=>hv(...qe,Re),nodeId:T,externalTree:N}),{sideX:Xe,sideY:ut}=oe.adaptiveOrigin||tA,Je=Se?o:"fixed",je=x.useMemo(()=>{const qe=B?{position:Je,[Xe]:Ee,[ut]:fe}:{position:Je,...Le};return Se||(qe.opacity=0),qe},[B,Je,Xe,Ee,ut,fe,Le,Se]),Qe=x.useRef(null);Oe(()=>{if(!S)return;const qe=F.current,ct=typeof qe=="function"?qe():qe,st=(Ov(ct)?ct.current:ct)||null||null;st!==Qe.current&&(Ge.setPositionReference(st),Qe.current=st)},[S,Ge,ue,F]),x.useEffect(()=>{if(!S)return;const qe=F.current;typeof qe!="function"&&Ov(qe)&&qe.current!==Qe.current&&(Ge.setPositionReference(qe.current),Qe.current=qe.current)},[S,Ge,ue,F]),x.useEffect(()=>{if(E&&S&&ve.domReference&&ve.floating)return hv(ve.domReference,ve.floating,H,Re)},[E,S,ve,H,Re]);const He=Ln($),rt=A0(r,He,he),gt=pa($)||"center",pt=!!((ot=oe.hide)!=null&&ot.referenceHidden);Oe(()=>{_&&S&&Se&&G(He)},[_,S,Se,He]);const Lt=x.useMemo(()=>{var qe,ct;return{position:"absolute",top:(qe=oe.arrow)==null?void 0:qe.y,left:(ct=oe.arrow)==null?void 0:ct.x}},[oe.arrow]),it=((Tt=oe.arrow)==null?void 0:Tt.centerOffset)!==0;return x.useMemo(()=>({positionerStyles:je,arrowStyles:Lt,arrowRef:te,arrowUncentered:it,side:rt,align:gt,physicalSide:He,anchorHidden:pt,refs:Ge,context:re,isPositioned:Se,update:H}),[je,Lt,te,it,rt,gt,He,pt,Ge,re,Se,H])}function Ov(l){return l!=null&&"current"in l}const C0=x.createContext(void 0);function Od(){const l=x.useContext(C0);if(!l)throw new Error(Fn(59));return l}function M0(l){return l==="starting"?OS:an}function Qs(l,i){l&&Object.assign(l.style,i)}const O0={position:"relative",maxHeight:"100%",overflowX:"hidden",overflowY:"auto"},lA={position:"fixed"},aA=x.forwardRef(function(i,o){const{anchor:r,positionMethod:c="absolute",className:f,render:d,side:p="bottom",align:v="center",sideOffset:m=0,alignOffset:y=0,collisionBoundary:h="clipping-ancestors",collisionPadding:E,arrowPadding:w=5,sticky:S=!1,disableAnchorTracking:R,alignItemWithTrigger:M=!0,collisionAvoidance:T=_S,...B}=i,{store:_,listRef:N,labelsRef:U,alignItemWithTriggerActiveRef:G,selectedItemTextRef:Y,valuesRef:D,initialValueRef:le,popupRef:ge,setValue:me}=fl(),ue=E0(),F=Ve(_,Be.open),ae=Ve(_,Be.mounted),he=Ve(_,Be.modal),C=Ve(_,Be.value),k=Ve(_,Be.openMethod),X=Ve(_,Be.positionerElement),I=Ve(_,Be.triggerElement),q=Ve(_,Be.isItemEqualToValue),A=Ve(_,Be.transitionStatus),O=x.useRef(null),W=x.useRef(null),[Q,te]=x.useState(M),P=ae&&Q&&k!=="touch";!ae&&Q!==M&&te(M),Oe(()=>{ae||(Be.scrollUpArrowVisible(_.state)&&_.set("scrollUpArrowVisible",!1),Be.scrollDownArrowVisible(_.state)&&_.set("scrollDownArrowVisible",!1))},[_,ae]),x.useImperativeHandle(G,()=>P),p0((P||he)&&F&&k!=="touch",I);const de=nA({anchor:r,floatingRootContext:ue,positionMethod:c,mounted:ae,side:p,sideOffset:m,align:v,alignOffset:y,arrowPadding:w,collisionBoundary:h,collisionPadding:E,sticky:S,disableAnchorTracking:R??P,collisionAvoidance:T,keepMounted:!0}),ce=P?"none":de.side,be=P?lA:de.positionerStyles,De=x.useMemo(()=>{const ve={};return F||(ve.pointerEvents="none"),{role:"presentation",hidden:!ae,style:{...be,...ve}}},[F,ae,be]),we={open:F,side:ce,align:de.align,anchorHidden:de.anchorHidden},Ye=Ze(ve=>{_.set("positionerElement",ve)}),xe=Jt("div",i,{ref:[o,Ye],state:we,stateAttributesMapping:tu,props:[De,M0(A),B]}),pe=x.useRef(0),Re=Ze(ve=>{if(ve.size===0&&pe.current===0||D.current.length===0)return;const Ee=pe.current;if(pe.current=ve.size,ve.size===Ee)return;const fe=zt(nr);if(Ee!==0&&!_.state.multiple&&C!==null&&Vo(D.current,C,q)===-1){const H=le.current,re=H!=null&&Vo(D.current,H,q)!==-1?H:null;me(re,fe),re===null&&(_.set("selectedIndex",null),Y.current=null)}if(Ee!==0&&_.state.multiple&&Array.isArray(C)){const oe=$=>Vo(D.current,$,q)!==-1,H=C.filter($=>oe($));(H.length!==C.length||H.some($=>!OR(C,$,q)))&&(me(H,fe),H.length===0&&(_.set("selectedIndex",null),Y.current=null))}if(F&&P){_.update({scrollUpArrowVisible:!1,scrollDownArrowVisible:!1});const oe={height:""};Qs(X,oe),Qs(ge.current,oe)}}),Ge=x.useMemo(()=>({...de,side:ce,alignItemWithTriggerActive:P,setControlledAlignItemWithTrigger:te,scrollUpArrowRef:O,scrollDownArrowRef:W}),[de,ce,P,te]);return L.jsx(KR,{elementsRef:N,labelsRef:U,onMapChange:Re,children:L.jsxs(C0.Provider,{value:Ge,children:[ae&&he&&L.jsx(h0,{inert:m0(!F),cutout:I}),xe]})})});function z0(l){const i=l.currentTarget.getBoundingClientRect();return i.top+1<=l.clientY&&l.clientY<=i.bottom-1&&i.left+1<=l.clientX&&l.clientX<=i.right-1}const Ms="base-ui-disable-scrollbar",od={className:Ms,getElement(l){return L.jsx("style",{nonce:l,href:Ms,precedence:"base-ui:low",children:`.${Ms}{scrollbar-width:none}.${Ms}::-webkit-scrollbar{display:none}`})}},iA=x.createContext(void 0);function oA(l){return x.useContext(iA)}function Yf(l,i=Number.MIN_SAFE_INTEGER,o=Number.MAX_SAFE_INTEGER){return Math.max(i,Math.min(l,o))}const rA=x.createContext(void 0),sA={disableStyleElements:!1};function uA(){return x.useContext(rA)??sA}const Dl=1,cA={...tu,...er},fA=x.forwardRef(function(i,o){const{render:r,className:c,finalFocus:f,...d}=i,{store:p,popupRef:v,onOpenChangeComplete:m,setOpen:y,valueRef:h,selectedItemTextRef:E,keyboardActiveRef:w,multiple:S,handleScrollArrowVisibility:R,scrollHandlerRef:M,highlightItemOnHover:T}=fl(),{side:B,align:_,alignItemWithTriggerActive:N,setControlledAlignItemWithTrigger:U,scrollDownArrowRef:G,scrollUpArrowRef:Y}=Od(),D=oA()!=null,le=E0(),{nonce:ge,disableStyleElements:me}=uA(),ue=jn(),F=Ve(p,Be.id),ae=Ve(p,Be.open),he=Ve(p,Be.mounted),C=Ve(p,Be.popupProps),k=Ve(p,Be.transitionStatus),X=Ve(p,Be.triggerElement),I=Ve(p,Be.positionerElement),q=Ve(p,Be.listElement),A=x.useRef(0),O=x.useRef(!1),W=x.useRef(0),Q=x.useRef(!1),te=x.useRef({}),P=Ps(),de=Ze(we=>{if(!I||!v.current||!Q.current)return;if(O.current||!N){R();return}const Ye=I.style.top==="0px",xe=I.style.bottom==="0px",pe=I.getBoundingClientRect().height,Re=kt(I),Ge=getComputedStyle(I),ve=parseFloat(Ge.marginTop),Ee=parseFloat(Ge.marginBottom),fe=zv(getComputedStyle(v.current)),oe=Math.min(Re.documentElement.clientHeight-ve-Ee,fe),H=we.scrollTop,$=Os(we);let re=0,Se=null,Le=!1,Xe=!1;const ut=je=>{I.style.height=`${je}px`},Je=(je,Qe)=>{const He=Yf(je,0,oe-pe);He>0&&ut(pe+He),we.scrollTop=Qe,oe-(pe+He)<=Dl&&(O.current=!0),R()};if(Ye){const je=$-H,Qe=pe+je,He=Math.min(Qe,oe);if(re=He,je<=Dl){Je(je,$);return}oe-He>Dl?Xe=!0:Le=!0}else if(xe){const je=H,Qe=pe+je,He=Math.min(Qe,oe),rt=Qe-oe;if(re=He,je<=Dl){Je(je,0);return}oe-He>Dl?Se=0:(Le=!0,H<$&&(Se=H-(je-rt)))}if(re=Math.ceil(re),re!==0&&ut(re),Xe||Se!=null){const je=Os(we),Qe=Xe?je:Yf(Se,0,je);Math.abs(we.scrollTop-Qe)>Dl&&(we.scrollTop=Qe)}(Le||re>=oe-Dl)&&(O.current=!0),R()});x.useImperativeHandle(M,()=>de,[de]),Ga({open:ae,ref:v,onComplete(){ae&&(m==null||m(!0))}});const ce={open:ae,transitionStatus:k,side:B,align:_};Oe(()=>{!I||!v.current||Object.keys(te.current).length||(te.current={top:I.style.top||"0",left:I.style.left||"0",right:I.style.right,height:I.style.height,bottom:I.style.bottom,minHeight:I.style.minHeight,maxHeight:I.style.maxHeight,marginTop:I.style.marginTop,marginBottom:I.style.marginBottom})},[v,I]),Oe(()=>{ae||N||(Q.current=!1,O.current=!1,A.current=0,W.current=0,Qs(I,te.current))},[ae,N,I,v]),Oe(()=>{const we=v.current;if(!(!ae||!X||!I||!we||p.state.transitionStatus==="ending")){if(!N){Q.current=!0,P.request(R),we.style.removeProperty("--transform-origin");return}queueMicrotask(()=>{var xe;const Ye=mA(we);we.style.removeProperty("--transform-origin");try{const pe=getComputedStyle(I),Re=getComputedStyle(we),Ge=kt(X),ve=en(I),Ee=dA(X),fe=zs(X.getBoundingClientRect(),Ee),oe=zs(I.getBoundingClientRect(),Ee),H=fe.left,$=fe.height,re=q||we,Se=re.scrollHeight,Le=parseFloat(Re.borderBottomWidth),Xe=parseFloat(pe.marginTop)||10,ut=parseFloat(pe.marginBottom)||10,Je=parseFloat(pe.minHeight)||100,je=zv(Re),Qe=5,He=5,rt=20,gt=Ge.documentElement.clientHeight-Xe-ut,pt=Ge.documentElement.clientWidth,Lt=gt-fe.bottom+$,it=E.current,ot=h.current;let Tt,qe=0,ct=0;if(it&&ot){const Pe=zs(ot.getBoundingClientRect(),Ee);Tt=zs(it.getBoundingClientRect(),Ee);const vt=Pe.left-H,Ht=Tt.left-oe.left,Vt=Pe.top-fe.top+Pe.height/2,pn=Tt.top-oe.top+Tt.height/2;qe=vt-Ht,ct=pn-Vt}const Ot=Lt+ct+ut+Le;let st=Math.min(gt,Ot);const _t=gt-Xe-ut,Te=Ot-st,$e=Math.max(Qe,H+qe),bt=pt-He,xt=Math.max(0,$e+oe.width-bt);I.style.left=`${$e-xt}px`,I.style.height=`${st}px`,I.style.maxHeight="auto",I.style.marginTop=`${Xe}px`,I.style.marginBottom=`${ut}px`,we.style.height="100%";const tn=Os(re),ft=Te>=tn-Dl;ft&&(st=Math.min(gt,oe.height)-(Te-tn));const ke=fe.top<rt||fe.bottom>gt-rt||Math.ceil(st)+Dl<Math.min(Se,Je),St=(((xe=ve.visualViewport)==null?void 0:xe.scale)??1)!==1&&md;if(ke||St){Q.current=!0,Qs(I,te.current),Ya.flushSync(()=>U(!1));return}if(ft){const Pe=Math.max(0,gt-Ot);I.style.top=oe.height>=_t?"0":`${Pe}px`,I.style.height=`${st}px`,re.scrollTop=Os(re),A.current=Math.max(Je,st)}else I.style.bottom="0",A.current=Math.max(Je,st),re.scrollTop=Te;if(Tt){const Pe=oe.top,vt=oe.height,Ht=Tt.top+Tt.height/2,Vt=vt>0?(Ht-Pe)/vt*100:50,pn=Yf(Vt,0,100);we.style.setProperty("--transform-origin",`50% ${pn}%`)}(A.current===gt||st>=je)&&(O.current=!0),R(),setTimeout(()=>{Q.current=!0})}finally{Ye()}})}},[p,ae,I,X,h,E,v,R,N,U,P,G,Y,q]),x.useEffect(()=>{if(!N||!I||!ae)return;const we=en(I);function Ye(xe){y(!1,zt(M2,xe))}return we.addEventListener("resize",Ye),()=>{we.removeEventListener("resize",Ye)}},[y,N,I,ae]);const be={...q?{role:"presentation","aria-orientation":void 0}:{role:"listbox","aria-multiselectable":S||void 0,id:`${F}-list`},onKeyDown(we){w.current=!0,D&&d0.has(we.key)&&we.stopPropagation()},onMouseMove(){w.current=!1},onPointerLeave(we){if(!T||z0(we)||we.pointerType==="touch")return;const Ye=we.currentTarget;ue.start(0,()=>{p.set("activeIndex",null),Ye.focus({preventScroll:!0})})},onScroll(we){q||de(we.currentTarget)},...N&&{style:q?{height:"100%"}:O0}},De=Jt("div",i,{ref:[o,v],state:ce,stateAttributesMapping:cA,props:[C,be,M0(k),{className:!q&&N?od.className:void 0},d]});return L.jsxs(x.Fragment,{children:[!me&&od.getElement(ge),L.jsx(Yy,{context:le,modal:!1,disabled:!he,returnFocus:f,restoreFocus:!0,children:De})]})});function zv(l){const i=l.maxHeight||"";return i.endsWith("px")&&parseFloat(i)||1/0}function Os(l){return Math.max(0,l.scrollHeight-l.clientHeight)}function dA(l){return Fy.getScale(l)}function zs(l,i){return Ko({x:l.x/i.x,y:l.y/i.y,width:l.width/i.x,height:l.height/i.y})}const _v=[["transform","none"],["scale","1"],["translate","0 0"]];function mA(l){const{style:i}=l,o={};for(const[r,c]of _v)o[r]=i.getPropertyValue(r),i.setProperty(r,c,"important");return()=>{for(const[r]of _v){const c=o[r];c?i.setProperty(r,c):i.removeProperty(r)}}}const hA=x.forwardRef(function(i,o){const{className:r,render:c,...f}=i,{store:d,scrollHandlerRef:p}=fl(),{alignItemWithTriggerActive:v}=Od(),m=Ve(d,Be.hasScrollArrows),y=Ve(d,Be.openMethod),h=Ve(d,Be.multiple),w={id:`${Ve(d,Be.id)}-list`,role:"listbox","aria-multiselectable":h||void 0,onScroll(R){var M;(M=p.current)==null||M.call(p,R.currentTarget)},...v&&{style:O0},className:m&&y!=="touch"?od.className:void 0},S=Ze(R=>{d.set("listElement",R)});return Jt("div",i,{ref:[o,S],props:[w,f]})});let _0=(function(l){return l[l.None=0]="None",l[l.GuessFromOrder=1]="GuessFromOrder",l})({});function gA(l={}){const{label:i,metadata:o,textRef:r,indexGuessBehavior:c,index:f}=l,{register:d,unregister:p,subscribeMapChange:v,elementsRef:m,labelsRef:y,nextIndexRef:h}=QR(),E=x.useRef(-1),[w,S]=x.useState(f??(c===_0.GuessFromOrder?()=>{if(E.current===-1){const T=h.current;h.current+=1,E.current=T}return E.current}:-1)),R=x.useRef(null),M=x.useCallback(T=>{var B;if(R.current=T,w!==-1&&T!==null&&(m.current[w]=T,y)){const _=i!==void 0;y.current[w]=_?i:((B=r==null?void 0:r.current)==null?void 0:B.textContent)??T.textContent}},[w,m,y,i,r]);return Oe(()=>{if(f!=null)return;const T=R.current;if(T)return d(T,o),()=>{p(T)}},[f,d,p,o]),Oe(()=>{if(f==null)return v(T=>{var _;const B=R.current?(_=T.get(R.current))==null?void 0:_.index:null;B!=null&&S(B)})},[f,v,S]),x.useMemo(()=>({ref:M,index:w}),[w,M])}const N0=x.createContext(void 0);function zd(){const l=x.useContext(N0);if(!l)throw new Error(Fn(57));return l}const pA=x.memo(x.forwardRef(function(i,o){const{render:r,className:c,value:f=null,label:d,disabled:p=!1,nativeButton:v=!1,...m}=i,y=x.useRef(null),h=gA({label:d,textRef:y,indexGuessBehavior:_0.GuessFromOrder}),{store:E,getItemProps:w,setOpen:S,setValue:R,selectionRef:M,typingRef:T,valuesRef:B,keyboardActiveRef:_,multiple:N,highlightItemOnHover:U}=fl(),G=jn(),Y=Ve(E,Be.isActive,h.index),D=Ve(E,Be.isSelected,h.index,f),le=Ve(E,Be.isSelectedByFocus,h.index),ge=Ve(E,Be.isItemEqualToValue),me=h.index,ue=me!==-1,F=x.useRef(null),ae=Tn(me);Oe(()=>{if(!ue)return;const P=B.current;return P[me]=f,()=>{delete P[me]}},[ue,me,f,B]),Oe(()=>{if(!ue)return;const P=E.state.value;let de=P;N&&Array.isArray(P)&&P.length>0&&(de=P[P.length-1]),de!==void 0&&ji(f,de,ge)&&E.set("selectedIndex",me)},[ue,me,N,ge,E,f]);const he={disabled:p,selected:D,highlighted:Y},C=w({active:Y,selected:D});C.onFocus=void 0,C.id=void 0;const k=x.useRef(null),X=x.useRef("mouse"),I=x.useRef(!1),{getButtonProps:q,buttonRef:A}=Fo({disabled:p,focusableWhenDisabled:!0,native:v,composite:!0});function O(P){const de=E.state.value;if(N){const ce=Array.isArray(de)?de:[],be=D?zR(ce,f,ge):[...ce,f];R(be,zt(Cf,P))}else R(f,zt(Cf,P)),S(!1,zt(Cf,P))}const W={role:"option","aria-selected":D,tabIndex:Y?0:-1,onFocus(){E.set("activeIndex",me)},onMouseEnter(){!_.current&&E.state.selectedIndex===null&&U&&E.set("activeIndex",me)},onMouseMove(){U&&E.set("activeIndex",me)},onMouseLeave(P){!U||_.current||z0(P)||G.start(0,()=>{E.state.activeIndex===me&&E.set("activeIndex",null)})},onTouchStart(){M.current={allowSelectedMouseUp:!1,allowUnselectedMouseUp:!1}},onKeyDown(P){k.current=P.key,E.set("activeIndex",me),P.key===" "&&T.current&&P.preventDefault()},onClick(P){I.current=!1,!(P.type==="keydown"&&k.current===null)&&(p||P.type==="keydown"&&k.current===" "&&T.current||X.current!=="touch"&&!Y||(k.current=null,O(P.nativeEvent)))},onPointerEnter(P){X.current=P.pointerType},onPointerDown(P){X.current=P.pointerType,I.current=!0},onMouseUp(){var ce;if(p)return;if(I.current){I.current=!1;return}const P=!M.current.allowSelectedMouseUp&&D,de=!M.current.allowUnselectedMouseUp&&!D;P||de||X.current!=="touch"&&!Y||(ce=F.current)==null||ce.click()}},Q=Jt("div",i,{ref:[A,o,h.ref,F],state:he,props:[C,W,m,q]}),te=x.useMemo(()=>({selected:D,indexRef:ae,textRef:y,selectedByFocus:le,hasRegistered:ue}),[D,ae,y,le,ue]);return L.jsx(N0.Provider,{value:te,children:Q})})),vA=x.forwardRef(function(i,o){const r=i.keepMounted??!1,{selected:c}=zd();return r||c?L.jsx(yA,{...i,ref:o}):null}),yA=x.memo(x.forwardRef((l,i)=>{const{render:o,className:r,keepMounted:c,...f}=l,{selected:d}=zd(),p=x.useRef(null),{transitionStatus:v,setMounted:m}=tr(d),h=Jt("span",l,{ref:[i,p],state:{selected:d,transitionStatus:v},props:[{"aria-hidden":!0,children:"✔️"},f],stateAttributesMapping:er});return Ga({open:d,ref:p,onComplete(){d||m(!1)}}),h})),bA=x.memo(x.forwardRef(function(i,o){const{indexRef:r,textRef:c,selectedByFocus:f,hasRegistered:d}=zd(),{selectedItemTextRef:p}=fl(),{className:v,render:m,...y}=i,h=x.useCallback(w=>{if(!w||!d)return;const S=p.current===null||!p.current.isConnected;(f||S&&r.current===0)&&(p.current=w)},[p,r,f,d]);return Jt("div",i,{ref:[h,o,c],props:y})})),D0=x.forwardRef(function(i,o){const{render:r,className:c,direction:f,keepMounted:d=!1,...p}=i,{store:v,popupRef:m,listRef:y,handleScrollArrowVisibility:h,scrollArrowsMountedCountRef:E}=fl(),{side:w,scrollDownArrowRef:S,scrollUpArrowRef:R}=Od(),M=f==="up"?Be.scrollUpArrowVisible:Be.scrollDownArrowVisible,T=Ve(v,M),B=Ve(v,Be.openMethod),_=T&&B!=="touch",N=jn(),U=f==="up"?R:S,{transitionStatus:G,setMounted:Y}=tr(_);Oe(()=>(E.current+=1,v.state.hasScrollArrows||v.set("hasScrollArrows",!0),()=>{E.current=Math.max(0,E.current-1),E.current===0&&v.state.hasScrollArrows&&v.set("hasScrollArrows",!1)}),[v,E]),Ga({open:_,ref:U,onComplete(){_||Y(!1)}});const ge=Jt("div",i,{ref:[o,U],state:{direction:f,visible:_,side:w,transitionStatus:G},props:[{"aria-hidden":!0,children:f==="up"?"▲":"▼",style:{position:"absolute"},onMouseMove(ue){if(ue.movementX===0&&ue.movementY===0||N.isStarted())return;v.set("activeIndex",null);function F(){var X;const ae=v.state.listElement??m.current;if(!ae)return;v.set("activeIndex",null),h();const he=ae.scrollTop===0,C=Math.round(ae.scrollTop+ae.clientHeight)>=ae.scrollHeight;if(y.current.length===0&&(f==="up"?v.set("scrollUpArrowVisible",!he):v.set("scrollDownArrowVisible",!C)),f==="up"&&he||f==="down"&&C){N.clear();return}if((v.state.listElement||m.current)&&y.current&&y.current.length>0){const I=y.current,q=((X=U.current)==null?void 0:X.offsetHeight)||0;if(f==="up"){let A=0;const O=ae.scrollTop+q;for(let Q=0;Q<I.length;Q+=1){const te=I[Q];if(te&&te.offsetTop>=O){A=Q;break}}const W=Math.max(0,A-1);if(W<A){const Q=I[W];Q&&(ae.scrollTop=Math.max(0,Q.offsetTop-q))}else ae.scrollTop=0}else{let A=I.length-1;const O=ae.scrollTop+ae.clientHeight-q;for(let Q=0;Q<I.length;Q+=1){const te=I[Q];if(te&&te.offsetTop+te.offsetHeight>O){A=Math.max(0,Q-1);break}}const W=Math.min(I.length-1,A+1);if(W>A){const Q=I[W];Q&&(ae.scrollTop=Q.offsetTop+Q.offsetHeight-ae.clientHeight+q)}else ae.scrollTop=ae.scrollHeight-ae.clientHeight}}N.start(40,F)}N.start(40,F)},onMouseLeave(){N.clear()}},p]});return _||d?ge:null}),xA=x.forwardRef(function(i,o){return L.jsx(D0,{...i,ref:o,direction:"down"})}),SA=x.forwardRef(function(i,o){return L.jsx(D0,{...i,ref:o,direction:"up"})}),Nv=kR;function Dv({className:l,...i}){return L.jsx(YR,{"data-slot":"select-value",className:mn("flex flex-1 text-left",l),...i})}function kv({className:l,size:i="default",children:o,...r}){return L.jsxs(HR,{"data-slot":"select-trigger","data-size":i,className:mn("flex w-fit items-center justify-between gap-1.5 rounded-lg border border-input bg-transparent py-2 pr-2 pl-2.5 text-sm whitespace-nowrap transition-colors outline-none select-none focus-visible:border-ring focus-visible:ring-3 focus-visible:ring-ring/50 disabled:cursor-not-allowed disabled:opacity-50 aria-invalid:border-destructive aria-invalid:ring-3 aria-invalid:ring-destructive/20 data-placeholder:text-muted-foreground data-[size=default]:h-8 data-[size=sm]:h-7 data-[size=sm]:rounded-[min(var(--radius-md),10px)] *:data-[slot=select-value]:line-clamp-1 *:data-[slot=select-value]:flex *:data-[slot=select-value]:items-center *:data-[slot=select-value]:gap-1.5 dark:bg-input/30 dark:hover:bg-input/50 dark:aria-invalid:border-destructive/50 dark:aria-invalid:ring-destructive/40 [&_svg]:pointer-events-none [&_svg]:shrink-0 [&_svg:not([class*='size-'])]:size-4",l),...r,children:[o,L.jsx(qR,{render:L.jsx(Gv,{className:"pointer-events-none size-4 text-muted-foreground"})})]})}function jv({className:l,children:i,side:o="bottom",sideOffset:r=4,align:c="center",alignOffset:f=0,alignItemWithTrigger:d=!0,...p}){return L.jsx(XR,{children:L.jsx(aA,{side:o,sideOffset:r,align:c,alignOffset:f,alignItemWithTrigger:d,className:"isolate z-50",children:L.jsxs(fA,{"data-slot":"select-content","data-align-trigger":d,className:mn("relative isolate z-50 max-h-(--available-height) w-(--anchor-width) min-w-36 origin-(--transform-origin) overflow-x-hidden overflow-y-auto rounded-lg bg-popover text-popover-foreground shadow-md ring-1 ring-foreground/10 duration-100 data-[align-trigger=true]:animate-none data-[side=bottom]:slide-in-from-top-2 data-[side=inline-end]:slide-in-from-left-2 data-[side=inline-start]:slide-in-from-right-2 data-[side=left]:slide-in-from-right-2 data-[side=right]:slide-in-from-left-2 data-[side=top]:slide-in-from-bottom-2 data-open:animate-in data-open:fade-in-0 data-open:zoom-in-95 data-closed:animate-out data-closed:fade-out-0 data-closed:zoom-out-95",l),...p,children:[L.jsx(EA,{}),L.jsx(hA,{children:i}),L.jsx(wA,{})]})})})}function Lv({className:l,children:i,...o}){return L.jsxs(pA,{"data-slot":"select-item",className:mn("relative flex w-full cursor-default items-center gap-1.5 rounded-md py-1 pr-8 pl-1.5 text-sm outline-hidden select-none focus:bg-accent focus:text-accent-foreground not-data-[variant=destructive]:focus:**:text-accent-foreground data-disabled:pointer-events-none data-disabled:opacity-50 [&_svg]:pointer-events-none [&_svg]:shrink-0 [&_svg:not([class*='size-'])]:size-4 *:[span]:last:flex *:[span]:last:items-center *:[span]:last:gap-2",l),...o,children:[L.jsx(bA,{className:"flex flex-1 shrink-0 gap-2 whitespace-nowrap",children:i}),L.jsx(vA,{render:L.jsx("span",{className:"pointer-events-none absolute right-2 flex size-4 items-center justify-center"}),children:L.jsx(qv,{className:"pointer-events-none"})})]})}function EA({className:l,...i}){return L.jsx(SA,{"data-slot":"select-scroll-up-button",className:mn("top-0 z-10 flex w-full cursor-default items-center justify-center bg-popover py-1 [&_svg:not([class*='size-'])]:size-4",l),...i,children:L.jsx(Gx,{})})}function wA({className:l,...i}){return L.jsx(xA,{"data-slot":"select-scroll-down-button",className:mn("bottom-0 z-10 flex w-full cursor-default items-center justify-center bg-popover py-1 [&_svg:not([class*='size-'])]:size-4",l),...i,children:L.jsx(Gv,{})})}function Uv({id:l,value:i,onChange:o,suggestions:r,placeholder:c}){const[f,d]=x.useState(!1),[p,v]=x.useState(""),m=x.useRef(null);x.useEffect(()=>{function h(E){m.current&&!m.current.contains(E.target)&&d(!1)}return document.addEventListener("mousedown",h),()=>document.removeEventListener("mousedown",h)},[]);const y=r.filter(h=>h.toLowerCase().includes((p||i).toLowerCase()));return L.jsxs("div",{ref:m,className:"relative",children:[L.jsx(Ny,{id:l,value:i,placeholder:c,onChange:h=>{o(h.target.value),v(h.target.value),d(!0)},onFocus:()=>d(!0)}),f&&y.length>0&&L.jsx("div",{className:"absolute z-50 mt-1 w-full rounded-md border bg-popover p-1 shadow-md",children:y.map(h=>L.jsx("button",{className:`w-full rounded px-2 py-1.5 text-left text-sm hover:bg-accent ${h===i?"bg-accent":""}`,onMouseDown:E=>{E.preventDefault(),o(h),v(""),d(!1)},children:h},h))})]})}function TA({open:l,onOpenChange:i}){const[o,r]=x.useState(null),[c,f]=x.useState([]),[d,p]=x.useState([]),[v,m]=x.useState({}),[y,h]=x.useState([]),[E,w]=x.useState([]),[S,R]=x.useState(!1);x.useEffect(()=>{l&&q2().then(T=>{r(T.config),f(T.modifier_key_options),p(T.toggle_key_options),m(T.key_labels),h(T.transcription_model_suggestions),w(T.fix_model_suggestions)})},[l]);async function M(){if(!o)return;R(!0);const T=await G2(o);R(!1),T?(Oi("Settings saved"),i(!1)):Oi.error("Failed to save settings")}return o?L.jsx(hR,{open:l,onOpenChange:i,children:L.jsxs(vR,{children:[L.jsx(yR,{children:L.jsx(xR,{children:"Settings"})}),L.jsxs("div",{className:"space-y-4",children:[L.jsxs("div",{className:"space-y-2",children:[L.jsx(Nl,{htmlFor:"modifier_key",children:"Modifier key"}),L.jsxs(Nv,{value:o.modifier_key,onValueChange:T=>T&&r({...o,modifier_key:T}),children:[L.jsx(kv,{id:"modifier_key",children:L.jsx(Dv,{children:v[o.modifier_key]||o.modifier_key})}),L.jsx(jv,{children:c.map(T=>L.jsx(Lv,{value:T,children:v[T]||T},T))})]})]}),L.jsxs("div",{className:"space-y-2",children:[L.jsx(Nl,{htmlFor:"toggle_key",children:"Toggle key"}),L.jsxs(Nv,{value:o.toggle_key,onValueChange:T=>T&&r({...o,toggle_key:T}),children:[L.jsx(kv,{id:"toggle_key",children:L.jsx(Dv,{children:v[o.toggle_key]||o.toggle_key})}),L.jsx(jv,{children:d.map(T=>L.jsx(Lv,{value:T,children:v[T]||T},T))})]})]}),L.jsxs("div",{className:"space-y-2",children:[L.jsx(Nl,{htmlFor:"transcription_model",children:"Transcription model"}),L.jsx(Uv,{id:"transcription_model",value:o.transcription_model,onChange:T=>r({...o,transcription_model:T}),suggestions:y})]}),L.jsxs("div",{className:"space-y-2",children:[L.jsx(Nl,{htmlFor:"fix_model",children:"Post-processing model"}),L.jsx(Uv,{id:"fix_model",value:o.fix_model,onChange:T=>r({...o,fix_model:T}),suggestions:E})]}),L.jsxs("div",{className:"flex items-center gap-2",children:[L.jsx(ko,{id:"no_fix",checked:o.no_fix,onCheckedChange:T=>r({...o,no_fix:T===!0})}),L.jsx(Nl,{htmlFor:"no_fix",children:"Disable post-processing"})]}),L.jsxs("div",{className:"flex items-center gap-2",children:[L.jsx(ko,{id:"sound",checked:o.sound,onCheckedChange:T=>r({...o,sound:T===!0})}),L.jsx(Nl,{htmlFor:"sound",children:"Play sound when mic is ready"})]}),L.jsxs("div",{className:"flex items-center gap-2",children:[L.jsx(ko,{id:"submit_with_return",checked:o.submit_with_return,onCheckedChange:T=>r({...o,submit_with_return:T===!0})}),L.jsxs(Nl,{htmlFor:"submit_with_return",children:["YOLO mode (submit with"," ",L.jsx("kbd",{className:"rounded border bg-muted px-1.5 py-0.5 text-xs font-mono",children:v[o.modifier_key]||o.modifier_key})," + ",L.jsx("kbd",{className:"rounded border bg-muted px-1.5 py-0.5 text-xs font-mono",children:"Return"}),")"]})]}),L.jsxs("div",{className:"flex items-center gap-2",children:[L.jsx(ko,{id:"save_audio",checked:o.save_audio,onCheckedChange:T=>r({...o,save_audio:T===!0})}),L.jsx(Nl,{htmlFor:"save_audio",children:"Save audio recordings"})]}),L.jsxs("div",{className:"flex items-center gap-2",children:[L.jsx(ko,{id:"mute_during_recording",checked:o.mute_during_recording,onCheckedChange:T=>r({...o,mute_during_recording:T===!0})}),L.jsx(Nl,{htmlFor:"mute_during_recording",children:"Mute audio output while recording"})]})]}),L.jsx(bR,{children:L.jsx(Go,{onClick:M,disabled:S,className:"w-full",children:S?"Saving...":"Save"})})]})}):null}const qf=30;function RA(){const[l,i]=x.useState(null),[o,r]=x.useState([]),[c,f]=x.useState(window.location.hash==="#settings"),[d,p]=x.useState(""),[v,m]=x.useState(0),[y,h]=x.useState(document.documentElement.classList.contains("dark"));function E(){const _=!y;h(_),document.documentElement.classList.toggle("dark",_)}const w=x.useMemo(()=>{if(!d.trim())return o;const _=d.toLowerCase();return o.filter(N=>N.raw_text.toLowerCase().includes(_)||N.fixed_text&&N.fixed_text.toLowerCase().includes(_)||N.app_name&&N.app_name.toLowerCase().includes(_))},[o,d]),S=Math.max(1,Math.ceil(w.length/qf)),R=Math.min(v,S-1),M=w.slice(R*qf,(R+1)*qf);function T(){V2().then(i),Y2(1e3).then(r)}function B(_){f(_),window.location.hash=_?"#settings":""}return x.useEffect(()=>{T();const _=setInterval(T,5e3),N=()=>T(),U=()=>{f(window.location.hash==="#settings")},G=window.matchMedia("(prefers-color-scheme: dark)"),Y=D=>{document.documentElement.classList.toggle("dark",D.matches)};return G.addEventListener("change",Y),window.addEventListener("focus",N),window.addEventListener("hashchange",U),()=>{clearInterval(_),window.removeEventListener("focus",N),window.removeEventListener("hashchange",U),G.removeEventListener("change",Y)}},[]),L.jsxs("div",{className:"flex flex-col h-screen max-w-3xl mx-auto px-4",children:[L.jsxs("header",{className:"flex justify-between items-center py-4 shrink-0",children:[L.jsxs("h1",{className:"text-xl font-semibold flex items-center gap-2",children:[L.jsx("img",{src:"/icon.svg",alt:"",className:"w-7 h-7 dark:invert"}),"Wordbird"]}),L.jsxs("div",{className:"flex items-center gap-1",children:[L.jsx("a",{href:"https://github.com/tillahoffmann/wordbird",target:"_blank",rel:"noopener noreferrer",className:"text-muted-foreground hover:text-foreground p-2 rounded-lg transition-colors",title:"GitHub",children:L.jsx("svg",{className:"w-5 h-5",fill:"currentColor",viewBox:"0 0 24 24",children:L.jsx("path",{d:"M12 0C5.37 0 0 5.37 0 12c0 5.31 3.435 9.795 8.205 11.385.6.105.825-.255.825-.57 0-.285-.015-1.23-.015-2.235-3.015.555-3.795-.735-4.035-1.41-.135-.345-.72-1.41-1.23-1.695-.42-.225-1.02-.78-.015-.795.945-.015 1.62.87 1.845 1.23 1.08 1.815 2.805 1.305 3.495.99.105-.78.42-1.305.765-1.605-2.67-.3-5.46-1.335-5.46-5.925 0-1.305.465-2.385 1.23-3.225-.12-.3-.54-1.53.12-3.18 0 0 1.005-.315 3.3 1.23.96-.27 1.98-.405 3-.405s2.04.135 3 .405c2.295-1.56 3.3-1.23 3.3-1.23.66 1.65.24 2.88.12 3.18.765.84 1.23 1.905 1.23 3.225 0 4.605-2.805 5.625-5.475 5.925.435.375.81 1.095.81 2.22 0 1.605-.015 2.895-.015 3.3 0 .315.225.69.825.57A12.02 12.02 0 0024 12c0-6.63-5.37-12-12-12z"})})}),L.jsx("button",{onClick:E,className:"text-muted-foreground hover:text-foreground p-2 rounded-lg transition-colors",title:y?"Light mode":"Dark mode",children:y?L.jsxs("svg",{className:"w-5 h-5",fill:"none",stroke:"currentColor",viewBox:"0 0 24 24",strokeWidth:2,strokeLinecap:"round",children:[L.jsx("circle",{cx:"12",cy:"12",r:"5"}),L.jsx("path",{d:"M12 1v2M12 21v2M4.22 4.22l1.42 1.42M18.36 18.36l1.42 1.42M1 12h2M21 12h2M4.22 19.78l1.42-1.42M18.36 5.64l1.42-1.42"})]}):L.jsx("svg",{className:"w-5 h-5",fill:"none",stroke:"currentColor",viewBox:"0 0 24 24",strokeWidth:2,strokeLinecap:"round",children:L.jsx("path",{d:"M21 12.79A9 9 0 1111.21 3 7 7 0 0021 12.79z"})})}),L.jsx("button",{onClick:()=>B(!0),className:"text-muted-foreground hover:text-foreground p-2 rounded-lg transition-colors",title:"Settings",children:L.jsx("svg",{className:"w-5 h-5",fill:"none",stroke:"currentColor",viewBox:"0 0 24 24",children:L.jsx("path",{strokeLinecap:"round",strokeLinejoin:"round",strokeWidth:2,d:"M12 6V4m0 2a2 2 0 100 4m0-4a2 2 0 110 4m-6 8a2 2 0 100-4m0 4a2 2 0 110-4m0 4v2m0-6V4m6 6v10m6-2a2 2 0 100-4m0 4a2 2 0 110-4m0 4v2m0-6V4"})})})]})]}),L.jsx(L2,{stats:l}),o.length>0&&L.jsx("div",{className:"shrink-0 mb-3",children:L.jsx(Ny,{placeholder:"Search transcriptions...",value:d,onChange:_=>{p(_.target.value),m(0)}})}),L.jsx("div",{className:"flex-1 overflow-y-auto min-h-0 pb-4",children:L.jsx(I2,{transcriptions:M,onDelete:T})}),S>1&&L.jsxs("div",{className:"shrink-0 flex items-center justify-center gap-2 py-3 border-t",children:[L.jsx(Go,{variant:"outline",size:"sm",onClick:()=>m(R-1),disabled:R===0,children:"Previous"}),L.jsxs("span",{className:"text-sm text-muted-foreground tabular-nums",children:[R+1," / ",S]}),L.jsx(Go,{variant:"outline",size:"sm",onClick:()=>m(R+1),disabled:R>=S-1,children:"Next"})]}),L.jsx(TA,{open:c,onOpenChange:B}),L.jsx(nS,{})]})}nx.createRoot(document.getElementById("root")).render(L.jsx(x.StrictMode,{children:L.jsx(RA,{})}));
